/* trust_level.c generated by valac 0.56.19, the Vala compiler
 * generated from trust_level.vala, do not modify */

#include "omemo.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gchar*
dino_plugins_omemo_trust_level_to_string (DinoPluginsOmemoTrustLevel self)
{
	gint val = 0;
	gchar* _tmp0_;
	gchar* result;
	val = (gint) self;
	_tmp0_ = g_strdup_printf ("%i", val);
	result = _tmp0_;
	return result;
}

static GType
dino_plugins_omemo_trust_level_get_type_once (void)
{
	static const GEnumValue values[] = {{DINO_PLUGINS_OMEMO_TRUST_LEVEL_VERIFIED, "DINO_PLUGINS_OMEMO_TRUST_LEVEL_VERIFIED", "verified"}, {DINO_PLUGINS_OMEMO_TRUST_LEVEL_TRUSTED, "DINO_PLUGINS_OMEMO_TRUST_LEVEL_TRUSTED", "trusted"}, {DINO_PLUGINS_OMEMO_TRUST_LEVEL_UNTRUSTED, "DINO_PLUGINS_OMEMO_TRUST_LEVEL_UNTRUSTED", "untrusted"}, {DINO_PLUGINS_OMEMO_TRUST_LEVEL_UNKNOWN, "DINO_PLUGINS_OMEMO_TRUST_LEVEL_UNKNOWN", "unknown"}, {0, NULL, NULL}};
	GType dino_plugins_omemo_trust_level_type_id;
	dino_plugins_omemo_trust_level_type_id = g_enum_register_static ("DinoPluginsOmemoTrustLevel", values);
	return dino_plugins_omemo_trust_level_type_id;
}

GType
dino_plugins_omemo_trust_level_get_type (void)
{
	static volatile gsize dino_plugins_omemo_trust_level_type_id__once = 0;
	if (g_once_init_enter (&dino_plugins_omemo_trust_level_type_id__once)) {
		GType dino_plugins_omemo_trust_level_type_id;
		dino_plugins_omemo_trust_level_type_id = dino_plugins_omemo_trust_level_get_type_once ();
		g_once_init_leave (&dino_plugins_omemo_trust_level_type_id__once, dino_plugins_omemo_trust_level_type_id);
	}
	return dino_plugins_omemo_trust_level_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

