/* session_info_type.c generated by valac 0.56.19, the Vala compiler
 * generated from session_info_type.vala, do not modify */

#include "xmpp-vala.h"
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_0_PROPERTY,
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI_PROPERTY,
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NUM_PROPERTIES
};
static GParamSpec* xmpp_xep_jingle_rtp_session_info_type_properties[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _xmpp_stanza_entry_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_stanza_entry_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
enum  {
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL,
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_MUTE_UPDATE_RECEIVED_SIGNAL,
	XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NUM_SIGNALS
};
static guint xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NUM_SIGNALS] = {0};

static gpointer xmpp_xep_jingle_rtp_session_info_type_parent_class = NULL;
static XmppXepJingleSessionInfoNsIface * xmpp_xep_jingle_rtp_session_info_type_xmpp_xep_jingle_session_info_ns_parent_iface = NULL;

static void xmpp_xep_jingle_rtp_session_info_type_real_handle_content_session_info (XmppXepJingleSessionInfoNs* base,
                                                                             XmppXmppStream* stream,
                                                                             XmppXepJingleSession* session,
                                                                             XmppStanzaNode* info,
                                                                             XmppIqStanza* iq,
                                                                             GError** error);
VALA_EXTERN void xmpp_xep_jingle_session_send_session_info (XmppXepJingleSession* self,
                                                XmppStanzaNode* child_node);
static void g_cclosure_user_marshal_VOID__OBJECT_ENUM (GClosure * closure,
                                                GValue * return_value,
                                                guint n_param_values,
                                                const GValue * param_values,
                                                gpointer invocation_hint,
                                                gpointer marshal_data);
static void g_cclosure_user_marshal_VOID__OBJECT_BOOLEAN_STRING (GClosure * closure,
                                                          GValue * return_value,
                                                          guint n_param_values,
                                                          const GValue * param_values,
                                                          gpointer invocation_hint,
                                                          gpointer marshal_data);
static GType xmpp_xep_jingle_rtp_session_info_type_get_type_once (void);
static void _vala_xmpp_xep_jingle_rtp_session_info_type_get_property (GObject * object,
                                                               guint property_id,
                                                               GValue * value,
                                                               GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static GType
xmpp_xep_jingle_rtp_call_session_info_get_type_once (void)
{
	static const GEnumValue values[] = {{XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_ACTIVE, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_ACTIVE", "active"}, {XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_HOLD, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_HOLD", "hold"}, {XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNHOLD, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNHOLD", "unhold"}, {XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_MUTE, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_MUTE", "mute"}, {XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNMUTE, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNMUTE", "unmute"}, {XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_RINGING, "XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_RINGING", "ringing"}, {0, NULL, NULL}};
	GType xmpp_xep_jingle_rtp_call_session_info_type_id;
	xmpp_xep_jingle_rtp_call_session_info_type_id = g_enum_register_static ("XmppXepJingleRtpCallSessionInfo", values);
	return xmpp_xep_jingle_rtp_call_session_info_type_id;
}

GType
xmpp_xep_jingle_rtp_call_session_info_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_rtp_call_session_info_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_rtp_call_session_info_type_id__once)) {
		GType xmpp_xep_jingle_rtp_call_session_info_type_id;
		xmpp_xep_jingle_rtp_call_session_info_type_id = xmpp_xep_jingle_rtp_call_session_info_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_rtp_call_session_info_type_id__once, xmpp_xep_jingle_rtp_call_session_info_type_id);
	}
	return xmpp_xep_jingle_rtp_call_session_info_type_id__once;
}

static void
xmpp_xep_jingle_rtp_session_info_type_real_handle_content_session_info (XmppXepJingleSessionInfoNs* base,
                                                                        XmppXmppStream* stream,
                                                                        XmppXepJingleSession* session,
                                                                        XmppStanzaNode* info,
                                                                        XmppIqStanza* iq,
                                                                        GError** error)
{
	XmppXepJingleRtpSessionInfoType * self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	GQuark _tmp3_ = 0U;
	static GQuark _tmp2_label0 = 0;
	static GQuark _tmp2_label1 = 0;
	static GQuark _tmp2_label2 = 0;
	static GQuark _tmp2_label3 = 0;
	static GQuark _tmp2_label4 = 0;
	static GQuark _tmp2_label5 = 0;
	self = (XmppXepJingleRtpSessionInfoType*) base;
	g_return_if_fail (stream != NULL);
	g_return_if_fail (session != NULL);
	g_return_if_fail (info != NULL);
	g_return_if_fail (iq != NULL);
	_tmp0_ = ((XmppStanzaEntry*) info)->name;
	_tmp1_ = _tmp0_;
	_tmp3_ = (NULL == _tmp1_) ? 0 : g_quark_from_string (_tmp1_);
	if (_tmp3_ == ((0 != _tmp2_label0) ? _tmp2_label0 : (_tmp2_label0 = g_quark_from_static_string ("active")))) {
		switch (0) {
			default:
			{
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_ACTIVE);
				break;
			}
		}
	} else if (_tmp3_ == ((0 != _tmp2_label1) ? _tmp2_label1 : (_tmp2_label1 = g_quark_from_static_string ("hold")))) {
		switch (0) {
			default:
			{
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_HOLD);
				break;
			}
		}
	} else if (_tmp3_ == ((0 != _tmp2_label2) ? _tmp2_label2 : (_tmp2_label2 = g_quark_from_static_string ("unhold")))) {
		switch (0) {
			default:
			{
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNHOLD);
				break;
			}
		}
	} else if (_tmp3_ == ((0 != _tmp2_label3) ? _tmp2_label3 : (_tmp2_label3 = g_quark_from_static_string ("mute")))) {
		switch (0) {
			default:
			{
				gchar* name = NULL;
				const gchar* _tmp4_;
				gchar* _tmp5_;
				const gchar* _tmp6_;
				_tmp4_ = xmpp_stanza_node_get_attribute (info, "name", NULL);
				_tmp5_ = g_strdup (_tmp4_);
				name = _tmp5_;
				_tmp6_ = name;
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_MUTE_UPDATE_RECEIVED_SIGNAL], 0, session, TRUE, _tmp6_);
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_MUTE);
				_g_free0 (name);
				break;
			}
		}
	} else if (_tmp3_ == ((0 != _tmp2_label4) ? _tmp2_label4 : (_tmp2_label4 = g_quark_from_static_string ("unmute")))) {
		switch (0) {
			default:
			{
				gchar* name = NULL;
				const gchar* _tmp7_;
				gchar* _tmp8_;
				const gchar* _tmp9_;
				_tmp7_ = xmpp_stanza_node_get_attribute (info, "name", NULL);
				_tmp8_ = g_strdup (_tmp7_);
				name = _tmp8_;
				_tmp9_ = name;
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_MUTE_UPDATE_RECEIVED_SIGNAL], 0, session, FALSE, _tmp9_);
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_UNMUTE);
				_g_free0 (name);
				break;
			}
		}
	} else if (_tmp3_ == ((0 != _tmp2_label5) ? _tmp2_label5 : (_tmp2_label5 = g_quark_from_static_string ("ringing")))) {
		switch (0) {
			default:
			{
				g_signal_emit (self, xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL], 0, session, XMPP_XEP_JINGLE_RTP_CALL_SESSION_INFO_RINGING);
				break;
			}
		}
	}
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
xmpp_xep_jingle_rtp_session_info_type_send_mute (XmppXepJingleRtpSessionInfoType* self,
                                                 XmppXepJingleSession* session,
                                                 gboolean mute,
                                                 const gchar* media)
{
	const gchar* _tmp0_ = NULL;
	gchar* node_name = NULL;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (session != NULL);
	g_return_if_fail (media != NULL);
	if (mute) {
		_tmp0_ = "mute";
	} else {
		_tmp0_ = "unmute";
	}
	_tmp1_ = g_strdup (_tmp0_);
	node_name = _tmp1_;
	{
		GeeList* _content_list = NULL;
		GeeList* _tmp2_;
		gint _content_size = 0;
		GeeList* _tmp3_;
		gint _tmp4_;
		gint _tmp5_;
		gint _content_index = 0;
		_tmp2_ = session->contents;
		_content_list = _tmp2_;
		_tmp3_ = _content_list;
		_tmp4_ = gee_collection_get_size ((GeeCollection*) _tmp3_);
		_tmp5_ = _tmp4_;
		_content_size = _tmp5_;
		_content_index = -1;
		while (TRUE) {
			gint _tmp6_;
			gint _tmp7_;
			XmppXepJingleContent* content = NULL;
			GeeList* _tmp8_;
			gpointer _tmp9_;
			XmppXepJingleRtpParameters* parameters = NULL;
			XmppXepJingleContent* _tmp10_;
			XmppXepJingleContentParameters* _tmp11_;
			XmppXepJingleRtpParameters* _tmp12_;
			gboolean _tmp13_ = FALSE;
			XmppXepJingleRtpParameters* _tmp14_;
			_content_index = _content_index + 1;
			_tmp6_ = _content_index;
			_tmp7_ = _content_size;
			if (!(_tmp6_ < _tmp7_)) {
				break;
			}
			_tmp8_ = _content_list;
			_tmp9_ = gee_list_get (_tmp8_, _content_index);
			content = (XmppXepJingleContent*) _tmp9_;
			_tmp10_ = content;
			_tmp11_ = _tmp10_->content_params;
			_tmp12_ = _g_object_ref0 (XMPP_XEP_JINGLE_RTP_IS_PARAMETERS (_tmp11_) ? ((XmppXepJingleRtpParameters*) _tmp11_) : NULL);
			parameters = _tmp12_;
			_tmp14_ = parameters;
			if (_tmp14_ != NULL) {
				XmppXepJingleRtpParameters* _tmp15_;
				const gchar* _tmp16_;
				const gchar* _tmp17_;
				_tmp15_ = parameters;
				_tmp16_ = xmpp_xep_jingle_rtp_parameters_get_media (_tmp15_);
				_tmp17_ = _tmp16_;
				_tmp13_ = g_strcmp0 (_tmp17_, media) == 0;
			} else {
				_tmp13_ = FALSE;
			}
			if (_tmp13_) {
				XmppStanzaNode* session_info_content = NULL;
				const gchar* _tmp18_;
				XmppStanzaNode* _tmp19_;
				XmppStanzaNode* _tmp20_;
				XmppStanzaNode* _tmp21_;
				XmppStanzaNode* _tmp22_;
				XmppXepJingleContent* _tmp23_;
				const gchar* _tmp24_;
				const gchar* _tmp25_;
				XmppStanzaNode* _tmp26_;
				XmppStanzaNode* _tmp27_;
				XmppXepJingleContent* _tmp28_;
				XmppXepJingleRole _tmp29_;
				XmppXepJingleRole _tmp30_;
				gchar* _tmp31_;
				gchar* _tmp32_;
				XmppStanzaNode* _tmp33_;
				XmppStanzaNode* _tmp34_;
				XmppStanzaNode* _tmp35_;
				_tmp18_ = node_name;
				_tmp19_ = xmpp_stanza_node_new_build (_tmp18_, XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI, NULL, NULL);
				_tmp20_ = _tmp19_;
				_tmp21_ = xmpp_stanza_node_add_self_xmlns (_tmp20_);
				_tmp22_ = _tmp21_;
				_tmp23_ = content;
				_tmp24_ = xmpp_xep_jingle_content_get_content_name (_tmp23_);
				_tmp25_ = _tmp24_;
				_tmp26_ = xmpp_stanza_node_put_attribute (_tmp22_, "name", _tmp25_, NULL);
				_tmp27_ = _tmp26_;
				_tmp28_ = content;
				_tmp29_ = xmpp_xep_jingle_content_get_content_creator (_tmp28_);
				_tmp30_ = _tmp29_;
				_tmp31_ = xmpp_xep_jingle_role_to_string (_tmp30_);
				_tmp32_ = _tmp31_;
				_tmp33_ = xmpp_stanza_node_put_attribute (_tmp27_, "creator", _tmp32_, NULL);
				_tmp34_ = _tmp33_;
				_g_free0 (_tmp32_);
				_xmpp_stanza_entry_unref0 (_tmp27_);
				_xmpp_stanza_entry_unref0 (_tmp22_);
				_xmpp_stanza_entry_unref0 (_tmp20_);
				session_info_content = _tmp34_;
				_tmp35_ = session_info_content;
				xmpp_xep_jingle_session_send_session_info (session, _tmp35_);
				_xmpp_stanza_entry_unref0 (session_info_content);
			}
			_g_object_unref0 (parameters);
			_g_object_unref0 (content);
		}
	}
	_g_free0 (node_name);
}

void
xmpp_xep_jingle_rtp_session_info_type_send_ringing (XmppXepJingleRtpSessionInfoType* self,
                                                    XmppXepJingleSession* session)
{
	XmppStanzaNode* session_info_content = NULL;
	XmppStanzaNode* _tmp0_;
	XmppStanzaNode* _tmp1_;
	XmppStanzaNode* _tmp2_;
	XmppStanzaNode* _tmp3_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (session != NULL);
	_tmp0_ = xmpp_stanza_node_new_build ("ringing", XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI, NULL, NULL);
	_tmp1_ = _tmp0_;
	_tmp2_ = xmpp_stanza_node_add_self_xmlns (_tmp1_);
	_tmp3_ = _tmp2_;
	_xmpp_stanza_entry_unref0 (_tmp1_);
	session_info_content = _tmp3_;
	xmpp_xep_jingle_session_send_session_info (session, session_info_content);
	_xmpp_stanza_entry_unref0 (session_info_content);
}

XmppXepJingleRtpSessionInfoType*
xmpp_xep_jingle_rtp_session_info_type_construct (GType object_type)
{
	XmppXepJingleRtpSessionInfoType * self = NULL;
	self = (XmppXepJingleRtpSessionInfoType*) g_object_new (object_type, NULL);
	return self;
}

XmppXepJingleRtpSessionInfoType*
xmpp_xep_jingle_rtp_session_info_type_new (void)
{
	return xmpp_xep_jingle_rtp_session_info_type_construct (XMPP_XEP_JINGLE_RTP_TYPE_SESSION_INFO_TYPE);
}

static const gchar*
xmpp_xep_jingle_rtp_session_info_type_real_get_ns_uri (XmppXepJingleSessionInfoNs* base)
{
	const gchar* result;
	XmppXepJingleRtpSessionInfoType* self;
	self = (XmppXepJingleRtpSessionInfoType*) base;
	result = XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI;
	return result;
}

static void
g_cclosure_user_marshal_VOID__OBJECT_ENUM (GClosure * closure,
                                           GValue * return_value,
                                           guint n_param_values,
                                           const GValue * param_values,
                                           gpointer invocation_hint,
                                           gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__OBJECT_ENUM) (gpointer data1, gpointer arg_1, gint arg_2, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_ENUM callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__OBJECT_ENUM) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_object (param_values + 1), g_value_get_enum (param_values + 2), data2);
}

static void
g_cclosure_user_marshal_VOID__OBJECT_BOOLEAN_STRING (GClosure * closure,
                                                     GValue * return_value,
                                                     guint n_param_values,
                                                     const GValue * param_values,
                                                     gpointer invocation_hint,
                                                     gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__OBJECT_BOOLEAN_STRING) (gpointer data1, gpointer arg_1, gboolean arg_2, const char* arg_3, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_BOOLEAN_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__OBJECT_BOOLEAN_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_object (param_values + 1), g_value_get_boolean (param_values + 2), g_value_get_string (param_values + 3), data2);
}

static void
xmpp_xep_jingle_rtp_session_info_type_class_init (XmppXepJingleRtpSessionInfoTypeClass * klass,
                                                  gpointer klass_data)
{
	xmpp_xep_jingle_rtp_session_info_type_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = _vala_xmpp_xep_jingle_rtp_session_info_type_get_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI_PROPERTY, xmpp_xep_jingle_rtp_session_info_type_properties[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI_PROPERTY] = g_param_spec_string ("ns-uri", "ns-uri", "ns-uri", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_INFO_RECEIVED_SIGNAL] = g_signal_new ("info-received", XMPP_XEP_JINGLE_RTP_TYPE_SESSION_INFO_TYPE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_ENUM, G_TYPE_NONE, 2, XMPP_XEP_JINGLE_TYPE_SESSION, XMPP_XEP_JINGLE_RTP_TYPE_CALL_SESSION_INFO);
	xmpp_xep_jingle_rtp_session_info_type_signals[XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_MUTE_UPDATE_RECEIVED_SIGNAL] = g_signal_new ("mute-update-received", XMPP_XEP_JINGLE_RTP_TYPE_SESSION_INFO_TYPE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_BOOLEAN_STRING, G_TYPE_NONE, 3, XMPP_XEP_JINGLE_TYPE_SESSION, G_TYPE_BOOLEAN, G_TYPE_STRING);
}

static void
xmpp_xep_jingle_rtp_session_info_type_xmpp_xep_jingle_session_info_ns_interface_init (XmppXepJingleSessionInfoNsIface * iface,
                                                                                      gpointer iface_data)
{
	xmpp_xep_jingle_rtp_session_info_type_xmpp_xep_jingle_session_info_ns_parent_iface = g_type_interface_peek_parent (iface);
	iface->handle_content_session_info = (void (*) (XmppXepJingleSessionInfoNs*, XmppXmppStream*, XmppXepJingleSession*, XmppStanzaNode*, XmppIqStanza*, GError**)) xmpp_xep_jingle_rtp_session_info_type_real_handle_content_session_info;
	iface->get_ns_uri = xmpp_xep_jingle_rtp_session_info_type_real_get_ns_uri;
}

static void
xmpp_xep_jingle_rtp_session_info_type_instance_init (XmppXepJingleRtpSessionInfoType * self,
                                                     gpointer klass)
{
}

static GType
xmpp_xep_jingle_rtp_session_info_type_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepJingleRtpSessionInfoTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_jingle_rtp_session_info_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XmppXepJingleRtpSessionInfoType), 0, (GInstanceInitFunc) xmpp_xep_jingle_rtp_session_info_type_instance_init, NULL };
	static const GInterfaceInfo xmpp_xep_jingle_session_info_ns_info = { (GInterfaceInitFunc) xmpp_xep_jingle_rtp_session_info_type_xmpp_xep_jingle_session_info_ns_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType xmpp_xep_jingle_rtp_session_info_type_type_id;
	xmpp_xep_jingle_rtp_session_info_type_type_id = g_type_register_static (G_TYPE_OBJECT, "XmppXepJingleRtpSessionInfoType", &g_define_type_info, 0);
	g_type_add_interface_static (xmpp_xep_jingle_rtp_session_info_type_type_id, XMPP_XEP_JINGLE_TYPE_SESSION_INFO_NS, &xmpp_xep_jingle_session_info_ns_info);
	return xmpp_xep_jingle_rtp_session_info_type_type_id;
}

GType
xmpp_xep_jingle_rtp_session_info_type_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_rtp_session_info_type_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_rtp_session_info_type_type_id__once)) {
		GType xmpp_xep_jingle_rtp_session_info_type_type_id;
		xmpp_xep_jingle_rtp_session_info_type_type_id = xmpp_xep_jingle_rtp_session_info_type_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_rtp_session_info_type_type_id__once, xmpp_xep_jingle_rtp_session_info_type_type_id);
	}
	return xmpp_xep_jingle_rtp_session_info_type_type_id__once;
}

static void
_vala_xmpp_xep_jingle_rtp_session_info_type_get_property (GObject * object,
                                                          guint property_id,
                                                          GValue * value,
                                                          GParamSpec * pspec)
{
	XmppXepJingleRtpSessionInfoType * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, XMPP_XEP_JINGLE_RTP_TYPE_SESSION_INFO_TYPE, XmppXepJingleRtpSessionInfoType);
	switch (property_id) {
		case XMPP_XEP_JINGLE_RTP_SESSION_INFO_TYPE_NS_URI_PROPERTY:
		g_value_set_string (value, xmpp_xep_jingle_session_info_ns_get_ns_uri ((XmppXepJingleSessionInfoNs*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

