/* size_request_box.c generated by valac 0.56.19, the Vala compiler
 * generated from size_request_box.vala, do not modify */

#include <gtk/gtk.h>
#include <glib-object.h>
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_TYPE_SIZE_REQUEST_BOX (dino_ui_size_request_box_get_type ())
#define DINO_UI_SIZE_REQUEST_BOX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBox))
#define DINO_UI_SIZE_REQUEST_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBoxClass))
#define DINO_UI_IS_SIZE_REQUEST_BOX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_SIZE_REQUEST_BOX))
#define DINO_UI_IS_SIZE_REQUEST_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_SIZE_REQUEST_BOX))
#define DINO_UI_SIZE_REQUEST_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBoxClass))

typedef struct _DinoUiSizeRequestBox DinoUiSizeRequestBox;
typedef struct _DinoUiSizeRequestBoxClass DinoUiSizeRequestBoxClass;
typedef struct _DinoUiSizeRequestBoxPrivate DinoUiSizeRequestBoxPrivate;
enum  {
	DINO_UI_SIZE_REQUEST_BOX_0_PROPERTY,
	DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY,
	DINO_UI_SIZE_REQUEST_BOX_NUM_PROPERTIES
};
static GParamSpec* dino_ui_size_request_box_properties[DINO_UI_SIZE_REQUEST_BOX_NUM_PROPERTIES];

#define DINO_UI_TYPE_SIZE_REQUEST_BIN (dino_ui_size_request_bin_get_type ())
#define DINO_UI_SIZE_REQUEST_BIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBin))
#define DINO_UI_SIZE_REQUEST_BIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBinClass))
#define DINO_UI_IS_SIZE_REQUEST_BIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_SIZE_REQUEST_BIN))
#define DINO_UI_IS_SIZE_REQUEST_BIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_SIZE_REQUEST_BIN))
#define DINO_UI_SIZE_REQUEST_BIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBinClass))

typedef struct _DinoUiSizeRequestBin DinoUiSizeRequestBin;
typedef struct _DinoUiSizeRequestBinClass DinoUiSizeRequestBinClass;
typedef struct _DinoUiSizeRequestBinPrivate DinoUiSizeRequestBinPrivate;
enum  {
	DINO_UI_SIZE_REQUEST_BIN_0_PROPERTY,
	DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY,
	DINO_UI_SIZE_REQUEST_BIN_NUM_PROPERTIES
};
static GParamSpec* dino_ui_size_request_bin_properties[DINO_UI_SIZE_REQUEST_BIN_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _DinoUiSizeRequestBox {
	GtkBox parent_instance;
	DinoUiSizeRequestBoxPrivate * priv;
};

struct _DinoUiSizeRequestBoxClass {
	GtkBoxClass parent_class;
};

struct _DinoUiSizeRequestBoxPrivate {
	GtkSizeRequestMode _size_request_mode;
};

struct _DinoUiSizeRequestBin {
	GtkWidget parent_instance;
	DinoUiSizeRequestBinPrivate * priv;
};

struct _DinoUiSizeRequestBinClass {
	GtkWidgetClass parent_class;
};

struct _DinoUiSizeRequestBinPrivate {
	GtkSizeRequestMode _size_request_mode;
};

static gint DinoUiSizeRequestBox_private_offset;
static gpointer dino_ui_size_request_box_parent_class = NULL;
static gint DinoUiSizeRequestBin_private_offset;
static gpointer dino_ui_size_request_bin_parent_class = NULL;

VALA_EXTERN GType dino_ui_size_request_box_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiSizeRequestBox, g_object_unref)
static GtkSizeRequestMode dino_ui_size_request_box_real_get_request_mode (GtkWidget* base);
VALA_EXTERN GtkSizeRequestMode dino_ui_size_request_box_get_size_request_mode (DinoUiSizeRequestBox* self);
VALA_EXTERN DinoUiSizeRequestBox* dino_ui_size_request_box_new (void);
VALA_EXTERN DinoUiSizeRequestBox* dino_ui_size_request_box_construct (GType object_type);
VALA_EXTERN void dino_ui_size_request_box_set_size_request_mode (DinoUiSizeRequestBox* self,
                                                     GtkSizeRequestMode value);
static void dino_ui_size_request_box_finalize (GObject * obj);
static GType dino_ui_size_request_box_get_type_once (void);
static void _vala_dino_ui_size_request_box_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec);
static void _vala_dino_ui_size_request_box_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec);
VALA_EXTERN GType dino_ui_size_request_bin_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiSizeRequestBin, g_object_unref)
static void dino_ui_size_request_bin_real_compute_expand_internal (GtkWidget* base,
                                                            gboolean* hexpand,
                                                            gboolean* vexpand);
static GtkSizeRequestMode dino_ui_size_request_bin_real_get_request_mode (GtkWidget* base);
VALA_EXTERN GtkSizeRequestMode dino_ui_size_request_bin_get_size_request_mode (DinoUiSizeRequestBin* self);
VALA_EXTERN DinoUiSizeRequestBin* dino_ui_size_request_bin_new (void);
VALA_EXTERN DinoUiSizeRequestBin* dino_ui_size_request_bin_construct (GType object_type);
VALA_EXTERN void dino_ui_size_request_bin_set_size_request_mode (DinoUiSizeRequestBin* self,
                                                     GtkSizeRequestMode value);
static GObject * dino_ui_size_request_bin_constructor (GType type,
                                                guint n_construct_properties,
                                                GObjectConstructParam * construct_properties);
static void dino_ui_size_request_bin_finalize (GObject * obj);
static GType dino_ui_size_request_bin_get_type_once (void);
static void _vala_dino_ui_size_request_bin_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec);
static void _vala_dino_ui_size_request_bin_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
dino_ui_size_request_box_get_instance_private (DinoUiSizeRequestBox* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiSizeRequestBox_private_offset);
}

static GtkSizeRequestMode
dino_ui_size_request_box_real_get_request_mode (GtkWidget* base)
{
	DinoUiSizeRequestBox * self;
	GtkSizeRequestMode _tmp0_;
	GtkSizeRequestMode result;
	self = (DinoUiSizeRequestBox*) base;
	_tmp0_ = self->priv->_size_request_mode;
	result = _tmp0_;
	return result;
}

DinoUiSizeRequestBox*
dino_ui_size_request_box_construct (GType object_type)
{
	DinoUiSizeRequestBox * self = NULL;
	self = (DinoUiSizeRequestBox*) g_object_new (object_type, NULL);
	return self;
}

DinoUiSizeRequestBox*
dino_ui_size_request_box_new (void)
{
	return dino_ui_size_request_box_construct (DINO_UI_TYPE_SIZE_REQUEST_BOX);
}

GtkSizeRequestMode
dino_ui_size_request_box_get_size_request_mode (DinoUiSizeRequestBox* self)
{
	GtkSizeRequestMode result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_size_request_mode;
	return result;
}

void
dino_ui_size_request_box_set_size_request_mode (DinoUiSizeRequestBox* self,
                                                GtkSizeRequestMode value)
{
	GtkSizeRequestMode old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_size_request_box_get_size_request_mode (self);
	if (old_value != value) {
		self->priv->_size_request_mode = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_size_request_box_properties[DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY]);
	}
}

static void
dino_ui_size_request_box_class_init (DinoUiSizeRequestBoxClass * klass,
                                     gpointer klass_data)
{
	dino_ui_size_request_box_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiSizeRequestBox_private_offset);
	((GtkWidgetClass *) klass)->get_request_mode = (GtkSizeRequestMode (*) (GtkWidget*)) dino_ui_size_request_box_real_get_request_mode;
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_size_request_box_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_size_request_box_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_size_request_box_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY, dino_ui_size_request_box_properties[DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY] = g_param_spec_enum ("size-request-mode", "size-request-mode", "size-request-mode", gtk_size_request_mode_get_type (), GTK_SIZE_REQUEST_CONSTANT_SIZE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_size_request_box_instance_init (DinoUiSizeRequestBox * self,
                                        gpointer klass)
{
	self->priv = dino_ui_size_request_box_get_instance_private (self);
	self->priv->_size_request_mode = GTK_SIZE_REQUEST_CONSTANT_SIZE;
}

static void
dino_ui_size_request_box_finalize (GObject * obj)
{
	DinoUiSizeRequestBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBox);
	G_OBJECT_CLASS (dino_ui_size_request_box_parent_class)->finalize (obj);
}

static GType
dino_ui_size_request_box_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiSizeRequestBoxClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_size_request_box_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiSizeRequestBox), 0, (GInstanceInitFunc) dino_ui_size_request_box_instance_init, NULL };
	GType dino_ui_size_request_box_type_id;
	dino_ui_size_request_box_type_id = g_type_register_static (gtk_box_get_type (), "DinoUiSizeRequestBox", &g_define_type_info, 0);
	DinoUiSizeRequestBox_private_offset = g_type_add_instance_private (dino_ui_size_request_box_type_id, sizeof (DinoUiSizeRequestBoxPrivate));
	return dino_ui_size_request_box_type_id;
}

GType
dino_ui_size_request_box_get_type (void)
{
	static volatile gsize dino_ui_size_request_box_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_size_request_box_type_id__once)) {
		GType dino_ui_size_request_box_type_id;
		dino_ui_size_request_box_type_id = dino_ui_size_request_box_get_type_once ();
		g_once_init_leave (&dino_ui_size_request_box_type_id__once, dino_ui_size_request_box_type_id);
	}
	return dino_ui_size_request_box_type_id__once;
}

static void
_vala_dino_ui_size_request_box_get_property (GObject * object,
                                             guint property_id,
                                             GValue * value,
                                             GParamSpec * pspec)
{
	DinoUiSizeRequestBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBox);
	switch (property_id) {
		case DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY:
		g_value_set_enum (value, dino_ui_size_request_box_get_size_request_mode (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_size_request_box_set_property (GObject * object,
                                             guint property_id,
                                             const GValue * value,
                                             GParamSpec * pspec)
{
	DinoUiSizeRequestBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_SIZE_REQUEST_BOX, DinoUiSizeRequestBox);
	switch (property_id) {
		case DINO_UI_SIZE_REQUEST_BOX_SIZE_REQUEST_MODE_PROPERTY:
		dino_ui_size_request_box_set_size_request_mode (self, g_value_get_enum (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_size_request_bin_get_instance_private (DinoUiSizeRequestBin* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiSizeRequestBin_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
dino_ui_size_request_bin_real_compute_expand_internal (GtkWidget* base,
                                                       gboolean* hexpand,
                                                       gboolean* vexpand)
{
	DinoUiSizeRequestBin * self;
	gboolean _vala_hexpand = FALSE;
	gboolean _vala_vexpand = FALSE;
	GtkWidget* child = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	self = (DinoUiSizeRequestBin*) base;
	_vala_hexpand = FALSE;
	_vala_vexpand = FALSE;
	_tmp0_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp1_ = _g_object_ref0 (_tmp0_);
	child = _tmp1_;
	while (TRUE) {
		GtkWidget* _tmp2_;
		gboolean _tmp3_ = FALSE;
		gboolean _tmp5_ = FALSE;
		GtkWidget* _tmp7_;
		GtkWidget* _tmp8_;
		GtkWidget* _tmp9_;
		_tmp2_ = child;
		if (!(_tmp2_ != NULL)) {
			break;
		}
		if (_vala_hexpand) {
			_tmp3_ = TRUE;
		} else {
			GtkWidget* _tmp4_;
			_tmp4_ = child;
			_tmp3_ = gtk_widget_compute_expand (_tmp4_, GTK_ORIENTATION_HORIZONTAL);
		}
		_vala_hexpand = _tmp3_;
		if (_vala_vexpand) {
			_tmp5_ = TRUE;
		} else {
			GtkWidget* _tmp6_;
			_tmp6_ = child;
			_tmp5_ = gtk_widget_compute_expand (_tmp6_, GTK_ORIENTATION_VERTICAL);
		}
		_vala_vexpand = _tmp5_;
		_tmp7_ = child;
		_tmp8_ = gtk_widget_get_next_sibling (_tmp7_);
		_tmp9_ = _g_object_ref0 (_tmp8_);
		_g_object_unref0 (child);
		child = _tmp9_;
	}
	_g_object_unref0 (child);
	if (hexpand) {
		*hexpand = _vala_hexpand;
	}
	if (vexpand) {
		*vexpand = _vala_vexpand;
	}
}

static GtkSizeRequestMode
dino_ui_size_request_bin_real_get_request_mode (GtkWidget* base)
{
	DinoUiSizeRequestBin * self;
	GtkSizeRequestMode _tmp0_;
	GtkSizeRequestMode result;
	self = (DinoUiSizeRequestBin*) base;
	_tmp0_ = self->priv->_size_request_mode;
	result = _tmp0_;
	return result;
}

DinoUiSizeRequestBin*
dino_ui_size_request_bin_construct (GType object_type)
{
	DinoUiSizeRequestBin * self = NULL;
	self = (DinoUiSizeRequestBin*) g_object_new (object_type, NULL);
	return self;
}

DinoUiSizeRequestBin*
dino_ui_size_request_bin_new (void)
{
	return dino_ui_size_request_bin_construct (DINO_UI_TYPE_SIZE_REQUEST_BIN);
}

GtkSizeRequestMode
dino_ui_size_request_bin_get_size_request_mode (DinoUiSizeRequestBin* self)
{
	GtkSizeRequestMode result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_size_request_mode;
	return result;
}

void
dino_ui_size_request_bin_set_size_request_mode (DinoUiSizeRequestBin* self,
                                                GtkSizeRequestMode value)
{
	GtkSizeRequestMode old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_size_request_bin_get_size_request_mode (self);
	if (old_value != value) {
		self->priv->_size_request_mode = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_size_request_bin_properties[DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY]);
	}
}

static GObject *
dino_ui_size_request_bin_constructor (GType type,
                                      guint n_construct_properties,
                                      GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	DinoUiSizeRequestBin * self;
	GtkBinLayout* _tmp0_;
	parent_class = G_OBJECT_CLASS (dino_ui_size_request_bin_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBin);
	_tmp0_ = (GtkBinLayout*) gtk_bin_layout_new ();
	gtk_widget_set_layout_manager ((GtkWidget*) self, (GtkLayoutManager*) _tmp0_);
	return obj;
}

static void
dino_ui_size_request_bin_class_init (DinoUiSizeRequestBinClass * klass,
                                     gpointer klass_data)
{
	dino_ui_size_request_bin_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiSizeRequestBin_private_offset);
	((GtkWidgetClass *) klass)->compute_expand = (void (*) (GtkWidget*, gboolean*, gboolean*)) dino_ui_size_request_bin_real_compute_expand_internal;
	((GtkWidgetClass *) klass)->get_request_mode = (GtkSizeRequestMode (*) (GtkWidget*)) dino_ui_size_request_bin_real_get_request_mode;
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_size_request_bin_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_size_request_bin_set_property;
	G_OBJECT_CLASS (klass)->constructor = dino_ui_size_request_bin_constructor;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_size_request_bin_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY, dino_ui_size_request_bin_properties[DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY] = g_param_spec_enum ("size-request-mode", "size-request-mode", "size-request-mode", gtk_size_request_mode_get_type (), GTK_SIZE_REQUEST_CONSTANT_SIZE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_size_request_bin_instance_init (DinoUiSizeRequestBin * self,
                                        gpointer klass)
{
	self->priv = dino_ui_size_request_bin_get_instance_private (self);
	self->priv->_size_request_mode = GTK_SIZE_REQUEST_CONSTANT_SIZE;
}

static void
dino_ui_size_request_bin_finalize (GObject * obj)
{
	DinoUiSizeRequestBin * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBin);
	G_OBJECT_CLASS (dino_ui_size_request_bin_parent_class)->finalize (obj);
}

static GType
dino_ui_size_request_bin_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiSizeRequestBinClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_size_request_bin_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiSizeRequestBin), 0, (GInstanceInitFunc) dino_ui_size_request_bin_instance_init, NULL };
	GType dino_ui_size_request_bin_type_id;
	dino_ui_size_request_bin_type_id = g_type_register_static (gtk_widget_get_type (), "DinoUiSizeRequestBin", &g_define_type_info, 0);
	DinoUiSizeRequestBin_private_offset = g_type_add_instance_private (dino_ui_size_request_bin_type_id, sizeof (DinoUiSizeRequestBinPrivate));
	return dino_ui_size_request_bin_type_id;
}

GType
dino_ui_size_request_bin_get_type (void)
{
	static volatile gsize dino_ui_size_request_bin_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_size_request_bin_type_id__once)) {
		GType dino_ui_size_request_bin_type_id;
		dino_ui_size_request_bin_type_id = dino_ui_size_request_bin_get_type_once ();
		g_once_init_leave (&dino_ui_size_request_bin_type_id__once, dino_ui_size_request_bin_type_id);
	}
	return dino_ui_size_request_bin_type_id__once;
}

static void
_vala_dino_ui_size_request_bin_get_property (GObject * object,
                                             guint property_id,
                                             GValue * value,
                                             GParamSpec * pspec)
{
	DinoUiSizeRequestBin * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBin);
	switch (property_id) {
		case DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY:
		g_value_set_enum (value, dino_ui_size_request_bin_get_size_request_mode (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_size_request_bin_set_property (GObject * object,
                                             guint property_id,
                                             const GValue * value,
                                             GParamSpec * pspec)
{
	DinoUiSizeRequestBin * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_SIZE_REQUEST_BIN, DinoUiSizeRequestBin);
	switch (property_id) {
		case DINO_UI_SIZE_REQUEST_BIN_SIZE_REQUEST_MODE_PROPERTY:
		dino_ui_size_request_bin_set_size_request_mode (self, g_value_get_enum (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

