/* register_plugin.c generated by valac 0.56.19, the Vala compiler
 * generated from register_plugin.vala, do not modify */

#include "http-files.h"
#include <gmodule.h>
#include <glib-object.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

GType
register_plugin (GModule* module)
{
	GType result;
	g_return_val_if_fail (module != NULL, G_TYPE_INVALID);
	result = DINO_PLUGINS_HTTP_FILES_TYPE_PLUGIN;
	return result;
}

