/* natural_size_increase.c generated by valac 0.56.19, the Vala compiler
 * generated from natural_size_increase.vala, do not modify */

#include <gtk/gtk.h>
#include <glib-object.h>
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_TYPE_NATURAL_SIZE_INCREASE (dino_ui_natural_size_increase_get_type ())
#define DINO_UI_NATURAL_SIZE_INCREASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncrease))
#define DINO_UI_NATURAL_SIZE_INCREASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncreaseClass))
#define DINO_UI_IS_NATURAL_SIZE_INCREASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_NATURAL_SIZE_INCREASE))
#define DINO_UI_IS_NATURAL_SIZE_INCREASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_NATURAL_SIZE_INCREASE))
#define DINO_UI_NATURAL_SIZE_INCREASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncreaseClass))

typedef struct _DinoUiNaturalSizeIncrease DinoUiNaturalSizeIncrease;
typedef struct _DinoUiNaturalSizeIncreaseClass DinoUiNaturalSizeIncreaseClass;
typedef struct _DinoUiNaturalSizeIncreasePrivate DinoUiNaturalSizeIncreasePrivate;
enum  {
	DINO_UI_NATURAL_SIZE_INCREASE_0_PROPERTY,
	DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY,
	DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY,
	DINO_UI_NATURAL_SIZE_INCREASE_NUM_PROPERTIES
};
static GParamSpec* dino_ui_natural_size_increase_properties[DINO_UI_NATURAL_SIZE_INCREASE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _DinoUiNaturalSizeIncrease {
	GtkWidget parent_instance;
	DinoUiNaturalSizeIncreasePrivate * priv;
};

struct _DinoUiNaturalSizeIncreaseClass {
	GtkWidgetClass parent_class;
};

struct _DinoUiNaturalSizeIncreasePrivate {
	guint _min_natural_height;
	guint _min_natural_width;
};

static gint DinoUiNaturalSizeIncrease_private_offset;
static gpointer dino_ui_natural_size_increase_parent_class = NULL;

VALA_EXTERN GType dino_ui_natural_size_increase_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiNaturalSizeIncrease, g_object_unref)
static void dino_ui_natural_size_increase_real_compute_expand_internal (GtkWidget* base,
                                                                 gboolean* hexpand,
                                                                 gboolean* vexpand);
static void dino_ui_natural_size_increase_real_measure (GtkWidget* base,
                                                 GtkOrientation orientation,
                                                 gint for_size,
                                                 gint* minimum,
                                                 gint* natural,
                                                 gint* minimum_baseline,
                                                 gint* natural_baseline);
VALA_EXTERN guint dino_ui_natural_size_increase_get_min_natural_width (DinoUiNaturalSizeIncrease* self);
VALA_EXTERN guint dino_ui_natural_size_increase_get_min_natural_height (DinoUiNaturalSizeIncrease* self);
static void dino_ui_natural_size_increase_real_size_allocate (GtkWidget* base,
                                                       gint width,
                                                       gint height,
                                                       gint baseline);
static GtkSizeRequestMode dino_ui_natural_size_increase_real_get_request_mode (GtkWidget* base);
static void dino_ui_natural_size_increase_real_dispose (GObject* base);
VALA_EXTERN DinoUiNaturalSizeIncrease* dino_ui_natural_size_increase_new (void);
VALA_EXTERN DinoUiNaturalSizeIncrease* dino_ui_natural_size_increase_construct (GType object_type);
VALA_EXTERN void dino_ui_natural_size_increase_set_min_natural_height (DinoUiNaturalSizeIncrease* self,
                                                           guint value);
VALA_EXTERN void dino_ui_natural_size_increase_set_min_natural_width (DinoUiNaturalSizeIncrease* self,
                                                          guint value);
static GObject * dino_ui_natural_size_increase_constructor (GType type,
                                                     guint n_construct_properties,
                                                     GObjectConstructParam * construct_properties);
static void _gtk_widget_queue_resize_g_object_notify (GObject* _sender,
                                               GParamSpec* pspec,
                                               gpointer self);
static void dino_ui_natural_size_increase_finalize (GObject * obj);
static GType dino_ui_natural_size_increase_get_type_once (void);
static void _vala_dino_ui_natural_size_increase_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_dino_ui_natural_size_increase_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
dino_ui_natural_size_increase_get_instance_private (DinoUiNaturalSizeIncrease* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiNaturalSizeIncrease_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
dino_ui_natural_size_increase_real_compute_expand_internal (GtkWidget* base,
                                                            gboolean* hexpand,
                                                            gboolean* vexpand)
{
	DinoUiNaturalSizeIncrease * self;
	gboolean _vala_hexpand = FALSE;
	gboolean _vala_vexpand = FALSE;
	GtkWidget* child = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	self = (DinoUiNaturalSizeIncrease*) base;
	_vala_hexpand = FALSE;
	_vala_vexpand = FALSE;
	_tmp0_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp1_ = _g_object_ref0 (_tmp0_);
	child = _tmp1_;
	while (TRUE) {
		GtkWidget* _tmp2_;
		gboolean _tmp3_ = FALSE;
		gboolean _tmp5_ = FALSE;
		GtkWidget* _tmp7_;
		GtkWidget* _tmp8_;
		GtkWidget* _tmp9_;
		_tmp2_ = child;
		if (!(_tmp2_ != NULL)) {
			break;
		}
		if (_vala_hexpand) {
			_tmp3_ = TRUE;
		} else {
			GtkWidget* _tmp4_;
			_tmp4_ = child;
			_tmp3_ = gtk_widget_compute_expand (_tmp4_, GTK_ORIENTATION_HORIZONTAL);
		}
		_vala_hexpand = _tmp3_;
		if (_vala_vexpand) {
			_tmp5_ = TRUE;
		} else {
			GtkWidget* _tmp6_;
			_tmp6_ = child;
			_tmp5_ = gtk_widget_compute_expand (_tmp6_, GTK_ORIENTATION_VERTICAL);
		}
		_vala_vexpand = _tmp5_;
		_tmp7_ = child;
		_tmp8_ = gtk_widget_get_next_sibling (_tmp7_);
		_tmp9_ = _g_object_ref0 (_tmp8_);
		_g_object_unref0 (child);
		child = _tmp9_;
	}
	_g_object_unref0 (child);
	if (hexpand) {
		*hexpand = _vala_hexpand;
	}
	if (vexpand) {
		*vexpand = _vala_vexpand;
	}
}

static void
dino_ui_natural_size_increase_real_measure (GtkWidget* base,
                                            GtkOrientation orientation,
                                            gint for_size,
                                            gint* minimum,
                                            gint* natural,
                                            gint* minimum_baseline,
                                            gint* natural_baseline)
{
	DinoUiNaturalSizeIncrease * self;
	gint _vala_minimum = 0;
	gint _vala_natural = 0;
	gint _vala_minimum_baseline = 0;
	gint _vala_natural_baseline = 0;
	GtkWidget* child = NULL;
	GtkWidget* _tmp2_;
	GtkWidget* _tmp3_;
	self = (DinoUiNaturalSizeIncrease*) base;
	_vala_minimum = 0;
	if (orientation == GTK_ORIENTATION_HORIZONTAL) {
		guint _tmp0_;
		_tmp0_ = self->priv->_min_natural_width;
		_vala_natural = (gint) _tmp0_;
	} else {
		guint _tmp1_;
		_tmp1_ = self->priv->_min_natural_height;
		_vala_natural = (gint) _tmp1_;
	}
	_vala_minimum_baseline = -1;
	_vala_natural_baseline = -1;
	_tmp2_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp3_ = _g_object_ref0 (_tmp2_);
	child = _tmp3_;
	while (TRUE) {
		GtkWidget* _tmp4_;
		GtkWidget* _tmp5_;
		GtkWidget* _tmp11_;
		GtkWidget* _tmp12_;
		GtkWidget* _tmp13_;
		_tmp4_ = child;
		if (!(_tmp4_ != NULL)) {
			break;
		}
		_tmp5_ = child;
		if (gtk_widget_should_layout (_tmp5_)) {
			gint child_min = 0;
			gint child_nat = 0;
			gint child_min_baseline = 0;
			gint child_nat_baseline = 0;
			GtkWidget* _tmp6_;
			gint _tmp7_ = 0;
			gint _tmp8_ = 0;
			gint _tmp9_ = 0;
			gint _tmp10_ = 0;
			child_min = 0;
			child_nat = 0;
			child_min_baseline = -1;
			child_nat_baseline = -1;
			_tmp6_ = child;
			gtk_widget_measure (_tmp6_, orientation, for_size, &_tmp7_, &_tmp8_, &_tmp9_, &_tmp10_);
			child_min = _tmp7_;
			child_nat = _tmp8_;
			child_min_baseline = _tmp9_;
			child_nat_baseline = _tmp10_;
			_vala_minimum = MAX (_vala_minimum, child_min);
			_vala_natural = MAX (_vala_natural, child_nat);
			if (child_min_baseline > 0) {
				_vala_minimum_baseline = MAX (_vala_minimum_baseline, child_min_baseline);
			}
			if (child_nat_baseline > 0) {
				_vala_natural_baseline = MAX (_vala_natural_baseline, child_nat_baseline);
			}
		}
		_tmp11_ = child;
		_tmp12_ = gtk_widget_get_next_sibling (_tmp11_);
		_tmp13_ = _g_object_ref0 (_tmp12_);
		_g_object_unref0 (child);
		child = _tmp13_;
	}
	_g_object_unref0 (child);
	if (minimum) {
		*minimum = _vala_minimum;
	}
	if (natural) {
		*natural = _vala_natural;
	}
	if (minimum_baseline) {
		*minimum_baseline = _vala_minimum_baseline;
	}
	if (natural_baseline) {
		*natural_baseline = _vala_natural_baseline;
	}
}

static void
dino_ui_natural_size_increase_real_size_allocate (GtkWidget* base,
                                                  gint width,
                                                  gint height,
                                                  gint baseline)
{
	DinoUiNaturalSizeIncrease * self;
	GtkWidget* child = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	self = (DinoUiNaturalSizeIncrease*) base;
	_tmp0_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp1_ = _g_object_ref0 (_tmp0_);
	child = _tmp1_;
	while (TRUE) {
		GtkWidget* _tmp2_;
		GtkWidget* _tmp3_;
		GtkWidget* _tmp5_;
		GtkWidget* _tmp6_;
		GtkWidget* _tmp7_;
		_tmp2_ = child;
		if (!(_tmp2_ != NULL)) {
			break;
		}
		_tmp3_ = child;
		if (gtk_widget_should_layout (_tmp3_)) {
			GtkWidget* _tmp4_;
			_tmp4_ = child;
			gtk_widget_allocate (_tmp4_, width, height, baseline, NULL);
		}
		_tmp5_ = child;
		_tmp6_ = gtk_widget_get_next_sibling (_tmp5_);
		_tmp7_ = _g_object_ref0 (_tmp6_);
		_g_object_unref0 (child);
		child = _tmp7_;
	}
	_g_object_unref0 (child);
}

static GtkSizeRequestMode
dino_ui_natural_size_increase_real_get_request_mode (GtkWidget* base)
{
	DinoUiNaturalSizeIncrease * self;
	GtkWidget* child = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
	GtkSizeRequestMode result;
	self = (DinoUiNaturalSizeIncrease*) base;
	_tmp0_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp1_ = _g_object_ref0 (_tmp0_);
	child = _tmp1_;
	_tmp2_ = child;
	if (_tmp2_ != NULL) {
		GtkWidget* _tmp3_;
		_tmp3_ = child;
		result = gtk_widget_get_request_mode (_tmp3_);
		_g_object_unref0 (child);
		return result;
	}
	result = GTK_SIZE_REQUEST_CONSTANT_SIZE;
	_g_object_unref0 (child);
	return result;
}

static void
dino_ui_natural_size_increase_real_dispose (GObject* base)
{
	DinoUiNaturalSizeIncrease * self;
	GtkWidget* child = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
	self = (DinoUiNaturalSizeIncrease*) base;
	_tmp0_ = gtk_widget_get_first_child ((GtkWidget*) self);
	_tmp1_ = _g_object_ref0 (_tmp0_);
	child = _tmp1_;
	_tmp2_ = child;
	if (_tmp2_ != NULL) {
		GtkWidget* _tmp3_;
		_tmp3_ = child;
		gtk_widget_unparent (_tmp3_);
	}
	_g_object_unref0 (child);
}

DinoUiNaturalSizeIncrease*
dino_ui_natural_size_increase_construct (GType object_type)
{
	DinoUiNaturalSizeIncrease * self = NULL;
	self = (DinoUiNaturalSizeIncrease*) g_object_new (object_type, NULL);
	return self;
}

DinoUiNaturalSizeIncrease*
dino_ui_natural_size_increase_new (void)
{
	return dino_ui_natural_size_increase_construct (DINO_UI_TYPE_NATURAL_SIZE_INCREASE);
}

guint
dino_ui_natural_size_increase_get_min_natural_height (DinoUiNaturalSizeIncrease* self)
{
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_min_natural_height;
	return result;
}

void
dino_ui_natural_size_increase_set_min_natural_height (DinoUiNaturalSizeIncrease* self,
                                                      guint value)
{
	guint old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_natural_size_increase_get_min_natural_height (self);
	if (old_value != value) {
		self->priv->_min_natural_height = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_natural_size_increase_properties[DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY]);
	}
}

guint
dino_ui_natural_size_increase_get_min_natural_width (DinoUiNaturalSizeIncrease* self)
{
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_min_natural_width;
	return result;
}

void
dino_ui_natural_size_increase_set_min_natural_width (DinoUiNaturalSizeIncrease* self,
                                                     guint value)
{
	guint old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_natural_size_increase_get_min_natural_width (self);
	if (old_value != value) {
		self->priv->_min_natural_width = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_natural_size_increase_properties[DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY]);
	}
}

static void
_gtk_widget_queue_resize_g_object_notify (GObject* _sender,
                                          GParamSpec* pspec,
                                          gpointer self)
{
	gtk_widget_queue_resize ((GtkWidget*) self);
}

static GObject *
dino_ui_natural_size_increase_constructor (GType type,
                                           guint n_construct_properties,
                                           GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	DinoUiNaturalSizeIncrease * self;
	parent_class = G_OBJECT_CLASS (dino_ui_natural_size_increase_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncrease);
	g_signal_connect_object ((GObject*) self, "notify", (GCallback) _gtk_widget_queue_resize_g_object_notify, (GtkWidget*) self, 0);
	return obj;
}

static void
dino_ui_natural_size_increase_class_init (DinoUiNaturalSizeIncreaseClass * klass,
                                          gpointer klass_data)
{
	dino_ui_natural_size_increase_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiNaturalSizeIncrease_private_offset);
	((GtkWidgetClass *) klass)->compute_expand = (void (*) (GtkWidget*, gboolean*, gboolean*)) dino_ui_natural_size_increase_real_compute_expand_internal;
	((GtkWidgetClass *) klass)->measure = (void (*) (GtkWidget*, GtkOrientation, gint, gint*, gint*, gint*, gint*)) dino_ui_natural_size_increase_real_measure;
	((GtkWidgetClass *) klass)->size_allocate = (void (*) (GtkWidget*, gint, gint, gint)) dino_ui_natural_size_increase_real_size_allocate;
	((GtkWidgetClass *) klass)->get_request_mode = (GtkSizeRequestMode (*) (GtkWidget*)) dino_ui_natural_size_increase_real_get_request_mode;
	((GObjectClass *) klass)->dispose = (void (*) (GObject*)) dino_ui_natural_size_increase_real_dispose;
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_natural_size_increase_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_natural_size_increase_set_property;
	G_OBJECT_CLASS (klass)->constructor = dino_ui_natural_size_increase_constructor;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_natural_size_increase_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY, dino_ui_natural_size_increase_properties[DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY] = g_param_spec_uint ("min-natural-height", "min-natural-height", "min-natural-height", 0, G_MAXUINT, (guint) 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY, dino_ui_natural_size_increase_properties[DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY] = g_param_spec_uint ("min-natural-width", "min-natural-width", "min-natural-width", 0, G_MAXUINT, (guint) 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_natural_size_increase_instance_init (DinoUiNaturalSizeIncrease * self,
                                             gpointer klass)
{
	self->priv = dino_ui_natural_size_increase_get_instance_private (self);
	self->priv->_min_natural_height = (guint) 0;
	self->priv->_min_natural_width = (guint) 0;
}

static void
dino_ui_natural_size_increase_finalize (GObject * obj)
{
	DinoUiNaturalSizeIncrease * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncrease);
	G_OBJECT_CLASS (dino_ui_natural_size_increase_parent_class)->finalize (obj);
}

static GType
dino_ui_natural_size_increase_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiNaturalSizeIncreaseClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_natural_size_increase_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiNaturalSizeIncrease), 0, (GInstanceInitFunc) dino_ui_natural_size_increase_instance_init, NULL };
	GType dino_ui_natural_size_increase_type_id;
	dino_ui_natural_size_increase_type_id = g_type_register_static (gtk_widget_get_type (), "DinoUiNaturalSizeIncrease", &g_define_type_info, 0);
	DinoUiNaturalSizeIncrease_private_offset = g_type_add_instance_private (dino_ui_natural_size_increase_type_id, sizeof (DinoUiNaturalSizeIncreasePrivate));
	return dino_ui_natural_size_increase_type_id;
}

GType
dino_ui_natural_size_increase_get_type (void)
{
	static volatile gsize dino_ui_natural_size_increase_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_natural_size_increase_type_id__once)) {
		GType dino_ui_natural_size_increase_type_id;
		dino_ui_natural_size_increase_type_id = dino_ui_natural_size_increase_get_type_once ();
		g_once_init_leave (&dino_ui_natural_size_increase_type_id__once, dino_ui_natural_size_increase_type_id);
	}
	return dino_ui_natural_size_increase_type_id__once;
}

static void
_vala_dino_ui_natural_size_increase_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	DinoUiNaturalSizeIncrease * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncrease);
	switch (property_id) {
		case DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY:
		g_value_set_uint (value, dino_ui_natural_size_increase_get_min_natural_height (self));
		break;
		case DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY:
		g_value_set_uint (value, dino_ui_natural_size_increase_get_min_natural_width (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_natural_size_increase_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	DinoUiNaturalSizeIncrease * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_NATURAL_SIZE_INCREASE, DinoUiNaturalSizeIncrease);
	switch (property_id) {
		case DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_HEIGHT_PROPERTY:
		dino_ui_natural_size_increase_set_min_natural_height (self, g_value_get_uint (value));
		break;
		case DINO_UI_NATURAL_SIZE_INCREASE_MIN_NATURAL_WIDTH_PROPERTY:
		dino_ui_natural_size_increase_set_min_natural_width (self, g_value_get_uint (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

