/* notifier_gnotifications.c generated by valac 0.56.19, the Vala compiler
 * generated from notifier_gnotifications.vala, do not modify */

#include <glib-object.h>
#include "libdino.h"
#include <gee.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <float.h>
#include <math.h>
#include "dino_i18n.h"
#include "xmpp-vala.h"
#include <cairo-gobject.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk/gdk.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER (dino_ui_gnotifications_notifier_get_type ())
#define DINO_UI_GNOTIFICATIONS_NOTIFIER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, DinoUiGNotificationsNotifier))
#define DINO_UI_GNOTIFICATIONS_NOTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, DinoUiGNotificationsNotifierClass))
#define DINO_UI_IS_GNOTIFICATIONS_NOTIFIER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER))
#define DINO_UI_IS_GNOTIFICATIONS_NOTIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER))
#define DINO_UI_GNOTIFICATIONS_NOTIFIER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, DinoUiGNotificationsNotifierClass))

typedef struct _DinoUiGNotificationsNotifier DinoUiGNotificationsNotifier;
typedef struct _DinoUiGNotificationsNotifierClass DinoUiGNotificationsNotifierClass;
typedef struct _DinoUiGNotificationsNotifierPrivate DinoUiGNotificationsNotifierPrivate;
enum  {
	DINO_UI_GNOTIFICATIONS_NOTIFIER_0_PROPERTY,
	DINO_UI_GNOTIFICATIONS_NOTIFIER_NUM_PROPERTIES
};
static GParamSpec* dino_ui_gnotifications_notifier_properties[DINO_UI_GNOTIFICATIONS_NOTIFIER_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _DinoUiGnotificationsNotifierNotifyMessageData DinoUiGnotificationsNotifierNotifyMessageData;
typedef struct _DinoUiGnotificationsNotifierNotifyFileData DinoUiGnotificationsNotifierNotifyFileData;
#define _g_variant_unref0(var) ((var == NULL) ? NULL : (var = (g_variant_unref (var), NULL)))
typedef struct _DinoUiGnotificationsNotifierNotifyContentItemData DinoUiGnotificationsNotifierNotifyContentItemData;
typedef struct _DinoUiGnotificationsNotifierNotifyCallData DinoUiGnotificationsNotifierNotifyCallData;
typedef struct _DinoUiGnotificationsNotifierRetractCallNotificationData DinoUiGnotificationsNotifierRetractCallNotificationData;
typedef struct _DinoUiGnotificationsNotifierNotifySubscriptionRequestData DinoUiGnotificationsNotifierNotifySubscriptionRequestData;
#define _dino_connection_manager_connection_error_unref0(var) ((var == NULL) ? NULL : (var = (dino_connection_manager_connection_error_unref (var), NULL)))
#define _xmpp_jid_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_jid_unref (var), NULL)))
typedef struct _DinoUiGnotificationsNotifierNotifyConnectionErrorData DinoUiGnotificationsNotifierNotifyConnectionErrorData;
typedef struct _DinoUiGnotificationsNotifierNotifyMucInviteData DinoUiGnotificationsNotifierNotifyMucInviteData;
typedef struct _DinoUiGnotificationsNotifierNotifyVoiceRequestData DinoUiGnotificationsNotifierNotifyVoiceRequestData;
typedef struct _DinoUiGnotificationsNotifierRetractContentItemNotificationsData DinoUiGnotificationsNotifierRetractContentItemNotificationsData;
typedef struct _DinoUiGnotificationsNotifierRetractConversationNotificationsData DinoUiGnotificationsNotifierRetractConversationNotificationsData;

#define DINO_UI_TYPE_COMPAT_AVATAR_DRAWER (dino_ui_compat_avatar_drawer_get_type ())
#define DINO_UI_COMPAT_AVATAR_DRAWER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_COMPAT_AVATAR_DRAWER, DinoUiCompatAvatarDrawer))
#define DINO_UI_COMPAT_AVATAR_DRAWER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_COMPAT_AVATAR_DRAWER, DinoUiCompatAvatarDrawerClass))
#define DINO_UI_IS_COMPAT_AVATAR_DRAWER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_COMPAT_AVATAR_DRAWER))
#define DINO_UI_IS_COMPAT_AVATAR_DRAWER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_COMPAT_AVATAR_DRAWER))
#define DINO_UI_COMPAT_AVATAR_DRAWER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_COMPAT_AVATAR_DRAWER, DinoUiCompatAvatarDrawerClass))

typedef struct _DinoUiCompatAvatarDrawer DinoUiCompatAvatarDrawer;
typedef struct _DinoUiCompatAvatarDrawerClass DinoUiCompatAvatarDrawerClass;

#define DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL (dino_ui_view_model_avatar_picture_model_get_type ())
#define DINO_UI_VIEW_MODEL_AVATAR_PICTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL, DinoUiViewModelAvatarPictureModel))
#define DINO_UI_VIEW_MODEL_AVATAR_PICTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL, DinoUiViewModelAvatarPictureModelClass))
#define DINO_UI_VIEW_MODEL_IS_AVATAR_PICTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL))
#define DINO_UI_VIEW_MODEL_IS_AVATAR_PICTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL))
#define DINO_UI_VIEW_MODEL_AVATAR_PICTURE_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_TYPE_AVATAR_PICTURE_MODEL, DinoUiViewModelAvatarPictureModelClass))

typedef struct _DinoUiViewModelAvatarPictureModel DinoUiViewModelAvatarPictureModel;
typedef struct _DinoUiViewModelAvatarPictureModelClass DinoUiViewModelAvatarPictureModelClass;

#define DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL (dino_ui_view_model_compat_avatar_picture_model_get_type ())
#define DINO_UI_VIEW_MODEL_COMPAT_AVATAR_PICTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL, DinoUiViewModelCompatAvatarPictureModel))
#define DINO_UI_VIEW_MODEL_COMPAT_AVATAR_PICTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL, DinoUiViewModelCompatAvatarPictureModelClass))
#define DINO_UI_VIEW_MODEL_IS_COMPAT_AVATAR_PICTURE_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL))
#define DINO_UI_VIEW_MODEL_IS_COMPAT_AVATAR_PICTURE_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL))
#define DINO_UI_VIEW_MODEL_COMPAT_AVATAR_PICTURE_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_TYPE_COMPAT_AVATAR_PICTURE_MODEL, DinoUiViewModelCompatAvatarPictureModelClass))

typedef struct _DinoUiViewModelCompatAvatarPictureModel DinoUiViewModelCompatAvatarPictureModel;
typedef struct _DinoUiViewModelCompatAvatarPictureModelClass DinoUiViewModelCompatAvatarPictureModelClass;
#define _cairo_surface_destroy0(var) ((var == NULL) ? NULL : (var = (cairo_surface_destroy (var), NULL)))
#define _dino_ui_compat_avatar_drawer_unref0(var) ((var == NULL) ? NULL : (var = (dino_ui_compat_avatar_drawer_unref (var), NULL)))
#define _g_bytes_unref0(var) ((var == NULL) ? NULL : (var = (g_bytes_unref (var), NULL)))
typedef struct _DinoUiGnotificationsNotifierGetConversationIconData DinoUiGnotificationsNotifierGetConversationIconData;
enum  {
	DINO_UI_GNOTIFICATIONS_NOTIFIER_CONVERSATION_SELECTED_SIGNAL,
	DINO_UI_GNOTIFICATIONS_NOTIFIER_NUM_SIGNALS
};
static guint dino_ui_gnotifications_notifier_signals[DINO_UI_GNOTIFICATIONS_NOTIFIER_NUM_SIGNALS] = {0};

struct _DinoUiGNotificationsNotifier {
	GObject parent_instance;
	DinoUiGNotificationsNotifierPrivate * priv;
};

struct _DinoUiGNotificationsNotifierClass {
	GObjectClass parent_class;
};

struct _DinoUiGNotificationsNotifierPrivate {
	DinoStreamInteractor* stream_interactor;
	GeeHashMap* notifications;
	GeeSet* active_conversation_ids;
	GeeSet* active_ids;
};

struct _DinoUiGnotificationsNotifierNotifyMessageData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesMessage* message;
	DinoEntitiesConversation* conversation;
	gchar* conversation_display_name;
	gchar* participant_display_name;
	gchar* text;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
	gchar* _tmp6_;
	const gchar* _tmp7_;
};

struct _DinoUiGnotificationsNotifierNotifyFileData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesFileTransfer* file_transfer;
	DinoEntitiesConversation* conversation;
	gboolean is_image;
	gchar* conversation_display_name;
	gchar* participant_display_name;
	gchar* text;
	gchar* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
	gchar* _tmp6_;
	const gchar* _tmp7_;
	const gchar* _tmp8_;
	const gchar* _tmp9_;
	gchar* _tmp10_;
	const gchar* _tmp11_;
	const gchar* _tmp12_;
	const gchar* _tmp13_;
	gchar* _tmp14_;
	const gchar* _tmp15_;
};

struct _DinoUiGnotificationsNotifierNotifyContentItemData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesConversation* conversation;
	gchar* title;
	gchar* body;
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp1_;
	GNotification* _tmp2_;
	GNotification* _tmp3_;
	GeeHashMap* _tmp4_;
	gpointer _tmp5_;
	GNotification* _tmp6_;
	gint _tmp7_;
	gint _tmp8_;
	GVariant* _tmp9_;
	GVariant* _tmp10_;
	GNotification* notification;
	GeeHashMap* _tmp11_;
	gpointer _tmp12_;
	GIcon* _tmp13_;
	GIcon* _tmp14_;
	GApplication* _tmp15_;
	gint _tmp16_;
	gint _tmp17_;
	gchar* _tmp18_;
	gchar* _tmp19_;
	GeeHashMap* _tmp20_;
	gpointer _tmp21_;
	GNotification* _tmp22_;
	GeeSet* _tmp23_;
	GeeSet* _tmp24_;
	gint _tmp25_;
	gint _tmp26_;
	gchar* _tmp27_;
	gchar* _tmp28_;
	GError* _inner_error0_;
};

struct _DinoUiGnotificationsNotifierNotifyCallData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesCall* call;
	DinoEntitiesConversation* conversation;
	gboolean video;
	gboolean multiparty;
	gchar* conversation_display_name;
	GNotification* notification;
	GNotification* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	gchar* body;
	gchar* _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	const gchar* _tmp7_;
	gchar* _tmp8_;
	GNotification* _tmp9_;
	const gchar* _tmp10_;
	GNotification* _tmp11_;
	GNotification* _tmp12_;
	gchar* _tmp13_;
	gchar** _tmp14_;
	gchar** _tmp15_;
	gint _tmp15__length1;
	GThemedIcon* _tmp16_;
	GThemedIcon* _tmp17_;
	GNotification* _tmp18_;
	gint _tmp19_;
	gint _tmp20_;
	GVariant* _tmp21_;
	GVariant* _tmp22_;
	GNotification* _tmp23_;
	const gchar* _tmp24_;
	gint _tmp25_;
	gint _tmp26_;
	GVariant* _tmp27_;
	GVariant* _tmp28_;
	GNotification* _tmp29_;
	const gchar* _tmp30_;
	gint _tmp31_;
	gint _tmp32_;
	GVariant* _tmp33_;
	GVariant* _tmp34_;
	GApplication* _tmp35_;
	gint _tmp36_;
	gint _tmp37_;
	gchar* _tmp38_;
	gchar* _tmp39_;
	GNotification* _tmp40_;
};

struct _DinoUiGnotificationsNotifierRetractCallNotificationData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesCall* call;
	DinoEntitiesConversation* conversation;
	GApplication* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
};

struct _DinoUiGnotificationsNotifierNotifySubscriptionRequestData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesConversation* conversation;
	GNotification* notification;
	const gchar* _tmp0_;
	GNotification* _tmp1_;
	XmppJid* _tmp2_;
	XmppJid* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	GIcon* _tmp6_;
	GIcon* _tmp7_;
	gint _tmp8_;
	gint _tmp9_;
	GVariant* _tmp10_;
	GVariant* _tmp11_;
	const gchar* _tmp12_;
	gint _tmp13_;
	gint _tmp14_;
	GVariant* _tmp15_;
	const gchar* _tmp16_;
	gint _tmp17_;
	gint _tmp18_;
	GVariant* _tmp19_;
	GApplication* _tmp20_;
	gint _tmp21_;
	gint _tmp22_;
	gchar* _tmp23_;
	gchar* _tmp24_;
	gchar* _tmp25_;
	gchar* _tmp26_;
	GeeSet* _tmp27_;
	gint _tmp28_;
	gint _tmp29_;
	gchar* _tmp30_;
	gchar* _tmp31_;
	gchar* _tmp32_;
	gchar* _tmp33_;
	GError* _inner_error0_;
};

struct _DinoUiGnotificationsNotifierNotifyConnectionErrorData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesAccount* account;
	DinoConnectionManagerConnectionError* _error_;
	GNotification* notification;
	const gchar* _tmp0_;
	XmppJid* _tmp1_;
	XmppJid* _tmp2_;
	XmppJid* _tmp3_;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	GNotification* _tmp7_;
	GNotification* _tmp8_;
	GNotification* _tmp9_;
	gint _tmp10_;
	gint _tmp11_;
	GVariant* _tmp12_;
	GVariant* _tmp13_;
	GNotification* _tmp14_;
	GNotification* _tmp15_;
	GApplication* _tmp16_;
	gint _tmp17_;
	gint _tmp18_;
	gchar* _tmp19_;
	gchar* _tmp20_;
	gchar* _tmp21_;
	gchar* _tmp22_;
	GNotification* _tmp23_;
};

struct _DinoUiGnotificationsNotifierNotifyMucInviteData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesAccount* account;
	XmppJid* room_jid;
	XmppJid* from_jid;
	gchar* inviter_display_name;
	DinoEntitiesConversation* direct_conversation;
	DinoEntitiesConversation* _tmp0_;
	gchar* display_room;
	XmppJid* _tmp1_;
	XmppJid* _tmp2_;
	XmppJid* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	GNotification* notification;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	GNotification* _tmp9_;
	GNotification* _tmp10_;
	gchar* body;
	const gchar* _tmp11_;
	gchar* _tmp12_;
	GIcon* _tmp13_;
	GIcon* _tmp14_;
	DinoEntitiesConversation* group_conversation;
	DinoStreamInteractor* _tmp15_;
	DinoModuleIdentity* _tmp16_;
	gpointer _tmp17_;
	DinoConversationManager* _tmp18_;
	DinoEntitiesConversationType _tmp19_;
	DinoEntitiesConversation* _tmp20_;
	DinoEntitiesConversation* _tmp21_;
	gint _tmp22_;
	gint _tmp23_;
	GVariant* _tmp24_;
	GVariant* _tmp25_;
	const gchar* _tmp26_;
	gint _tmp27_;
	gint _tmp28_;
	GVariant* _tmp29_;
	const gchar* _tmp30_;
	gint _tmp31_;
	gint _tmp32_;
	GVariant* _tmp33_;
	GApplication* _tmp34_;
	GError* _inner_error0_;
};

struct _DinoUiGnotificationsNotifierNotifyVoiceRequestData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesConversation* conversation;
	XmppJid* from_jid;
	gchar* display_name;
	DinoStreamInteractor* _tmp0_;
	gchar* _tmp1_;
	gchar* display_room;
	DinoStreamInteractor* _tmp2_;
	gchar* _tmp3_;
	GNotification* notification;
	const gchar* _tmp4_;
	GNotification* _tmp5_;
	gchar* body;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	GIcon* _tmp8_;
	GIcon* _tmp9_;
	const gchar* _tmp10_;
	gint _tmp11_;
	gint _tmp12_;
	GVariant* _tmp13_;
	const gchar* _tmp14_;
	gint _tmp15_;
	gint _tmp16_;
	GVariant* _tmp17_;
	GApplication* _tmp18_;
	GError* _inner_error0_;
};

struct _DinoUiGnotificationsNotifierRetractContentItemNotificationsData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	GeeSet* _tmp0_;
	GeeList* conversations;
	DinoStreamInteractor* _tmp1_;
	DinoModuleIdentity* _tmp2_;
	gpointer _tmp3_;
	DinoConversationManager* _tmp4_;
	GeeList* _tmp5_;
	GeeList* _tmp6_;
	GeeList* _conversation_list;
	GeeList* _tmp7_;
	gint _conversation_size;
	GeeList* _tmp8_;
	gint _tmp9_;
	gint _tmp10_;
	gint _conversation_index;
	gint _tmp11_;
	gint _tmp12_;
	DinoEntitiesConversation* conversation;
	GeeList* _tmp13_;
	gpointer _tmp14_;
	GApplication* _tmp15_;
	DinoEntitiesConversation* _tmp16_;
	gint _tmp17_;
	gint _tmp18_;
	gchar* _tmp19_;
	gchar* _tmp20_;
	GeeHashSet* _tmp21_;
	GeeIterator* _id_it;
	GeeSet* _tmp22_;
	GeeIterator* _tmp23_;
	GeeIterator* _tmp24_;
	gchar* id;
	GeeIterator* _tmp25_;
	gpointer _tmp26_;
	GApplication* _tmp27_;
	const gchar* _tmp28_;
	GeeSet* _tmp29_;
};

struct _DinoUiGnotificationsNotifierRetractConversationNotificationsData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesConversation* conversation;
	gchar* subscription_id;
	gint _tmp0_;
	gint _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	GeeSet* _tmp6_;
	const gchar* _tmp7_;
	GApplication* _tmp8_;
	const gchar* _tmp9_;
};

struct _DinoUiGnotificationsNotifierGetConversationIconData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiGNotificationsNotifier* self;
	DinoEntitiesConversation* conversation;
	GIcon* result;
	DinoUiCompatAvatarDrawer* drawer;
	DinoStreamInteractor* _tmp0_;
	DinoUiViewModelCompatAvatarPictureModel* _tmp1_;
	DinoUiViewModelCompatAvatarPictureModel* _tmp2_;
	DinoUiViewModelCompatAvatarPictureModel* _tmp3_;
	DinoUiCompatAvatarDrawer* _tmp4_;
	DinoUiCompatAvatarDrawer* _tmp5_;
	cairo_surface_t* surface;
	cairo_surface_t* _tmp6_;
	GdkPixbuf* avatar;
	GdkPixbuf* _tmp7_;
	guint8* buffer;
	gint buffer_length1;
	gint _buffer_size_;
	guint8* _tmp8_;
	gsize _tmp9_;
	GBytes* _tmp10_;
	GBytes* _tmp11_;
	GBytesIcon* _tmp12_;
	GIcon* _tmp13_;
	GError* _inner_error0_;
};

static gint DinoUiGNotificationsNotifier_private_offset;
static gpointer dino_ui_gnotifications_notifier_parent_class = NULL;
static DinoNotificationProviderIface * dino_ui_gnotifications_notifier_dino_notification_provider_parent_iface = NULL;

VALA_EXTERN GType dino_ui_gnotifications_notifier_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiGNotificationsNotifier, g_object_unref)
static guint _dino_entities_conversation_hash_func_gee_hash_data_func (gconstpointer v,
                                                                gpointer self);
static gboolean _dino_entities_conversation_equals_func_gee_equal_data_func (gconstpointer a,
                                                                      gconstpointer b,
                                                                      gpointer self);
VALA_EXTERN DinoUiGNotificationsNotifier* dino_ui_gnotifications_notifier_new (DinoStreamInteractor* stream_interactor);
VALA_EXTERN DinoUiGNotificationsNotifier* dino_ui_gnotifications_notifier_construct (GType object_type,
                                                                         DinoStreamInteractor* stream_interactor);
static gdouble dino_ui_gnotifications_notifier_real_get_priority (DinoNotificationProvider* base);
static void dino_ui_gnotifications_notifier_real_notify_message_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_message (DinoNotificationProvider* base,
                                                          DinoEntitiesMessage* message,
                                                          DinoEntitiesConversation* conversation,
                                                          const gchar* conversation_display_name,
                                                          const gchar* participant_display_name,
                                                          GAsyncReadyCallback _callback_,
                                                          gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_message_co (DinoUiGnotificationsNotifierNotifyMessageData* _data_);
static void dino_ui_gnotifications_notifier_notify_content_item (DinoUiGNotificationsNotifier* self,
                                                          DinoEntitiesConversation* conversation,
                                                          const gchar* title,
                                                          const gchar* body,
                                                          GAsyncReadyCallback _callback_,
                                                          gpointer _user_data_);
static void dino_ui_gnotifications_notifier_notify_content_item_finish (DinoUiGNotificationsNotifier* self,
                                                                 GAsyncResult* _res_);
static void dino_ui_gnotifications_notifier_notify_message_ready (GObject* source_object,
                                                           GAsyncResult* _res_,
                                                           gpointer _user_data_);
static void dino_ui_gnotifications_notifier_real_notify_file_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_file (DinoNotificationProvider* base,
                                                       DinoEntitiesFileTransfer* file_transfer,
                                                       DinoEntitiesConversation* conversation,
                                                       gboolean is_image,
                                                       const gchar* conversation_display_name,
                                                       const gchar* participant_display_name,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_file_co (DinoUiGnotificationsNotifierNotifyFileData* _data_);
static void dino_ui_gnotifications_notifier_notify_file_ready (GObject* source_object,
                                                        GAsyncResult* _res_,
                                                        gpointer _user_data_);
static void dino_ui_gnotifications_notifier_notify_content_item_data_free (gpointer _data);
static gboolean dino_ui_gnotifications_notifier_notify_content_item_co (DinoUiGnotificationsNotifierNotifyContentItemData* _data_);
static void dino_ui_gnotifications_notifier_get_conversation_icon (DinoUiGNotificationsNotifier* self,
                                                            DinoEntitiesConversation* conversation,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_);
static GIcon* dino_ui_gnotifications_notifier_get_conversation_icon_finish (DinoUiGNotificationsNotifier* self,
                                                                     GAsyncResult* _res_,
                                                                     GError** error);
static void dino_ui_gnotifications_notifier_notify_content_item_ready (GObject* source_object,
                                                                GAsyncResult* _res_,
                                                                gpointer _user_data_);
static void dino_ui_gnotifications_notifier_real_notify_call_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_call (DinoNotificationProvider* base,
                                                       DinoEntitiesCall* call,
                                                       DinoEntitiesConversation* conversation,
                                                       gboolean video,
                                                       gboolean multiparty,
                                                       const gchar* conversation_display_name,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_call_co (DinoUiGnotificationsNotifierNotifyCallData* _data_);
static void dino_ui_gnotifications_notifier_real_retract_call_notification_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_retract_call_notification (DinoNotificationProvider* base,
                                                                     DinoEntitiesCall* call,
                                                                     DinoEntitiesConversation* conversation,
                                                                     GAsyncReadyCallback _callback_,
                                                                     gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_retract_call_notification_co (DinoUiGnotificationsNotifierRetractCallNotificationData* _data_);
static void dino_ui_gnotifications_notifier_real_notify_subscription_request_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_subscription_request (DinoNotificationProvider* base,
                                                                       DinoEntitiesConversation* conversation,
                                                                       GAsyncReadyCallback _callback_,
                                                                       gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_subscription_request_co (DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_);
static void dino_ui_gnotifications_notifier_notify_subscription_request_ready (GObject* source_object,
                                                                        GAsyncResult* _res_,
                                                                        gpointer _user_data_);
static GVariant* _variant_new5 (gint value);
static GVariant* _variant_new6 (gint value);
static void dino_ui_gnotifications_notifier_real_notify_connection_error_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_connection_error (DinoNotificationProvider* base,
                                                                   DinoEntitiesAccount* account,
                                                                   DinoConnectionManagerConnectionError* _error_,
                                                                   GAsyncReadyCallback _callback_,
                                                                   gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_connection_error_co (DinoUiGnotificationsNotifierNotifyConnectionErrorData* _data_);
static void dino_ui_gnotifications_notifier_real_notify_muc_invite_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_muc_invite (DinoNotificationProvider* base,
                                                             DinoEntitiesAccount* account,
                                                             XmppJid* room_jid,
                                                             XmppJid* from_jid,
                                                             const gchar* inviter_display_name,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_muc_invite_co (DinoUiGnotificationsNotifierNotifyMucInviteData* _data_);
static void dino_ui_gnotifications_notifier_notify_muc_invite_ready (GObject* source_object,
                                                              GAsyncResult* _res_,
                                                              gpointer _user_data_);
static GVariant* _variant_new7 (gint value);
static GVariant* _variant_new8 (gint value);
static void dino_ui_gnotifications_notifier_real_notify_voice_request_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_notify_voice_request (DinoNotificationProvider* base,
                                                                DinoEntitiesConversation* conversation,
                                                                XmppJid* from_jid,
                                                                GAsyncReadyCallback _callback_,
                                                                gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_notify_voice_request_co (DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_);
VALA_EXTERN gchar* dino_ui_util_get_participant_display_name (DinoStreamInteractor* stream_interactor,
                                                  DinoEntitiesConversation* conversation,
                                                  XmppJid* participant,
                                                  gboolean me_is_me);
VALA_EXTERN gchar* dino_ui_util_get_conversation_display_name (DinoStreamInteractor* stream_interactor,
                                                   DinoEntitiesConversation* conversation);
static void dino_ui_gnotifications_notifier_notify_voice_request_ready (GObject* source_object,
                                                                 GAsyncResult* _res_,
                                                                 gpointer _user_data_);
static GVariant* _variant_new9 (gint value);
static GVariant* _variant_new10 (gint value);
static void dino_ui_gnotifications_notifier_real_retract_content_item_notifications_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_retract_content_item_notifications (DinoNotificationProvider* base,
                                                                              GAsyncReadyCallback _callback_,
                                                                              gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_retract_content_item_notifications_co (DinoUiGnotificationsNotifierRetractContentItemNotificationsData* _data_);
static void dino_ui_gnotifications_notifier_real_retract_conversation_notifications_data_free (gpointer _data);
static void dino_ui_gnotifications_notifier_real_retract_conversation_notifications (DinoNotificationProvider* base,
                                                                              DinoEntitiesConversation* conversation,
                                                                              GAsyncReadyCallback _callback_,
                                                                              gpointer _user_data_);
static gboolean dino_ui_gnotifications_notifier_real_retract_conversation_notifications_co (DinoUiGnotificationsNotifierRetractConversationNotificationsData* _data_);
static void dino_ui_gnotifications_notifier_get_conversation_icon_data_free (gpointer _data);
static gboolean dino_ui_gnotifications_notifier_get_conversation_icon_co (DinoUiGnotificationsNotifierGetConversationIconData* _data_);
VALA_EXTERN gpointer dino_ui_compat_avatar_drawer_ref (gpointer instance);
VALA_EXTERN void dino_ui_compat_avatar_drawer_unref (gpointer instance);
VALA_EXTERN GParamSpec* dino_ui_param_spec_compat_avatar_drawer (const gchar* name,
                                                     const gchar* nick,
                                                     const gchar* blurb,
                                                     GType object_type,
                                                     GParamFlags flags);
VALA_EXTERN void dino_ui_value_set_compat_avatar_drawer (GValue* value,
                                             gpointer v_object);
VALA_EXTERN void dino_ui_value_take_compat_avatar_drawer (GValue* value,
                                              gpointer v_object);
VALA_EXTERN gpointer dino_ui_value_get_compat_avatar_drawer (const GValue* value);
VALA_EXTERN GType dino_ui_compat_avatar_drawer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiCompatAvatarDrawer, dino_ui_compat_avatar_drawer_unref)
VALA_EXTERN DinoUiViewModelCompatAvatarPictureModel* dino_ui_view_model_compat_avatar_picture_model_new (DinoStreamInteractor* stream_interactor);
VALA_EXTERN DinoUiViewModelCompatAvatarPictureModel* dino_ui_view_model_compat_avatar_picture_model_construct (GType object_type,
                                                                                                   DinoStreamInteractor* stream_interactor);
VALA_EXTERN GType dino_ui_view_model_avatar_picture_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelAvatarPictureModel, g_object_unref)
VALA_EXTERN GType dino_ui_view_model_compat_avatar_picture_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelCompatAvatarPictureModel, g_object_unref)
VALA_EXTERN DinoUiViewModelCompatAvatarPictureModel* dino_ui_view_model_compat_avatar_picture_model_set_conversation (DinoUiViewModelCompatAvatarPictureModel* self,
                                                                                                          DinoEntitiesConversation* conversation);
VALA_EXTERN DinoUiCompatAvatarDrawer* dino_ui_compat_avatar_drawer_new (void);
VALA_EXTERN DinoUiCompatAvatarDrawer* dino_ui_compat_avatar_drawer_construct (GType object_type);
VALA_EXTERN void dino_ui_compat_avatar_drawer_set_model (DinoUiCompatAvatarDrawer* self,
                                             DinoUiViewModelAvatarPictureModel* value);
VALA_EXTERN void dino_ui_compat_avatar_drawer_set_width_request (DinoUiCompatAvatarDrawer* self,
                                                     gint value);
VALA_EXTERN void dino_ui_compat_avatar_drawer_set_height_request (DinoUiCompatAvatarDrawer* self,
                                                      gint value);
VALA_EXTERN cairo_surface_t* dino_ui_compat_avatar_drawer_draw_image_surface (DinoUiCompatAvatarDrawer* self);
static void dino_ui_gnotifications_notifier_finalize (GObject * obj);
static GType dino_ui_gnotifications_notifier_get_type_once (void);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
dino_ui_gnotifications_notifier_get_instance_private (DinoUiGNotificationsNotifier* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiGNotificationsNotifier_private_offset);
}

static guint
_dino_entities_conversation_hash_func_gee_hash_data_func (gconstpointer v,
                                                          gpointer self)
{
	guint result;
	result = dino_entities_conversation_hash_func ((DinoEntitiesConversation*) v);
	return result;
}

static gboolean
_dino_entities_conversation_equals_func_gee_equal_data_func (gconstpointer a,
                                                             gconstpointer b,
                                                             gpointer self)
{
	gboolean result;
	result = dino_entities_conversation_equals_func ((DinoEntitiesConversation*) a, (DinoEntitiesConversation*) b);
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

DinoUiGNotificationsNotifier*
dino_ui_gnotifications_notifier_construct (GType object_type,
                                           DinoStreamInteractor* stream_interactor)
{
	DinoUiGNotificationsNotifier * self = NULL;
	DinoStreamInteractor* _tmp0_;
	g_return_val_if_fail (stream_interactor != NULL, NULL);
	self = (DinoUiGNotificationsNotifier*) g_object_new (object_type, NULL);
	_tmp0_ = _g_object_ref0 (stream_interactor);
	_g_object_unref0 (self->priv->stream_interactor);
	self->priv->stream_interactor = _tmp0_;
	return self;
}

DinoUiGNotificationsNotifier*
dino_ui_gnotifications_notifier_new (DinoStreamInteractor* stream_interactor)
{
	return dino_ui_gnotifications_notifier_construct (DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, stream_interactor);
}

static gdouble
dino_ui_gnotifications_notifier_real_get_priority (DinoNotificationProvider* base)
{
	DinoUiGNotificationsNotifier * self;
	gdouble result;
	self = (DinoUiGNotificationsNotifier*) base;
	result = (gdouble) 0;
	return result;
}

static void
dino_ui_gnotifications_notifier_real_notify_message_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyMessageData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->message);
	_g_object_unref0 (_data_->conversation);
	_g_free0 (_data_->conversation_display_name);
	_g_free0 (_data_->participant_display_name);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyMessageData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_notify_message (DinoNotificationProvider* base,
                                                     DinoEntitiesMessage* message,
                                                     DinoEntitiesConversation* conversation,
                                                     const gchar* conversation_display_name,
                                                     const gchar* participant_display_name,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyMessageData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesMessage* _tmp1_;
	DinoEntitiesConversation* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	g_return_if_fail (message != NULL);
	g_return_if_fail (conversation != NULL);
	g_return_if_fail (conversation_display_name != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyMessageData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_message_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (message);
	_g_object_unref0 (_data_->message);
	_data_->message = _tmp1_;
	_tmp2_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp2_;
	_tmp3_ = g_strdup (conversation_display_name);
	_g_free0 (_data_->conversation_display_name);
	_data_->conversation_display_name = _tmp3_;
	_tmp4_ = g_strdup (participant_display_name);
	_g_free0 (_data_->participant_display_name);
	_data_->participant_display_name = _tmp4_;
	dino_ui_gnotifications_notifier_real_notify_message_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_message_finish (DinoNotificationProvider* base,
                                                       GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyMessageData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static const gchar*
string_to_string (const gchar* self)
{
	const gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self;
	return result;
}

static void
dino_ui_gnotifications_notifier_notify_message_ready (GObject* source_object,
                                                      GAsyncResult* _res_,
                                                      gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyMessageData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_real_notify_message_co (_data_);
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_message_co (DinoUiGnotificationsNotifierNotifyMessageData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = dino_entities_message_get_body (_data_->message);
	_data_->_tmp1_ = _data_->_tmp0_;
	_data_->_tmp2_ = g_strdup (_data_->_tmp1_);
	_data_->text = _data_->_tmp2_;
	if (_data_->participant_display_name != NULL) {
		_data_->_tmp3_ = string_to_string (_data_->participant_display_name);
		_data_->_tmp4_ = _data_->text;
		_data_->_tmp5_ = string_to_string (_data_->_tmp4_);
		_data_->_tmp6_ = g_strconcat (_data_->_tmp3_, ": ", _data_->_tmp5_, NULL);
		_g_free0 (_data_->text);
		_data_->text = _data_->_tmp6_;
	}
	_data_->_tmp7_ = _data_->text;
	_data_->_state_ = 1;
	dino_ui_gnotifications_notifier_notify_content_item (_data_->self, _data_->conversation, _data_->conversation_display_name, _data_->_tmp7_, dino_ui_gnotifications_notifier_notify_message_ready, _data_);
	return FALSE;
	_state_1:
	dino_ui_gnotifications_notifier_notify_content_item_finish (_data_->self, _data_->_res_);
	_g_free0 (_data_->text);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_file_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyFileData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->file_transfer);
	_g_object_unref0 (_data_->conversation);
	_g_free0 (_data_->conversation_display_name);
	_g_free0 (_data_->participant_display_name);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyFileData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_notify_file (DinoNotificationProvider* base,
                                                  DinoEntitiesFileTransfer* file_transfer,
                                                  DinoEntitiesConversation* conversation,
                                                  gboolean is_image,
                                                  const gchar* conversation_display_name,
                                                  const gchar* participant_display_name,
                                                  GAsyncReadyCallback _callback_,
                                                  gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyFileData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesFileTransfer* _tmp1_;
	DinoEntitiesConversation* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	g_return_if_fail (file_transfer != NULL);
	g_return_if_fail (conversation != NULL);
	g_return_if_fail (conversation_display_name != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyFileData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_file_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (file_transfer);
	_g_object_unref0 (_data_->file_transfer);
	_data_->file_transfer = _tmp1_;
	_tmp2_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp2_;
	_data_->is_image = is_image;
	_tmp3_ = g_strdup (conversation_display_name);
	_g_free0 (_data_->conversation_display_name);
	_data_->conversation_display_name = _tmp3_;
	_tmp4_ = g_strdup (participant_display_name);
	_g_free0 (_data_->participant_display_name);
	_data_->participant_display_name = _tmp4_;
	dino_ui_gnotifications_notifier_real_notify_file_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_file_finish (DinoNotificationProvider* base,
                                                    GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyFileData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_gnotifications_notifier_notify_file_ready (GObject* source_object,
                                                   GAsyncResult* _res_,
                                                   gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyFileData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_real_notify_file_co (_data_);
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_file_co (DinoUiGnotificationsNotifierNotifyFileData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = g_strdup ("");
	_data_->text = _data_->_tmp0_;
	_data_->_tmp1_ = dino_entities_file_transfer_get_direction (_data_->file_transfer);
	_data_->_tmp2_ = _data_->_tmp1_;
	if (_data_->_tmp2_ == DINO_ENTITIES_MESSAGE_DIRECTION_SENT) {
		if (_data_->is_image) {
			_data_->_tmp4_ = dino_gettext ("Image sent");
			_data_->_tmp3_ = _data_->_tmp4_;
		} else {
			_data_->_tmp5_ = dino_gettext ("File sent");
			_data_->_tmp3_ = _data_->_tmp5_;
		}
		_data_->_tmp6_ = g_strdup (_data_->_tmp3_);
		_g_free0 (_data_->text);
		_data_->text = _data_->_tmp6_;
	} else {
		if (_data_->is_image) {
			_data_->_tmp8_ = dino_gettext ("Image received");
			_data_->_tmp7_ = _data_->_tmp8_;
		} else {
			_data_->_tmp9_ = dino_gettext ("File received");
			_data_->_tmp7_ = _data_->_tmp9_;
		}
		_data_->_tmp10_ = g_strdup (_data_->_tmp7_);
		_g_free0 (_data_->text);
		_data_->text = _data_->_tmp10_;
	}
	if (_data_->participant_display_name != NULL) {
		_data_->_tmp11_ = string_to_string (_data_->participant_display_name);
		_data_->_tmp12_ = _data_->text;
		_data_->_tmp13_ = string_to_string (_data_->_tmp12_);
		_data_->_tmp14_ = g_strconcat (_data_->_tmp11_, ": ", _data_->_tmp13_, NULL);
		_g_free0 (_data_->text);
		_data_->text = _data_->_tmp14_;
	}
	_data_->_tmp15_ = _data_->text;
	_data_->_state_ = 1;
	dino_ui_gnotifications_notifier_notify_content_item (_data_->self, _data_->conversation, _data_->conversation_display_name, _data_->_tmp15_, dino_ui_gnotifications_notifier_notify_file_ready, _data_);
	return FALSE;
	_state_1:
	dino_ui_gnotifications_notifier_notify_content_item_finish (_data_->self, _data_->_res_);
	_g_free0 (_data_->text);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_notify_content_item_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyContentItemData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->conversation);
	_g_free0 (_data_->title);
	_g_free0 (_data_->body);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyContentItemData, _data_);
}

static void
dino_ui_gnotifications_notifier_notify_content_item (DinoUiGNotificationsNotifier* self,
                                                     DinoEntitiesConversation* conversation,
                                                     const gchar* title,
                                                     const gchar* body,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyContentItemData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesConversation* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (conversation != NULL);
	g_return_if_fail (title != NULL);
	g_return_if_fail (body != NULL);
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyContentItemData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_notify_content_item_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp1_;
	_tmp2_ = g_strdup (title);
	_g_free0 (_data_->title);
	_data_->title = _tmp2_;
	_tmp3_ = g_strdup (body);
	_g_free0 (_data_->body);
	_data_->body = _tmp3_;
	dino_ui_gnotifications_notifier_notify_content_item_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_content_item_finish (DinoUiGNotificationsNotifier* self,
                                                            GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyContentItemData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_gnotifications_notifier_notify_content_item_ready (GObject* source_object,
                                                           GAsyncResult* _res_,
                                                           gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyContentItemData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_notify_content_item_co (_data_);
}

static gboolean
dino_ui_gnotifications_notifier_notify_content_item_co (DinoUiGnotificationsNotifierNotifyContentItemData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->notifications;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _data_->_tmp0_, _data_->conversation)) {
		_data_->_tmp1_ = _data_->self->priv->notifications;
		_data_->_tmp2_ = g_notification_new ("");
		_data_->_tmp3_ = _data_->_tmp2_;
		gee_abstract_map_set ((GeeAbstractMap*) _data_->_tmp1_, _data_->conversation, _data_->_tmp3_);
		_g_object_unref0 (_data_->_tmp3_);
		_data_->_tmp4_ = _data_->self->priv->notifications;
		_data_->_tmp5_ = gee_abstract_map_get ((GeeAbstractMap*) _data_->_tmp4_, _data_->conversation);
		_data_->_tmp6_ = (GNotification*) _data_->_tmp5_;
		_data_->_tmp7_ = dino_entities_conversation_get_id (_data_->conversation);
		_data_->_tmp8_ = _data_->_tmp7_;
		_data_->_tmp9_ = g_variant_new_int32 ((gint32) _data_->_tmp8_);
		g_variant_ref_sink (_data_->_tmp9_);
		_data_->_tmp10_ = _data_->_tmp9_;
		g_notification_set_default_action_and_target_value (_data_->_tmp6_, "app.open-conversation", _data_->_tmp10_);
		_g_variant_unref0 (_data_->_tmp10_);
		_g_object_unref0 (_data_->_tmp6_);
	}
	_data_->_tmp11_ = _data_->self->priv->notifications;
	_data_->_tmp12_ = gee_abstract_map_get ((GeeAbstractMap*) _data_->_tmp11_, _data_->conversation);
	_data_->notification = (GNotification*) _data_->_tmp12_;
	g_notification_set_title (_data_->notification, _data_->title);
	g_notification_set_body (_data_->notification, _data_->body);
	{
		_data_->_state_ = 1;
		dino_ui_gnotifications_notifier_get_conversation_icon (_data_->self, _data_->conversation, dino_ui_gnotifications_notifier_notify_content_item_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp14_ = dino_ui_gnotifications_notifier_get_conversation_icon_finish (_data_->self, _data_->_res_, &_data_->_inner_error0_);
		_data_->_tmp13_ = _data_->_tmp14_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		g_notification_set_icon (_data_->notification, _data_->_tmp13_);
		_g_object_unref0 (_data_->_tmp13_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_data_->_inner_error0_);
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_object_unref0 (_data_->notification);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp15_ = g_application_get_default ();
	_data_->_tmp16_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp17_ = _data_->_tmp16_;
	_data_->_tmp18_ = g_strdup_printf ("%i", _data_->_tmp17_);
	_data_->_tmp19_ = _data_->_tmp18_;
	_data_->_tmp20_ = _data_->self->priv->notifications;
	_data_->_tmp21_ = gee_abstract_map_get ((GeeAbstractMap*) _data_->_tmp20_, _data_->conversation);
	_data_->_tmp22_ = (GNotification*) _data_->_tmp21_;
	g_application_send_notification (_data_->_tmp15_, _data_->_tmp19_, _data_->_tmp22_);
	_g_object_unref0 (_data_->_tmp22_);
	_g_free0 (_data_->_tmp19_);
	_data_->_tmp23_ = _data_->self->priv->active_conversation_ids;
	if (_data_->_tmp23_ != NULL) {
		_data_->_tmp24_ = _data_->self->priv->active_conversation_ids;
		_data_->_tmp25_ = dino_entities_conversation_get_id (_data_->conversation);
		_data_->_tmp26_ = _data_->_tmp25_;
		_data_->_tmp27_ = g_strdup_printf ("%i", _data_->_tmp26_);
		_data_->_tmp28_ = _data_->_tmp27_;
		gee_collection_add ((GeeCollection*) _data_->_tmp24_, _data_->_tmp28_);
		_g_free0 (_data_->_tmp28_);
	}
	_g_object_unref0 (_data_->notification);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_call_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyCallData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->call);
	_g_object_unref0 (_data_->conversation);
	_g_free0 (_data_->conversation_display_name);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyCallData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_notify_call (DinoNotificationProvider* base,
                                                  DinoEntitiesCall* call,
                                                  DinoEntitiesConversation* conversation,
                                                  gboolean video,
                                                  gboolean multiparty,
                                                  const gchar* conversation_display_name,
                                                  GAsyncReadyCallback _callback_,
                                                  gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyCallData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesCall* _tmp1_;
	DinoEntitiesConversation* _tmp2_;
	gchar* _tmp3_;
	g_return_if_fail (call != NULL);
	g_return_if_fail (conversation != NULL);
	g_return_if_fail (conversation_display_name != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyCallData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_call_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (call);
	_g_object_unref0 (_data_->call);
	_data_->call = _tmp1_;
	_tmp2_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp2_;
	_data_->video = video;
	_data_->multiparty = multiparty;
	_tmp3_ = g_strdup (conversation_display_name);
	_g_free0 (_data_->conversation_display_name);
	_data_->conversation_display_name = _tmp3_;
	dino_ui_gnotifications_notifier_real_notify_call_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_call_finish (DinoNotificationProvider* base,
                                                    GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyCallData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_call_co (DinoUiGnotificationsNotifierNotifyCallData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = g_notification_new (_data_->conversation_display_name);
	_data_->notification = _data_->_tmp0_;
	if (_data_->video) {
		_data_->_tmp2_ = dino_gettext ("Incoming video call");
		_data_->_tmp1_ = _data_->_tmp2_;
	} else {
		_data_->_tmp3_ = dino_gettext ("Incoming call");
		_data_->_tmp1_ = _data_->_tmp3_;
	}
	_data_->_tmp4_ = g_strdup (_data_->_tmp1_);
	_data_->body = _data_->_tmp4_;
	if (_data_->multiparty) {
		if (_data_->video) {
			_data_->_tmp6_ = dino_gettext ("Incoming video group call");
			_data_->_tmp5_ = _data_->_tmp6_;
		} else {
			_data_->_tmp7_ = dino_gettext ("Incoming group call");
			_data_->_tmp5_ = _data_->_tmp7_;
		}
		_data_->_tmp8_ = g_strdup (_data_->_tmp5_);
		_g_free0 (_data_->body);
		_data_->body = _data_->_tmp8_;
	}
	_data_->_tmp9_ = _data_->notification;
	_data_->_tmp10_ = _data_->body;
	g_notification_set_body (_data_->_tmp9_, _data_->_tmp10_);
	_data_->_tmp11_ = _data_->notification;
	g_notification_set_urgent (_data_->_tmp11_, TRUE);
	_data_->_tmp12_ = _data_->notification;
	_data_->_tmp13_ = g_strdup ("call-start-symbolic");
	_data_->_tmp14_ = g_new0 (gchar*, 1 + 1);
	_data_->_tmp14_[0] = _data_->_tmp13_;
	_data_->_tmp15_ = _data_->_tmp14_;
	_data_->_tmp15__length1 = 1;
	_data_->_tmp16_ = (GThemedIcon*) g_themed_icon_new_from_names (_data_->_tmp15_, 1);
	_data_->_tmp17_ = _data_->_tmp16_;
	g_notification_set_icon (_data_->_tmp12_, (GIcon*) _data_->_tmp17_);
	_g_object_unref0 (_data_->_tmp17_);
	_data_->_tmp15_ = (_vala_array_free (_data_->_tmp15_, _data_->_tmp15__length1, (GDestroyNotify) g_free), NULL);
	_data_->_tmp18_ = _data_->notification;
	_data_->_tmp19_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp20_ = _data_->_tmp19_;
	_data_->_tmp21_ = g_variant_new_int32 ((gint32) _data_->_tmp20_);
	g_variant_ref_sink (_data_->_tmp21_);
	_data_->_tmp22_ = _data_->_tmp21_;
	g_notification_set_default_action_and_target_value (_data_->_tmp18_, "app.open-conversation", _data_->_tmp22_);
	_g_variant_unref0 (_data_->_tmp22_);
	_data_->_tmp23_ = _data_->notification;
	_data_->_tmp24_ = dino_gettext ("Reject");
	_data_->_tmp25_ = dino_entities_call_get_id (_data_->call);
	_data_->_tmp26_ = _data_->_tmp25_;
	_data_->_tmp27_ = g_variant_new_int32 ((gint32) _data_->_tmp26_);
	g_variant_ref_sink (_data_->_tmp27_);
	_data_->_tmp28_ = _data_->_tmp27_;
	g_notification_add_button_with_target_value (_data_->_tmp23_, _data_->_tmp24_, "app.reject-call", _data_->_tmp28_);
	_g_variant_unref0 (_data_->_tmp28_);
	_data_->_tmp29_ = _data_->notification;
	_data_->_tmp30_ = dino_gettext ("Accept");
	_data_->_tmp31_ = dino_entities_call_get_id (_data_->call);
	_data_->_tmp32_ = _data_->_tmp31_;
	_data_->_tmp33_ = g_variant_new_int32 ((gint32) _data_->_tmp32_);
	g_variant_ref_sink (_data_->_tmp33_);
	_data_->_tmp34_ = _data_->_tmp33_;
	g_notification_add_button_with_target_value (_data_->_tmp29_, _data_->_tmp30_, "app.accept-call", _data_->_tmp34_);
	_g_variant_unref0 (_data_->_tmp34_);
	_data_->_tmp35_ = g_application_get_default ();
	_data_->_tmp36_ = dino_entities_call_get_id (_data_->call);
	_data_->_tmp37_ = _data_->_tmp36_;
	_data_->_tmp38_ = g_strdup_printf ("%i", _data_->_tmp37_);
	_data_->_tmp39_ = _data_->_tmp38_;
	_data_->_tmp40_ = _data_->notification;
	g_application_send_notification (_data_->_tmp35_, _data_->_tmp39_, _data_->_tmp40_);
	_g_free0 (_data_->_tmp39_);
	_g_free0 (_data_->body);
	_g_object_unref0 (_data_->notification);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_retract_call_notification_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierRetractCallNotificationData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->call);
	_g_object_unref0 (_data_->conversation);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierRetractCallNotificationData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_retract_call_notification (DinoNotificationProvider* base,
                                                                DinoEntitiesCall* call,
                                                                DinoEntitiesConversation* conversation,
                                                                GAsyncReadyCallback _callback_,
                                                                gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierRetractCallNotificationData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesCall* _tmp1_;
	DinoEntitiesConversation* _tmp2_;
	g_return_if_fail (call != NULL);
	g_return_if_fail (conversation != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierRetractCallNotificationData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_retract_call_notification_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (call);
	_g_object_unref0 (_data_->call);
	_data_->call = _tmp1_;
	_tmp2_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp2_;
	dino_ui_gnotifications_notifier_real_retract_call_notification_co (_data_);
}

static void
dino_ui_gnotifications_notifier_retract_call_notification_finish (DinoNotificationProvider* base,
                                                                  GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierRetractCallNotificationData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static gboolean
dino_ui_gnotifications_notifier_real_retract_call_notification_co (DinoUiGnotificationsNotifierRetractCallNotificationData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = g_application_get_default ();
	_data_->_tmp1_ = dino_entities_call_get_id (_data_->call);
	_data_->_tmp2_ = _data_->_tmp1_;
	_data_->_tmp3_ = g_strdup_printf ("%i", _data_->_tmp2_);
	_data_->_tmp4_ = _data_->_tmp3_;
	g_application_withdraw_notification (_data_->_tmp0_, _data_->_tmp4_);
	_g_free0 (_data_->_tmp4_);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_subscription_request_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->conversation);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifySubscriptionRequestData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_notify_subscription_request (DinoNotificationProvider* base,
                                                                  DinoEntitiesConversation* conversation,
                                                                  GAsyncReadyCallback _callback_,
                                                                  gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesConversation* _tmp1_;
	g_return_if_fail (conversation != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifySubscriptionRequestData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_subscription_request_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp1_;
	dino_ui_gnotifications_notifier_real_notify_subscription_request_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_subscription_request_finish (DinoNotificationProvider* base,
                                                                    GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_gnotifications_notifier_notify_subscription_request_ready (GObject* source_object,
                                                                   GAsyncResult* _res_,
                                                                   gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_real_notify_subscription_request_co (_data_);
}

static GVariant*
_variant_new5 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static GVariant*
_variant_new6 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_subscription_request_co (DinoUiGnotificationsNotifierNotifySubscriptionRequestData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = dino_gettext ("Subscription request");
	_data_->_tmp1_ = g_notification_new (_data_->_tmp0_);
	_data_->notification = _data_->_tmp1_;
	_data_->_tmp2_ = dino_entities_conversation_get_counterpart (_data_->conversation);
	_data_->_tmp3_ = _data_->_tmp2_;
	_data_->_tmp4_ = xmpp_jid_to_string (_data_->_tmp3_);
	_data_->_tmp5_ = _data_->_tmp4_;
	g_notification_set_body (_data_->notification, _data_->_tmp5_);
	_g_free0 (_data_->_tmp5_);
	{
		_data_->_state_ = 1;
		dino_ui_gnotifications_notifier_get_conversation_icon (_data_->self, _data_->conversation, dino_ui_gnotifications_notifier_notify_subscription_request_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp7_ = dino_ui_gnotifications_notifier_get_conversation_icon_finish (_data_->self, _data_->_res_, &_data_->_inner_error0_);
		_data_->_tmp6_ = _data_->_tmp7_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		g_notification_set_icon (_data_->notification, _data_->_tmp6_);
		_g_object_unref0 (_data_->_tmp6_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_data_->_inner_error0_);
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_object_unref0 (_data_->notification);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp8_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp9_ = _data_->_tmp8_;
	_data_->_tmp10_ = g_variant_new_int32 ((gint32) _data_->_tmp9_);
	g_variant_ref_sink (_data_->_tmp10_);
	_data_->_tmp11_ = _data_->_tmp10_;
	g_notification_set_default_action_and_target_value (_data_->notification, "app.open-conversation", _data_->_tmp11_);
	_g_variant_unref0 (_data_->_tmp11_);
	_data_->_tmp12_ = dino_gettext ("Accept");
	_data_->_tmp13_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp14_ = _data_->_tmp13_;
	_data_->_tmp15_ = _variant_new5 (_data_->_tmp14_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp12_, "app.accept-subscription", _data_->_tmp15_);
	_g_variant_unref0 (_data_->_tmp15_);
	_data_->_tmp16_ = dino_gettext ("Deny");
	_data_->_tmp17_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp18_ = _data_->_tmp17_;
	_data_->_tmp19_ = _variant_new6 (_data_->_tmp18_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp16_, "app.deny-subscription", _data_->_tmp19_);
	_g_variant_unref0 (_data_->_tmp19_);
	_data_->_tmp20_ = g_application_get_default ();
	_data_->_tmp21_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp22_ = _data_->_tmp21_;
	_data_->_tmp23_ = g_strdup_printf ("%i", _data_->_tmp22_);
	_data_->_tmp24_ = _data_->_tmp23_;
	_data_->_tmp25_ = g_strconcat (_data_->_tmp24_, "-subscription", NULL);
	_data_->_tmp26_ = _data_->_tmp25_;
	g_application_send_notification (_data_->_tmp20_, _data_->_tmp26_, _data_->notification);
	_g_free0 (_data_->_tmp26_);
	_g_free0 (_data_->_tmp24_);
	_data_->_tmp27_ = _data_->self->priv->active_ids;
	_data_->_tmp28_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp29_ = _data_->_tmp28_;
	_data_->_tmp30_ = g_strdup_printf ("%i", _data_->_tmp29_);
	_data_->_tmp31_ = _data_->_tmp30_;
	_data_->_tmp32_ = g_strconcat (_data_->_tmp31_, "-subscription", NULL);
	_data_->_tmp33_ = _data_->_tmp32_;
	gee_collection_add ((GeeCollection*) _data_->_tmp27_, _data_->_tmp33_);
	_g_free0 (_data_->_tmp33_);
	_g_free0 (_data_->_tmp31_);
	_g_object_unref0 (_data_->notification);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_connection_error_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyConnectionErrorData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->account);
	_dino_connection_manager_connection_error_unref0 (_data_->_error_);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyConnectionErrorData, _data_);
}

static gpointer
_dino_connection_manager_connection_error_ref0 (gpointer self)
{
	return self ? dino_connection_manager_connection_error_ref (self) : NULL;
}

static void
dino_ui_gnotifications_notifier_real_notify_connection_error (DinoNotificationProvider* base,
                                                              DinoEntitiesAccount* account,
                                                              DinoConnectionManagerConnectionError* _error_,
                                                              GAsyncReadyCallback _callback_,
                                                              gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyConnectionErrorData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesAccount* _tmp1_;
	DinoConnectionManagerConnectionError* _tmp2_;
	g_return_if_fail (account != NULL);
	g_return_if_fail (_error_ != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyConnectionErrorData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_connection_error_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (account);
	_g_object_unref0 (_data_->account);
	_data_->account = _tmp1_;
	_tmp2_ = _dino_connection_manager_connection_error_ref0 (_error_);
	_dino_connection_manager_connection_error_unref0 (_data_->_error_);
	_data_->_error_ = _tmp2_;
	dino_ui_gnotifications_notifier_real_notify_connection_error_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_connection_error_finish (DinoNotificationProvider* base,
                                                                GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyConnectionErrorData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_connection_error_co (DinoUiGnotificationsNotifierNotifyConnectionErrorData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = dino_gettext ("Could not connect to %s");
	_data_->_tmp1_ = dino_entities_account_get_bare_jid (_data_->account);
	_data_->_tmp2_ = _data_->_tmp1_;
	_data_->_tmp3_ = _data_->_tmp2_;
	_data_->_tmp4_ = _data_->_tmp3_->domainpart;
	_data_->_tmp5_ = g_strdup_printf (_data_->_tmp0_, _data_->_tmp4_);
	_data_->_tmp6_ = _data_->_tmp5_;
	_data_->_tmp7_ = g_notification_new (_data_->_tmp6_);
	_data_->_tmp8_ = _data_->_tmp7_;
	_g_free0 (_data_->_tmp6_);
	_xmpp_jid_unref0 (_data_->_tmp3_);
	_data_->notification = _data_->_tmp8_;
	_data_->_tmp9_ = _data_->notification;
	_data_->_tmp10_ = dino_entities_account_get_id (_data_->account);
	_data_->_tmp11_ = _data_->_tmp10_;
	_data_->_tmp12_ = g_variant_new_int32 ((gint32) _data_->_tmp11_);
	g_variant_ref_sink (_data_->_tmp12_);
	_data_->_tmp13_ = _data_->_tmp12_;
	g_notification_set_default_action_and_target_value (_data_->_tmp9_, "app.preferences-account", _data_->_tmp13_);
	_g_variant_unref0 (_data_->_tmp13_);
	switch (_data_->_error_->source) {
		case DINO_CONNECTION_MANAGER_CONNECTION_ERROR_SOURCE_SASL:
		{
			_data_->_tmp14_ = _data_->notification;
			g_notification_set_body (_data_->_tmp14_, "Wrong password");
			break;
		}
		case DINO_CONNECTION_MANAGER_CONNECTION_ERROR_SOURCE_TLS:
		{
			_data_->_tmp15_ = _data_->notification;
			g_notification_set_body (_data_->_tmp15_, "Invalid TLS certificate");
			break;
		}
		default:
		{
			break;
		}
	}
	_data_->_tmp16_ = g_application_get_default ();
	_data_->_tmp17_ = dino_entities_account_get_id (_data_->account);
	_data_->_tmp18_ = _data_->_tmp17_;
	_data_->_tmp19_ = g_strdup_printf ("%i", _data_->_tmp18_);
	_data_->_tmp20_ = _data_->_tmp19_;
	_data_->_tmp21_ = g_strconcat (_data_->_tmp20_, "-connection-error", NULL);
	_data_->_tmp22_ = _data_->_tmp21_;
	_data_->_tmp23_ = _data_->notification;
	g_application_send_notification (_data_->_tmp16_, _data_->_tmp22_, _data_->_tmp23_);
	_g_free0 (_data_->_tmp22_);
	_g_free0 (_data_->_tmp20_);
	_g_object_unref0 (_data_->notification);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_muc_invite_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyMucInviteData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->account);
	_xmpp_jid_unref0 (_data_->room_jid);
	_xmpp_jid_unref0 (_data_->from_jid);
	_g_free0 (_data_->inviter_display_name);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyMucInviteData, _data_);
}

static gpointer
_xmpp_jid_ref0 (gpointer self)
{
	return self ? xmpp_jid_ref (self) : NULL;
}

static void
dino_ui_gnotifications_notifier_real_notify_muc_invite (DinoNotificationProvider* base,
                                                        DinoEntitiesAccount* account,
                                                        XmppJid* room_jid,
                                                        XmppJid* from_jid,
                                                        const gchar* inviter_display_name,
                                                        GAsyncReadyCallback _callback_,
                                                        gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyMucInviteData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesAccount* _tmp1_;
	XmppJid* _tmp2_;
	XmppJid* _tmp3_;
	gchar* _tmp4_;
	g_return_if_fail (account != NULL);
	g_return_if_fail (room_jid != NULL);
	g_return_if_fail (from_jid != NULL);
	g_return_if_fail (inviter_display_name != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyMucInviteData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_muc_invite_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (account);
	_g_object_unref0 (_data_->account);
	_data_->account = _tmp1_;
	_tmp2_ = _xmpp_jid_ref0 (room_jid);
	_xmpp_jid_unref0 (_data_->room_jid);
	_data_->room_jid = _tmp2_;
	_tmp3_ = _xmpp_jid_ref0 (from_jid);
	_xmpp_jid_unref0 (_data_->from_jid);
	_data_->from_jid = _tmp3_;
	_tmp4_ = g_strdup (inviter_display_name);
	_g_free0 (_data_->inviter_display_name);
	_data_->inviter_display_name = _tmp4_;
	dino_ui_gnotifications_notifier_real_notify_muc_invite_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_muc_invite_finish (DinoNotificationProvider* base,
                                                          GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyMucInviteData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_gnotifications_notifier_notify_muc_invite_ready (GObject* source_object,
                                                         GAsyncResult* _res_,
                                                         gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyMucInviteData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_real_notify_muc_invite_co (_data_);
}

static GVariant*
_variant_new7 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static GVariant*
_variant_new8 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_muc_invite_co (DinoUiGnotificationsNotifierNotifyMucInviteData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = dino_entities_conversation_new (_data_->from_jid, _data_->account, DINO_ENTITIES_CONVERSATION_TYPE_CHAT);
	_data_->direct_conversation = _data_->_tmp0_;
	_data_->_tmp1_ = xmpp_jid_get_bare_jid (_data_->room_jid);
	_data_->_tmp2_ = _data_->_tmp1_;
	_data_->_tmp3_ = _data_->_tmp2_;
	_data_->_tmp4_ = xmpp_jid_to_string (_data_->_tmp3_);
	_data_->_tmp5_ = _data_->_tmp4_;
	_xmpp_jid_unref0 (_data_->_tmp3_);
	_data_->display_room = _data_->_tmp5_;
	_data_->_tmp6_ = dino_gettext ("Invitation to %s");
	_data_->_tmp7_ = g_strdup_printf (_data_->_tmp6_, _data_->display_room);
	_data_->_tmp8_ = _data_->_tmp7_;
	_data_->_tmp9_ = g_notification_new (_data_->_tmp8_);
	_data_->_tmp10_ = _data_->_tmp9_;
	_g_free0 (_data_->_tmp8_);
	_data_->notification = _data_->_tmp10_;
	_data_->_tmp11_ = dino_gettext ("%s invited you to %s");
	_data_->_tmp12_ = g_strdup_printf (_data_->_tmp11_, _data_->inviter_display_name, _data_->display_room);
	_data_->body = _data_->_tmp12_;
	g_notification_set_body (_data_->notification, _data_->body);
	{
		_data_->_state_ = 1;
		dino_ui_gnotifications_notifier_get_conversation_icon (_data_->self, _data_->direct_conversation, dino_ui_gnotifications_notifier_notify_muc_invite_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp14_ = dino_ui_gnotifications_notifier_get_conversation_icon_finish (_data_->self, _data_->_res_, &_data_->_inner_error0_);
		_data_->_tmp13_ = _data_->_tmp14_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		g_notification_set_icon (_data_->notification, _data_->_tmp13_);
		_g_object_unref0 (_data_->_tmp13_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_data_->_inner_error0_);
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_free0 (_data_->body);
		_g_object_unref0 (_data_->notification);
		_g_free0 (_data_->display_room);
		_g_object_unref0 (_data_->direct_conversation);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp15_ = _data_->self->priv->stream_interactor;
	_data_->_tmp16_ = dino_conversation_manager_IDENTITY;
	_data_->_tmp17_ = dino_stream_interactor_get_module (_data_->_tmp15_, DINO_TYPE_CONVERSATION_MANAGER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp16_);
	_data_->_tmp18_ = (DinoConversationManager*) _data_->_tmp17_;
	_data_->_tmp19_ = DINO_ENTITIES_CONVERSATION_TYPE_GROUPCHAT;
	_data_->_tmp20_ = dino_conversation_manager_create_conversation (_data_->_tmp18_, _data_->room_jid, _data_->account, &_data_->_tmp19_);
	_data_->_tmp21_ = _data_->_tmp20_;
	_g_object_unref0 (_data_->_tmp18_);
	_data_->group_conversation = _data_->_tmp21_;
	_data_->_tmp22_ = dino_entities_conversation_get_id (_data_->group_conversation);
	_data_->_tmp23_ = _data_->_tmp22_;
	_data_->_tmp24_ = g_variant_new_int32 ((gint32) _data_->_tmp23_);
	g_variant_ref_sink (_data_->_tmp24_);
	_data_->_tmp25_ = _data_->_tmp24_;
	g_notification_set_default_action_and_target_value (_data_->notification, "app.open-muc-join", _data_->_tmp25_);
	_g_variant_unref0 (_data_->_tmp25_);
	_data_->_tmp26_ = dino_gettext ("Deny");
	_data_->_tmp27_ = dino_entities_conversation_get_id (_data_->group_conversation);
	_data_->_tmp28_ = _data_->_tmp27_;
	_data_->_tmp29_ = _variant_new7 (_data_->_tmp28_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp26_, "app.deny-invite", _data_->_tmp29_);
	_g_variant_unref0 (_data_->_tmp29_);
	_data_->_tmp30_ = dino_gettext ("Accept");
	_data_->_tmp31_ = dino_entities_conversation_get_id (_data_->group_conversation);
	_data_->_tmp32_ = _data_->_tmp31_;
	_data_->_tmp33_ = _variant_new8 (_data_->_tmp32_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp30_, "app.open-muc-join", _data_->_tmp33_);
	_g_variant_unref0 (_data_->_tmp33_);
	_data_->_tmp34_ = g_application_get_default ();
	g_application_send_notification (_data_->_tmp34_, NULL, _data_->notification);
	_g_object_unref0 (_data_->group_conversation);
	_g_free0 (_data_->body);
	_g_object_unref0 (_data_->notification);
	_g_free0 (_data_->display_room);
	_g_object_unref0 (_data_->direct_conversation);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_notify_voice_request_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->conversation);
	_xmpp_jid_unref0 (_data_->from_jid);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierNotifyVoiceRequestData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_notify_voice_request (DinoNotificationProvider* base,
                                                           DinoEntitiesConversation* conversation,
                                                           XmppJid* from_jid,
                                                           GAsyncReadyCallback _callback_,
                                                           gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesConversation* _tmp1_;
	XmppJid* _tmp2_;
	g_return_if_fail (conversation != NULL);
	g_return_if_fail (from_jid != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierNotifyVoiceRequestData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_notify_voice_request_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp1_;
	_tmp2_ = _xmpp_jid_ref0 (from_jid);
	_xmpp_jid_unref0 (_data_->from_jid);
	_data_->from_jid = _tmp2_;
	dino_ui_gnotifications_notifier_real_notify_voice_request_co (_data_);
}

static void
dino_ui_gnotifications_notifier_notify_voice_request_finish (DinoNotificationProvider* base,
                                                             GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_gnotifications_notifier_notify_voice_request_ready (GObject* source_object,
                                                            GAsyncResult* _res_,
                                                            gpointer _user_data_)
{
	DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_gnotifications_notifier_real_notify_voice_request_co (_data_);
}

static GVariant*
_variant_new9 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static GVariant*
_variant_new10 (gint value)
{
	return g_variant_ref_sink (g_variant_new_int32 (value));
}

static gboolean
dino_ui_gnotifications_notifier_real_notify_voice_request_co (DinoUiGnotificationsNotifierNotifyVoiceRequestData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->stream_interactor;
	_data_->_tmp1_ = dino_ui_util_get_participant_display_name (_data_->_tmp0_, _data_->conversation, _data_->from_jid, FALSE);
	_data_->display_name = _data_->_tmp1_;
	_data_->_tmp2_ = _data_->self->priv->stream_interactor;
	_data_->_tmp3_ = dino_ui_util_get_conversation_display_name (_data_->_tmp2_, _data_->conversation);
	_data_->display_room = _data_->_tmp3_;
	_data_->_tmp4_ = dino_gettext ("Permission request");
	_data_->_tmp5_ = g_notification_new (_data_->_tmp4_);
	_data_->notification = _data_->_tmp5_;
	_data_->_tmp6_ = dino_gettext ("%s requests the permission to write in %s");
	_data_->_tmp7_ = g_strdup_printf (_data_->_tmp6_, _data_->display_name, _data_->display_room);
	_data_->body = _data_->_tmp7_;
	g_notification_set_body (_data_->notification, _data_->body);
	{
		_data_->_state_ = 1;
		dino_ui_gnotifications_notifier_get_conversation_icon (_data_->self, _data_->conversation, dino_ui_gnotifications_notifier_notify_voice_request_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp9_ = dino_ui_gnotifications_notifier_get_conversation_icon_finish (_data_->self, _data_->_res_, &_data_->_inner_error0_);
		_data_->_tmp8_ = _data_->_tmp9_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		g_notification_set_icon (_data_->notification, _data_->_tmp8_);
		_g_object_unref0 (_data_->_tmp8_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_data_->_inner_error0_);
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_free0 (_data_->body);
		_g_object_unref0 (_data_->notification);
		_g_free0 (_data_->display_room);
		_g_free0 (_data_->display_name);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp10_ = dino_gettext ("Deny");
	_data_->_tmp11_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp12_ = _data_->_tmp11_;
	_data_->_tmp13_ = _variant_new9 (_data_->_tmp12_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp10_, "app.deny-voice-request", _data_->_tmp13_);
	_g_variant_unref0 (_data_->_tmp13_);
	_data_->_tmp14_ = dino_gettext ("Accept");
	_data_->_tmp15_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp16_ = _data_->_tmp15_;
	_data_->_tmp17_ = _variant_new10 (_data_->_tmp16_);
	g_notification_add_button_with_target_value (_data_->notification, _data_->_tmp14_, "app.accept-voice-request", _data_->_tmp17_);
	_g_variant_unref0 (_data_->_tmp17_);
	_data_->_tmp18_ = g_application_get_default ();
	g_application_send_notification (_data_->_tmp18_, NULL, _data_->notification);
	_g_free0 (_data_->body);
	_g_object_unref0 (_data_->notification);
	_g_free0 (_data_->display_room);
	_g_free0 (_data_->display_name);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_retract_content_item_notifications_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierRetractContentItemNotificationsData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierRetractContentItemNotificationsData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_retract_content_item_notifications (DinoNotificationProvider* base,
                                                                         GAsyncReadyCallback _callback_,
                                                                         gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierRetractContentItemNotificationsData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierRetractContentItemNotificationsData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_retract_content_item_notifications_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	dino_ui_gnotifications_notifier_real_retract_content_item_notifications_co (_data_);
}

static void
dino_ui_gnotifications_notifier_retract_content_item_notifications_finish (DinoNotificationProvider* base,
                                                                           GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierRetractContentItemNotificationsData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static gboolean
dino_ui_gnotifications_notifier_real_retract_content_item_notifications_co (DinoUiGnotificationsNotifierRetractContentItemNotificationsData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->active_conversation_ids;
	if (_data_->_tmp0_ == NULL) {
		_data_->_tmp1_ = _data_->self->priv->stream_interactor;
		_data_->_tmp2_ = dino_conversation_manager_IDENTITY;
		_data_->_tmp3_ = dino_stream_interactor_get_module (_data_->_tmp1_, DINO_TYPE_CONVERSATION_MANAGER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp2_);
		_data_->_tmp4_ = (DinoConversationManager*) _data_->_tmp3_;
		_data_->_tmp5_ = dino_conversation_manager_get_active_conversations (_data_->_tmp4_, NULL);
		_data_->_tmp6_ = _data_->_tmp5_;
		_g_object_unref0 (_data_->_tmp4_);
		_data_->conversations = _data_->_tmp6_;
		{
			_data_->_tmp7_ = _data_->conversations;
			_data_->_conversation_list = _data_->_tmp7_;
			_data_->_tmp8_ = _data_->_conversation_list;
			_data_->_tmp9_ = gee_collection_get_size ((GeeCollection*) _data_->_tmp8_);
			_data_->_tmp10_ = _data_->_tmp9_;
			_data_->_conversation_size = _data_->_tmp10_;
			_data_->_conversation_index = -1;
			while (TRUE) {
				_data_->_conversation_index = _data_->_conversation_index + 1;
				_data_->_tmp11_ = _data_->_conversation_index;
				_data_->_tmp12_ = _data_->_conversation_size;
				if (!(_data_->_tmp11_ < _data_->_tmp12_)) {
					break;
				}
				_data_->_tmp13_ = _data_->_conversation_list;
				_data_->_tmp14_ = gee_list_get (_data_->_tmp13_, _data_->_conversation_index);
				_data_->conversation = (DinoEntitiesConversation*) _data_->_tmp14_;
				_data_->_tmp15_ = g_application_get_default ();
				_data_->_tmp16_ = _data_->conversation;
				_data_->_tmp17_ = dino_entities_conversation_get_id (_data_->_tmp16_);
				_data_->_tmp18_ = _data_->_tmp17_;
				_data_->_tmp19_ = g_strdup_printf ("%i", _data_->_tmp18_);
				_data_->_tmp20_ = _data_->_tmp19_;
				g_application_withdraw_notification (_data_->_tmp15_, _data_->_tmp20_);
				_g_free0 (_data_->_tmp20_);
				_g_object_unref0 (_data_->conversation);
			}
		}
		_data_->_tmp21_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL, NULL, NULL, NULL);
		_g_object_unref0 (_data_->self->priv->active_conversation_ids);
		_data_->self->priv->active_conversation_ids = (GeeSet*) _data_->_tmp21_;
		_g_object_unref0 (_data_->conversations);
	} else {
		{
			_data_->_tmp22_ = _data_->self->priv->active_conversation_ids;
			_data_->_tmp23_ = gee_iterable_iterator ((GeeIterable*) _data_->_tmp22_);
			_data_->_id_it = _data_->_tmp23_;
			while (TRUE) {
				_data_->_tmp24_ = _data_->_id_it;
				if (!gee_iterator_next (_data_->_tmp24_)) {
					break;
				}
				_data_->_tmp25_ = _data_->_id_it;
				_data_->_tmp26_ = gee_iterator_get (_data_->_tmp25_);
				_data_->id = (gchar*) _data_->_tmp26_;
				_data_->_tmp27_ = g_application_get_default ();
				_data_->_tmp28_ = _data_->id;
				g_application_withdraw_notification (_data_->_tmp27_, _data_->_tmp28_);
				_g_free0 (_data_->id);
			}
			_g_object_unref0 (_data_->_id_it);
		}
		_data_->_tmp29_ = _data_->self->priv->active_conversation_ids;
		gee_collection_clear ((GeeCollection*) _data_->_tmp29_);
	}
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_real_retract_conversation_notifications_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierRetractConversationNotificationsData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->conversation);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierRetractConversationNotificationsData, _data_);
}

static void
dino_ui_gnotifications_notifier_real_retract_conversation_notifications (DinoNotificationProvider* base,
                                                                         DinoEntitiesConversation* conversation,
                                                                         GAsyncReadyCallback _callback_,
                                                                         gpointer _user_data_)
{
	DinoUiGNotificationsNotifier * self;
	DinoUiGnotificationsNotifierRetractConversationNotificationsData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesConversation* _tmp1_;
	g_return_if_fail (conversation != NULL);
	self = (DinoUiGNotificationsNotifier*) base;
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierRetractConversationNotificationsData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_real_retract_conversation_notifications_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp1_;
	dino_ui_gnotifications_notifier_real_retract_conversation_notifications_co (_data_);
}

static void
dino_ui_gnotifications_notifier_retract_conversation_notifications_finish (DinoNotificationProvider* base,
                                                                           GAsyncResult* _res_)
{
	DinoUiGnotificationsNotifierRetractConversationNotificationsData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static gboolean
dino_ui_gnotifications_notifier_real_retract_conversation_notifications_co (DinoUiGnotificationsNotifierRetractConversationNotificationsData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = dino_entities_conversation_get_id (_data_->conversation);
	_data_->_tmp1_ = _data_->_tmp0_;
	_data_->_tmp2_ = g_strdup_printf ("%i", _data_->_tmp1_);
	_data_->_tmp3_ = _data_->_tmp2_;
	_data_->_tmp4_ = g_strconcat (_data_->_tmp3_, "-subscription", NULL);
	_data_->_tmp5_ = _data_->_tmp4_;
	_g_free0 (_data_->_tmp3_);
	_data_->subscription_id = _data_->_tmp5_;
	_data_->_tmp6_ = _data_->self->priv->active_ids;
	_data_->_tmp7_ = _data_->subscription_id;
	if (gee_collection_contains ((GeeCollection*) _data_->_tmp6_, _data_->_tmp7_)) {
		_data_->_tmp8_ = g_application_get_default ();
		_data_->_tmp9_ = _data_->subscription_id;
		g_application_withdraw_notification (_data_->_tmp8_, _data_->_tmp9_);
	}
	_g_free0 (_data_->subscription_id);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_get_conversation_icon_data_free (gpointer _data)
{
	DinoUiGnotificationsNotifierGetConversationIconData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->conversation);
	_g_object_unref0 (_data_->result);
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiGnotificationsNotifierGetConversationIconData, _data_);
}

static void
dino_ui_gnotifications_notifier_get_conversation_icon (DinoUiGNotificationsNotifier* self,
                                                       DinoEntitiesConversation* conversation,
                                                       GAsyncReadyCallback _callback_,
                                                       gpointer _user_data_)
{
	DinoUiGnotificationsNotifierGetConversationIconData* _data_;
	DinoUiGNotificationsNotifier* _tmp0_;
	DinoEntitiesConversation* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (conversation != NULL);
	_data_ = g_slice_new0 (DinoUiGnotificationsNotifierGetConversationIconData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_gnotifications_notifier_get_conversation_icon_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (conversation);
	_g_object_unref0 (_data_->conversation);
	_data_->conversation = _tmp1_;
	dino_ui_gnotifications_notifier_get_conversation_icon_co (_data_);
}

static GIcon*
dino_ui_gnotifications_notifier_get_conversation_icon_finish (DinoUiGNotificationsNotifier* self,
                                                              GAsyncResult* _res_,
                                                              GError** error)
{
	GIcon* result;
	DinoUiGnotificationsNotifierGetConversationIconData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
	if (NULL == _data_) {
		return NULL;
	}
	result = _data_->result;
	_data_->result = NULL;
	return result;
}

static gboolean
dino_ui_gnotifications_notifier_get_conversation_icon_co (DinoUiGnotificationsNotifierGetConversationIconData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->stream_interactor;
	_data_->_tmp1_ = dino_ui_view_model_compat_avatar_picture_model_new (_data_->_tmp0_);
	_data_->_tmp2_ = _data_->_tmp1_;
	_data_->_tmp3_ = dino_ui_view_model_compat_avatar_picture_model_set_conversation (_data_->_tmp2_, _data_->conversation);
	_data_->_tmp4_ = NULL;
	_data_->_tmp4_ = dino_ui_compat_avatar_drawer_new ();
	dino_ui_compat_avatar_drawer_set_model (_data_->_tmp4_, (DinoUiViewModelAvatarPictureModel*) _data_->_tmp3_);
	_g_object_unref0 (_data_->_tmp3_);
	dino_ui_compat_avatar_drawer_set_width_request (_data_->_tmp4_, 40);
	dino_ui_compat_avatar_drawer_set_height_request (_data_->_tmp4_, 40);
	_data_->_tmp5_ = _data_->_tmp4_;
	_g_object_unref0 (_data_->_tmp2_);
	_data_->drawer = _data_->_tmp5_;
	_data_->_tmp6_ = dino_ui_compat_avatar_drawer_draw_image_surface (_data_->drawer);
	_data_->surface = _data_->_tmp6_;
	_data_->_tmp7_ = gdk_pixbuf_get_from_surface (_data_->surface, 0, 0, cairo_image_surface_get_width (_data_->surface), cairo_image_surface_get_height (_data_->surface));
	_data_->avatar = _data_->_tmp7_;
	_data_->buffer_length1 = 0;
	_data_->_buffer_size_ = 0;
	_data_->_tmp8_ = NULL;
	_data_->_tmp9_ = 0;
	gdk_pixbuf_save_to_buffer (_data_->avatar, &_data_->_tmp8_, &_data_->_tmp9_, "png", &_data_->_inner_error0_, NULL);
	_data_->buffer = (g_free (_data_->buffer), NULL);
	_data_->buffer = _data_->_tmp8_;
	_data_->buffer_length1 = _data_->_tmp9_;
	_data_->_buffer_size_ = _data_->buffer_length1;
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		g_task_return_error (_data_->_async_result, _data_->_inner_error0_);
		_data_->buffer = (g_free (_data_->buffer), NULL);
		_g_object_unref0 (_data_->avatar);
		_cairo_surface_destroy0 (_data_->surface);
		_dino_ui_compat_avatar_drawer_unref0 (_data_->drawer);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp10_ = g_bytes_new (_data_->buffer, _data_->buffer_length1);
	_data_->_tmp11_ = _data_->_tmp10_;
	_data_->_tmp12_ = (GBytesIcon*) g_bytes_icon_new (_data_->_tmp11_);
	_data_->_tmp13_ = (GIcon*) _data_->_tmp12_;
	_g_bytes_unref0 (_data_->_tmp11_);
	_data_->result = _data_->_tmp13_;
	_data_->buffer = (g_free (_data_->buffer), NULL);
	_g_object_unref0 (_data_->avatar);
	_cairo_surface_destroy0 (_data_->surface);
	_dino_ui_compat_avatar_drawer_unref0 (_data_->drawer);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_gnotifications_notifier_class_init (DinoUiGNotificationsNotifierClass * klass,
                                            gpointer klass_data)
{
	dino_ui_gnotifications_notifier_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiGNotificationsNotifier_private_offset);
	G_OBJECT_CLASS (klass)->finalize = dino_ui_gnotifications_notifier_finalize;
	dino_ui_gnotifications_notifier_signals[DINO_UI_GNOTIFICATIONS_NOTIFIER_CONVERSATION_SELECTED_SIGNAL] = g_signal_new ("conversation-selected", DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, DINO_ENTITIES_TYPE_CONVERSATION);
}

static void
dino_ui_gnotifications_notifier_dino_notification_provider_interface_init (DinoNotificationProviderIface * iface,
                                                                           gpointer iface_data)
{
	dino_ui_gnotifications_notifier_dino_notification_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_priority = (gdouble (*) (DinoNotificationProvider*)) dino_ui_gnotifications_notifier_real_get_priority;
	iface->notify_message = (void (*) (DinoNotificationProvider*, DinoEntitiesMessage*, DinoEntitiesConversation*, const gchar*, const gchar*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_message;
	iface->notify_message_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_message_finish;
	iface->notify_file = (void (*) (DinoNotificationProvider*, DinoEntitiesFileTransfer*, DinoEntitiesConversation*, gboolean, const gchar*, const gchar*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_file;
	iface->notify_file_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_file_finish;
	iface->notify_call = (void (*) (DinoNotificationProvider*, DinoEntitiesCall*, DinoEntitiesConversation*, gboolean, gboolean, const gchar*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_call;
	iface->notify_call_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_call_finish;
	iface->retract_call_notification = (void (*) (DinoNotificationProvider*, DinoEntitiesCall*, DinoEntitiesConversation*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_retract_call_notification;
	iface->retract_call_notification_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_retract_call_notification_finish;
	iface->notify_subscription_request = (void (*) (DinoNotificationProvider*, DinoEntitiesConversation*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_subscription_request;
	iface->notify_subscription_request_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_subscription_request_finish;
	iface->notify_connection_error = (void (*) (DinoNotificationProvider*, DinoEntitiesAccount*, DinoConnectionManagerConnectionError*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_connection_error;
	iface->notify_connection_error_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_connection_error_finish;
	iface->notify_muc_invite = (void (*) (DinoNotificationProvider*, DinoEntitiesAccount*, XmppJid*, XmppJid*, const gchar*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_muc_invite;
	iface->notify_muc_invite_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_muc_invite_finish;
	iface->notify_voice_request = (void (*) (DinoNotificationProvider*, DinoEntitiesConversation*, XmppJid*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_notify_voice_request;
	iface->notify_voice_request_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_notify_voice_request_finish;
	iface->retract_content_item_notifications = (void (*) (DinoNotificationProvider*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_retract_content_item_notifications;
	iface->retract_content_item_notifications_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_retract_content_item_notifications_finish;
	iface->retract_conversation_notifications = (void (*) (DinoNotificationProvider*, DinoEntitiesConversation*, GAsyncReadyCallback, gpointer)) dino_ui_gnotifications_notifier_real_retract_conversation_notifications;
	iface->retract_conversation_notifications_finish = (void (*) (DinoNotificationProvider*, GAsyncResult*)) dino_ui_gnotifications_notifier_retract_conversation_notifications_finish;
}

static void
dino_ui_gnotifications_notifier_instance_init (DinoUiGNotificationsNotifier * self,
                                               gpointer klass)
{
	GeeHashMap* _tmp0_;
	GeeHashSet* _tmp1_;
	self->priv = dino_ui_gnotifications_notifier_get_instance_private (self);
	_tmp0_ = gee_hash_map_new (DINO_ENTITIES_TYPE_CONVERSATION, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, g_notification_get_type (), (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _dino_entities_conversation_hash_func_gee_hash_data_func, NULL, NULL, _dino_entities_conversation_equals_func_gee_equal_data_func, NULL, NULL, NULL, NULL, NULL);
	self->priv->notifications = _tmp0_;
	self->priv->active_conversation_ids = NULL;
	_tmp1_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL, NULL, NULL, NULL);
	self->priv->active_ids = (GeeSet*) _tmp1_;
}

static void
dino_ui_gnotifications_notifier_finalize (GObject * obj)
{
	DinoUiGNotificationsNotifier * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_GNOTIFICATIONS_NOTIFIER, DinoUiGNotificationsNotifier);
	_g_object_unref0 (self->priv->stream_interactor);
	_g_object_unref0 (self->priv->notifications);
	_g_object_unref0 (self->priv->active_conversation_ids);
	_g_object_unref0 (self->priv->active_ids);
	G_OBJECT_CLASS (dino_ui_gnotifications_notifier_parent_class)->finalize (obj);
}

static GType
dino_ui_gnotifications_notifier_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiGNotificationsNotifierClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_gnotifications_notifier_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiGNotificationsNotifier), 0, (GInstanceInitFunc) dino_ui_gnotifications_notifier_instance_init, NULL };
	static const GInterfaceInfo dino_notification_provider_info = { (GInterfaceInitFunc) dino_ui_gnotifications_notifier_dino_notification_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType dino_ui_gnotifications_notifier_type_id;
	dino_ui_gnotifications_notifier_type_id = g_type_register_static (G_TYPE_OBJECT, "DinoUiGNotificationsNotifier", &g_define_type_info, 0);
	g_type_add_interface_static (dino_ui_gnotifications_notifier_type_id, DINO_TYPE_NOTIFICATION_PROVIDER, &dino_notification_provider_info);
	DinoUiGNotificationsNotifier_private_offset = g_type_add_instance_private (dino_ui_gnotifications_notifier_type_id, sizeof (DinoUiGNotificationsNotifierPrivate));
	return dino_ui_gnotifications_notifier_type_id;
}

GType
dino_ui_gnotifications_notifier_get_type (void)
{
	static volatile gsize dino_ui_gnotifications_notifier_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_gnotifications_notifier_type_id__once)) {
		GType dino_ui_gnotifications_notifier_type_id;
		dino_ui_gnotifications_notifier_type_id = dino_ui_gnotifications_notifier_get_type_once ();
		g_once_init_leave (&dino_ui_gnotifications_notifier_type_id__once, dino_ui_gnotifications_notifier_type_id);
	}
	return dino_ui_gnotifications_notifier_type_id__once;
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

