/* conversation_titlebar.c generated by valac 0.56.19, the Vala compiler
 * generated from conversation_titlebar.vala, do not modify */

#include <glib-object.h>
#include <adwaita.h>
#include <gtk/gtk.h>
#include <pango/pango.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_TYPE_CONVERSATION_TITLEBAR (dino_ui_conversation_titlebar_get_type ())
#define DINO_UI_CONVERSATION_TITLEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebar))
#define DINO_UI_CONVERSATION_TITLEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebarClass))
#define DINO_UI_IS_CONVERSATION_TITLEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_CONVERSATION_TITLEBAR))
#define DINO_UI_IS_CONVERSATION_TITLEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_CONVERSATION_TITLEBAR))
#define DINO_UI_CONVERSATION_TITLEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebarClass))

typedef struct _DinoUiConversationTitlebar DinoUiConversationTitlebar;
typedef struct _DinoUiConversationTitlebarClass DinoUiConversationTitlebarClass;
typedef struct _DinoUiConversationTitlebarPrivate DinoUiConversationTitlebarPrivate;
enum  {
	DINO_UI_CONVERSATION_TITLEBAR_0_PROPERTY,
	DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY,
	DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY,
	DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY,
	DINO_UI_CONVERSATION_TITLEBAR_NUM_PROPERTIES
};
static GParamSpec* dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _pango_attr_list_unref0(var) ((var == NULL) ? NULL : (var = (pango_attr_list_unref (var), NULL)))
enum  {
	DINO_UI_CONVERSATION_TITLEBAR_BACK_PRESSED_SIGNAL,
	DINO_UI_CONVERSATION_TITLEBAR_NUM_SIGNALS
};
static guint dino_ui_conversation_titlebar_signals[DINO_UI_CONVERSATION_TITLEBAR_NUM_SIGNALS] = {0};

struct _DinoUiConversationTitlebar {
	GObject parent_instance;
	DinoUiConversationTitlebarPrivate * priv;
	AdwHeaderBar* header_bar;
};

struct _DinoUiConversationTitlebarClass {
	GObjectClass parent_class;
};

struct _DinoUiConversationTitlebarPrivate {
	GtkLabel* title_label;
	GtkLabel* subtitle_label;
	GtkRevealer* back_revealer;
};

static gint DinoUiConversationTitlebar_private_offset;
static gpointer dino_ui_conversation_titlebar_parent_class = NULL;

VALA_EXTERN GType dino_ui_conversation_titlebar_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiConversationTitlebar, g_object_unref)
VALA_EXTERN DinoUiConversationTitlebar* dino_ui_conversation_titlebar_new (void);
VALA_EXTERN DinoUiConversationTitlebar* dino_ui_conversation_titlebar_construct (GType object_type);
static void __lambda151_ (DinoUiConversationTitlebar* self);
static void ___lambda151__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
VALA_EXTERN void dino_ui_conversation_titlebar_insert_button (DinoUiConversationTitlebar* self,
                                                  GtkWidget* button);
VALA_EXTERN GtkWidget* dino_ui_conversation_titlebar_get_widget (DinoUiConversationTitlebar* self);
VALA_EXTERN const gchar* dino_ui_conversation_titlebar_get_title (DinoUiConversationTitlebar* self);
VALA_EXTERN void dino_ui_conversation_titlebar_set_title (DinoUiConversationTitlebar* self,
                                              const gchar* value);
VALA_EXTERN const gchar* dino_ui_conversation_titlebar_get_subtitle (DinoUiConversationTitlebar* self);
VALA_EXTERN void dino_ui_conversation_titlebar_set_subtitle (DinoUiConversationTitlebar* self,
                                                 const gchar* value);
VALA_EXTERN gboolean dino_ui_conversation_titlebar_get_back_button_visible (DinoUiConversationTitlebar* self);
VALA_EXTERN void dino_ui_conversation_titlebar_set_back_button_visible (DinoUiConversationTitlebar* self,
                                                            gboolean value);
static void dino_ui_conversation_titlebar_finalize (GObject * obj);
static GType dino_ui_conversation_titlebar_get_type_once (void);
static void _vala_dino_ui_conversation_titlebar_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_dino_ui_conversation_titlebar_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
dino_ui_conversation_titlebar_get_instance_private (DinoUiConversationTitlebar* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiConversationTitlebar_private_offset);
}

static void
__lambda151_ (DinoUiConversationTitlebar* self)
{
	g_signal_emit (self, dino_ui_conversation_titlebar_signals[DINO_UI_CONVERSATION_TITLEBAR_BACK_PRESSED_SIGNAL], 0);
}

static void
___lambda151__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda151_ ((DinoUiConversationTitlebar*) self);
}

DinoUiConversationTitlebar*
dino_ui_conversation_titlebar_construct (GType object_type)
{
	DinoUiConversationTitlebar * self = NULL;
	GtkBox* titles_box = NULL;
	GtkBox* _tmp0_ = NULL;
	GtkLabel* _tmp1_;
	PangoAttrList* _tmp2_;
	PangoAttrList* _tmp3_;
	GtkLabel* _tmp4_;
	PangoAttrList* _tmp5_;
	PangoAttrList* _tmp6_;
	PangoAttribute* _tmp7_;
	GtkLabel* _tmp8_;
	GtkLabel* _tmp9_;
	PangoAttrList* _tmp10_;
	PangoAttrList* _tmp11_;
	GtkLabel* _tmp12_;
	PangoAttrList* _tmp13_;
	PangoAttrList* _tmp14_;
	PangoAttribute* _tmp15_;
	GtkLabel* _tmp16_;
	GtkLabel* _tmp17_;
	GtkRevealer* _tmp18_ = NULL;
	GtkButton* back_button = NULL;
	GtkButton* _tmp19_ = NULL;
	GtkStyleContext* _tmp20_;
	GtkRevealer* _tmp21_;
	AdwHeaderBar* _tmp22_;
	GtkRevealer* _tmp23_;
	AdwHeaderBar* _tmp24_;
	self = (DinoUiConversationTitlebar*) g_object_new (object_type, NULL);
	_tmp0_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	gtk_widget_set_valign ((GtkWidget*) _tmp0_, GTK_ALIGN_CENTER);
	g_object_ref_sink (_tmp0_);
	titles_box = _tmp0_;
	_tmp1_ = self->priv->title_label;
	_tmp2_ = pango_attr_list_new ();
	_tmp3_ = _tmp2_;
	gtk_label_set_attributes (_tmp1_, _tmp3_);
	_pango_attr_list_unref0 (_tmp3_);
	_tmp4_ = self->priv->title_label;
	_tmp5_ = gtk_label_get_attributes (_tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
	pango_attr_list_insert (_tmp6_, _tmp7_);
	_tmp8_ = self->priv->title_label;
	gtk_box_append (titles_box, (GtkWidget*) _tmp8_);
	_tmp9_ = self->priv->subtitle_label;
	_tmp10_ = pango_attr_list_new ();
	_tmp11_ = _tmp10_;
	gtk_label_set_attributes (_tmp9_, _tmp11_);
	_pango_attr_list_unref0 (_tmp11_);
	_tmp12_ = self->priv->subtitle_label;
	_tmp13_ = gtk_label_get_attributes (_tmp12_);
	_tmp14_ = _tmp13_;
	_tmp15_ = pango_attr_scale_new (PANGO_SCALE_SMALL);
	pango_attr_list_insert (_tmp14_, _tmp15_);
	_tmp16_ = self->priv->subtitle_label;
	gtk_widget_add_css_class ((GtkWidget*) _tmp16_, "dim-label");
	_tmp17_ = self->priv->subtitle_label;
	gtk_box_append (titles_box, (GtkWidget*) _tmp17_);
	_tmp18_ = (GtkRevealer*) gtk_revealer_new ();
	gtk_widget_set_visible ((GtkWidget*) _tmp18_, TRUE);
	gtk_revealer_set_transition_type (_tmp18_, GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT);
	gtk_revealer_set_transition_duration (_tmp18_, (guint) 200);
	gtk_widget_set_can_focus ((GtkWidget*) _tmp18_, FALSE);
	gtk_revealer_set_reveal_child (_tmp18_, FALSE);
	g_object_ref_sink (_tmp18_);
	_g_object_unref0 (self->priv->back_revealer);
	self->priv->back_revealer = _tmp18_;
	_tmp19_ = (GtkButton*) gtk_button_new_from_icon_name ("go-previous-symbolic");
	gtk_widget_set_visible ((GtkWidget*) _tmp19_, TRUE);
	gtk_widget_set_valign ((GtkWidget*) _tmp19_, GTK_ALIGN_CENTER);
	gtk_button_set_use_underline (_tmp19_, TRUE);
	g_object_ref_sink (_tmp19_);
	back_button = _tmp19_;
	_tmp20_ = gtk_widget_get_style_context ((GtkWidget*) back_button);
	gtk_style_context_add_class (_tmp20_, "image-button");
	g_signal_connect_object (back_button, "clicked", (GCallback) ___lambda151__gtk_button_clicked, self, 0);
	_tmp21_ = self->priv->back_revealer;
	gtk_revealer_set_child (_tmp21_, (GtkWidget*) back_button);
	_tmp22_ = self->header_bar;
	_tmp23_ = self->priv->back_revealer;
	adw_header_bar_pack_start (_tmp22_, (GtkWidget*) _tmp23_);
	_tmp24_ = self->header_bar;
	adw_header_bar_set_title_widget (_tmp24_, (GtkWidget*) titles_box);
	_g_object_unref0 (back_button);
	_g_object_unref0 (titles_box);
	return self;
}

DinoUiConversationTitlebar*
dino_ui_conversation_titlebar_new (void)
{
	return dino_ui_conversation_titlebar_construct (DINO_UI_TYPE_CONVERSATION_TITLEBAR);
}

void
dino_ui_conversation_titlebar_insert_button (DinoUiConversationTitlebar* self,
                                             GtkWidget* button)
{
	AdwHeaderBar* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);
	_tmp0_ = self->header_bar;
	adw_header_bar_pack_end (_tmp0_, button);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

GtkWidget*
dino_ui_conversation_titlebar_get_widget (DinoUiConversationTitlebar* self)
{
	AdwHeaderBar* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->header_bar;
	_tmp1_ = _g_object_ref0 ((GtkWidget*) _tmp0_);
	result = _tmp1_;
	return result;
}

const gchar*
dino_ui_conversation_titlebar_get_title (DinoUiConversationTitlebar* self)
{
	const gchar* result;
	GtkLabel* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->title_label;
	_tmp1_ = gtk_label_get_label (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_conversation_titlebar_set_title (DinoUiConversationTitlebar* self,
                                         const gchar* value)
{
	GtkLabel* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->title_label;
	gtk_label_set_label (_tmp0_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY]);
}

const gchar*
dino_ui_conversation_titlebar_get_subtitle (DinoUiConversationTitlebar* self)
{
	const gchar* result;
	GtkLabel* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->subtitle_label;
	_tmp1_ = gtk_label_get_label (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_conversation_titlebar_set_subtitle (DinoUiConversationTitlebar* self,
                                            const gchar* value)
{
	GtkLabel* _tmp0_;
	GtkLabel* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->subtitle_label;
	gtk_label_set_label (_tmp0_, value);
	_tmp1_ = self->priv->subtitle_label;
	gtk_widget_set_visible ((GtkWidget*) _tmp1_, value != NULL);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY]);
}

gboolean
dino_ui_conversation_titlebar_get_back_button_visible (DinoUiConversationTitlebar* self)
{
	gboolean result;
	GtkRevealer* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->back_revealer;
	_tmp1_ = gtk_revealer_get_reveal_child (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_conversation_titlebar_set_back_button_visible (DinoUiConversationTitlebar* self,
                                                       gboolean value)
{
	GtkRevealer* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->back_revealer;
	gtk_revealer_set_reveal_child (_tmp0_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY]);
}

static void
dino_ui_conversation_titlebar_class_init (DinoUiConversationTitlebarClass * klass,
                                          gpointer klass_data)
{
	dino_ui_conversation_titlebar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiConversationTitlebar_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_conversation_titlebar_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_conversation_titlebar_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_conversation_titlebar_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY] = g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY] = g_param_spec_string ("subtitle", "subtitle", "subtitle", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY, dino_ui_conversation_titlebar_properties[DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY] = g_param_spec_boolean ("back-button-visible", "back-button-visible", "back-button-visible", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	dino_ui_conversation_titlebar_signals[DINO_UI_CONVERSATION_TITLEBAR_BACK_PRESSED_SIGNAL] = g_signal_new ("back-pressed", DINO_UI_TYPE_CONVERSATION_TITLEBAR, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
dino_ui_conversation_titlebar_instance_init (DinoUiConversationTitlebar * self,
                                             gpointer klass)
{
	AdwHeaderBar* _tmp0_;
	GtkLabel* _tmp1_ = NULL;
	GtkLabel* _tmp2_ = NULL;
	self->priv = dino_ui_conversation_titlebar_get_instance_private (self);
	_tmp0_ = (AdwHeaderBar*) adw_header_bar_new ();
	g_object_ref_sink (_tmp0_);
	self->header_bar = _tmp0_;
	_tmp1_ = (GtkLabel*) gtk_label_new ("");
	gtk_label_set_ellipsize (_tmp1_, PANGO_ELLIPSIZE_END);
	g_object_ref_sink (_tmp1_);
	self->priv->title_label = _tmp1_;
	_tmp2_ = (GtkLabel*) gtk_label_new ("");
	gtk_label_set_use_markup (_tmp2_, TRUE);
	gtk_label_set_ellipsize (_tmp2_, PANGO_ELLIPSIZE_END);
	gtk_widget_set_visible ((GtkWidget*) _tmp2_, FALSE);
	g_object_ref_sink (_tmp2_);
	self->priv->subtitle_label = _tmp2_;
}

static void
dino_ui_conversation_titlebar_finalize (GObject * obj)
{
	DinoUiConversationTitlebar * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebar);
	_g_object_unref0 (self->header_bar);
	_g_object_unref0 (self->priv->title_label);
	_g_object_unref0 (self->priv->subtitle_label);
	_g_object_unref0 (self->priv->back_revealer);
	G_OBJECT_CLASS (dino_ui_conversation_titlebar_parent_class)->finalize (obj);
}

static GType
dino_ui_conversation_titlebar_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiConversationTitlebarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_conversation_titlebar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiConversationTitlebar), 0, (GInstanceInitFunc) dino_ui_conversation_titlebar_instance_init, NULL };
	GType dino_ui_conversation_titlebar_type_id;
	dino_ui_conversation_titlebar_type_id = g_type_register_static (G_TYPE_OBJECT, "DinoUiConversationTitlebar", &g_define_type_info, 0);
	DinoUiConversationTitlebar_private_offset = g_type_add_instance_private (dino_ui_conversation_titlebar_type_id, sizeof (DinoUiConversationTitlebarPrivate));
	return dino_ui_conversation_titlebar_type_id;
}

GType
dino_ui_conversation_titlebar_get_type (void)
{
	static volatile gsize dino_ui_conversation_titlebar_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_conversation_titlebar_type_id__once)) {
		GType dino_ui_conversation_titlebar_type_id;
		dino_ui_conversation_titlebar_type_id = dino_ui_conversation_titlebar_get_type_once ();
		g_once_init_leave (&dino_ui_conversation_titlebar_type_id__once, dino_ui_conversation_titlebar_type_id);
	}
	return dino_ui_conversation_titlebar_type_id__once;
}

static void
_vala_dino_ui_conversation_titlebar_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	DinoUiConversationTitlebar * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebar);
	switch (property_id) {
		case DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY:
		g_value_set_string (value, dino_ui_conversation_titlebar_get_title (self));
		break;
		case DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY:
		g_value_set_string (value, dino_ui_conversation_titlebar_get_subtitle (self));
		break;
		case DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY:
		g_value_set_boolean (value, dino_ui_conversation_titlebar_get_back_button_visible (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_conversation_titlebar_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	DinoUiConversationTitlebar * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_CONVERSATION_TITLEBAR, DinoUiConversationTitlebar);
	switch (property_id) {
		case DINO_UI_CONVERSATION_TITLEBAR_TITLE_PROPERTY:
		dino_ui_conversation_titlebar_set_title (self, g_value_get_string (value));
		break;
		case DINO_UI_CONVERSATION_TITLEBAR_SUBTITLE_PROPERTY:
		dino_ui_conversation_titlebar_set_subtitle (self, g_value_get_string (value));
		break;
		case DINO_UI_CONVERSATION_TITLEBAR_BACK_BUTTON_VISIBLE_PROPERTY:
		dino_ui_conversation_titlebar_set_back_button_visible (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

