/* 0177_jingle_raw_udp.c generated by valac 0.56.19, the Vala compiler
 * generated from 0177_jingle_raw_udp.vala, do not modify */

#include "xmpp-vala.h"
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <gee.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	XMPP_XEP_JINGLE_RAW_UDP_MODULE_0_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_MODULE_NS_URI_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_MODULE_TYPE__PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_MODULE_PRIORITY_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_MODULE_NUM_PROPERTIES
};
static GParamSpec* xmpp_xep_jingle_raw_udp_module_properties[XMPP_XEP_JINGLE_RAW_UDP_MODULE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _xmpp_jid_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_jid_unref (var), NULL)))
typedef struct _XmppXepJingleRawUdpModuleIsTransportAvailableData XmppXepJingleRawUdpModuleIsTransportAvailableData;
enum  {
	XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_0_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NS_URI_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_COMPONENTS_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NUM_PROPERTIES
};
static GParamSpec* xmpp_xep_jingle_raw_udp_transport_parameters_properties[XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _xmpp_stanza_entry_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_stanza_entry_unref (var), NULL)))
enum  {
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_0_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY,
	XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_NUM_PROPERTIES
};
static GParamSpec* xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_NUM_PROPERTIES];

struct _XmppXepJingleRawUdpModulePrivate {
	XmppXepJingleRawUdpGetLocalIpAddresses get_local_ip_addresses_impl;
	gpointer get_local_ip_addresses_impl_target;
	GDestroyNotify get_local_ip_addresses_impl_target_destroy_notify;
};

struct _XmppXepJingleRawUdpModuleIsTransportAvailableData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	XmppXepJingleRawUdpModule* self;
	XmppXmppStream* stream;
	guint8 components;
	XmppJid* full_jid;
	gboolean result;
	XmppModuleIdentity* _tmp0_;
	gpointer _tmp1_;
	XmppXepServiceDiscoveryModule* _tmp2_;
	gboolean _tmp3_;
};

struct _XmppXepJingleRawUdpTransportParametersPrivate {
	guint8 _components;
};

struct _XmppXepJingleRawUdpCandidatePrivate {
	gint _component;
	gint _generation;
	gchar* _id;
	GInetAddress* _ip;
	guint _port;
};

static gint XmppXepJingleRawUdpModule_private_offset;
static gpointer xmpp_xep_jingle_raw_udp_module_parent_class = NULL;
XmppModuleIdentity* xmpp_xep_jingle_raw_udp_module_IDENTITY = NULL;
static XmppXepJingleTransportIface * xmpp_xep_jingle_raw_udp_module_xmpp_xep_jingle_transport_parent_iface = NULL;
static gint XmppXepJingleRawUdpTransportParameters_private_offset;
static gpointer xmpp_xep_jingle_raw_udp_transport_parameters_parent_class = NULL;
static XmppXepJingleTransportParametersIface * xmpp_xep_jingle_raw_udp_transport_parameters_xmpp_xep_jingle_transport_parameters_parent_iface = NULL;
static gint XmppXepJingleRawUdpCandidate_private_offset;
static gpointer xmpp_xep_jingle_raw_udp_candidate_parent_class = NULL;

static void xmpp_xep_jingle_raw_udp_module_real_attach (XmppXmppStreamModule* base,
                                                 XmppXmppStream* stream);
static void xmpp_xep_jingle_raw_udp_module_real_detach (XmppXmppStreamModule* base,
                                                 XmppXmppStream* stream);
static gchar* xmpp_xep_jingle_raw_udp_module_real_get_ns (XmppXmppStreamModule* base);
static gchar* xmpp_xep_jingle_raw_udp_module_real_get_id (XmppXmppStreamModule* base);
static void xmpp_xep_jingle_raw_udp_module_real_is_transport_available_data_free (gpointer _data);
static void xmpp_xep_jingle_raw_udp_module_real_is_transport_available (XmppXepJingleTransport* base,
                                                                 XmppXmppStream* stream,
                                                                 guint8 components,
                                                                 XmppJid* full_jid,
                                                                 GAsyncReadyCallback _callback_,
                                                                 gpointer _user_data_);
static gboolean xmpp_xep_jingle_raw_udp_module_real_is_transport_available_co (XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_);
static void xmpp_xep_jingle_raw_udp_module_is_transport_available_ready (GObject* source_object,
                                                                  GAsyncResult* _res_,
                                                                  gpointer _user_data_);
static XmppXepJingleTransportParameters* xmpp_xep_jingle_raw_udp_module_real_create_transport_parameters (XmppXepJingleTransport* base,
                                                                                                   XmppXmppStream* stream,
                                                                                                   guint8 components,
                                                                                                   XmppJid* local_full_jid,
                                                                                                   XmppJid* peer_full_jid,
                                                                                                   GError** error);
static XmppXepJingleTransportParameters* xmpp_xep_jingle_raw_udp_module_real_parse_transport_parameters (XmppXepJingleTransport* base,
                                                                                                  XmppXmppStream* stream,
                                                                                                  guint8 components,
                                                                                                  XmppJid* local_full_jid,
                                                                                                  XmppJid* peer_full_jid,
                                                                                                  XmppStanzaNode* transport,
                                                                                                  GError** error);
static void xmpp_xep_jingle_raw_udp_module_finalize (GObject * obj);
static GType xmpp_xep_jingle_raw_udp_module_get_type_once (void);
static void _vala_xmpp_xep_jingle_raw_udp_module_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void xmpp_xep_jingle_raw_udp_transport_parameters_real_set_content (XmppXepJingleTransportParameters* base,
                                                                    XmppXepJingleContent* content);
static XmppStanzaNode* xmpp_xep_jingle_raw_udp_transport_parameters_real_to_transport_stanza_node (XmppXepJingleTransportParameters* base,
                                                                                            const gchar* action_type);
static void xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_accept (XmppXepJingleTransportParameters* base,
                                                                                XmppStanzaNode* transport,
                                                                                GError** error);
static void xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_info (XmppXepJingleTransportParameters* base,
                                                                              XmppStanzaNode* transport,
                                                                              GError** error);
static void xmpp_xep_jingle_raw_udp_transport_parameters_real_create_transport_connection (XmppXepJingleTransportParameters* base,
                                                                                    XmppXmppStream* stream,
                                                                                    XmppXepJingleContent* content);
static void xmpp_xep_jingle_raw_udp_transport_parameters_finalize (GObject * obj);
static GType xmpp_xep_jingle_raw_udp_transport_parameters_get_type_once (void);
static void _vala_xmpp_xep_jingle_raw_udp_transport_parameters_get_property (GObject * object,
                                                                      guint property_id,
                                                                      GValue * value,
                                                                      GParamSpec * pspec);
static void xmpp_xep_jingle_raw_udp_candidate_finalize (GObject * obj);
static GType xmpp_xep_jingle_raw_udp_candidate_get_type_once (void);
static void _vala_xmpp_xep_jingle_raw_udp_candidate_get_property (GObject * object,
                                                           guint property_id,
                                                           GValue * value,
                                                           GParamSpec * pspec);
static void _vala_xmpp_xep_jingle_raw_udp_candidate_set_property (GObject * object,
                                                           guint property_id,
                                                           const GValue * value,
                                                           GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
xmpp_xep_jingle_raw_udp_module_get_instance_private (XmppXepJingleRawUdpModule* self)
{
	return G_STRUCT_MEMBER_P (self, XmppXepJingleRawUdpModule_private_offset);
}

static void
xmpp_xep_jingle_raw_udp_module_real_attach (XmppXmppStreamModule* base,
                                            XmppXmppStream* stream)
{
	XmppXepJingleRawUdpModule * self;
	XmppModuleIdentity* _tmp0_;
	gpointer _tmp1_;
	XmppXepJingleModule* _tmp2_;
	XmppModuleIdentity* _tmp3_;
	gpointer _tmp4_;
	XmppXepServiceDiscoveryModule* _tmp5_;
	self = (XmppXepJingleRawUdpModule*) base;
	g_return_if_fail (stream != NULL);
	_tmp0_ = xmpp_xep_jingle_module_IDENTITY;
	_tmp1_ = xmpp_xmpp_stream_get_module (stream, XMPP_XEP_JINGLE_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = (XmppXepJingleModule*) _tmp1_;
	xmpp_xep_jingle_module_register_transport (_tmp2_, (XmppXepJingleTransport*) self);
	_g_object_unref0 (_tmp2_);
	_tmp3_ = xmpp_xep_service_discovery_module_IDENTITY;
	_tmp4_ = xmpp_xmpp_stream_get_module (stream, XMPP_XEP_SERVICE_DISCOVERY_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp3_);
	_tmp5_ = (XmppXepServiceDiscoveryModule*) _tmp4_;
	xmpp_xep_service_discovery_module_add_feature (_tmp5_, stream, XMPP_XEP_JINGLE_RAW_UDP_NS_URI);
	_g_object_unref0 (_tmp5_);
}

static void
xmpp_xep_jingle_raw_udp_module_real_detach (XmppXmppStreamModule* base,
                                            XmppXmppStream* stream)
{
	XmppXepJingleRawUdpModule * self;
	XmppModuleIdentity* _tmp0_;
	gpointer _tmp1_;
	XmppXepServiceDiscoveryModule* _tmp2_;
	self = (XmppXepJingleRawUdpModule*) base;
	g_return_if_fail (stream != NULL);
	_tmp0_ = xmpp_xep_service_discovery_module_IDENTITY;
	_tmp1_ = xmpp_xmpp_stream_get_module (stream, XMPP_XEP_SERVICE_DISCOVERY_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = (XmppXepServiceDiscoveryModule*) _tmp1_;
	xmpp_xep_service_discovery_module_remove_feature (_tmp2_, stream, XMPP_XEP_JINGLE_RAW_UDP_NS_URI);
	_g_object_unref0 (_tmp2_);
}

static gchar*
xmpp_xep_jingle_raw_udp_module_real_get_ns (XmppXmppStreamModule* base)
{
	XmppXepJingleRawUdpModule * self;
	gchar* _tmp0_;
	gchar* result;
	self = (XmppXepJingleRawUdpModule*) base;
	_tmp0_ = g_strdup (XMPP_XEP_JINGLE_RAW_UDP_NS_URI);
	result = _tmp0_;
	return result;
}

static gchar*
xmpp_xep_jingle_raw_udp_module_real_get_id (XmppXmppStreamModule* base)
{
	XmppXepJingleRawUdpModule * self;
	XmppModuleIdentity* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* result;
	self = (XmppXepJingleRawUdpModule*) base;
	_tmp0_ = xmpp_xep_jingle_raw_udp_module_IDENTITY;
	_tmp1_ = xmpp_module_identity_get_id (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup (_tmp2_);
	result = _tmp3_;
	return result;
}

static void
xmpp_xep_jingle_raw_udp_module_real_is_transport_available_data_free (gpointer _data)
{
	XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->stream);
	_xmpp_jid_unref0 (_data_->full_jid);
	_g_object_unref0 (_data_->self);
	g_slice_free (XmppXepJingleRawUdpModuleIsTransportAvailableData, _data_);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static gpointer
_xmpp_jid_ref0 (gpointer self)
{
	return self ? xmpp_jid_ref (self) : NULL;
}

static void
xmpp_xep_jingle_raw_udp_module_real_is_transport_available (XmppXepJingleTransport* base,
                                                            XmppXmppStream* stream,
                                                            guint8 components,
                                                            XmppJid* full_jid,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_)
{
	XmppXepJingleRawUdpModule * self;
	XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_;
	XmppXepJingleRawUdpModule* _tmp0_;
	XmppXmppStream* _tmp1_;
	XmppJid* _tmp2_;
	g_return_if_fail (stream != NULL);
	g_return_if_fail (full_jid != NULL);
	self = (XmppXepJingleRawUdpModule*) base;
	_data_ = g_slice_new0 (XmppXepJingleRawUdpModuleIsTransportAvailableData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, xmpp_xep_jingle_raw_udp_module_real_is_transport_available_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	_tmp1_ = _g_object_ref0 (stream);
	_g_object_unref0 (_data_->stream);
	_data_->stream = _tmp1_;
	_data_->components = components;
	_tmp2_ = _xmpp_jid_ref0 (full_jid);
	_xmpp_jid_unref0 (_data_->full_jid);
	_data_->full_jid = _tmp2_;
	xmpp_xep_jingle_raw_udp_module_real_is_transport_available_co (_data_);
}

static gboolean
xmpp_xep_jingle_raw_udp_module_is_transport_available_finish (XmppXepJingleTransport* base,
                                                              GAsyncResult* _res_)
{
	gboolean result;
	XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
	result = _data_->result;
	return result;
}

static void
xmpp_xep_jingle_raw_udp_module_is_transport_available_ready (GObject* source_object,
                                                             GAsyncResult* _res_,
                                                             gpointer _user_data_)
{
	XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	xmpp_xep_jingle_raw_udp_module_real_is_transport_available_co (_data_);
}

static gboolean
xmpp_xep_jingle_raw_udp_module_real_is_transport_available_co (XmppXepJingleRawUdpModuleIsTransportAvailableData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = xmpp_xep_service_discovery_module_IDENTITY;
	_data_->_tmp1_ = xmpp_xmpp_stream_get_module (_data_->stream, XMPP_XEP_SERVICE_DISCOVERY_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp0_);
	_data_->_tmp2_ = (XmppXepServiceDiscoveryModule*) _data_->_tmp1_;
	_data_->_state_ = 1;
	xmpp_xep_service_discovery_module_has_entity_feature (_data_->_tmp2_, _data_->stream, _data_->full_jid, XMPP_XEP_JINGLE_RAW_UDP_NS_URI, xmpp_xep_jingle_raw_udp_module_is_transport_available_ready, _data_);
	return FALSE;
	_state_1:
	_data_->_tmp3_ = xmpp_xep_service_discovery_module_has_entity_feature_finish (_data_->_tmp2_, _data_->_res_);
	_g_object_unref0 (_data_->_tmp2_);
	_data_->result = _data_->_tmp3_;
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

void
xmpp_xep_jingle_raw_udp_module_set_local_ip_address_handler (XmppXepJingleRawUdpModule* self,
                                                             XmppXepJingleRawUdpGetLocalIpAddresses get_local_ip_addresses,
                                                             gpointer get_local_ip_addresses_target,
                                                             GDestroyNotify get_local_ip_addresses_target_destroy_notify)
{
	XmppXepJingleRawUdpGetLocalIpAddresses _tmp0_;
	gpointer _tmp0__target;
	GDestroyNotify _tmp0__target_destroy_notify;
	g_return_if_fail (self != NULL);
	_tmp0_ = get_local_ip_addresses;
	_tmp0__target = get_local_ip_addresses_target;
	_tmp0__target_destroy_notify = get_local_ip_addresses_target_destroy_notify;
	get_local_ip_addresses = NULL;
	get_local_ip_addresses_target = NULL;
	get_local_ip_addresses_target_destroy_notify = NULL;
	(self->priv->get_local_ip_addresses_impl_target_destroy_notify == NULL) ? NULL : (self->priv->get_local_ip_addresses_impl_target_destroy_notify (self->priv->get_local_ip_addresses_impl_target), NULL);
	self->priv->get_local_ip_addresses_impl = NULL;
	self->priv->get_local_ip_addresses_impl_target = NULL;
	self->priv->get_local_ip_addresses_impl_target_destroy_notify = NULL;
	self->priv->get_local_ip_addresses_impl = _tmp0_;
	self->priv->get_local_ip_addresses_impl_target = _tmp0__target;
	self->priv->get_local_ip_addresses_impl_target_destroy_notify = _tmp0__target_destroy_notify;
	(get_local_ip_addresses_target_destroy_notify == NULL) ? NULL : (get_local_ip_addresses_target_destroy_notify (get_local_ip_addresses_target), NULL);
	get_local_ip_addresses = NULL;
	get_local_ip_addresses_target = NULL;
	get_local_ip_addresses_target_destroy_notify = NULL;
}

GeeList*
xmpp_xep_jingle_raw_udp_module_get_local_ip_addresses (XmppXepJingleRawUdpModule* self)
{
	XmppXepJingleRawUdpGetLocalIpAddresses _tmp0_;
	gpointer _tmp0__target;
	XmppXepJingleRawUdpGetLocalIpAddresses _tmp2_;
	gpointer _tmp2__target;
	GeeList* _tmp3_;
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->get_local_ip_addresses_impl;
	_tmp0__target = self->priv->get_local_ip_addresses_impl_target;
	if (_tmp0_ == NULL) {
		GeeList* _tmp1_;
		_tmp1_ = gee_list_empty (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free);
		result = _tmp1_;
		return result;
	}
	_tmp2_ = self->priv->get_local_ip_addresses_impl;
	_tmp2__target = self->priv->get_local_ip_addresses_impl_target;
	_tmp3_ = _tmp2_ (_tmp2__target);
	result = _tmp3_;
	return result;
}

static XmppXepJingleTransportParameters*
xmpp_xep_jingle_raw_udp_module_real_create_transport_parameters (XmppXepJingleTransport* base,
                                                                 XmppXmppStream* stream,
                                                                 guint8 components,
                                                                 XmppJid* local_full_jid,
                                                                 XmppJid* peer_full_jid,
                                                                 GError** error)
{
	XmppXepJingleRawUdpModule * self;
	XmppXepJingleRawUdpTransportParameters* _tmp0_;
	XmppXepJingleTransportParameters* result;
	self = (XmppXepJingleRawUdpModule*) base;
	g_return_val_if_fail (stream != NULL, NULL);
	g_return_val_if_fail (local_full_jid != NULL, NULL);
	g_return_val_if_fail (peer_full_jid != NULL, NULL);
	_tmp0_ = xmpp_xep_jingle_raw_udp_transport_parameters_new (components, NULL);
	result = (XmppXepJingleTransportParameters*) _tmp0_;
	return result;
}

static XmppXepJingleTransportParameters*
xmpp_xep_jingle_raw_udp_module_real_parse_transport_parameters (XmppXepJingleTransport* base,
                                                                XmppXmppStream* stream,
                                                                guint8 components,
                                                                XmppJid* local_full_jid,
                                                                XmppJid* peer_full_jid,
                                                                XmppStanzaNode* transport,
                                                                GError** error)
{
	XmppXepJingleRawUdpModule * self;
	XmppXepJingleRawUdpTransportParameters* _tmp0_;
	XmppXepJingleTransportParameters* result;
	self = (XmppXepJingleRawUdpModule*) base;
	g_return_val_if_fail (stream != NULL, NULL);
	g_return_val_if_fail (local_full_jid != NULL, NULL);
	g_return_val_if_fail (peer_full_jid != NULL, NULL);
	g_return_val_if_fail (transport != NULL, NULL);
	_tmp0_ = xmpp_xep_jingle_raw_udp_transport_parameters_new (components, transport);
	result = (XmppXepJingleTransportParameters*) _tmp0_;
	return result;
}

XmppXepJingleRawUdpModule*
xmpp_xep_jingle_raw_udp_module_construct (GType object_type)
{
	XmppXepJingleRawUdpModule * self = NULL;
	self = (XmppXepJingleRawUdpModule*) xmpp_xmpp_stream_module_construct (object_type);
	return self;
}

XmppXepJingleRawUdpModule*
xmpp_xep_jingle_raw_udp_module_new (void)
{
	return xmpp_xep_jingle_raw_udp_module_construct (XMPP_XEP_JINGLE_RAW_UDP_TYPE_MODULE);
}

static const gchar*
xmpp_xep_jingle_raw_udp_module_real_get_ns_uri (XmppXepJingleTransport* base)
{
	const gchar* result;
	XmppXepJingleRawUdpModule* self;
	self = (XmppXepJingleRawUdpModule*) base;
	result = XMPP_XEP_JINGLE_RAW_UDP_NS_URI;
	return result;
}

static XmppXepJingleTransportType
xmpp_xep_jingle_raw_udp_module_real_get_type_ (XmppXepJingleTransport* base)
{
	XmppXepJingleTransportType result;
	XmppXepJingleRawUdpModule* self;
	self = (XmppXepJingleRawUdpModule*) base;
	result = XMPP_XEP_JINGLE_TRANSPORT_TYPE_DATAGRAM;
	return result;
}

static gint
xmpp_xep_jingle_raw_udp_module_real_get_priority (XmppXepJingleTransport* base)
{
	gint result;
	XmppXepJingleRawUdpModule* self;
	self = (XmppXepJingleRawUdpModule*) base;
	result = 1;
	return result;
}

static void
xmpp_xep_jingle_raw_udp_module_class_init (XmppXepJingleRawUdpModuleClass * klass,
                                           gpointer klass_data)
{
	XmppModuleIdentity* _tmp0_;
	xmpp_xep_jingle_raw_udp_module_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &XmppXepJingleRawUdpModule_private_offset);
	((XmppXmppStreamModuleClass *) klass)->attach = (void (*) (XmppXmppStreamModule*, XmppXmppStream*)) xmpp_xep_jingle_raw_udp_module_real_attach;
	((XmppXmppStreamModuleClass *) klass)->detach = (void (*) (XmppXmppStreamModule*, XmppXmppStream*)) xmpp_xep_jingle_raw_udp_module_real_detach;
	((XmppXmppStreamModuleClass *) klass)->get_ns = (gchar* (*) (XmppXmppStreamModule*)) xmpp_xep_jingle_raw_udp_module_real_get_ns;
	((XmppXmppStreamModuleClass *) klass)->get_id = (gchar* (*) (XmppXmppStreamModule*)) xmpp_xep_jingle_raw_udp_module_real_get_id;
	G_OBJECT_CLASS (klass)->get_property = _vala_xmpp_xep_jingle_raw_udp_module_get_property;
	G_OBJECT_CLASS (klass)->finalize = xmpp_xep_jingle_raw_udp_module_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_MODULE_NS_URI_PROPERTY, xmpp_xep_jingle_raw_udp_module_properties[XMPP_XEP_JINGLE_RAW_UDP_MODULE_NS_URI_PROPERTY] = g_param_spec_string ("ns-uri", "ns-uri", "ns-uri", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_MODULE_TYPE__PROPERTY, xmpp_xep_jingle_raw_udp_module_properties[XMPP_XEP_JINGLE_RAW_UDP_MODULE_TYPE__PROPERTY] = g_param_spec_enum ("type-", "type-", "type-", XMPP_XEP_JINGLE_TYPE_TRANSPORT_TYPE, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_MODULE_PRIORITY_PROPERTY, xmpp_xep_jingle_raw_udp_module_properties[XMPP_XEP_JINGLE_RAW_UDP_MODULE_PRIORITY_PROPERTY] = g_param_spec_int ("priority", "priority", "priority", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	_tmp0_ = xmpp_module_identity_new (XMPP_XEP_JINGLE_RAW_UDP_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, XMPP_XEP_JINGLE_RAW_UDP_NS_URI, "0177_jingle_raw_udp");
	xmpp_xep_jingle_raw_udp_module_IDENTITY = _tmp0_;
}

static void
xmpp_xep_jingle_raw_udp_module_xmpp_xep_jingle_transport_interface_init (XmppXepJingleTransportIface * iface,
                                                                         gpointer iface_data)
{
	xmpp_xep_jingle_raw_udp_module_xmpp_xep_jingle_transport_parent_iface = g_type_interface_peek_parent (iface);
	iface->is_transport_available = (void (*) (XmppXepJingleTransport*, XmppXmppStream*, guint8, XmppJid*, GAsyncReadyCallback, gpointer)) xmpp_xep_jingle_raw_udp_module_real_is_transport_available;
	iface->is_transport_available_finish = (gboolean (*) (XmppXepJingleTransport*, GAsyncResult*)) xmpp_xep_jingle_raw_udp_module_is_transport_available_finish;
	iface->create_transport_parameters = (XmppXepJingleTransportParameters* (*) (XmppXepJingleTransport*, XmppXmppStream*, guint8, XmppJid*, XmppJid*, GError**)) xmpp_xep_jingle_raw_udp_module_real_create_transport_parameters;
	iface->parse_transport_parameters = (XmppXepJingleTransportParameters* (*) (XmppXepJingleTransport*, XmppXmppStream*, guint8, XmppJid*, XmppJid*, XmppStanzaNode*, GError**)) xmpp_xep_jingle_raw_udp_module_real_parse_transport_parameters;
	iface->get_ns_uri = xmpp_xep_jingle_raw_udp_module_real_get_ns_uri;
	iface->get_type_ = xmpp_xep_jingle_raw_udp_module_real_get_type_;
	iface->get_priority = xmpp_xep_jingle_raw_udp_module_real_get_priority;
}

static void
xmpp_xep_jingle_raw_udp_module_instance_init (XmppXepJingleRawUdpModule * self,
                                              gpointer klass)
{
	self->priv = xmpp_xep_jingle_raw_udp_module_get_instance_private (self);
	self->priv->get_local_ip_addresses_impl = NULL;
	self->priv->get_local_ip_addresses_impl_target = NULL;
	self->priv->get_local_ip_addresses_impl_target_destroy_notify = NULL;
}

static void
xmpp_xep_jingle_raw_udp_module_finalize (GObject * obj)
{
	XmppXepJingleRawUdpModule * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, XMPP_XEP_JINGLE_RAW_UDP_TYPE_MODULE, XmppXepJingleRawUdpModule);
	(self->priv->get_local_ip_addresses_impl_target_destroy_notify == NULL) ? NULL : (self->priv->get_local_ip_addresses_impl_target_destroy_notify (self->priv->get_local_ip_addresses_impl_target), NULL);
	self->priv->get_local_ip_addresses_impl = NULL;
	self->priv->get_local_ip_addresses_impl_target = NULL;
	self->priv->get_local_ip_addresses_impl_target_destroy_notify = NULL;
	G_OBJECT_CLASS (xmpp_xep_jingle_raw_udp_module_parent_class)->finalize (obj);
}

static GType
xmpp_xep_jingle_raw_udp_module_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepJingleRawUdpModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_jingle_raw_udp_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XmppXepJingleRawUdpModule), 0, (GInstanceInitFunc) xmpp_xep_jingle_raw_udp_module_instance_init, NULL };
	static const GInterfaceInfo xmpp_xep_jingle_transport_info = { (GInterfaceInitFunc) xmpp_xep_jingle_raw_udp_module_xmpp_xep_jingle_transport_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType xmpp_xep_jingle_raw_udp_module_type_id;
	xmpp_xep_jingle_raw_udp_module_type_id = g_type_register_static (XMPP_TYPE_XMPP_STREAM_MODULE, "XmppXepJingleRawUdpModule", &g_define_type_info, 0);
	g_type_add_interface_static (xmpp_xep_jingle_raw_udp_module_type_id, XMPP_XEP_JINGLE_TYPE_TRANSPORT, &xmpp_xep_jingle_transport_info);
	XmppXepJingleRawUdpModule_private_offset = g_type_add_instance_private (xmpp_xep_jingle_raw_udp_module_type_id, sizeof (XmppXepJingleRawUdpModulePrivate));
	return xmpp_xep_jingle_raw_udp_module_type_id;
}

GType
xmpp_xep_jingle_raw_udp_module_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_raw_udp_module_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_raw_udp_module_type_id__once)) {
		GType xmpp_xep_jingle_raw_udp_module_type_id;
		xmpp_xep_jingle_raw_udp_module_type_id = xmpp_xep_jingle_raw_udp_module_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_raw_udp_module_type_id__once, xmpp_xep_jingle_raw_udp_module_type_id);
	}
	return xmpp_xep_jingle_raw_udp_module_type_id__once;
}

static void
_vala_xmpp_xep_jingle_raw_udp_module_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	XmppXepJingleRawUdpModule * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, XMPP_XEP_JINGLE_RAW_UDP_TYPE_MODULE, XmppXepJingleRawUdpModule);
	switch (property_id) {
		case XMPP_XEP_JINGLE_RAW_UDP_MODULE_NS_URI_PROPERTY:
		g_value_set_string (value, xmpp_xep_jingle_transport_get_ns_uri ((XmppXepJingleTransport*) self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_MODULE_TYPE__PROPERTY:
		g_value_set_enum (value, xmpp_xep_jingle_transport_get_type_ ((XmppXepJingleTransport*) self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_MODULE_PRIORITY_PROPERTY:
		g_value_set_int (value, xmpp_xep_jingle_transport_get_priority ((XmppXepJingleTransport*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
xmpp_xep_jingle_raw_udp_transport_parameters_get_instance_private (XmppXepJingleRawUdpTransportParameters* self)
{
	return G_STRUCT_MEMBER_P (self, XmppXepJingleRawUdpTransportParameters_private_offset);
}

XmppXepJingleRawUdpTransportParameters*
xmpp_xep_jingle_raw_udp_transport_parameters_construct (GType object_type,
                                                        guint8 components,
                                                        XmppStanzaNode* node)
{
	XmppXepJingleRawUdpTransportParameters * self = NULL;
	self = (XmppXepJingleRawUdpTransportParameters*) g_object_new (object_type, NULL);
	if (node != NULL) {
		{
			GeeList* _candidate_node_list = NULL;
			GeeList* _tmp0_;
			gint _candidate_node_size = 0;
			GeeList* _tmp1_;
			gint _tmp2_;
			gint _tmp3_;
			gint _candidate_node_index = 0;
			_tmp0_ = xmpp_stanza_node_get_subnodes (node, "candidate", NULL, FALSE);
			_candidate_node_list = _tmp0_;
			_tmp1_ = _candidate_node_list;
			_tmp2_ = gee_collection_get_size ((GeeCollection*) _tmp1_);
			_tmp3_ = _tmp2_;
			_candidate_node_size = _tmp3_;
			_candidate_node_index = -1;
			while (TRUE) {
				gint _tmp4_;
				gint _tmp5_;
				XmppStanzaNode* candidate_node = NULL;
				GeeList* _tmp6_;
				gpointer _tmp7_;
				XmppXepJingleRawUdpCandidate* candidate = NULL;
				XmppXepJingleRawUdpCandidate* _tmp8_;
				gchar* component_str = NULL;
				XmppStanzaNode* _tmp9_;
				const gchar* _tmp10_;
				gchar* _tmp11_;
				XmppXepJingleRawUdpCandidate* _tmp12_;
				const gchar* _tmp13_;
				gchar* generation_str = NULL;
				XmppStanzaNode* _tmp14_;
				const gchar* _tmp15_;
				gchar* _tmp16_;
				XmppXepJingleRawUdpCandidate* _tmp17_;
				const gchar* _tmp18_;
				XmppXepJingleRawUdpCandidate* _tmp19_;
				XmppStanzaNode* _tmp20_;
				const gchar* _tmp21_;
				gchar* ip_str = NULL;
				XmppStanzaNode* _tmp22_;
				const gchar* _tmp23_;
				gchar* _tmp24_;
				XmppXepJingleRawUdpCandidate* _tmp25_;
				const gchar* _tmp26_;
				GInetAddress* _tmp27_;
				GInetAddress* _tmp28_;
				gchar* port_str = NULL;
				XmppStanzaNode* _tmp29_;
				const gchar* _tmp30_;
				gchar* _tmp31_;
				XmppXepJingleRawUdpCandidate* _tmp32_;
				const gchar* _tmp33_;
				GeeList* _tmp34_;
				XmppXepJingleRawUdpCandidate* _tmp35_;
				_candidate_node_index = _candidate_node_index + 1;
				_tmp4_ = _candidate_node_index;
				_tmp5_ = _candidate_node_size;
				if (!(_tmp4_ < _tmp5_)) {
					break;
				}
				_tmp6_ = _candidate_node_list;
				_tmp7_ = gee_list_get (_tmp6_, _candidate_node_index);
				candidate_node = (XmppStanzaNode*) _tmp7_;
				_tmp8_ = xmpp_xep_jingle_raw_udp_candidate_new ();
				candidate = _tmp8_;
				_tmp9_ = candidate_node;
				_tmp10_ = xmpp_stanza_node_get_attribute (_tmp9_, "component", NULL);
				_tmp11_ = g_strdup (_tmp10_);
				component_str = _tmp11_;
				_tmp12_ = candidate;
				_tmp13_ = component_str;
				xmpp_xep_jingle_raw_udp_candidate_set_component (_tmp12_, atoi (_tmp13_));
				_tmp14_ = candidate_node;
				_tmp15_ = xmpp_stanza_node_get_attribute (_tmp14_, "generation", NULL);
				_tmp16_ = g_strdup (_tmp15_);
				generation_str = _tmp16_;
				_tmp17_ = candidate;
				_tmp18_ = generation_str;
				xmpp_xep_jingle_raw_udp_candidate_set_generation (_tmp17_, atoi (_tmp18_));
				_tmp19_ = candidate;
				_tmp20_ = candidate_node;
				_tmp21_ = xmpp_stanza_node_get_attribute (_tmp20_, "generation", NULL);
				xmpp_xep_jingle_raw_udp_candidate_set_id (_tmp19_, _tmp21_);
				_tmp22_ = candidate_node;
				_tmp23_ = xmpp_stanza_node_get_attribute (_tmp22_, "ip", NULL);
				_tmp24_ = g_strdup (_tmp23_);
				ip_str = _tmp24_;
				_tmp25_ = candidate;
				_tmp26_ = ip_str;
				_tmp27_ = g_inet_address_new_from_string (_tmp26_);
				_tmp28_ = _tmp27_;
				xmpp_xep_jingle_raw_udp_candidate_set_ip (_tmp25_, _tmp28_);
				_g_object_unref0 (_tmp28_);
				_tmp29_ = candidate_node;
				_tmp30_ = xmpp_stanza_node_get_attribute (_tmp29_, "port", NULL);
				_tmp31_ = g_strdup (_tmp30_);
				port_str = _tmp31_;
				_tmp32_ = candidate;
				_tmp33_ = port_str;
				xmpp_xep_jingle_raw_udp_candidate_set_port (_tmp32_, (guint) atoi (_tmp33_));
				_tmp34_ = self->remote_candidates;
				_tmp35_ = candidate;
				gee_collection_add ((GeeCollection*) _tmp34_, _tmp35_);
				_g_free0 (port_str);
				_g_free0 (ip_str);
				_g_free0 (generation_str);
				_g_free0 (component_str);
				_g_object_unref0 (candidate);
				_xmpp_stanza_entry_unref0 (candidate_node);
			}
			_g_object_unref0 (_candidate_node_list);
		}
	}
	return self;
}

XmppXepJingleRawUdpTransportParameters*
xmpp_xep_jingle_raw_udp_transport_parameters_new (guint8 components,
                                                  XmppStanzaNode* node)
{
	return xmpp_xep_jingle_raw_udp_transport_parameters_construct (XMPP_XEP_JINGLE_RAW_UDP_TYPE_TRANSPORT_PARAMETERS, components, node);
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_real_set_content (XmppXepJingleTransportParameters* base,
                                                               XmppXepJingleContent* content)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	g_return_if_fail (content != NULL);
}

static XmppStanzaNode*
xmpp_xep_jingle_raw_udp_transport_parameters_real_to_transport_stanza_node (XmppXepJingleTransportParameters* base,
                                                                            const gchar* action_type)
{
	XmppXepJingleRawUdpTransportParameters * self;
	XmppStanzaNode* transport_node = NULL;
	XmppStanzaNode* _tmp0_;
	XmppStanzaNode* _tmp1_;
	XmppStanzaNode* _tmp2_;
	XmppStanzaNode* _tmp3_;
	XmppStanzaNode* result;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	g_return_val_if_fail (action_type != NULL, NULL);
	_tmp0_ = xmpp_stanza_node_new_build ("transport", XMPP_XEP_JINGLE_RAW_UDP_NS_URI, NULL, NULL);
	_tmp1_ = _tmp0_;
	_tmp2_ = xmpp_stanza_node_add_self_xmlns (_tmp1_);
	_tmp3_ = _tmp2_;
	_xmpp_stanza_entry_unref0 (_tmp1_);
	transport_node = _tmp3_;
	{
		GeeList* _candidate_list = NULL;
		GeeList* _tmp4_;
		gint _candidate_size = 0;
		GeeList* _tmp5_;
		gint _tmp6_;
		gint _tmp7_;
		gint _candidate_index = 0;
		_tmp4_ = self->own_candidates;
		_candidate_list = _tmp4_;
		_tmp5_ = _candidate_list;
		_tmp6_ = gee_collection_get_size ((GeeCollection*) _tmp5_);
		_tmp7_ = _tmp6_;
		_candidate_size = _tmp7_;
		_candidate_index = -1;
		while (TRUE) {
			gint _tmp8_;
			gint _tmp9_;
			XmppXepJingleRawUdpCandidate* candidate = NULL;
			GeeList* _tmp10_;
			gpointer _tmp11_;
			XmppStanzaNode* _tmp12_;
			XmppStanzaNode* _tmp13_;
			XmppStanzaNode* _tmp14_;
			XmppXepJingleRawUdpCandidate* _tmp15_;
			gint _tmp16_;
			gint _tmp17_;
			gchar* _tmp18_;
			gchar* _tmp19_;
			XmppStanzaNode* _tmp20_;
			XmppStanzaNode* _tmp21_;
			XmppXepJingleRawUdpCandidate* _tmp22_;
			const gchar* _tmp23_;
			const gchar* _tmp24_;
			XmppStanzaNode* _tmp25_;
			XmppStanzaNode* _tmp26_;
			XmppXepJingleRawUdpCandidate* _tmp27_;
			GInetAddress* _tmp28_;
			GInetAddress* _tmp29_;
			gchar* _tmp30_;
			gchar* _tmp31_;
			XmppStanzaNode* _tmp32_;
			XmppStanzaNode* _tmp33_;
			XmppXepJingleRawUdpCandidate* _tmp34_;
			guint _tmp35_;
			guint _tmp36_;
			gchar* _tmp37_;
			gchar* _tmp38_;
			XmppStanzaNode* _tmp39_;
			XmppStanzaNode* _tmp40_;
			XmppStanzaNode* _tmp41_;
			XmppStanzaNode* _tmp42_;
			_candidate_index = _candidate_index + 1;
			_tmp8_ = _candidate_index;
			_tmp9_ = _candidate_size;
			if (!(_tmp8_ < _tmp9_)) {
				break;
			}
			_tmp10_ = _candidate_list;
			_tmp11_ = gee_list_get (_tmp10_, _candidate_index);
			candidate = (XmppXepJingleRawUdpCandidate*) _tmp11_;
			_tmp12_ = transport_node;
			_tmp13_ = xmpp_stanza_node_new_build ("candidate", XMPP_XEP_JINGLE_RAW_UDP_NS_URI, NULL, NULL);
			_tmp14_ = _tmp13_;
			_tmp15_ = candidate;
			_tmp16_ = xmpp_xep_jingle_raw_udp_candidate_get_generation (_tmp15_);
			_tmp17_ = _tmp16_;
			_tmp18_ = g_strdup_printf ("%i", _tmp17_);
			_tmp19_ = _tmp18_;
			_tmp20_ = xmpp_stanza_node_put_attribute (_tmp14_, "generation", _tmp19_, NULL);
			_tmp21_ = _tmp20_;
			_tmp22_ = candidate;
			_tmp23_ = xmpp_xep_jingle_raw_udp_candidate_get_id (_tmp22_);
			_tmp24_ = _tmp23_;
			_tmp25_ = xmpp_stanza_node_put_attribute (_tmp21_, "id", _tmp24_, NULL);
			_tmp26_ = _tmp25_;
			_tmp27_ = candidate;
			_tmp28_ = xmpp_xep_jingle_raw_udp_candidate_get_ip (_tmp27_);
			_tmp29_ = _tmp28_;
			_tmp30_ = g_inet_address_to_string (_tmp29_);
			_tmp31_ = _tmp30_;
			_tmp32_ = xmpp_stanza_node_put_attribute (_tmp26_, "ip", _tmp31_, NULL);
			_tmp33_ = _tmp32_;
			_tmp34_ = candidate;
			_tmp35_ = xmpp_xep_jingle_raw_udp_candidate_get_port (_tmp34_);
			_tmp36_ = _tmp35_;
			_tmp37_ = g_strdup_printf ("%u", _tmp36_);
			_tmp38_ = _tmp37_;
			_tmp39_ = xmpp_stanza_node_put_attribute (_tmp33_, "port", _tmp38_, NULL);
			_tmp40_ = _tmp39_;
			_tmp41_ = xmpp_stanza_node_put_node (_tmp12_, _tmp40_);
			_tmp42_ = _tmp41_;
			_xmpp_stanza_entry_unref0 (_tmp42_);
			_xmpp_stanza_entry_unref0 (_tmp40_);
			_g_free0 (_tmp38_);
			_xmpp_stanza_entry_unref0 (_tmp33_);
			_g_free0 (_tmp31_);
			_xmpp_stanza_entry_unref0 (_tmp26_);
			_xmpp_stanza_entry_unref0 (_tmp21_);
			_g_free0 (_tmp19_);
			_xmpp_stanza_entry_unref0 (_tmp14_);
			_g_object_unref0 (candidate);
		}
	}
	result = transport_node;
	return result;
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_accept (XmppXepJingleTransportParameters* base,
                                                                           XmppStanzaNode* transport,
                                                                           GError** error)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	g_return_if_fail (transport != NULL);
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_info (XmppXepJingleTransportParameters* base,
                                                                         XmppStanzaNode* transport,
                                                                         GError** error)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	g_return_if_fail (transport != NULL);
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_real_create_transport_connection (XmppXepJingleTransportParameters* base,
                                                                               XmppXmppStream* stream,
                                                                               XmppXepJingleContent* content)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	g_return_if_fail (stream != NULL);
	g_return_if_fail (content != NULL);
}

static const gchar*
xmpp_xep_jingle_raw_udp_transport_parameters_real_get_ns_uri (XmppXepJingleTransportParameters* base)
{
	const gchar* result;
	XmppXepJingleRawUdpTransportParameters* self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	result = XMPP_XEP_JINGLE_RAW_UDP_NS_URI;
	return result;
}

static guint8
xmpp_xep_jingle_raw_udp_transport_parameters_real_get_components (XmppXepJingleTransportParameters* base)
{
	guint8 result;
	XmppXepJingleRawUdpTransportParameters* self;
	self = (XmppXepJingleRawUdpTransportParameters*) base;
	result = self->priv->_components;
	return result;
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_class_init (XmppXepJingleRawUdpTransportParametersClass * klass,
                                                         gpointer klass_data)
{
	xmpp_xep_jingle_raw_udp_transport_parameters_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &XmppXepJingleRawUdpTransportParameters_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_xmpp_xep_jingle_raw_udp_transport_parameters_get_property;
	G_OBJECT_CLASS (klass)->finalize = xmpp_xep_jingle_raw_udp_transport_parameters_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NS_URI_PROPERTY, xmpp_xep_jingle_raw_udp_transport_parameters_properties[XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NS_URI_PROPERTY] = g_param_spec_string ("ns-uri", "ns-uri", "ns-uri", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_COMPONENTS_PROPERTY, xmpp_xep_jingle_raw_udp_transport_parameters_properties[XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_COMPONENTS_PROPERTY] = g_param_spec_uchar ("components", "components", "components", 0, G_MAXUINT8, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_xmpp_xep_jingle_transport_parameters_interface_init (XmppXepJingleTransportParametersIface * iface,
                                                                                                  gpointer iface_data)
{
	xmpp_xep_jingle_raw_udp_transport_parameters_xmpp_xep_jingle_transport_parameters_parent_iface = g_type_interface_peek_parent (iface);
	iface->set_content = (void (*) (XmppXepJingleTransportParameters*, XmppXepJingleContent*)) xmpp_xep_jingle_raw_udp_transport_parameters_real_set_content;
	iface->to_transport_stanza_node = (XmppStanzaNode* (*) (XmppXepJingleTransportParameters*, const gchar*)) xmpp_xep_jingle_raw_udp_transport_parameters_real_to_transport_stanza_node;
	iface->handle_transport_accept = (void (*) (XmppXepJingleTransportParameters*, XmppStanzaNode*, GError**)) xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_accept;
	iface->handle_transport_info = (void (*) (XmppXepJingleTransportParameters*, XmppStanzaNode*, GError**)) xmpp_xep_jingle_raw_udp_transport_parameters_real_handle_transport_info;
	iface->create_transport_connection = (void (*) (XmppXepJingleTransportParameters*, XmppXmppStream*, XmppXepJingleContent*)) xmpp_xep_jingle_raw_udp_transport_parameters_real_create_transport_connection;
	iface->get_ns_uri = xmpp_xep_jingle_raw_udp_transport_parameters_real_get_ns_uri;
	iface->get_components = xmpp_xep_jingle_raw_udp_transport_parameters_real_get_components;
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_instance_init (XmppXepJingleRawUdpTransportParameters * self,
                                                            gpointer klass)
{
	GeeArrayList* _tmp0_;
	GeeArrayList* _tmp1_;
	self->priv = xmpp_xep_jingle_raw_udp_transport_parameters_get_instance_private (self);
	_tmp0_ = gee_array_list_new (XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
	self->remote_candidates = (GeeList*) _tmp0_;
	_tmp1_ = gee_array_list_new (XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
	self->own_candidates = (GeeList*) _tmp1_;
}

static void
xmpp_xep_jingle_raw_udp_transport_parameters_finalize (GObject * obj)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, XMPP_XEP_JINGLE_RAW_UDP_TYPE_TRANSPORT_PARAMETERS, XmppXepJingleRawUdpTransportParameters);
	_g_object_unref0 (self->remote_candidates);
	_g_object_unref0 (self->own_candidates);
	G_OBJECT_CLASS (xmpp_xep_jingle_raw_udp_transport_parameters_parent_class)->finalize (obj);
}

static GType
xmpp_xep_jingle_raw_udp_transport_parameters_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepJingleRawUdpTransportParametersClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_jingle_raw_udp_transport_parameters_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XmppXepJingleRawUdpTransportParameters), 0, (GInstanceInitFunc) xmpp_xep_jingle_raw_udp_transport_parameters_instance_init, NULL };
	static const GInterfaceInfo xmpp_xep_jingle_transport_parameters_info = { (GInterfaceInitFunc) xmpp_xep_jingle_raw_udp_transport_parameters_xmpp_xep_jingle_transport_parameters_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType xmpp_xep_jingle_raw_udp_transport_parameters_type_id;
	xmpp_xep_jingle_raw_udp_transport_parameters_type_id = g_type_register_static (G_TYPE_OBJECT, "XmppXepJingleRawUdpTransportParameters", &g_define_type_info, 0);
	g_type_add_interface_static (xmpp_xep_jingle_raw_udp_transport_parameters_type_id, XMPP_XEP_JINGLE_TYPE_TRANSPORT_PARAMETERS, &xmpp_xep_jingle_transport_parameters_info);
	XmppXepJingleRawUdpTransportParameters_private_offset = g_type_add_instance_private (xmpp_xep_jingle_raw_udp_transport_parameters_type_id, sizeof (XmppXepJingleRawUdpTransportParametersPrivate));
	return xmpp_xep_jingle_raw_udp_transport_parameters_type_id;
}

GType
xmpp_xep_jingle_raw_udp_transport_parameters_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_raw_udp_transport_parameters_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_raw_udp_transport_parameters_type_id__once)) {
		GType xmpp_xep_jingle_raw_udp_transport_parameters_type_id;
		xmpp_xep_jingle_raw_udp_transport_parameters_type_id = xmpp_xep_jingle_raw_udp_transport_parameters_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_raw_udp_transport_parameters_type_id__once, xmpp_xep_jingle_raw_udp_transport_parameters_type_id);
	}
	return xmpp_xep_jingle_raw_udp_transport_parameters_type_id__once;
}

static void
_vala_xmpp_xep_jingle_raw_udp_transport_parameters_get_property (GObject * object,
                                                                 guint property_id,
                                                                 GValue * value,
                                                                 GParamSpec * pspec)
{
	XmppXepJingleRawUdpTransportParameters * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, XMPP_XEP_JINGLE_RAW_UDP_TYPE_TRANSPORT_PARAMETERS, XmppXepJingleRawUdpTransportParameters);
	switch (property_id) {
		case XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_NS_URI_PROPERTY:
		g_value_set_string (value, xmpp_xep_jingle_transport_parameters_get_ns_uri ((XmppXepJingleTransportParameters*) self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_TRANSPORT_PARAMETERS_COMPONENTS_PROPERTY:
		g_value_set_uchar (value, xmpp_xep_jingle_transport_parameters_get_components ((XmppXepJingleTransportParameters*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
xmpp_xep_jingle_raw_udp_candidate_get_instance_private (XmppXepJingleRawUdpCandidate* self)
{
	return G_STRUCT_MEMBER_P (self, XmppXepJingleRawUdpCandidate_private_offset);
}

XmppXepJingleRawUdpCandidate*
xmpp_xep_jingle_raw_udp_candidate_construct (GType object_type)
{
	XmppXepJingleRawUdpCandidate * self = NULL;
	self = (XmppXepJingleRawUdpCandidate*) g_object_new (object_type, NULL);
	return self;
}

XmppXepJingleRawUdpCandidate*
xmpp_xep_jingle_raw_udp_candidate_new (void)
{
	return xmpp_xep_jingle_raw_udp_candidate_construct (XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE);
}

gint
xmpp_xep_jingle_raw_udp_candidate_get_component (XmppXepJingleRawUdpCandidate* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, -1);
	result = self->priv->_component;
	return result;
}

void
xmpp_xep_jingle_raw_udp_candidate_set_component (XmppXepJingleRawUdpCandidate* self,
                                                 gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = xmpp_xep_jingle_raw_udp_candidate_get_component (self);
	if (old_value != value) {
		self->priv->_component = value;
		g_object_notify_by_pspec ((GObject *) self, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY]);
	}
}

gint
xmpp_xep_jingle_raw_udp_candidate_get_generation (XmppXepJingleRawUdpCandidate* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, -1);
	result = self->priv->_generation;
	return result;
}

void
xmpp_xep_jingle_raw_udp_candidate_set_generation (XmppXepJingleRawUdpCandidate* self,
                                                  gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = xmpp_xep_jingle_raw_udp_candidate_get_generation (self);
	if (old_value != value) {
		self->priv->_generation = value;
		g_object_notify_by_pspec ((GObject *) self, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY]);
	}
}

const gchar*
xmpp_xep_jingle_raw_udp_candidate_get_id (XmppXepJingleRawUdpCandidate* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_id;
	result = _tmp0_;
	return result;
}

void
xmpp_xep_jingle_raw_udp_candidate_set_id (XmppXepJingleRawUdpCandidate* self,
                                          const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = xmpp_xep_jingle_raw_udp_candidate_get_id (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_id);
		self->priv->_id = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY]);
	}
}

GInetAddress*
xmpp_xep_jingle_raw_udp_candidate_get_ip (XmppXepJingleRawUdpCandidate* self)
{
	GInetAddress* result;
	GInetAddress* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_ip;
	result = _tmp0_;
	return result;
}

void
xmpp_xep_jingle_raw_udp_candidate_set_ip (XmppXepJingleRawUdpCandidate* self,
                                          GInetAddress* value)
{
	GInetAddress* old_value;
	g_return_if_fail (self != NULL);
	old_value = xmpp_xep_jingle_raw_udp_candidate_get_ip (self);
	if (old_value != value) {
		GInetAddress* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_ip);
		self->priv->_ip = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY]);
	}
}

guint
xmpp_xep_jingle_raw_udp_candidate_get_port (XmppXepJingleRawUdpCandidate* self)
{
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_port;
	return result;
}

void
xmpp_xep_jingle_raw_udp_candidate_set_port (XmppXepJingleRawUdpCandidate* self,
                                            guint value)
{
	guint old_value;
	g_return_if_fail (self != NULL);
	old_value = xmpp_xep_jingle_raw_udp_candidate_get_port (self);
	if (old_value != value) {
		self->priv->_port = value;
		g_object_notify_by_pspec ((GObject *) self, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY]);
	}
}

static void
xmpp_xep_jingle_raw_udp_candidate_class_init (XmppXepJingleRawUdpCandidateClass * klass,
                                              gpointer klass_data)
{
	xmpp_xep_jingle_raw_udp_candidate_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &XmppXepJingleRawUdpCandidate_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_xmpp_xep_jingle_raw_udp_candidate_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_xmpp_xep_jingle_raw_udp_candidate_set_property;
	G_OBJECT_CLASS (klass)->finalize = xmpp_xep_jingle_raw_udp_candidate_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY] = g_param_spec_int ("component", "component", "component", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY] = g_param_spec_int ("generation", "generation", "generation", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY] = g_param_spec_string ("id", "id", "id", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY] = g_param_spec_object ("ip", "ip", "ip", g_inet_address_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY, xmpp_xep_jingle_raw_udp_candidate_properties[XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY] = g_param_spec_uint ("port", "port", "port", 0, G_MAXUINT, 0U, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
xmpp_xep_jingle_raw_udp_candidate_instance_init (XmppXepJingleRawUdpCandidate * self,
                                                 gpointer klass)
{
	self->priv = xmpp_xep_jingle_raw_udp_candidate_get_instance_private (self);
}

static void
xmpp_xep_jingle_raw_udp_candidate_finalize (GObject * obj)
{
	XmppXepJingleRawUdpCandidate * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE, XmppXepJingleRawUdpCandidate);
	_g_free0 (self->priv->_id);
	_g_object_unref0 (self->priv->_ip);
	G_OBJECT_CLASS (xmpp_xep_jingle_raw_udp_candidate_parent_class)->finalize (obj);
}

static GType
xmpp_xep_jingle_raw_udp_candidate_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepJingleRawUdpCandidateClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_jingle_raw_udp_candidate_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XmppXepJingleRawUdpCandidate), 0, (GInstanceInitFunc) xmpp_xep_jingle_raw_udp_candidate_instance_init, NULL };
	GType xmpp_xep_jingle_raw_udp_candidate_type_id;
	xmpp_xep_jingle_raw_udp_candidate_type_id = g_type_register_static (G_TYPE_OBJECT, "XmppXepJingleRawUdpCandidate", &g_define_type_info, 0);
	XmppXepJingleRawUdpCandidate_private_offset = g_type_add_instance_private (xmpp_xep_jingle_raw_udp_candidate_type_id, sizeof (XmppXepJingleRawUdpCandidatePrivate));
	return xmpp_xep_jingle_raw_udp_candidate_type_id;
}

GType
xmpp_xep_jingle_raw_udp_candidate_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_raw_udp_candidate_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_raw_udp_candidate_type_id__once)) {
		GType xmpp_xep_jingle_raw_udp_candidate_type_id;
		xmpp_xep_jingle_raw_udp_candidate_type_id = xmpp_xep_jingle_raw_udp_candidate_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_raw_udp_candidate_type_id__once, xmpp_xep_jingle_raw_udp_candidate_type_id);
	}
	return xmpp_xep_jingle_raw_udp_candidate_type_id__once;
}

static void
_vala_xmpp_xep_jingle_raw_udp_candidate_get_property (GObject * object,
                                                      guint property_id,
                                                      GValue * value,
                                                      GParamSpec * pspec)
{
	XmppXepJingleRawUdpCandidate * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE, XmppXepJingleRawUdpCandidate);
	switch (property_id) {
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY:
		g_value_set_int (value, xmpp_xep_jingle_raw_udp_candidate_get_component (self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY:
		g_value_set_int (value, xmpp_xep_jingle_raw_udp_candidate_get_generation (self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY:
		g_value_set_string (value, xmpp_xep_jingle_raw_udp_candidate_get_id (self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY:
		g_value_set_object (value, xmpp_xep_jingle_raw_udp_candidate_get_ip (self));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY:
		g_value_set_uint (value, xmpp_xep_jingle_raw_udp_candidate_get_port (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_xmpp_xep_jingle_raw_udp_candidate_set_property (GObject * object,
                                                      guint property_id,
                                                      const GValue * value,
                                                      GParamSpec * pspec)
{
	XmppXepJingleRawUdpCandidate * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, XMPP_XEP_JINGLE_RAW_UDP_TYPE_CANDIDATE, XmppXepJingleRawUdpCandidate);
	switch (property_id) {
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_COMPONENT_PROPERTY:
		xmpp_xep_jingle_raw_udp_candidate_set_component (self, g_value_get_int (value));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_GENERATION_PROPERTY:
		xmpp_xep_jingle_raw_udp_candidate_set_generation (self, g_value_get_int (value));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_ID_PROPERTY:
		xmpp_xep_jingle_raw_udp_candidate_set_id (self, g_value_get_string (value));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_IP_PROPERTY:
		xmpp_xep_jingle_raw_udp_candidate_set_ip (self, g_value_get_object (value));
		break;
		case XMPP_XEP_JINGLE_RAW_UDP_CANDIDATE_PORT_PROPERTY:
		xmpp_xep_jingle_raw_udp_candidate_set_port (self, g_value_get_uint (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

