/* random.c generated by valac 0.56.19, the Vala compiler
 * generated from random.vala, do not modify */

#include "crypto-vala.h"
#include <glib.h>
#include <gcrypt.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

void
crypto_randomize (guint8* buffer,
                  gint buffer_length1)
{
	gcry_randomize (buffer, (gint) buffer_length1, GCRY_VERY_STRONG_RANDOM);
}

