/* status_code.c generated by valac 0.56.19, the Vala compiler
 * generated from status_code.vala, do not modify */

#include "xmpp-vala.h"
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

const XmppXepMucStatusCode XMPP_XEP_MUC_ROOM_CONFIGURATION_CODES[4] = {XMPP_XEP_MUC_STATUS_CODE_LOGGING_ENABLED, XMPP_XEP_MUC_STATUS_CODE_LOGGING_DISABLED, XMPP_XEP_MUC_STATUS_CODE_NON_ANONYMOUS, XMPP_XEP_MUC_STATUS_CODE_SEMI_ANONYMOUS};
const XmppXepMucStatusCode XMPP_XEP_MUC_USER_REMOVED_CODES[5] = {XMPP_XEP_MUC_STATUS_CODE_BANNED, XMPP_XEP_MUC_STATUS_CODE_KICKED, XMPP_XEP_MUC_STATUS_CODE_REMOVED_AFFILIATION_CHANGE, XMPP_XEP_MUC_STATUS_CODE_REMOVED_MEMBERS_ONLY, XMPP_XEP_MUC_STATUS_CODE_REMOVED_SHUTDOWN};

static GType
xmpp_xep_muc_status_code_get_type_once (void)
{
	static const GEnumValue values[] = {{XMPP_XEP_MUC_STATUS_CODE_JID_VISIBLE, "XMPP_XEP_MUC_STATUS_CODE_JID_VISIBLE", "jid-visible"}, {XMPP_XEP_MUC_STATUS_CODE_AFFILIATION_CHANGED, "XMPP_XEP_MUC_STATUS_CODE_AFFILIATION_CHANGED", "affiliation-changed"}, {XMPP_XEP_MUC_STATUS_CODE_SHOWS_UNAVIABLE_MEMBERS, "XMPP_XEP_MUC_STATUS_CODE_SHOWS_UNAVIABLE_MEMBERS", "shows-unaviable-members"}, {XMPP_XEP_MUC_STATUS_CODE_SHOWS_UNAVIABLE_MEMBERS_NOT, "XMPP_XEP_MUC_STATUS_CODE_SHOWS_UNAVIABLE_MEMBERS_NOT", "shows-unaviable-members-not"}, {XMPP_XEP_MUC_STATUS_CODE_CONFIG_CHANGE_NON_PRIVACY, "XMPP_XEP_MUC_STATUS_CODE_CONFIG_CHANGE_NON_PRIVACY", "config-change-non-privacy"}, {XMPP_XEP_MUC_STATUS_CODE_SELF_PRESENCE, "XMPP_XEP_MUC_STATUS_CODE_SELF_PRESENCE", "self-presence"}, {XMPP_XEP_MUC_STATUS_CODE_LOGGING_ENABLED, "XMPP_XEP_MUC_STATUS_CODE_LOGGING_ENABLED", "logging-enabled"}, {XMPP_XEP_MUC_STATUS_CODE_LOGGING_DISABLED, "XMPP_XEP_MUC_STATUS_CODE_LOGGING_DISABLED", "logging-disabled"}, {XMPP_XEP_MUC_STATUS_CODE_NON_ANONYMOUS, "XMPP_XEP_MUC_STATUS_CODE_NON_ANONYMOUS", "non-anonymous"}, {XMPP_XEP_MUC_STATUS_CODE_SEMI_ANONYMOUS, "XMPP_XEP_MUC_STATUS_CODE_SEMI_ANONYMOUS", "semi-anonymous"}, {XMPP_XEP_MUC_STATUS_CODE_NEW_ROOM_CREATED, "XMPP_XEP_MUC_STATUS_CODE_NEW_ROOM_CREATED", "new-room-created"}, {XMPP_XEP_MUC_STATUS_CODE_MODIFIED_NICK, "XMPP_XEP_MUC_STATUS_CODE_MODIFIED_NICK", "modified-nick"}, {XMPP_XEP_MUC_STATUS_CODE_BANNED, "XMPP_XEP_MUC_STATUS_CODE_BANNED", "banned"}, {XMPP_XEP_MUC_STATUS_CODE_ROOM_NICKNAME, "XMPP_XEP_MUC_STATUS_CODE_ROOM_NICKNAME", "room-nickname"}, {XMPP_XEP_MUC_STATUS_CODE_KICKED, "XMPP_XEP_MUC_STATUS_CODE_KICKED", "kicked"}, {XMPP_XEP_MUC_STATUS_CODE_REMOVED_AFFILIATION_CHANGE, "XMPP_XEP_MUC_STATUS_CODE_REMOVED_AFFILIATION_CHANGE", "removed-affiliation-change"}, {XMPP_XEP_MUC_STATUS_CODE_REMOVED_MEMBERS_ONLY, "XMPP_XEP_MUC_STATUS_CODE_REMOVED_MEMBERS_ONLY", "removed-members-only"}, {XMPP_XEP_MUC_STATUS_CODE_REMOVED_SHUTDOWN, "XMPP_XEP_MUC_STATUS_CODE_REMOVED_SHUTDOWN", "removed-shutdown"}, {0, NULL, NULL}};
	GType xmpp_xep_muc_status_code_type_id;
	xmpp_xep_muc_status_code_type_id = g_enum_register_static ("XmppXepMucStatusCode", values);
	return xmpp_xep_muc_status_code_type_id;
}

GType
xmpp_xep_muc_status_code_get_type (void)
{
	static volatile gsize xmpp_xep_muc_status_code_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_muc_status_code_type_id__once)) {
		GType xmpp_xep_muc_status_code_type_id;
		xmpp_xep_muc_status_code_type_id = xmpp_xep_muc_status_code_get_type_once ();
		g_once_init_leave (&xmpp_xep_muc_status_code_type_id__once, xmpp_xep_muc_status_code_type_id);
	}
	return xmpp_xep_muc_status_code_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

