/* notifications.c generated by valac 0.56.19, the Vala compiler
 * generated from notifications.vala, do not modify */

#include "libdino.h"
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	DINO_DBUS_NOTIFICATIONS_ACTION_INVOKED_SIGNAL,
	DINO_DBUS_NOTIFICATIONS_NOTIFICATION_CLOSED_SIGNAL,
	DINO_DBUS_NOTIFICATIONS_NUM_SIGNALS
};
static guint dino_dbus_notifications_signals[DINO_DBUS_NOTIFICATIONS_NUM_SIGNALS] = {0};
typedef GDBusProxy DinoDBusNotificationsProxy;
typedef GDBusProxyClass DinoDBusNotificationsProxyClass;
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _DinoDbusNotificationsNotifyReadyData DinoDbusNotificationsNotifyReadyData;
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))
typedef struct _DinoDbusNotificationsGetCapabilitiesReadyData DinoDbusNotificationsGetCapabilitiesReadyData;
typedef struct _DinoDbusNotificationsCloseNotificationReadyData DinoDbusNotificationsCloseNotificationReadyData;
typedef struct _DinoDbusNotificationsGetServerInformationReadyData DinoDbusNotificationsGetServerInformationReadyData;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _DinoGetNotificationsDbusData DinoGetNotificationsDbusData;

struct _DinoDbusNotificationsNotifyReadyData {
	GDBusMethodInvocation* _invocation_;
	gchar* app_name;
	guint32 replaces_id;
	gchar* app_icon;
	gchar* summary;
	gchar* body;
	gchar** actions;
	gint actions_length1;
	GHashTable* hints;
	gint32 expire_timeout;
};

struct _DinoDbusNotificationsGetCapabilitiesReadyData {
	GDBusMethodInvocation* _invocation_;
};

struct _DinoDbusNotificationsCloseNotificationReadyData {
	GDBusMethodInvocation* _invocation_;
	guint id;
};

struct _DinoDbusNotificationsGetServerInformationReadyData {
	GDBusMethodInvocation* _invocation_;
};

struct _DinoGetNotificationsDbusData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoDBusNotifications* result;
	DinoDBusNotifications* _tmp0_;
	DinoDBusNotifications* _tmp1_;
	DinoDBusNotifications* _tmp2_;
	GError* e;
	GError* _tmp3_;
	const gchar* _tmp4_;
	GError* _inner_error0_;
};

static void g_cclosure_user_marshal_VOID__UINT_STRING (GClosure * closure,
                                                GValue * return_value,
                                                guint n_param_values,
                                                const GValue * param_values,
                                                gpointer invocation_hint,
                                                gpointer marshal_data);
static void g_cclosure_user_marshal_VOID__UINT_UINT (GClosure * closure,
                                              GValue * return_value,
                                              guint n_param_values,
                                              const GValue * param_values,
                                              gpointer invocation_hint,
                                              gpointer marshal_data);
static GType dino_dbus_notifications_get_type_once (void);
static void dino_dbus_notifications_proxy_g_signal (GDBusProxy* proxy,
                                             const gchar* sender_name,
                                             const gchar* signal_name,
                                             GVariant* parameters);
static void _dbus_handle_dino_dbus_notifications_action_invoked (DinoDBusNotifications* self,
                                                          GVariant* parameters);
static void _dbus_handle_dino_dbus_notifications_notification_closed (DinoDBusNotifications* self,
                                                               GVariant* parameters);
static void _vala_g_async_ready_callback (GObject *source_object,
                                   GAsyncResult *res,
                                   void *user_data);
static void dino_dbus_notifications_proxy_notify_async (DinoDBusNotifications* self,
                                                 const gchar* app_name,
                                                 guint32 replaces_id,
                                                 const gchar* app_icon,
                                                 const gchar* summary,
                                                 const gchar* body,
                                                 gchar** actions,
                                                 gint actions_length1,
                                                 GHashTable* hints,
                                                 gint32 expire_timeout,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
static guint32 dino_dbus_notifications_proxy_notify_finish (DinoDBusNotifications* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
static void dino_dbus_notifications_proxy_get_capabilities_async (DinoDBusNotifications* self,
                                                           GAsyncReadyCallback _callback_,
                                                           gpointer _user_data_);
static void dino_dbus_notifications_proxy_get_capabilities_finish (DinoDBusNotifications* self,
                                                            GAsyncResult* _res_,
                                                            gchar*** capabilities,
                                                            gint* capabilities_length1,
                                                            GError** error);
static void dino_dbus_notifications_proxy_close_notification_async (DinoDBusNotifications* self,
                                                             guint id,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
static void dino_dbus_notifications_proxy_close_notification_finish (DinoDBusNotifications* self,
                                                              GAsyncResult* _res_,
                                                              GError** error);
static void dino_dbus_notifications_proxy_get_server_information_async (DinoDBusNotifications* self,
                                                                 GAsyncReadyCallback _callback_,
                                                                 gpointer _user_data_);
static void dino_dbus_notifications_proxy_get_server_information_finish (DinoDBusNotifications* self,
                                                                  GAsyncResult* _res_,
                                                                  gchar** name,
                                                                  gchar** vendor,
                                                                  gchar** version,
                                                                  gchar** spec_version,
                                                                  GError** error);
static void dino_dbus_notifications_proxy_dino_dbus_notifications_interface_init (DinoDBusNotificationsIface* iface);
static void _dbus_dino_dbus_notifications_notify (DinoDBusNotifications* self,
                                           GVariant* _parameters_,
                                           GDBusMethodInvocation* invocation);
static void _dbus_dino_dbus_notifications_notify_ready (GObject * source_object,
                                                 GAsyncResult * _res_,
                                                 gpointer _user_data_);
static void _dbus_dino_dbus_notifications_get_capabilities (DinoDBusNotifications* self,
                                                     GVariant* _parameters_,
                                                     GDBusMethodInvocation* invocation);
static void _dbus_dino_dbus_notifications_get_capabilities_ready (GObject * source_object,
                                                           GAsyncResult * _res_,
                                                           gpointer _user_data_);
static void _dbus_dino_dbus_notifications_close_notification (DinoDBusNotifications* self,
                                                       GVariant* _parameters_,
                                                       GDBusMethodInvocation* invocation);
static void _dbus_dino_dbus_notifications_close_notification_ready (GObject * source_object,
                                                             GAsyncResult * _res_,
                                                             gpointer _user_data_);
static void _dbus_dino_dbus_notifications_get_server_information (DinoDBusNotifications* self,
                                                           GVariant* _parameters_,
                                                           GDBusMethodInvocation* invocation);
static void _dbus_dino_dbus_notifications_get_server_information_ready (GObject * source_object,
                                                                 GAsyncResult * _res_,
                                                                 gpointer _user_data_);
static void dino_dbus_notifications_dbus_interface_method_call (GDBusConnection* connection,
                                                         const gchar* sender,
                                                         const gchar* object_path,
                                                         const gchar* interface_name,
                                                         const gchar* method_name,
                                                         GVariant* parameters,
                                                         GDBusMethodInvocation* invocation,
                                                         gpointer user_data);
static GVariant* dino_dbus_notifications_dbus_interface_get_property (GDBusConnection* connection,
                                                               const gchar* sender,
                                                               const gchar* object_path,
                                                               const gchar* interface_name,
                                                               const gchar* property_name,
                                                               GError** error,
                                                               gpointer user_data);
static gboolean dino_dbus_notifications_dbus_interface_set_property (GDBusConnection* connection,
                                                              const gchar* sender,
                                                              const gchar* object_path,
                                                              const gchar* interface_name,
                                                              const gchar* property_name,
                                                              GVariant* value,
                                                              GError** error,
                                                              gpointer user_data);
static void _dbus_dino_dbus_notifications_action_invoked (GObject* _sender,
                                                   guint32 key,
                                                   const gchar* action_key,
                                                   gpointer* _data);
static void _dbus_dino_dbus_notifications_notification_closed (GObject* _sender,
                                                        guint32 id,
                                                        guint32 reason,
                                                        gpointer* _data);
static void _dino_dbus_notifications_unregister_object (gpointer user_data);
static void dino_get_notifications_dbus_data_free (gpointer _data);
static gboolean dino_get_notifications_dbus_co (DinoGetNotificationsDbusData* _data_);
static void dino_get_notifications_dbus_ready (GObject* source_object,
                                        GAsyncResult* _res_,
                                        gpointer _user_data_);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);

static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_app_name = {-1, "app_name", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_replaces_id = {-1, "replaces_id", "u", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_app_icon = {-1, "app_icon", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_summary = {-1, "summary", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_body = {-1, "body", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_actions = {-1, "actions", "as", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_hints = {-1, "hints", "a{sv}", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_expire_timeout = {-1, "expire_timeout", "i", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notify_result = {-1, "result", "u", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_notify_in[] = {&_dino_dbus_notifications_dbus_arg_info_notify_app_name, &_dino_dbus_notifications_dbus_arg_info_notify_replaces_id, &_dino_dbus_notifications_dbus_arg_info_notify_app_icon, &_dino_dbus_notifications_dbus_arg_info_notify_summary, &_dino_dbus_notifications_dbus_arg_info_notify_body, &_dino_dbus_notifications_dbus_arg_info_notify_actions, &_dino_dbus_notifications_dbus_arg_info_notify_hints, &_dino_dbus_notifications_dbus_arg_info_notify_expire_timeout, NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_notify_out[] = {&_dino_dbus_notifications_dbus_arg_info_notify_result, NULL};
static const GDBusMethodInfo _dino_dbus_notifications_dbus_method_info_notify = {-1, "Notify", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_notify_in), (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_notify_out), NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_get_capabilities_capabilities = {-1, "capabilities", "as", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_get_capabilities_in[] = {NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_get_capabilities_out[] = {&_dino_dbus_notifications_dbus_arg_info_get_capabilities_capabilities, NULL};
static const GDBusMethodInfo _dino_dbus_notifications_dbus_method_info_get_capabilities = {-1, "GetCapabilities", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_get_capabilities_in), (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_get_capabilities_out), NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_close_notification_id = {-1, "id", "u", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_close_notification_in[] = {&_dino_dbus_notifications_dbus_arg_info_close_notification_id, NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_close_notification_out[] = {NULL};
static const GDBusMethodInfo _dino_dbus_notifications_dbus_method_info_close_notification = {-1, "CloseNotification", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_close_notification_in), (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_close_notification_out), NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_get_server_information_name = {-1, "name", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_get_server_information_vendor = {-1, "vendor", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_get_server_information_version = {-1, "version", "s", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_get_server_information_spec_version = {-1, "spec_version", "s", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_get_server_information_in[] = {NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_get_server_information_out[] = {&_dino_dbus_notifications_dbus_arg_info_get_server_information_name, &_dino_dbus_notifications_dbus_arg_info_get_server_information_vendor, &_dino_dbus_notifications_dbus_arg_info_get_server_information_version, &_dino_dbus_notifications_dbus_arg_info_get_server_information_spec_version, NULL};
static const GDBusMethodInfo _dino_dbus_notifications_dbus_method_info_get_server_information = {-1, "GetServerInformation", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_get_server_information_in), (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_get_server_information_out), NULL};
static const GDBusMethodInfo * const _dino_dbus_notifications_dbus_method_info[] = {&_dino_dbus_notifications_dbus_method_info_notify, &_dino_dbus_notifications_dbus_method_info_get_capabilities, &_dino_dbus_notifications_dbus_method_info_close_notification, &_dino_dbus_notifications_dbus_method_info_get_server_information, NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_action_invoked_key = {-1, "key", "u", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_action_invoked_action_key = {-1, "action_key", "s", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_action_invoked[] = {&_dino_dbus_notifications_dbus_arg_info_action_invoked_key, &_dino_dbus_notifications_dbus_arg_info_action_invoked_action_key, NULL};
static const GDBusSignalInfo _dino_dbus_notifications_dbus_signal_info_action_invoked = {-1, "ActionInvoked", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_action_invoked), NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notification_closed_id = {-1, "id", "u", NULL};
static const GDBusArgInfo _dino_dbus_notifications_dbus_arg_info_notification_closed_reason = {-1, "reason", "u", NULL};
static const GDBusArgInfo * const _dino_dbus_notifications_dbus_arg_info_notification_closed[] = {&_dino_dbus_notifications_dbus_arg_info_notification_closed_id, &_dino_dbus_notifications_dbus_arg_info_notification_closed_reason, NULL};
static const GDBusSignalInfo _dino_dbus_notifications_dbus_signal_info_notification_closed = {-1, "NotificationClosed", (GDBusArgInfo **) (&_dino_dbus_notifications_dbus_arg_info_notification_closed), NULL};
static const GDBusSignalInfo * const _dino_dbus_notifications_dbus_signal_info[] = {&_dino_dbus_notifications_dbus_signal_info_action_invoked, &_dino_dbus_notifications_dbus_signal_info_notification_closed, NULL};
static const GDBusPropertyInfo * const _dino_dbus_notifications_dbus_property_info[] = {NULL};
static const GDBusInterfaceInfo _dino_dbus_notifications_dbus_interface_info = {-1, "org.freedesktop.Notifications", (GDBusMethodInfo **) (&_dino_dbus_notifications_dbus_method_info), (GDBusSignalInfo **) (&_dino_dbus_notifications_dbus_signal_info), (GDBusPropertyInfo **) (&_dino_dbus_notifications_dbus_property_info), NULL};
static const GDBusInterfaceVTable _dino_dbus_notifications_dbus_interface_vtable = {dino_dbus_notifications_dbus_interface_method_call, dino_dbus_notifications_dbus_interface_get_property, dino_dbus_notifications_dbus_interface_set_property};

void
dino_dbus_notifications_notify (DinoDBusNotifications* self,
                                const gchar* app_name,
                                guint32 replaces_id,
                                const gchar* app_icon,
                                const gchar* summary,
                                const gchar* body,
                                gchar** actions,
                                gint actions_length1,
                                GHashTable* hints,
                                gint32 expire_timeout,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->notify) {
		_iface_->notify (self, app_name, replaces_id, app_icon, summary, body, actions, actions_length1, hints, expire_timeout, _callback_, _user_data_);
	}
}

guint32
dino_dbus_notifications_notify_finish (DinoDBusNotifications* self,
                                       GAsyncResult* _res_,
                                       GError** error)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->notify_finish) {
		return _iface_->notify_finish (self, _res_, error);
	}
	return 0U;
}

void
dino_dbus_notifications_get_capabilities (DinoDBusNotifications* self,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->get_capabilities) {
		_iface_->get_capabilities (self, _callback_, _user_data_);
	}
}

void
dino_dbus_notifications_get_capabilities_finish (DinoDBusNotifications* self,
                                                 GAsyncResult* _res_,
                                                 gchar*** capabilities,
                                                 gint* capabilities_length1,
                                                 GError** error)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->get_capabilities_finish) {
		_iface_->get_capabilities_finish (self, _res_, capabilities, capabilities_length1, error);
	}
}

void
dino_dbus_notifications_close_notification (DinoDBusNotifications* self,
                                            guint id,
                                            GAsyncReadyCallback _callback_,
                                            gpointer _user_data_)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->close_notification) {
		_iface_->close_notification (self, id, _callback_, _user_data_);
	}
}

void
dino_dbus_notifications_close_notification_finish (DinoDBusNotifications* self,
                                                   GAsyncResult* _res_,
                                                   GError** error)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->close_notification_finish) {
		_iface_->close_notification_finish (self, _res_, error);
	}
}

void
dino_dbus_notifications_get_server_information (DinoDBusNotifications* self,
                                                GAsyncReadyCallback _callback_,
                                                gpointer _user_data_)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->get_server_information) {
		_iface_->get_server_information (self, _callback_, _user_data_);
	}
}

void
dino_dbus_notifications_get_server_information_finish (DinoDBusNotifications* self,
                                                       GAsyncResult* _res_,
                                                       gchar** name,
                                                       gchar** vendor,
                                                       gchar** version,
                                                       gchar** spec_version,
                                                       GError** error)
{
	DinoDBusNotificationsIface* _iface_;
	_iface_ = DINO_DBUS_NOTIFICATIONS_GET_INTERFACE (self);
	if (_iface_->get_server_information_finish) {
		_iface_->get_server_information_finish (self, _res_, name, vendor, version, spec_version, error);
	}
}

static void
g_cclosure_user_marshal_VOID__UINT_STRING (GClosure * closure,
                                           GValue * return_value,
                                           guint n_param_values,
                                           const GValue * param_values,
                                           gpointer invocation_hint,
                                           gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__UINT_STRING) (gpointer data1, guint32 arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__UINT_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__UINT_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_uint (param_values + 1), g_value_get_string (param_values + 2), data2);
}

static void
g_cclosure_user_marshal_VOID__UINT_UINT (GClosure * closure,
                                         GValue * return_value,
                                         guint n_param_values,
                                         const GValue * param_values,
                                         gpointer invocation_hint,
                                         gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__UINT_UINT) (gpointer data1, guint32 arg_1, guint32 arg_2, gpointer data2);
	register GMarshalFunc_VOID__UINT_UINT callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__UINT_UINT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_uint (param_values + 1), g_value_get_uint (param_values + 2), data2);
}

static void
dino_dbus_notifications_default_init (DinoDBusNotificationsIface * iface,
                                      gpointer iface_data)
{
	dino_dbus_notifications_signals[DINO_DBUS_NOTIFICATIONS_ACTION_INVOKED_SIGNAL] = g_signal_new ("action-invoked", DINO_TYPE_DBUS_NOTIFICATIONS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__UINT_STRING, G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_STRING);
	dino_dbus_notifications_signals[DINO_DBUS_NOTIFICATIONS_NOTIFICATION_CLOSED_SIGNAL] = g_signal_new ("notification-closed", DINO_TYPE_DBUS_NOTIFICATIONS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__UINT_UINT, G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
}

static GType
dino_dbus_notifications_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoDBusNotificationsIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_dbus_notifications_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType dino_dbus_notifications_type_id;
	dino_dbus_notifications_type_id = g_type_register_static (G_TYPE_INTERFACE, "DinoDBusNotifications", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (dino_dbus_notifications_type_id, G_TYPE_OBJECT);
	g_type_set_qdata (dino_dbus_notifications_type_id, g_quark_from_static_string ("vala-dbus-proxy-type"), (void*) dino_dbus_notifications_proxy_get_type);
	g_type_set_qdata (dino_dbus_notifications_type_id, g_quark_from_static_string ("vala-dbus-interface-name"), "org.freedesktop.Notifications");
	g_type_set_qdata (dino_dbus_notifications_type_id, g_quark_from_static_string ("vala-dbus-interface-info"), (void*) (&_dino_dbus_notifications_dbus_interface_info));
	g_type_set_qdata (dino_dbus_notifications_type_id, g_quark_from_static_string ("vala-dbus-register-object"), (void*) dino_dbus_notifications_register_object);
	return dino_dbus_notifications_type_id;
}

GType
dino_dbus_notifications_get_type (void)
{
	static volatile gsize dino_dbus_notifications_type_id__once = 0;
	if (g_once_init_enter (&dino_dbus_notifications_type_id__once)) {
		GType dino_dbus_notifications_type_id;
		dino_dbus_notifications_type_id = dino_dbus_notifications_get_type_once ();
		g_once_init_leave (&dino_dbus_notifications_type_id__once, dino_dbus_notifications_type_id);
	}
	return dino_dbus_notifications_type_id__once;
}

G_DEFINE_TYPE_EXTENDED (DinoDBusNotificationsProxy, dino_dbus_notifications_proxy, G_TYPE_DBUS_PROXY, 0, G_IMPLEMENT_INTERFACE (DINO_TYPE_DBUS_NOTIFICATIONS, dino_dbus_notifications_proxy_dino_dbus_notifications_interface_init) )
static void
dino_dbus_notifications_proxy_class_init (DinoDBusNotificationsProxyClass* klass)
{
	G_DBUS_PROXY_CLASS (klass)->g_signal = dino_dbus_notifications_proxy_g_signal;
}

static void
_dbus_handle_dino_dbus_notifications_action_invoked (DinoDBusNotifications* self,
                                                     GVariant* parameters)
{
	GVariantIter _arguments_iter;
	guint32 key = 0U;
	GVariant* _tmp1_;
	gchar* action_key = NULL;
	GVariant* _tmp2_;
	g_variant_iter_init (&_arguments_iter, parameters);
	_tmp1_ = g_variant_iter_next_value (&_arguments_iter);
	key = g_variant_get_uint32 (_tmp1_);
	g_variant_unref (_tmp1_);
	_tmp2_ = g_variant_iter_next_value (&_arguments_iter);
	action_key = g_variant_dup_string (_tmp2_, NULL);
	g_variant_unref (_tmp2_);
	g_signal_emit_by_name (self, "action-invoked", key, action_key);
	_g_free0 (action_key);
}

static void
_dbus_handle_dino_dbus_notifications_notification_closed (DinoDBusNotifications* self,
                                                          GVariant* parameters)
{
	GVariantIter _arguments_iter;
	guint32 id = 0U;
	GVariant* _tmp3_;
	guint32 reason = 0U;
	GVariant* _tmp4_;
	g_variant_iter_init (&_arguments_iter, parameters);
	_tmp3_ = g_variant_iter_next_value (&_arguments_iter);
	id = g_variant_get_uint32 (_tmp3_);
	g_variant_unref (_tmp3_);
	_tmp4_ = g_variant_iter_next_value (&_arguments_iter);
	reason = g_variant_get_uint32 (_tmp4_);
	g_variant_unref (_tmp4_);
	g_signal_emit_by_name (self, "notification-closed", id, reason);
}

static void
dino_dbus_notifications_proxy_g_signal (GDBusProxy* proxy,
                                        const gchar* sender_name,
                                        const gchar* signal_name,
                                        GVariant* parameters)
{
	if (strcmp (signal_name, "ActionInvoked") == 0) {
		_dbus_handle_dino_dbus_notifications_action_invoked ((DinoDBusNotifications*) proxy, parameters);
	} else if (strcmp (signal_name, "NotificationClosed") == 0) {
		_dbus_handle_dino_dbus_notifications_notification_closed ((DinoDBusNotifications*) proxy, parameters);
	}
}

static void
dino_dbus_notifications_proxy_init (DinoDBusNotificationsProxy* self)
{
	g_dbus_proxy_set_interface_info (G_DBUS_PROXY (self), (GDBusInterfaceInfo *) (&_dino_dbus_notifications_dbus_interface_info));
}

static void
_vala_g_async_ready_callback (GObject *source_object,
                              GAsyncResult *res,
                              void *user_data)
{
	g_task_return_pointer (user_data, g_object_ref (res), g_object_unref);
	g_object_unref (user_data);
}

static void
dino_dbus_notifications_proxy_notify_async (DinoDBusNotifications* self,
                                            const gchar* app_name,
                                            guint32 replaces_id,
                                            const gchar* app_icon,
                                            const gchar* summary,
                                            const gchar* body,
                                            gchar** actions,
                                            gint actions_length1,
                                            GHashTable* hints,
                                            gint32 expire_timeout,
                                            GAsyncReadyCallback _callback_,
                                            gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	gchar** _tmp5_;
	GVariantBuilder _tmp6_;
	gint _tmp7_;
	GVariantBuilder _tmp8_;
	GHashTableIter _tmp9_;
	gpointer _tmp10_;
	gpointer _tmp11_;
	G_DBUS_ERROR;
	G_IO_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.Notifications", "Notify");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (app_name));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_uint32 (replaces_id));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (app_icon));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (summary));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (body));
	_tmp5_ = actions;
	g_variant_builder_init (&_tmp6_, G_VARIANT_TYPE ("as"));
	for (_tmp7_ = 0; _tmp7_ < actions_length1; _tmp7_++) {
		g_variant_builder_add_value (&_tmp6_, g_variant_new_string (*_tmp5_));
		_tmp5_++;
	}
	g_variant_builder_add_value (&_arguments_builder, g_variant_builder_end (&_tmp6_));
	g_hash_table_iter_init (&_tmp9_, hints);
	g_variant_builder_init (&_tmp8_, G_VARIANT_TYPE ("a{sv}"));
	while (g_hash_table_iter_next (&_tmp9_, &_tmp10_, &_tmp11_)) {
		gchar* _key;
		GVariant* _value;
		_key = (gchar*) _tmp10_;
		_value = (GVariant*) _tmp11_;
		g_variant_builder_add (&_tmp8_, "{?*}", g_variant_new_string (_key), g_variant_new_variant (_value));
	}
	g_variant_builder_add_value (&_arguments_builder, g_variant_builder_end (&_tmp8_));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_int32 (expire_timeout));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static guint32
dino_dbus_notifications_proxy_notify_finish (DinoDBusNotifications* self,
                                             GAsyncResult* _res_,
                                             GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	guint32 _result = 0U;
	GVariant* _tmp14_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		guint32 _tmp12_ = 0U;
		return _tmp12_;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		guint32 _tmp13_ = 0U;
		g_object_unref (_reply_message);
		return _tmp13_;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_tmp14_ = g_variant_iter_next_value (&_reply_iter);
	_result = g_variant_get_uint32 (_tmp14_);
	g_variant_unref (_tmp14_);
	g_object_unref (_reply_message);
	return _result;
}

static void
dino_dbus_notifications_proxy_get_capabilities_async (DinoDBusNotifications* self,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.Notifications", "GetCapabilities");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static void
dino_dbus_notifications_proxy_get_capabilities_finish (DinoDBusNotifications* self,
                                                       GAsyncResult* _res_,
                                                       gchar*** capabilities,
                                                       gint* capabilities_length1,
                                                       GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	gchar** _vala_capabilities = NULL;
	gint _vala_capabilities_length1;
	GVariant* _tmp15_;
	gchar** _tmp16_;
	gint _tmp16__length;
	gint _tmp16__size;
	gint _tmp16__length1;
	GVariantIter _tmp17_;
	GVariant* _tmp18_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_vala_capabilities_length1 = 0;
	_tmp15_ = g_variant_iter_next_value (&_reply_iter);
	_tmp16_ = g_new (gchar*, 5);
	_tmp16__length = 0;
	_tmp16__size = 4;
	_tmp16__length1 = 0;
	g_variant_iter_init (&_tmp17_, _tmp15_);
	for (; (_tmp18_ = g_variant_iter_next_value (&_tmp17_)) != NULL; _tmp16__length1++) {
		if (_tmp16__size == _tmp16__length) {
			_tmp16__size = 2 * _tmp16__size;
			_tmp16_ = g_renew (gchar*, _tmp16_, _tmp16__size + 1);
		}
		_tmp16_[_tmp16__length++] = g_variant_dup_string (_tmp18_, NULL);
		g_variant_unref (_tmp18_);
	}
	_vala_capabilities_length1 = _tmp16__length1;
	_tmp16_[_tmp16__length] = NULL;
	_vala_capabilities = _tmp16_;
	g_variant_unref (_tmp15_);
	*capabilities = _vala_capabilities;
	*capabilities_length1 = _vala_capabilities_length1;
	g_object_unref (_reply_message);
}

static void
dino_dbus_notifications_proxy_close_notification_async (DinoDBusNotifications* self,
                                                        guint id,
                                                        GAsyncReadyCallback _callback_,
                                                        gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	G_IO_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.Notifications", "CloseNotification");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_uint32 (id));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static void
dino_dbus_notifications_proxy_close_notification_finish (DinoDBusNotifications* self,
                                                         GAsyncResult* _res_,
                                                         GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return;
	}
	g_object_unref (_reply_message);
}

static void
dino_dbus_notifications_proxy_get_server_information_async (DinoDBusNotifications* self,
                                                            GAsyncReadyCallback _callback_,
                                                            gpointer _user_data_)
{
	GDBusMessage *_message;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	G_DBUS_ERROR;
	G_IO_ERROR;
	_message = g_dbus_message_new_method_call (g_dbus_proxy_get_name ((GDBusProxy *) self), g_dbus_proxy_get_object_path ((GDBusProxy *) self), "org.freedesktop.Notifications", "GetServerInformation");
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_message_set_body (_message, _arguments);
	if (_callback_ != NULL) {
		g_dbus_connection_send_message_with_reply (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, g_dbus_proxy_get_default_timeout ((GDBusProxy *) self), NULL, NULL, _vala_g_async_ready_callback, g_task_new ((GObject *) self, NULL, _callback_, _user_data_));
	} else {
		g_dbus_message_set_flags (_message, G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED);
		g_dbus_connection_send_message (g_dbus_proxy_get_connection ((GDBusProxy *) self), _message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	}
	g_object_unref (_message);
}

static void
dino_dbus_notifications_proxy_get_server_information_finish (DinoDBusNotifications* self,
                                                             GAsyncResult* _res_,
                                                             gchar** name,
                                                             gchar** vendor,
                                                             gchar** version,
                                                             gchar** spec_version,
                                                             GError** error)
{
	GAsyncResult *_inner_res;
	GDBusMessage *_reply_message;
	GVariant *_reply;
	GVariantIter _reply_iter;
	gchar* _vala_name = NULL;
	GVariant* _tmp19_;
	gchar* _vala_vendor = NULL;
	GVariant* _tmp20_;
	gchar* _vala_version = NULL;
	GVariant* _tmp21_;
	gchar* _vala_spec_version = NULL;
	GVariant* _tmp22_;
	_inner_res = g_task_propagate_pointer ((GTask *) _res_, NULL);
	_reply_message = g_dbus_connection_send_message_with_reply_finish (g_dbus_proxy_get_connection ((GDBusProxy *) self), _inner_res, error);
	g_object_unref (_inner_res);
	if (!_reply_message) {
		return;
	}
	if (g_dbus_message_to_gerror (_reply_message, error)) {
		g_object_unref (_reply_message);
		return;
	}
	_reply = g_dbus_message_get_body (_reply_message);
	g_variant_iter_init (&_reply_iter, _reply);
	_tmp19_ = g_variant_iter_next_value (&_reply_iter);
	_vala_name = g_variant_dup_string (_tmp19_, NULL);
	g_variant_unref (_tmp19_);
	*name = _vala_name;
	_tmp20_ = g_variant_iter_next_value (&_reply_iter);
	_vala_vendor = g_variant_dup_string (_tmp20_, NULL);
	g_variant_unref (_tmp20_);
	*vendor = _vala_vendor;
	_tmp21_ = g_variant_iter_next_value (&_reply_iter);
	_vala_version = g_variant_dup_string (_tmp21_, NULL);
	g_variant_unref (_tmp21_);
	*version = _vala_version;
	_tmp22_ = g_variant_iter_next_value (&_reply_iter);
	_vala_spec_version = g_variant_dup_string (_tmp22_, NULL);
	g_variant_unref (_tmp22_);
	*spec_version = _vala_spec_version;
	g_object_unref (_reply_message);
}

static void
dino_dbus_notifications_proxy_dino_dbus_notifications_interface_init (DinoDBusNotificationsIface* iface)
{
	iface->notify = dino_dbus_notifications_proxy_notify_async;
	iface->notify_finish = dino_dbus_notifications_proxy_notify_finish;
	iface->get_capabilities = dino_dbus_notifications_proxy_get_capabilities_async;
	iface->get_capabilities_finish = dino_dbus_notifications_proxy_get_capabilities_finish;
	iface->close_notification = dino_dbus_notifications_proxy_close_notification_async;
	iface->close_notification_finish = dino_dbus_notifications_proxy_close_notification_finish;
	iface->get_server_information = dino_dbus_notifications_proxy_get_server_information_async;
	iface->get_server_information_finish = dino_dbus_notifications_proxy_get_server_information_finish;
}

static void
_dbus_dino_dbus_notifications_notify (DinoDBusNotifications* self,
                                      GVariant* _parameters_,
                                      GDBusMethodInvocation* invocation)
{
	GVariantIter _arguments_iter;
	DinoDbusNotificationsNotifyReadyData* _ready_data;
	GVariant* _tmp23_;
	GVariant* _tmp24_;
	GVariant* _tmp25_;
	GVariant* _tmp26_;
	GVariant* _tmp27_;
	GVariant* _tmp28_;
	gchar** _tmp29_;
	gint _tmp29__length;
	gint _tmp29__size;
	gint _tmp29__length1;
	GVariantIter _tmp30_;
	GVariant* _tmp31_;
	GVariant* _tmp32_;
	GHashTable* _tmp33_;
	GVariantIter _tmp34_;
	GVariant* _tmp35_;
	GVariant* _tmp36_;
	GVariant* _tmp37_;
	g_variant_iter_init (&_arguments_iter, _parameters_);
	_ready_data = g_slice_new0 (DinoDbusNotificationsNotifyReadyData);
	_ready_data->_invocation_ = invocation;
	_tmp23_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->app_name = g_variant_dup_string (_tmp23_, NULL);
	g_variant_unref (_tmp23_);
	_tmp24_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->replaces_id = g_variant_get_uint32 (_tmp24_);
	g_variant_unref (_tmp24_);
	_tmp25_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->app_icon = g_variant_dup_string (_tmp25_, NULL);
	g_variant_unref (_tmp25_);
	_tmp26_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->summary = g_variant_dup_string (_tmp26_, NULL);
	g_variant_unref (_tmp26_);
	_tmp27_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->body = g_variant_dup_string (_tmp27_, NULL);
	g_variant_unref (_tmp27_);
	_tmp28_ = g_variant_iter_next_value (&_arguments_iter);
	_tmp29_ = g_new (gchar*, 5);
	_tmp29__length = 0;
	_tmp29__size = 4;
	_tmp29__length1 = 0;
	g_variant_iter_init (&_tmp30_, _tmp28_);
	for (; (_tmp31_ = g_variant_iter_next_value (&_tmp30_)) != NULL; _tmp29__length1++) {
		if (_tmp29__size == _tmp29__length) {
			_tmp29__size = 2 * _tmp29__size;
			_tmp29_ = g_renew (gchar*, _tmp29_, _tmp29__size + 1);
		}
		_tmp29_[_tmp29__length++] = g_variant_dup_string (_tmp31_, NULL);
		g_variant_unref (_tmp31_);
	}
	_ready_data->actions_length1 = _tmp29__length1;
	_tmp29_[_tmp29__length] = NULL;
	_ready_data->actions = _tmp29_;
	g_variant_unref (_tmp28_);
	_tmp32_ = g_variant_iter_next_value (&_arguments_iter);
	_tmp33_ = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_variant_unref);
	g_variant_iter_init (&_tmp34_, _tmp32_);
	while (g_variant_iter_loop (&_tmp34_, "{?*}", &_tmp35_, &_tmp36_)) {
		g_hash_table_insert (_tmp33_, g_variant_dup_string (_tmp35_, NULL), g_variant_get_variant (_tmp36_));
	}
	_ready_data->hints = _tmp33_;
	g_variant_unref (_tmp32_);
	_tmp37_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->expire_timeout = g_variant_get_int32 (_tmp37_);
	g_variant_unref (_tmp37_);
	dino_dbus_notifications_notify (self, _ready_data->app_name, _ready_data->replaces_id, _ready_data->app_icon, _ready_data->summary, _ready_data->body, _ready_data->actions, _ready_data->actions_length1, _ready_data->hints, _ready_data->expire_timeout, (GAsyncReadyCallback) _dbus_dino_dbus_notifications_notify_ready, _ready_data);
	_error:
	;
}

static void
_dbus_dino_dbus_notifications_notify_ready (GObject * source_object,
                                            GAsyncResult * _res_,
                                            gpointer _user_data_)
{
	DinoDbusNotificationsNotifyReadyData* _ready_data;
	GDBusMethodInvocation* invocation;
	GError* error = NULL;
	GDBusMessage* _reply_message = NULL;
	GVariant* _reply;
	GVariantBuilder _reply_builder;
	guint32 result;
	_ready_data = _user_data_;
	invocation = _ready_data->_invocation_;
	result = dino_dbus_notifications_notify_finish ((DinoDBusNotifications*) source_object, _res_, &error);
	if (error) {
		g_dbus_method_invocation_take_error (invocation, error);
		goto _error;
	}
	_reply_message = g_dbus_message_new_method_reply (g_dbus_method_invocation_get_message (invocation));
	g_variant_builder_init (&_reply_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_reply_builder, g_variant_new_uint32 (result));
	_reply = g_variant_builder_end (&_reply_builder);
	g_dbus_message_set_body (_reply_message, _reply);
	g_dbus_connection_send_message (g_dbus_method_invocation_get_connection (invocation), _reply_message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	g_object_unref (invocation);
	g_object_unref (_reply_message);
	_error:
	_g_free0 (_ready_data->app_name);
	_g_free0 (_ready_data->app_icon);
	_g_free0 (_ready_data->summary);
	_g_free0 (_ready_data->body);
	_ready_data->actions = (_vala_array_free (_ready_data->actions, _ready_data->actions_length1, (GDestroyNotify) g_free), NULL);
	_g_hash_table_unref0 (_ready_data->hints);
	g_slice_free (DinoDbusNotificationsNotifyReadyData, _ready_data);
}

static void
_dbus_dino_dbus_notifications_get_capabilities (DinoDBusNotifications* self,
                                                GVariant* _parameters_,
                                                GDBusMethodInvocation* invocation)
{
	GVariantIter _arguments_iter;
	DinoDbusNotificationsGetCapabilitiesReadyData* _ready_data;
	g_variant_iter_init (&_arguments_iter, _parameters_);
	_ready_data = g_slice_new0 (DinoDbusNotificationsGetCapabilitiesReadyData);
	_ready_data->_invocation_ = invocation;
	dino_dbus_notifications_get_capabilities (self, (GAsyncReadyCallback) _dbus_dino_dbus_notifications_get_capabilities_ready, _ready_data);
}

static void
_dbus_dino_dbus_notifications_get_capabilities_ready (GObject * source_object,
                                                      GAsyncResult * _res_,
                                                      gpointer _user_data_)
{
	DinoDbusNotificationsGetCapabilitiesReadyData* _ready_data;
	GDBusMethodInvocation* invocation;
	GError* error = NULL;
	GDBusMessage* _reply_message = NULL;
	GVariant* _reply;
	GVariantBuilder _reply_builder;
	gchar** capabilities = NULL;
	gint capabilities_length1 = 0;
	gchar** _tmp38_;
	GVariantBuilder _tmp39_;
	gint _tmp40_;
	_ready_data = _user_data_;
	invocation = _ready_data->_invocation_;
	dino_dbus_notifications_get_capabilities_finish ((DinoDBusNotifications*) source_object, _res_, &capabilities, &capabilities_length1, &error);
	if (error) {
		g_dbus_method_invocation_take_error (invocation, error);
		goto _error;
	}
	_reply_message = g_dbus_message_new_method_reply (g_dbus_method_invocation_get_message (invocation));
	g_variant_builder_init (&_reply_builder, G_VARIANT_TYPE_TUPLE);
	_tmp38_ = capabilities;
	g_variant_builder_init (&_tmp39_, G_VARIANT_TYPE ("as"));
	for (_tmp40_ = 0; _tmp40_ < capabilities_length1; _tmp40_++) {
		g_variant_builder_add_value (&_tmp39_, g_variant_new_string (*_tmp38_));
		_tmp38_++;
	}
	g_variant_builder_add_value (&_reply_builder, g_variant_builder_end (&_tmp39_));
	_reply = g_variant_builder_end (&_reply_builder);
	g_dbus_message_set_body (_reply_message, _reply);
	g_dbus_connection_send_message (g_dbus_method_invocation_get_connection (invocation), _reply_message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	g_object_unref (invocation);
	g_object_unref (_reply_message);
	_error:
	capabilities = (_vala_array_free (capabilities, capabilities_length1, (GDestroyNotify) g_free), NULL);
	g_slice_free (DinoDbusNotificationsGetCapabilitiesReadyData, _ready_data);
}

static void
_dbus_dino_dbus_notifications_close_notification (DinoDBusNotifications* self,
                                                  GVariant* _parameters_,
                                                  GDBusMethodInvocation* invocation)
{
	GVariantIter _arguments_iter;
	DinoDbusNotificationsCloseNotificationReadyData* _ready_data;
	GVariant* _tmp41_;
	g_variant_iter_init (&_arguments_iter, _parameters_);
	_ready_data = g_slice_new0 (DinoDbusNotificationsCloseNotificationReadyData);
	_ready_data->_invocation_ = invocation;
	_tmp41_ = g_variant_iter_next_value (&_arguments_iter);
	_ready_data->id = g_variant_get_uint32 (_tmp41_);
	g_variant_unref (_tmp41_);
	dino_dbus_notifications_close_notification (self, _ready_data->id, (GAsyncReadyCallback) _dbus_dino_dbus_notifications_close_notification_ready, _ready_data);
}

static void
_dbus_dino_dbus_notifications_close_notification_ready (GObject * source_object,
                                                        GAsyncResult * _res_,
                                                        gpointer _user_data_)
{
	DinoDbusNotificationsCloseNotificationReadyData* _ready_data;
	GDBusMethodInvocation* invocation;
	GError* error = NULL;
	GDBusMessage* _reply_message = NULL;
	GVariant* _reply;
	GVariantBuilder _reply_builder;
	_ready_data = _user_data_;
	invocation = _ready_data->_invocation_;
	dino_dbus_notifications_close_notification_finish ((DinoDBusNotifications*) source_object, _res_, &error);
	if (error) {
		g_dbus_method_invocation_take_error (invocation, error);
		goto _error;
	}
	_reply_message = g_dbus_message_new_method_reply (g_dbus_method_invocation_get_message (invocation));
	g_variant_builder_init (&_reply_builder, G_VARIANT_TYPE_TUPLE);
	_reply = g_variant_builder_end (&_reply_builder);
	g_dbus_message_set_body (_reply_message, _reply);
	g_dbus_connection_send_message (g_dbus_method_invocation_get_connection (invocation), _reply_message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	g_object_unref (invocation);
	g_object_unref (_reply_message);
	_error:
	g_slice_free (DinoDbusNotificationsCloseNotificationReadyData, _ready_data);
}

static void
_dbus_dino_dbus_notifications_get_server_information (DinoDBusNotifications* self,
                                                      GVariant* _parameters_,
                                                      GDBusMethodInvocation* invocation)
{
	GVariantIter _arguments_iter;
	DinoDbusNotificationsGetServerInformationReadyData* _ready_data;
	g_variant_iter_init (&_arguments_iter, _parameters_);
	_ready_data = g_slice_new0 (DinoDbusNotificationsGetServerInformationReadyData);
	_ready_data->_invocation_ = invocation;
	dino_dbus_notifications_get_server_information (self, (GAsyncReadyCallback) _dbus_dino_dbus_notifications_get_server_information_ready, _ready_data);
}

static void
_dbus_dino_dbus_notifications_get_server_information_ready (GObject * source_object,
                                                            GAsyncResult * _res_,
                                                            gpointer _user_data_)
{
	DinoDbusNotificationsGetServerInformationReadyData* _ready_data;
	GDBusMethodInvocation* invocation;
	GError* error = NULL;
	GDBusMessage* _reply_message = NULL;
	GVariant* _reply;
	GVariantBuilder _reply_builder;
	gchar* name = NULL;
	gchar* vendor = NULL;
	gchar* version = NULL;
	gchar* spec_version = NULL;
	_ready_data = _user_data_;
	invocation = _ready_data->_invocation_;
	dino_dbus_notifications_get_server_information_finish ((DinoDBusNotifications*) source_object, _res_, &name, &vendor, &version, &spec_version, &error);
	if (error) {
		g_dbus_method_invocation_take_error (invocation, error);
		goto _error;
	}
	_reply_message = g_dbus_message_new_method_reply (g_dbus_method_invocation_get_message (invocation));
	g_variant_builder_init (&_reply_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_reply_builder, g_variant_new_string (name));
	g_variant_builder_add_value (&_reply_builder, g_variant_new_string (vendor));
	g_variant_builder_add_value (&_reply_builder, g_variant_new_string (version));
	g_variant_builder_add_value (&_reply_builder, g_variant_new_string (spec_version));
	_reply = g_variant_builder_end (&_reply_builder);
	g_dbus_message_set_body (_reply_message, _reply);
	g_dbus_connection_send_message (g_dbus_method_invocation_get_connection (invocation), _reply_message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, NULL, NULL);
	g_object_unref (invocation);
	g_object_unref (_reply_message);
	_error:
	_g_free0 (name);
	_g_free0 (vendor);
	_g_free0 (version);
	_g_free0 (spec_version);
	g_slice_free (DinoDbusNotificationsGetServerInformationReadyData, _ready_data);
}

static void
dino_dbus_notifications_dbus_interface_method_call (GDBusConnection* connection,
                                                    const gchar* sender,
                                                    const gchar* object_path,
                                                    const gchar* interface_name,
                                                    const gchar* method_name,
                                                    GVariant* parameters,
                                                    GDBusMethodInvocation* invocation,
                                                    gpointer user_data)
{
	gpointer* data;
	gpointer object;
	data = user_data;
	object = data[0];
	if (strcmp (method_name, "Notify") == 0) {
		_dbus_dino_dbus_notifications_notify (object, parameters, invocation);
	} else if (strcmp (method_name, "GetCapabilities") == 0) {
		_dbus_dino_dbus_notifications_get_capabilities (object, parameters, invocation);
	} else if (strcmp (method_name, "CloseNotification") == 0) {
		_dbus_dino_dbus_notifications_close_notification (object, parameters, invocation);
	} else if (strcmp (method_name, "GetServerInformation") == 0) {
		_dbus_dino_dbus_notifications_get_server_information (object, parameters, invocation);
	} else {
		g_object_unref (invocation);
	}
}

static GVariant*
dino_dbus_notifications_dbus_interface_get_property (GDBusConnection* connection,
                                                     const gchar* sender,
                                                     const gchar* object_path,
                                                     const gchar* interface_name,
                                                     const gchar* property_name,
                                                     GError** error,
                                                     gpointer user_data)
{
	return NULL;
}

static gboolean
dino_dbus_notifications_dbus_interface_set_property (GDBusConnection* connection,
                                                     const gchar* sender,
                                                     const gchar* object_path,
                                                     const gchar* interface_name,
                                                     const gchar* property_name,
                                                     GVariant* value,
                                                     GError** error,
                                                     gpointer user_data)
{
	return FALSE;
}

static void
_dbus_dino_dbus_notifications_action_invoked (GObject* _sender,
                                              guint32 key,
                                              const gchar* action_key,
                                              gpointer* _data)
{
	GDBusConnection * _connection;
	const gchar * _path;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	_connection = _data[1];
	_path = _data[2];
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_uint32 (key));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_string (action_key));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_connection_emit_signal (_connection, NULL, _path, "org.freedesktop.Notifications", "ActionInvoked", _arguments, NULL);
}

static void
_dbus_dino_dbus_notifications_notification_closed (GObject* _sender,
                                                   guint32 id,
                                                   guint32 reason,
                                                   gpointer* _data)
{
	GDBusConnection * _connection;
	const gchar * _path;
	GVariant *_arguments;
	GVariantBuilder _arguments_builder;
	_connection = _data[1];
	_path = _data[2];
	g_variant_builder_init (&_arguments_builder, G_VARIANT_TYPE_TUPLE);
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_uint32 (id));
	g_variant_builder_add_value (&_arguments_builder, g_variant_new_uint32 (reason));
	_arguments = g_variant_builder_end (&_arguments_builder);
	g_dbus_connection_emit_signal (_connection, NULL, _path, "org.freedesktop.Notifications", "NotificationClosed", _arguments, NULL);
}

guint
dino_dbus_notifications_register_object (gpointer object,
                                         GDBusConnection* connection,
                                         const gchar* path,
                                         GError** error)
{
	guint result;
	gpointer *data;
	data = g_new (gpointer, 3);
	data[0] = g_object_ref (object);
	data[1] = g_object_ref (connection);
	data[2] = g_strdup (path);
	result = g_dbus_connection_register_object (connection, path, (GDBusInterfaceInfo *) (&_dino_dbus_notifications_dbus_interface_info), &_dino_dbus_notifications_dbus_interface_vtable, data, _dino_dbus_notifications_unregister_object, error);
	if (!result) {
		return 0;
	}
	g_signal_connect (object, "action-invoked", (GCallback) _dbus_dino_dbus_notifications_action_invoked, data);
	g_signal_connect (object, "notification-closed", (GCallback) _dbus_dino_dbus_notifications_notification_closed, data);
	return result;
}

static void
_dino_dbus_notifications_unregister_object (gpointer user_data)
{
	gpointer* data;
	data = user_data;
	g_signal_handlers_disconnect_by_func (data[0], _dbus_dino_dbus_notifications_action_invoked, data);
	g_signal_handlers_disconnect_by_func (data[0], _dbus_dino_dbus_notifications_notification_closed, data);
	g_object_unref (data[0]);
	g_object_unref (data[1]);
	g_free (data[2]);
	g_free (data);
}

static void
dino_get_notifications_dbus_data_free (gpointer _data)
{
	DinoGetNotificationsDbusData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->result);
	g_slice_free (DinoGetNotificationsDbusData, _data_);
}

void
dino_get_notifications_dbus (GAsyncReadyCallback _callback_,
                             gpointer _user_data_)
{
	DinoGetNotificationsDbusData* _data_;
	_data_ = g_slice_new0 (DinoGetNotificationsDbusData);
	_data_->_async_result = g_task_new (NULL, NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_get_notifications_dbus_data_free);
	dino_get_notifications_dbus_co (_data_);
}

DinoDBusNotifications*
dino_get_notifications_dbus_finish (GAsyncResult* _res_)
{
	DinoDBusNotifications* result;
	DinoGetNotificationsDbusData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
	result = _data_->result;
	_data_->result = NULL;
	return result;
}

static void
dino_get_notifications_dbus_ready (GObject* source_object,
                                   GAsyncResult* _res_,
                                   gpointer _user_data_)
{
	DinoGetNotificationsDbusData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_get_notifications_dbus_co (_data_);
}

static gboolean
dino_get_notifications_dbus_co (DinoGetNotificationsDbusData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	{
		_data_->_state_ = 1;
		g_async_initable_new_async (DINO_TYPE_DBUS_NOTIFICATIONS_PROXY, 0, NULL, dino_get_notifications_dbus_ready, _data_, "g-flags", 0, "g-name", "org.freedesktop.Notifications", "g-bus-type", G_BUS_TYPE_SESSION, "g-object-path", "/org/freedesktop/Notifications", "g-interface-name", "org.freedesktop.Notifications", NULL);
		return FALSE;
		_state_1:
		_data_->_tmp1_ = (DinoDBusNotifications*) g_async_initable_new_finish ((GAsyncInitable *) _data_->_source_object_, _data_->_res_, &_data_->_inner_error0_);
		_data_->_tmp0_ = (DinoDBusNotifications*) _data_->_tmp1_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			if (_data_->_inner_error0_->domain == G_IO_ERROR) {
				goto __catch0_g_io_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
			g_clear_error (&_data_->_inner_error0_);
			g_object_unref (_data_->_async_result);
			return FALSE;
		}
		_data_->_tmp2_ = _data_->_tmp0_;
		_data_->_tmp0_ = NULL;
		_data_->result = _data_->_tmp2_;
		_g_object_unref0 (_data_->_tmp0_);
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	goto __finally0;
	__catch0_g_io_error:
	{
		_data_->e = _data_->_inner_error0_;
		_data_->_inner_error0_ = NULL;
		_data_->_tmp3_ = _data_->e;
		_data_->_tmp4_ = _data_->_tmp3_->message;
		g_warning ("notifications.vala:24: Couldn't get org.freedesktop.Notifications DBus" \
" instance: %s\n", _data_->_tmp4_);
		_g_error_free0 (_data_->e);
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->result = NULL;
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

