/* encryption.c generated by valac 0.56.19, the Vala compiler
 * generated from encryption.vala, do not modify */

#include "libdino.h"
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

gboolean
dino_entities_encryption_is_some (DinoEntitiesEncryption self)
{
	gboolean result;
	result = self != DINO_ENTITIES_ENCRYPTION_NONE;
	return result;
}

static GType
dino_entities_encryption_get_type_once (void)
{
	static const GEnumValue values[] = {{DINO_ENTITIES_ENCRYPTION_NONE, "DINO_ENTITIES_ENCRYPTION_NONE", "none"}, {DINO_ENTITIES_ENCRYPTION_PGP, "DINO_ENTITIES_ENCRYPTION_PGP", "pgp"}, {DINO_ENTITIES_ENCRYPTION_OMEMO, "DINO_ENTITIES_ENCRYPTION_OMEMO", "omemo"}, {DINO_ENTITIES_ENCRYPTION_DTLS_SRTP, "DINO_ENTITIES_ENCRYPTION_DTLS_SRTP", "dtls-srtp"}, {DINO_ENTITIES_ENCRYPTION_SRTP, "DINO_ENTITIES_ENCRYPTION_SRTP", "srtp"}, {DINO_ENTITIES_ENCRYPTION_UNKNOWN, "DINO_ENTITIES_ENCRYPTION_UNKNOWN", "unknown"}, {0, NULL, NULL}};
	GType dino_entities_encryption_type_id;
	dino_entities_encryption_type_id = g_enum_register_static ("DinoEntitiesEncryption", values);
	return dino_entities_encryption_type_id;
}

GType
dino_entities_encryption_get_type (void)
{
	static volatile gsize dino_entities_encryption_type_id__once = 0;
	if (g_once_init_enter (&dino_entities_encryption_type_id__once)) {
		GType dino_entities_encryption_type_id;
		dino_entities_encryption_type_id = dino_entities_encryption_get_type_once ();
		g_once_init_leave (&dino_entities_encryption_type_id__once, dino_entities_encryption_type_id);
	}
	return dino_entities_encryption_type_id__once;
}

