/* module.c generated by valac 0.56.19, the Vala compiler
 * generated from module.vala, do not modify */

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gio/gio.h>
#include "xmpp-vala.h"
#include <glib-object.h>

#define XMPP_XEP_VCARD_NS_URI "vcard-temp"
#define XMPP_XEP_VCARD_NS_URI_UPDATE XMPP_XEP_VCARD_NS_URI ":x:update"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _xmpp_jid_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_jid_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_bytes_unref0(var) ((var == NULL) ? NULL : (var = (g_bytes_unref (var), NULL)))
#define _xmpp_stanza_entry_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_stanza_entry_unref (var), NULL)))
typedef struct _XmppXepVcardFetchImageData XmppXepVcardFetchImageData;
enum  {
	XMPP_XEP_VCARD_MODULE_0_PROPERTY,
	XMPP_XEP_VCARD_MODULE_NUM_PROPERTIES
};
static GParamSpec* xmpp_xep_vcard_module_properties[XMPP_XEP_VCARD_MODULE_NUM_PROPERTIES];
enum  {
	XMPP_XEP_VCARD_MODULE_RECEIVED_AVATAR_HASH_SIGNAL,
	XMPP_XEP_VCARD_MODULE_NUM_SIGNALS
};
static guint xmpp_xep_vcard_module_signals[XMPP_XEP_VCARD_MODULE_NUM_SIGNALS] = {0};

struct _XmppXepVcardFetchImageData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	XmppXmppStream* stream;
	XmppJid* jid;
	gchar* hash;
	GBytes* result;
	XmppIqStanza* iq;
	XmppStanzaNode* _tmp0_;
	XmppStanzaNode* _tmp1_;
	XmppStanzaNode* _tmp2_;
	XmppStanzaNode* _tmp3_;
	XmppJid* _tmp4_;
	XmppIqStanza* _tmp5_;
	XmppIqStanza* _tmp6_;
	XmppIqStanza* iq_res;
	XmppModuleIdentity* _tmp7_;
	gpointer _tmp8_;
	XmppIqModule* _tmp9_;
	XmppIqStanza* _tmp10_;
	XmppIqStanza* _tmp11_;
	XmppIqStanza* _tmp12_;
	XmppIqStanza* _tmp13_;
	gchar* res;
	XmppIqStanza* _tmp14_;
	XmppStanzaNode* _tmp15_;
	const gchar* _tmp16_;
	gchar* _tmp17_;
	gchar* _tmp18_;
	const gchar* _tmp19_;
	gchar* _tmp20_;
	gchar* _tmp21_;
	const gchar* _tmp22_;
	GBytes* content;
	const gchar* _tmp23_;
	gsize _tmp24_;
	guchar* _tmp25_;
	GBytes* _tmp26_;
	gchar* sha1;
	GBytes* _tmp27_;
	gchar* _tmp28_;
	const gchar* _tmp29_;
	GError* _inner_error0_;
};

static gpointer xmpp_xep_vcard_module_parent_class = NULL;
XmppModuleIdentity* xmpp_xep_vcard_module_IDENTITY = NULL;

static void xmpp_xep_vcard_fetch_image_data_free (gpointer _data);
static gboolean xmpp_xep_vcard_fetch_image_co (XmppXepVcardFetchImageData* _data_);
static void xmpp_xep_vcard_fetch_image_ready (GObject* source_object,
                                       GAsyncResult* _res_,
                                       gpointer _user_data_);
static void xmpp_xep_vcard_module_real_attach (XmppXmppStreamModule* base,
                                        XmppXmppStream* stream);
static void xmpp_xep_vcard_module_on_received_presence (XmppXepVCardModule* self,
                                                 XmppXmppStream* stream,
                                                 XmppPresenceStanza* presence);
static void _xmpp_xep_vcard_module_on_received_presence_xmpp_presence_module_received_presence (XmppPresenceModule* _sender,
                                                                                         XmppXmppStream* stream,
                                                                                         XmppPresenceStanza* presence,
                                                                                         gpointer self);
static void xmpp_xep_vcard_module_real_detach (XmppXmppStreamModule* base,
                                        XmppXmppStream* stream);
static gchar* xmpp_xep_vcard_module_real_get_ns (XmppXmppStreamModule* base);
static gchar* xmpp_xep_vcard_module_real_get_id (XmppXmppStreamModule* base);
static void g_cclosure_user_marshal_VOID__OBJECT_XMPP_JID_STRING (GClosure * closure,
                                                           GValue * return_value,
                                                           guint n_param_values,
                                                           const GValue * param_values,
                                                           gpointer invocation_hint,
                                                           gpointer marshal_data);
static void xmpp_xep_vcard_module_finalize (GObject * obj);
static GType xmpp_xep_vcard_module_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static void
xmpp_xep_vcard_fetch_image_data_free (gpointer _data)
{
	XmppXepVcardFetchImageData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->stream);
	_xmpp_jid_unref0 (_data_->jid);
	_g_free0 (_data_->hash);
	_g_bytes_unref0 (_data_->result);
	g_slice_free (XmppXepVcardFetchImageData, _data_);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static gpointer
_xmpp_jid_ref0 (gpointer self)
{
	return self ? xmpp_jid_ref (self) : NULL;
}

void
xmpp_xep_vcard_fetch_image (XmppXmppStream* stream,
                            XmppJid* jid,
                            const gchar* hash,
                            GAsyncReadyCallback _callback_,
                            gpointer _user_data_)
{
	XmppXepVcardFetchImageData* _data_;
	XmppXmppStream* _tmp0_;
	XmppJid* _tmp1_;
	gchar* _tmp2_;
	g_return_if_fail (stream != NULL);
	g_return_if_fail (jid != NULL);
	g_return_if_fail (hash != NULL);
	_data_ = g_slice_new0 (XmppXepVcardFetchImageData);
	_data_->_async_result = g_task_new (NULL, NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, xmpp_xep_vcard_fetch_image_data_free);
	_tmp0_ = _g_object_ref0 (stream);
	_g_object_unref0 (_data_->stream);
	_data_->stream = _tmp0_;
	_tmp1_ = _xmpp_jid_ref0 (jid);
	_xmpp_jid_unref0 (_data_->jid);
	_data_->jid = _tmp1_;
	_tmp2_ = g_strdup (hash);
	_g_free0 (_data_->hash);
	_data_->hash = _tmp2_;
	xmpp_xep_vcard_fetch_image_co (_data_);
}

GBytes*
xmpp_xep_vcard_fetch_image_finish (GAsyncResult* _res_)
{
	GBytes* result;
	XmppXepVcardFetchImageData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
	result = _data_->result;
	_data_->result = NULL;
	return result;
}

static void
xmpp_xep_vcard_fetch_image_ready (GObject* source_object,
                                  GAsyncResult* _res_,
                                  gpointer _user_data_)
{
	XmppXepVcardFetchImageData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	xmpp_xep_vcard_fetch_image_co (_data_);
}

static const gchar*
string_to_string (const gchar* self)
{
	const gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self;
	return result;
}

static gboolean
xmpp_xep_vcard_fetch_image_co (XmppXepVcardFetchImageData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = xmpp_stanza_node_new_build ("vCard", XMPP_XEP_VCARD_NS_URI, NULL, NULL);
	_data_->_tmp1_ = _data_->_tmp0_;
	_data_->_tmp2_ = xmpp_stanza_node_add_self_xmlns (_data_->_tmp1_);
	_data_->_tmp3_ = _data_->_tmp2_;
	_data_->_tmp4_ = _xmpp_jid_ref0 (_data_->jid);
	_data_->_tmp5_ = NULL;
	_data_->_tmp5_ = xmpp_iq_stanza_new_get (_data_->_tmp3_, NULL);
	xmpp_stanza_set_to ((XmppStanza*) _data_->_tmp5_, _data_->_tmp4_);
	_xmpp_jid_unref0 (_data_->_tmp4_);
	_data_->_tmp6_ = _data_->_tmp5_;
	_xmpp_stanza_entry_unref0 (_data_->_tmp3_);
	_xmpp_stanza_entry_unref0 (_data_->_tmp1_);
	_data_->iq = _data_->_tmp6_;
	_data_->_tmp7_ = xmpp_iq_module_IDENTITY;
	_data_->_tmp8_ = xmpp_xmpp_stream_get_module (_data_->stream, XMPP_IQ_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp7_);
	_data_->_tmp9_ = (XmppIqModule*) _data_->_tmp8_;
	_data_->_tmp10_ = _data_->iq;
	_data_->_state_ = 1;
	xmpp_iq_module_send_iq_async (_data_->_tmp9_, _data_->stream, _data_->_tmp10_, G_PRIORITY_DEFAULT, NULL, xmpp_xep_vcard_fetch_image_ready, _data_);
	return FALSE;
	_state_1:
	_data_->_tmp11_ = xmpp_iq_module_send_iq_finish (_data_->_tmp9_, _data_->_res_, &_data_->_inner_error0_);
	_data_->_tmp12_ = _data_->_tmp11_;
	_g_object_unref0 (_data_->_tmp9_);
	_data_->iq_res = _data_->_tmp12_;
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_object_unref0 (_data_->iq);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp13_ = _data_->iq_res;
	if (xmpp_stanza_is_error ((XmppStanza*) _data_->_tmp13_)) {
		_data_->result = NULL;
		_g_object_unref0 (_data_->iq_res);
		_g_object_unref0 (_data_->iq);
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp14_ = _data_->iq_res;
	_data_->_tmp15_ = ((XmppStanza*) _data_->_tmp14_)->stanza;
	_data_->_tmp16_ = string_to_string (XMPP_XEP_VCARD_NS_URI);
	_data_->_tmp17_ = g_strconcat (_data_->_tmp16_, ":vCard", NULL);
	_data_->_tmp18_ = _data_->_tmp17_;
	_data_->_tmp19_ = xmpp_stanza_node_get_deep_string_content (_data_->_tmp15_, _data_->_tmp18_, "PHOTO", "BINVAL", NULL);
	_data_->_tmp20_ = g_strdup (_data_->_tmp19_);
	_data_->_tmp21_ = _data_->_tmp20_;
	_g_free0 (_data_->_tmp18_);
	_data_->res = _data_->_tmp21_;
	_data_->_tmp22_ = _data_->res;
	if (_data_->_tmp22_ == NULL) {
		_data_->result = NULL;
		_g_free0 (_data_->res);
		_g_object_unref0 (_data_->iq_res);
		_g_object_unref0 (_data_->iq);
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp23_ = _data_->res;
	_data_->_tmp24_ = 0;
	_data_->_tmp25_ = g_base64_decode (_data_->_tmp23_, &_data_->_tmp24_);
	_data_->_tmp26_ = g_bytes_new_take (_data_->_tmp25_, _data_->_tmp24_);
	_data_->content = _data_->_tmp26_;
	_data_->_tmp27_ = _data_->content;
	_data_->_tmp28_ = g_compute_checksum_for_bytes (G_CHECKSUM_SHA1, _data_->_tmp27_);
	_data_->sha1 = _data_->_tmp28_;
	_data_->_tmp29_ = _data_->sha1;
	if (g_strcmp0 (_data_->_tmp29_, _data_->hash) != 0) {
		_data_->result = NULL;
		_g_free0 (_data_->sha1);
		_g_bytes_unref0 (_data_->content);
		_g_free0 (_data_->res);
		_g_object_unref0 (_data_->iq_res);
		_g_object_unref0 (_data_->iq);
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->result = _data_->content;
	_g_free0 (_data_->sha1);
	_g_free0 (_data_->res);
	_g_object_unref0 (_data_->iq_res);
	_g_object_unref0 (_data_->iq);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
_xmpp_xep_vcard_module_on_received_presence_xmpp_presence_module_received_presence (XmppPresenceModule* _sender,
                                                                                    XmppXmppStream* stream,
                                                                                    XmppPresenceStanza* presence,
                                                                                    gpointer self)
{
	xmpp_xep_vcard_module_on_received_presence ((XmppXepVCardModule*) self, stream, presence);
}

static void
xmpp_xep_vcard_module_real_attach (XmppXmppStreamModule* base,
                                   XmppXmppStream* stream)
{
	XmppXepVCardModule * self;
	XmppModuleIdentity* _tmp0_;
	gpointer _tmp1_;
	XmppPresenceModule* _tmp2_;
	self = (XmppXepVCardModule*) base;
	g_return_if_fail (stream != NULL);
	_tmp0_ = xmpp_presence_module_IDENTITY;
	_tmp1_ = xmpp_xmpp_stream_get_module (stream, XMPP_PRESENCE_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = (XmppPresenceModule*) _tmp1_;
	g_signal_connect_object (_tmp2_, "received-presence", (GCallback) _xmpp_xep_vcard_module_on_received_presence_xmpp_presence_module_received_presence, self, 0);
	_g_object_unref0 (_tmp2_);
}

static void
xmpp_xep_vcard_module_real_detach (XmppXmppStreamModule* base,
                                   XmppXmppStream* stream)
{
	XmppXepVCardModule * self;
	XmppModuleIdentity* _tmp0_;
	gpointer _tmp1_;
	XmppPresenceModule* _tmp2_;
	guint _tmp3_;
	self = (XmppXepVCardModule*) base;
	g_return_if_fail (stream != NULL);
	_tmp0_ = xmpp_presence_module_IDENTITY;
	_tmp1_ = xmpp_xmpp_stream_get_module (stream, XMPP_PRESENCE_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = (XmppPresenceModule*) _tmp1_;
	g_signal_parse_name ("received-presence", XMPP_PRESENCE_TYPE_MODULE, &_tmp3_, NULL, FALSE);
	g_signal_handlers_disconnect_matched (_tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _xmpp_xep_vcard_module_on_received_presence_xmpp_presence_module_received_presence, self);
	_g_object_unref0 (_tmp2_);
}

static gchar*
xmpp_xep_vcard_module_real_get_ns (XmppXmppStreamModule* base)
{
	XmppXepVCardModule * self;
	gchar* _tmp0_;
	gchar* result;
	self = (XmppXepVCardModule*) base;
	_tmp0_ = g_strdup (XMPP_XEP_VCARD_NS_URI);
	result = _tmp0_;
	return result;
}

static gchar*
xmpp_xep_vcard_module_real_get_id (XmppXmppStreamModule* base)
{
	XmppXepVCardModule * self;
	XmppModuleIdentity* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* result;
	self = (XmppXepVCardModule*) base;
	_tmp0_ = xmpp_xep_vcard_module_IDENTITY;
	_tmp1_ = xmpp_module_identity_get_id (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup (_tmp2_);
	result = _tmp3_;
	return result;
}

static void
xmpp_xep_vcard_module_on_received_presence (XmppXepVCardModule* self,
                                            XmppXmppStream* stream,
                                            XmppPresenceStanza* presence)
{
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	XmppStanzaNode* update_node = NULL;
	XmppStanzaNode* _tmp2_;
	XmppStanzaNode* _tmp3_;
	XmppStanzaNode* _tmp4_;
	XmppStanzaNode* photo_node = NULL;
	XmppStanzaNode* _tmp5_;
	XmppStanzaNode* _tmp6_;
	XmppStanzaNode* _tmp7_;
	gchar* sha1 = NULL;
	XmppStanzaNode* _tmp8_;
	const gchar* _tmp9_;
	gchar* _tmp10_;
	const gchar* _tmp11_;
	XmppJid* _tmp12_;
	XmppJid* _tmp13_;
	XmppJid* _tmp14_;
	const gchar* _tmp15_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (stream != NULL);
	g_return_if_fail (presence != NULL);
	_tmp0_ = xmpp_stanza_get_type_ ((XmppStanza*) presence);
	_tmp1_ = _tmp0_;
	if (g_strcmp0 (_tmp1_, XMPP_PRESENCE_STANZA_TYPE_AVAILABLE) != 0) {
		return;
	}
	_tmp2_ = ((XmppStanza*) presence)->stanza;
	_tmp3_ = xmpp_stanza_node_get_subnode (_tmp2_, "x", XMPP_XEP_VCARD_NS_URI_UPDATE, FALSE);
	update_node = _tmp3_;
	_tmp4_ = update_node;
	if (_tmp4_ == NULL) {
		_xmpp_stanza_entry_unref0 (update_node);
		return;
	}
	_tmp5_ = update_node;
	_tmp6_ = xmpp_stanza_node_get_subnode (_tmp5_, "photo", XMPP_XEP_VCARD_NS_URI_UPDATE, FALSE);
	photo_node = _tmp6_;
	_tmp7_ = photo_node;
	if (_tmp7_ == NULL) {
		_xmpp_stanza_entry_unref0 (photo_node);
		_xmpp_stanza_entry_unref0 (update_node);
		return;
	}
	_tmp8_ = photo_node;
	_tmp9_ = xmpp_stanza_entry_get_string_content ((XmppStanzaEntry*) _tmp8_);
	_tmp10_ = g_strdup (_tmp9_);
	sha1 = _tmp10_;
	_tmp11_ = sha1;
	if (_tmp11_ == NULL) {
		_g_free0 (sha1);
		_xmpp_stanza_entry_unref0 (photo_node);
		_xmpp_stanza_entry_unref0 (update_node);
		return;
	}
	_tmp12_ = xmpp_stanza_get_from ((XmppStanza*) presence);
	_tmp13_ = _tmp12_;
	_tmp14_ = _tmp13_;
	_tmp15_ = sha1;
	g_signal_emit (self, xmpp_xep_vcard_module_signals[XMPP_XEP_VCARD_MODULE_RECEIVED_AVATAR_HASH_SIGNAL], 0, stream, _tmp14_, _tmp15_);
	_xmpp_jid_unref0 (_tmp14_);
	_g_free0 (sha1);
	_xmpp_stanza_entry_unref0 (photo_node);
	_xmpp_stanza_entry_unref0 (update_node);
}

XmppXepVCardModule*
xmpp_xep_vcard_module_construct (GType object_type)
{
	XmppXepVCardModule * self = NULL;
	self = (XmppXepVCardModule*) xmpp_xmpp_stream_module_construct (object_type);
	return self;
}

XmppXepVCardModule*
xmpp_xep_vcard_module_new (void)
{
	return xmpp_xep_vcard_module_construct (XMPP_XEP_VCARD_TYPE_MODULE);
}

static void
g_cclosure_user_marshal_VOID__OBJECT_XMPP_JID_STRING (GClosure * closure,
                                                      GValue * return_value,
                                                      guint n_param_values,
                                                      const GValue * param_values,
                                                      gpointer invocation_hint,
                                                      gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__OBJECT_XMPP_JID_STRING) (gpointer data1, gpointer arg_1, gpointer arg_2, const char* arg_3, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_XMPP_JID_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__OBJECT_XMPP_JID_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_object (param_values + 1), xmpp_value_get_jid (param_values + 2), g_value_get_string (param_values + 3), data2);
}

static void
xmpp_xep_vcard_module_class_init (XmppXepVCardModuleClass * klass,
                                  gpointer klass_data)
{
	XmppModuleIdentity* _tmp0_;
	xmpp_xep_vcard_module_parent_class = g_type_class_peek_parent (klass);
	((XmppXmppStreamModuleClass *) klass)->attach = (void (*) (XmppXmppStreamModule*, XmppXmppStream*)) xmpp_xep_vcard_module_real_attach;
	((XmppXmppStreamModuleClass *) klass)->detach = (void (*) (XmppXmppStreamModule*, XmppXmppStream*)) xmpp_xep_vcard_module_real_detach;
	((XmppXmppStreamModuleClass *) klass)->get_ns = (gchar* (*) (XmppXmppStreamModule*)) xmpp_xep_vcard_module_real_get_ns;
	((XmppXmppStreamModuleClass *) klass)->get_id = (gchar* (*) (XmppXmppStreamModule*)) xmpp_xep_vcard_module_real_get_id;
	G_OBJECT_CLASS (klass)->finalize = xmpp_xep_vcard_module_finalize;
	xmpp_xep_vcard_module_signals[XMPP_XEP_VCARD_MODULE_RECEIVED_AVATAR_HASH_SIGNAL] = g_signal_new ("received-avatar-hash", XMPP_XEP_VCARD_TYPE_MODULE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_XMPP_JID_STRING, G_TYPE_NONE, 3, XMPP_TYPE_XMPP_STREAM, XMPP_TYPE_JID, G_TYPE_STRING);
	_tmp0_ = xmpp_module_identity_new (XMPP_XEP_VCARD_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, XMPP_XEP_VCARD_NS_URI, "0153_vcard_based_avatars");
	xmpp_xep_vcard_module_IDENTITY = _tmp0_;
}

static void
xmpp_xep_vcard_module_instance_init (XmppXepVCardModule * self,
                                     gpointer klass)
{
}

static void
xmpp_xep_vcard_module_finalize (GObject * obj)
{
	XmppXepVCardModule * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, XMPP_XEP_VCARD_TYPE_MODULE, XmppXepVCardModule);
	G_OBJECT_CLASS (xmpp_xep_vcard_module_parent_class)->finalize (obj);
}

static GType
xmpp_xep_vcard_module_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepVCardModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_vcard_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (XmppXepVCardModule), 0, (GInstanceInitFunc) xmpp_xep_vcard_module_instance_init, NULL };
	GType xmpp_xep_vcard_module_type_id;
	xmpp_xep_vcard_module_type_id = g_type_register_static (XMPP_TYPE_XMPP_STREAM_MODULE, "XmppXepVCardModule", &g_define_type_info, 0);
	return xmpp_xep_vcard_module_type_id;
}

GType
xmpp_xep_vcard_module_get_type (void)
{
	static volatile gsize xmpp_xep_vcard_module_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_vcard_module_type_id__once)) {
		GType xmpp_xep_vcard_module_type_id;
		xmpp_xep_vcard_module_type_id = xmpp_xep_vcard_module_get_type_once ();
		g_once_init_leave (&xmpp_xep_vcard_module_type_id__once, xmpp_xep_vcard_module_type_id);
	}
	return xmpp_xep_vcard_module_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

