/* conference_details_fragment.c generated by valac 0.56.19, the Vala compiler
 * generated from conference_details_fragment.vala, do not modify */

#include <gtk/gtk.h>
#include <glib-object.h>
#include <glib.h>
#include "libdino.h"
#include "xmpp-vala.h"
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include "dino_i18n.h"
#include <adwaita.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT (dino_ui_conference_details_fragment_get_type ())
#define DINO_UI_CONFERENCE_DETAILS_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragment))
#define DINO_UI_CONFERENCE_DETAILS_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragmentClass))
#define DINO_UI_IS_CONFERENCE_DETAILS_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT))
#define DINO_UI_IS_CONFERENCE_DETAILS_FRAGMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT))
#define DINO_UI_CONFERENCE_DETAILS_FRAGMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragmentClass))

typedef struct _DinoUiConferenceDetailsFragment DinoUiConferenceDetailsFragment;
typedef struct _DinoUiConferenceDetailsFragmentClass DinoUiConferenceDetailsFragmentClass;
typedef struct _DinoUiConferenceDetailsFragmentPrivate DinoUiConferenceDetailsFragmentPrivate;

#define DINO_UI_TYPE_ACCOUNT_COMBO_BOX (dino_ui_account_combo_box_get_type ())
#define DINO_UI_ACCOUNT_COMBO_BOX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_ACCOUNT_COMBO_BOX, DinoUiAccountComboBox))
#define DINO_UI_ACCOUNT_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_ACCOUNT_COMBO_BOX, DinoUiAccountComboBoxClass))
#define DINO_UI_IS_ACCOUNT_COMBO_BOX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_ACCOUNT_COMBO_BOX))
#define DINO_UI_IS_ACCOUNT_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_ACCOUNT_COMBO_BOX))
#define DINO_UI_ACCOUNT_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_ACCOUNT_COMBO_BOX, DinoUiAccountComboBoxClass))

typedef struct _DinoUiAccountComboBox DinoUiAccountComboBox;
typedef struct _DinoUiAccountComboBoxClass DinoUiAccountComboBoxClass;
enum  {
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_0_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NUM_PROPERTIES
};
static GParamSpec* dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _xmpp_jid_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_jid_unref (var), NULL)))

#define DINO_UI_TYPE_APPLICATION (dino_ui_application_get_type ())
#define DINO_UI_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_APPLICATION, DinoUiApplication))
#define DINO_UI_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_APPLICATION, DinoUiApplicationClass))
#define DINO_UI_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_APPLICATION))
#define DINO_UI_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_APPLICATION))
#define DINO_UI_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_APPLICATION, DinoUiApplicationClass))

typedef struct _DinoUiApplication DinoUiApplication;
typedef struct _DinoUiApplicationClass DinoUiApplicationClass;
typedef struct _DinoUiApplicationPrivate DinoUiApplicationPrivate;

#define DINO_UI_TYPE_MAIN_WINDOW (dino_ui_main_window_get_type ())
#define DINO_UI_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_MAIN_WINDOW, DinoUiMainWindow))
#define DINO_UI_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_MAIN_WINDOW, DinoUiMainWindowClass))
#define DINO_UI_IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_MAIN_WINDOW))
#define DINO_UI_IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_MAIN_WINDOW))
#define DINO_UI_MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_MAIN_WINDOW, DinoUiMainWindowClass))

typedef struct _DinoUiMainWindow DinoUiMainWindow;
typedef struct _DinoUiMainWindowClass DinoUiMainWindowClass;

#define DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER (dino_ui_main_window_controller_get_type ())
#define DINO_UI_MAIN_WINDOW_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER, DinoUiMainWindowController))
#define DINO_UI_MAIN_WINDOW_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER, DinoUiMainWindowControllerClass))
#define DINO_UI_IS_MAIN_WINDOW_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER))
#define DINO_UI_IS_MAIN_WINDOW_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER))
#define DINO_UI_MAIN_WINDOW_CONTROLLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_TYPE_MAIN_WINDOW_CONTROLLER, DinoUiMainWindowControllerClass))

typedef struct _DinoUiMainWindowController DinoUiMainWindowController;
typedef struct _DinoUiMainWindowControllerClass DinoUiMainWindowControllerClass;
#define _xmpp_xep_muc_join_result_unref0(var) ((var == NULL) ? NULL : (var = (xmpp_xep_muc_join_result_unref (var), NULL)))
typedef struct _DinoUiConferenceDetailsFragmentOnOkButtonClickedData DinoUiConferenceDetailsFragmentOnOkButtonClickedData;
enum  {
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JOINED_SIGNAL,
	DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NUM_SIGNALS
};
static guint dino_ui_conference_details_fragment_signals[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NUM_SIGNALS] = {0};

struct _DinoUiConferenceDetailsFragment {
	GtkBox parent_instance;
	DinoUiConferenceDetailsFragmentPrivate * priv;
};

struct _DinoUiConferenceDetailsFragmentClass {
	GtkBoxClass parent_class;
};

struct _DinoUiConferenceDetailsFragmentPrivate {
	gboolean _done;
	gboolean _fragment_active;
	GtkStack* accounts_stack;
	GtkButton* accounts_button;
	GtkLabel* accounts_label;
	DinoUiAccountComboBox* account_combobox;
	GtkStack* jid_stack;
	GtkButton* jid_button;
	GtkLabel* jid_label;
	GtkEntry* jid_entry;
	GtkStack* nick_stack;
	GtkButton* nick_button;
	GtkLabel* nick_label;
	GtkEntry* nick_entry;
	GtkStack* password_stack;
	GtkButton* password_button;
	GtkLabel* password_label;
	GtkLabel* password_text_label;
	GtkEntry* password_entry;
	GtkRevealer* notification_revealer;
	GtkButton* notification_button;
	GtkLabel* notification_label;
	DinoStreamInteractor* stream_interactor;
	GtkButton* ok_button_;
};

struct _DinoUiApplication {
	AdwApplication parent_instance;
	DinoUiApplicationPrivate * priv;
	DinoUiMainWindow* window;
	DinoUiMainWindowController* controller;
};

struct _DinoUiApplicationClass {
	AdwApplicationClass parent_class;
};

struct _DinoUiConferenceDetailsFragmentOnOkButtonClickedData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	DinoUiConferenceDetailsFragment* self;
	gboolean _tmp0_;
	GtkButton* _tmp1_;
	GtkButton* _tmp2_;
	const gchar* _tmp3_;
	GtkButton* _tmp4_;
	GtkButton* _tmp5_;
	gchar* label_text;
	gchar* _tmp6_;
	XmppJid* parsed_jid;
	const gchar* _tmp7_;
	const gchar* _tmp8_;
	XmppJid* _tmp9_;
	XmppXepMucJoinResult* join_result;
	DinoStreamInteractor* _tmp10_;
	DinoModuleIdentity* _tmp11_;
	gpointer _tmp12_;
	DinoMucManager* _tmp13_;
	DinoEntitiesAccount* _tmp14_;
	DinoEntitiesAccount* _tmp15_;
	DinoEntitiesAccount* _tmp16_;
	XmppJid* _tmp17_;
	const gchar* _tmp18_;
	const gchar* _tmp19_;
	const gchar* _tmp20_;
	const gchar* _tmp21_;
	XmppXepMucJoinResult* _tmp22_;
	XmppXepMucJoinResult* _tmp23_;
	GtkButton* _tmp24_;
	GtkButton* _tmp25_;
	const gchar* _tmp26_;
	GtkButton* _tmp27_;
	GtkButton* _tmp28_;
	gboolean _tmp29_;
	XmppXepMucJoinResult* _tmp30_;
	XmppXepMucJoinResult* _tmp31_;
	const gchar* _tmp32_;
	DinoEntitiesConversation* conversation;
	DinoStreamInteractor* _tmp33_;
	DinoModuleIdentity* _tmp34_;
	gpointer _tmp35_;
	DinoConversationManager* _tmp36_;
	XmppJid* _tmp37_;
	DinoEntitiesAccount* _tmp38_;
	DinoEntitiesAccount* _tmp39_;
	DinoEntitiesAccount* _tmp40_;
	DinoEntitiesConversationType _tmp41_;
	DinoEntitiesConversation* _tmp42_;
	DinoEntitiesConversation* _tmp43_;
	DinoUiApplication* app;
	GApplication* _tmp44_;
	DinoUiApplication* _tmp45_;
	DinoUiApplication* _tmp46_;
	DinoUiMainWindowController* _tmp47_;
	DinoEntitiesConversation* _tmp48_;
	XmppXepMucJoinResult* _tmp49_;
	XmppXepMucMucEnterError* _tmp50_;
	XmppXepMucJoinResult* _tmp51_;
	XmppXepMucMucEnterError* _tmp52_;
	const gchar* _tmp53_;
	gchar* _tmp54_;
	GtkLabel* _tmp55_;
	GtkStack* _tmp56_;
	const gchar* _tmp57_;
	gchar* _tmp58_;
	const gchar* _tmp59_;
	gchar* _tmp60_;
	const gchar* _tmp61_;
	gchar* _tmp62_;
	const gchar* _tmp63_;
	gchar* _tmp64_;
	const gchar* _tmp65_;
	gchar* _tmp66_;
	const gchar* _tmp67_;
	gchar* _tmp68_;
	XmppXepMucJoinResult* _tmp69_;
	const gchar* _tmp70_;
	XmppJid* _tmp71_;
	const gchar* _tmp72_;
	const gchar* _tmp73_;
	XmppJid* _tmp74_;
	const gchar* _tmp75_;
	const gchar* _tmp76_;
	gchar* _tmp77_;
	const gchar* _tmp78_;
	gchar* _tmp79_;
	GtkLabel* _tmp80_;
	const gchar* _tmp81_;
	GtkRevealer* _tmp82_;
	GError* _inner_error0_;
};

static gint DinoUiConferenceDetailsFragment_private_offset;
static gpointer dino_ui_conference_details_fragment_parent_class = NULL;

VALA_EXTERN GType dino_ui_conference_details_fragment_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiConferenceDetailsFragment, g_object_unref)
VALA_EXTERN GType dino_ui_account_combo_box_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiAccountComboBox, g_object_unref)
VALA_EXTERN DinoUiConferenceDetailsFragment* dino_ui_conference_details_fragment_new (DinoStreamInteractor* stream_interactor);
VALA_EXTERN DinoUiConferenceDetailsFragment* dino_ui_conference_details_fragment_construct (GType object_type,
                                                                                DinoStreamInteractor* stream_interactor);
VALA_EXTERN void dino_ui_account_combo_box_initialize (DinoUiAccountComboBox* self,
                                           DinoStreamInteractor* stream_interactor);
static void __lambda291_ (DinoUiConferenceDetailsFragment* self);
static void dino_ui_conference_details_fragment_set_active_stack (DinoUiConferenceDetailsFragment* self,
                                                           GtkStack* stack);
static void ___lambda291__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static void __lambda292_ (DinoUiConferenceDetailsFragment* self);
static void ___lambda292__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static void __lambda293_ (DinoUiConferenceDetailsFragment* self);
static void ___lambda293__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static void __lambda294_ (DinoUiConferenceDetailsFragment* self);
static void ___lambda294__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static void __lambda295_ (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN DinoEntitiesAccount* dino_ui_account_combo_box_get_selected (DinoUiAccountComboBox* self);
static void ___lambda295__gtk_combo_box_changed (GtkComboBox* _sender,
                                          gpointer self);
static void __lambda296_ (DinoUiConferenceDetailsFragment* self);
static void dino_ui_conference_details_fragment_check_if_done (DinoUiConferenceDetailsFragment* self);
static void ___lambda296__gtk_editable_changed (GtkEditable* _sender,
                                         gpointer self);
static void __lambda297_ (DinoUiConferenceDetailsFragment* self);
static void ___lambda297__gtk_editable_changed (GtkEditable* _sender,
                                         gpointer self);
static void __lambda298_ (DinoUiConferenceDetailsFragment* self);
static void ___lambda298__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
VALA_EXTERN void dino_ui_conference_details_fragment_clear (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN void dino_ui_conference_details_fragment_set_jid (DinoUiConferenceDetailsFragment* self,
                                                  const gchar* value);
VALA_EXTERN void dino_ui_conference_details_fragment_set_nick (DinoUiConferenceDetailsFragment* self,
                                                   const gchar* value);
VALA_EXTERN void dino_ui_conference_details_fragment_set_password (DinoUiConferenceDetailsFragment* self,
                                                       const gchar* value);
VALA_EXTERN void dino_ui_conference_details_fragment_reset_editable (DinoUiConferenceDetailsFragment* self);
static void dino_ui_conference_details_fragment_on_ok_button_clicked_data_free (gpointer _data);
static void dino_ui_conference_details_fragment_on_ok_button_clicked (DinoUiConferenceDetailsFragment* self,
                                                               GAsyncReadyCallback _callback_,
                                                               gpointer _user_data_);
static void dino_ui_conference_details_fragment_on_ok_button_clicked_finish (DinoUiConferenceDetailsFragment* self,
                                                                      GAsyncResult* _res_);
static gboolean dino_ui_conference_details_fragment_on_ok_button_clicked_co (DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_);
VALA_EXTERN gboolean dino_ui_conference_details_fragment_get_fragment_active (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN GtkButton* dino_ui_conference_details_fragment_get_ok_button (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN const gchar* dino_ui_conference_details_fragment_get_jid (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN DinoEntitiesAccount* dino_ui_conference_details_fragment_get_account (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN const gchar* dino_ui_conference_details_fragment_get_nick (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN const gchar* dino_ui_conference_details_fragment_get_password (DinoUiConferenceDetailsFragment* self);
static void dino_ui_conference_details_fragment_on_ok_button_clicked_ready (GObject* source_object,
                                                                     GAsyncResult* _res_,
                                                                     gpointer _user_data_);
VALA_EXTERN GType dino_ui_application_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiApplication, g_object_unref)
VALA_EXTERN GType dino_ui_main_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiMainWindow, g_object_unref)
VALA_EXTERN GType dino_ui_main_window_controller_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiMainWindowController, g_object_unref)
VALA_EXTERN void dino_ui_main_window_controller_select_conversation (DinoUiMainWindowController* self,
                                                         DinoEntitiesConversation* conversation,
                                                         gboolean do_reset_search,
                                                         gboolean default_initialize_conversation);
static void dino_ui_conference_details_fragment_set_done (DinoUiConferenceDetailsFragment* self,
                                                   gboolean value);
VALA_EXTERN gboolean dino_ui_conference_details_fragment_get_done (DinoUiConferenceDetailsFragment* self);
VALA_EXTERN void dino_ui_conference_details_fragment_set_account (DinoUiConferenceDetailsFragment* self,
                                                      DinoEntitiesAccount* value);
VALA_EXTERN void dino_ui_account_combo_box_set_selected (DinoUiAccountComboBox* self,
                                             DinoEntitiesAccount* value);
VALA_EXTERN void dino_ui_conference_details_fragment_set_fragment_active (DinoUiConferenceDetailsFragment* self,
                                                              gboolean value);
VALA_EXTERN void dino_ui_conference_details_fragment_set_ok_button (DinoUiConferenceDetailsFragment* self,
                                                        GtkButton* value);
static void _dino_ui_conference_details_fragment_ok_button____lambda299_ (DinoUiConferenceDetailsFragment* self);
static void __dino_ui_conference_details_fragment_ok_button____lambda299__gtk_button_clicked (GtkButton* _sender,
                                                                                       gpointer self);
static void dino_ui_conference_details_fragment_finalize (GObject * obj);
static GType dino_ui_conference_details_fragment_get_type_once (void);
static void _vala_dino_ui_conference_details_fragment_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec);
static void _vala_dino_ui_conference_details_fragment_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec);

static inline gpointer
dino_ui_conference_details_fragment_get_instance_private (DinoUiConferenceDetailsFragment* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiConferenceDetailsFragment_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
__lambda291_ (DinoUiConferenceDetailsFragment* self)
{
	GtkStack* _tmp0_;
	_tmp0_ = self->priv->accounts_stack;
	dino_ui_conference_details_fragment_set_active_stack (self, _tmp0_);
}

static void
___lambda291__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda291_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda292_ (DinoUiConferenceDetailsFragment* self)
{
	GtkStack* _tmp0_;
	_tmp0_ = self->priv->jid_stack;
	dino_ui_conference_details_fragment_set_active_stack (self, _tmp0_);
}

static void
___lambda292__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda292_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda293_ (DinoUiConferenceDetailsFragment* self)
{
	GtkStack* _tmp0_;
	_tmp0_ = self->priv->nick_stack;
	dino_ui_conference_details_fragment_set_active_stack (self, _tmp0_);
}

static void
___lambda293__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda293_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda294_ (DinoUiConferenceDetailsFragment* self)
{
	GtkStack* _tmp0_;
	_tmp0_ = self->priv->password_stack;
	dino_ui_conference_details_fragment_set_active_stack (self, _tmp0_);
}

static void
___lambda294__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda294_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda295_ (DinoUiConferenceDetailsFragment* self)
{
	GtkLabel* _tmp0_;
	DinoUiAccountComboBox* _tmp1_;
	DinoEntitiesAccount* _tmp2_;
	DinoEntitiesAccount* _tmp3_;
	XmppJid* _tmp4_;
	XmppJid* _tmp5_;
	XmppJid* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	_tmp0_ = self->priv->accounts_label;
	_tmp1_ = self->priv->account_combobox;
	_tmp2_ = dino_ui_account_combo_box_get_selected (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = dino_entities_account_get_bare_jid (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = _tmp5_;
	_tmp7_ = xmpp_jid_to_string (_tmp6_);
	_tmp8_ = _tmp7_;
	gtk_label_set_label (_tmp0_, _tmp8_);
	_g_free0 (_tmp8_);
	_xmpp_jid_unref0 (_tmp6_);
}

static void
___lambda295__gtk_combo_box_changed (GtkComboBox* _sender,
                                     gpointer self)
{
	__lambda295_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda296_ (DinoUiConferenceDetailsFragment* self)
{
	dino_ui_conference_details_fragment_check_if_done (self);
}

static void
___lambda296__gtk_editable_changed (GtkEditable* _sender,
                                    gpointer self)
{
	__lambda296_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda297_ (DinoUiConferenceDetailsFragment* self)
{
	dino_ui_conference_details_fragment_check_if_done (self);
}

static void
___lambda297__gtk_editable_changed (GtkEditable* _sender,
                                    gpointer self)
{
	__lambda297_ ((DinoUiConferenceDetailsFragment*) self);
}

static void
__lambda298_ (DinoUiConferenceDetailsFragment* self)
{
	GtkRevealer* _tmp0_;
	_tmp0_ = self->priv->notification_revealer;
	gtk_revealer_set_reveal_child (_tmp0_, FALSE);
}

static void
___lambda298__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda298_ ((DinoUiConferenceDetailsFragment*) self);
}

DinoUiConferenceDetailsFragment*
dino_ui_conference_details_fragment_construct (GType object_type,
                                               DinoStreamInteractor* stream_interactor)
{
	DinoUiConferenceDetailsFragment * self = NULL;
	DinoStreamInteractor* _tmp0_;
	DinoUiAccountComboBox* _tmp1_;
	GtkButton* _tmp2_;
	GtkButton* _tmp3_;
	GtkButton* _tmp4_;
	GtkButton* _tmp5_;
	DinoUiAccountComboBox* _tmp6_;
	GtkLabel* _tmp7_;
	DinoUiAccountComboBox* _tmp8_;
	DinoEntitiesAccount* _tmp9_;
	DinoEntitiesAccount* _tmp10_;
	XmppJid* _tmp11_;
	XmppJid* _tmp12_;
	XmppJid* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	GtkEntry* _tmp16_;
	GtkEntry* _tmp17_;
	GtkButton* _tmp18_;
	g_return_val_if_fail (stream_interactor != NULL, NULL);
	self = (DinoUiConferenceDetailsFragment*) g_object_new (object_type, NULL);
	_tmp0_ = _g_object_ref0 (stream_interactor);
	_g_object_unref0 (self->priv->stream_interactor);
	self->priv->stream_interactor = _tmp0_;
	_tmp1_ = self->priv->account_combobox;
	dino_ui_account_combo_box_initialize (_tmp1_, stream_interactor);
	_tmp2_ = self->priv->accounts_button;
	g_signal_connect_object (_tmp2_, "clicked", (GCallback) ___lambda291__gtk_button_clicked, self, 0);
	_tmp3_ = self->priv->jid_button;
	g_signal_connect_object (_tmp3_, "clicked", (GCallback) ___lambda292__gtk_button_clicked, self, 0);
	_tmp4_ = self->priv->nick_button;
	g_signal_connect_object (_tmp4_, "clicked", (GCallback) ___lambda293__gtk_button_clicked, self, 0);
	_tmp5_ = self->priv->password_button;
	g_signal_connect_object (_tmp5_, "clicked", (GCallback) ___lambda294__gtk_button_clicked, self, 0);
	_tmp6_ = self->priv->account_combobox;
	g_signal_connect_object ((GtkComboBox*) _tmp6_, "changed", (GCallback) ___lambda295__gtk_combo_box_changed, self, 0);
	_tmp7_ = self->priv->accounts_label;
	_tmp8_ = self->priv->account_combobox;
	_tmp9_ = dino_ui_account_combo_box_get_selected (_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = dino_entities_account_get_bare_jid (_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = _tmp12_;
	_tmp14_ = xmpp_jid_to_string (_tmp13_);
	_tmp15_ = _tmp14_;
	gtk_label_set_label (_tmp7_, _tmp15_);
	_g_free0 (_tmp15_);
	_xmpp_jid_unref0 (_tmp13_);
	_tmp16_ = self->priv->jid_entry;
	g_signal_connect_object ((GtkEditable*) _tmp16_, "changed", (GCallback) ___lambda296__gtk_editable_changed, self, 0);
	_tmp17_ = self->priv->nick_entry;
	g_signal_connect_object ((GtkEditable*) _tmp17_, "changed", (GCallback) ___lambda297__gtk_editable_changed, self, 0);
	dino_ui_conference_details_fragment_check_if_done (self);
	_tmp18_ = self->priv->notification_button;
	g_signal_connect_object (_tmp18_, "clicked", (GCallback) ___lambda298__gtk_button_clicked, self, 0);
	dino_ui_conference_details_fragment_clear (self);
	return self;
}

DinoUiConferenceDetailsFragment*
dino_ui_conference_details_fragment_new (DinoStreamInteractor* stream_interactor)
{
	return dino_ui_conference_details_fragment_construct (DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, stream_interactor);
}

void
dino_ui_conference_details_fragment_clear (DinoUiConferenceDetailsFragment* self)
{
	GtkLabel* _tmp0_;
	GtkStack* _tmp1_;
	GtkRevealer* _tmp2_;
	g_return_if_fail (self != NULL);
	dino_ui_conference_details_fragment_set_jid (self, "");
	dino_ui_conference_details_fragment_set_nick (self, "");
	dino_ui_conference_details_fragment_set_password (self, "");
	_tmp0_ = self->priv->password_text_label;
	gtk_widget_set_visible ((GtkWidget*) _tmp0_, FALSE);
	_tmp1_ = self->priv->password_stack;
	gtk_widget_set_visible ((GtkWidget*) _tmp1_, FALSE);
	_tmp2_ = self->priv->notification_revealer;
	gtk_revealer_set_reveal_child (_tmp2_, FALSE);
	dino_ui_conference_details_fragment_reset_editable (self);
}

void
dino_ui_conference_details_fragment_reset_editable (DinoUiConferenceDetailsFragment* self)
{
	GtkStack* _tmp0_;
	GtkStack* _tmp1_;
	GtkStack* _tmp2_;
	GtkStack* _tmp3_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->jid_stack;
	gtk_stack_set_visible_child_name (_tmp0_, "entry");
	_tmp1_ = self->priv->accounts_stack;
	gtk_stack_set_visible_child_name (_tmp1_, "entry");
	_tmp2_ = self->priv->nick_stack;
	gtk_stack_set_visible_child_name (_tmp2_, "entry");
	_tmp3_ = self->priv->password_stack;
	gtk_stack_set_visible_child_name (_tmp3_, "entry");
}

static void
dino_ui_conference_details_fragment_on_ok_button_clicked_data_free (gpointer _data)
{
	DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->self);
	g_slice_free (DinoUiConferenceDetailsFragmentOnOkButtonClickedData, _data_);
}

static void
dino_ui_conference_details_fragment_on_ok_button_clicked (DinoUiConferenceDetailsFragment* self,
                                                          GAsyncReadyCallback _callback_,
                                                          gpointer _user_data_)
{
	DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_;
	DinoUiConferenceDetailsFragment* _tmp0_;
	g_return_if_fail (self != NULL);
	_data_ = g_slice_new0 (DinoUiConferenceDetailsFragmentOnOkButtonClickedData);
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, _callback_, _user_data_);
	g_task_set_task_data (_data_->_async_result, _data_, dino_ui_conference_details_fragment_on_ok_button_clicked_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	dino_ui_conference_details_fragment_on_ok_button_clicked_co (_data_);
}

static void
dino_ui_conference_details_fragment_on_ok_button_clicked_finish (DinoUiConferenceDetailsFragment* self,
                                                                 GAsyncResult* _res_)
{
	DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_;
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
}

static void
dino_ui_conference_details_fragment_on_ok_button_clicked_ready (GObject* source_object,
                                                                GAsyncResult* _res_,
                                                                gpointer _user_data_)
{
	DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_;
	_data_ = _user_data_;
	_data_->_source_object_ = source_object;
	_data_->_res_ = _res_;
	dino_ui_conference_details_fragment_on_ok_button_clicked_co (_data_);
}

static gboolean
dino_ui_conference_details_fragment_on_ok_button_clicked_co (DinoUiConferenceDetailsFragmentOnOkButtonClickedData* _data_)
{
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		case 1:
		goto _state_1;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = _data_->self->priv->_fragment_active;
	if (!_data_->_tmp0_) {
		g_task_return_pointer (_data_->_async_result, _data_, NULL);
		if (_data_->_state_ != 0) {
			while (!g_task_get_completed (_data_->_async_result)) {
				g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
			}
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp1_ = dino_ui_conference_details_fragment_get_ok_button (_data_->self);
	_data_->_tmp2_ = _data_->_tmp1_;
	_data_->_tmp3_ = dino_gettext ("Joining…");
	gtk_button_set_label (_data_->_tmp2_, _data_->_tmp3_);
	_data_->_tmp4_ = dino_ui_conference_details_fragment_get_ok_button (_data_->self);
	_data_->_tmp5_ = _data_->_tmp4_;
	gtk_widget_set_sensitive ((GtkWidget*) _data_->_tmp5_, FALSE);
	_data_->_tmp6_ = g_strdup ("");
	_data_->label_text = _data_->_tmp6_;
	{
		_data_->_tmp7_ = dino_ui_conference_details_fragment_get_jid (_data_->self);
		_data_->_tmp8_ = _data_->_tmp7_;
		_data_->_tmp9_ = xmpp_jid_new (_data_->_tmp8_, &_data_->_inner_error0_);
		_data_->parsed_jid = _data_->_tmp9_;
		if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
			if (_data_->_inner_error0_->domain == XMPP_INVALID_JID_ERROR) {
				goto __catch0_xmpp_invalid_jid_error;
			}
			_g_free0 (_data_->label_text);
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
			g_clear_error (&_data_->_inner_error0_);
			g_object_unref (_data_->_async_result);
			return FALSE;
		}
		_data_->_tmp10_ = _data_->self->priv->stream_interactor;
		_data_->_tmp11_ = dino_muc_manager_IDENTITY;
		_data_->_tmp12_ = dino_stream_interactor_get_module (_data_->_tmp10_, DINO_TYPE_MUC_MANAGER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp11_);
		_data_->_tmp13_ = (DinoMucManager*) _data_->_tmp12_;
		_data_->_tmp14_ = dino_ui_conference_details_fragment_get_account (_data_->self);
		_data_->_tmp15_ = _data_->_tmp14_;
		_data_->_tmp16_ = _data_->_tmp15_;
		_data_->_tmp17_ = _data_->parsed_jid;
		_data_->_tmp18_ = dino_ui_conference_details_fragment_get_nick (_data_->self);
		_data_->_tmp19_ = _data_->_tmp18_;
		_data_->_tmp20_ = dino_ui_conference_details_fragment_get_password (_data_->self);
		_data_->_tmp21_ = _data_->_tmp20_;
		_data_->_state_ = 1;
		dino_muc_manager_join (_data_->_tmp13_, _data_->_tmp16_, _data_->_tmp17_, _data_->_tmp19_, _data_->_tmp21_, FALSE, NULL, dino_ui_conference_details_fragment_on_ok_button_clicked_ready, _data_);
		return FALSE;
		_state_1:
		_data_->_tmp22_ = dino_muc_manager_join_finish (_data_->_tmp13_, _data_->_res_);
		_data_->_tmp23_ = _data_->_tmp22_;
		_g_object_unref0 (_data_->_tmp16_);
		_g_object_unref0 (_data_->_tmp13_);
		_data_->join_result = _data_->_tmp23_;
		_data_->_tmp24_ = dino_ui_conference_details_fragment_get_ok_button (_data_->self);
		_data_->_tmp25_ = _data_->_tmp24_;
		_data_->_tmp26_ = dino_gettext ("Join");
		gtk_button_set_label (_data_->_tmp25_, _data_->_tmp26_);
		_data_->_tmp27_ = dino_ui_conference_details_fragment_get_ok_button (_data_->self);
		_data_->_tmp28_ = _data_->_tmp27_;
		gtk_widget_set_sensitive ((GtkWidget*) _data_->_tmp28_, TRUE);
		_data_->_tmp30_ = _data_->join_result;
		if (_data_->_tmp30_ == NULL) {
			_data_->_tmp29_ = TRUE;
		} else {
			_data_->_tmp31_ = _data_->join_result;
			_data_->_tmp32_ = _data_->_tmp31_->nick;
			_data_->_tmp29_ = _data_->_tmp32_ != NULL;
		}
		if (_data_->_tmp29_) {
			_data_->_tmp33_ = _data_->self->priv->stream_interactor;
			_data_->_tmp34_ = dino_conversation_manager_IDENTITY;
			_data_->_tmp35_ = dino_stream_interactor_get_module (_data_->_tmp33_, DINO_TYPE_CONVERSATION_MANAGER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _data_->_tmp34_);
			_data_->_tmp36_ = (DinoConversationManager*) _data_->_tmp35_;
			_data_->_tmp37_ = _data_->parsed_jid;
			_data_->_tmp38_ = dino_ui_conference_details_fragment_get_account (_data_->self);
			_data_->_tmp39_ = _data_->_tmp38_;
			_data_->_tmp40_ = _data_->_tmp39_;
			_data_->_tmp41_ = DINO_ENTITIES_CONVERSATION_TYPE_GROUPCHAT;
			_data_->_tmp42_ = dino_conversation_manager_create_conversation (_data_->_tmp36_, _data_->_tmp37_, _data_->_tmp40_, &_data_->_tmp41_);
			_data_->_tmp43_ = _data_->_tmp42_;
			_g_object_unref0 (_data_->_tmp40_);
			_g_object_unref0 (_data_->_tmp36_);
			_data_->conversation = _data_->_tmp43_;
			_data_->_tmp44_ = g_application_get_default ();
			_data_->_tmp45_ = _g_object_ref0 (DINO_UI_IS_APPLICATION (_data_->_tmp44_) ? ((DinoUiApplication*) _data_->_tmp44_) : NULL);
			_data_->app = _data_->_tmp45_;
			_data_->_tmp46_ = _data_->app;
			_data_->_tmp47_ = _data_->_tmp46_->controller;
			_data_->_tmp48_ = _data_->conversation;
			dino_ui_main_window_controller_select_conversation (_data_->_tmp47_, _data_->_tmp48_, TRUE, TRUE);
			g_signal_emit (_data_->self, dino_ui_conference_details_fragment_signals[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JOINED_SIGNAL], 0);
			_g_object_unref0 (_data_->app);
			_g_object_unref0 (_data_->conversation);
			_xmpp_xep_muc_join_result_unref0 (_data_->join_result);
			_xmpp_jid_unref0 (_data_->parsed_jid);
			_g_free0 (_data_->label_text);
			g_task_return_pointer (_data_->_async_result, _data_, NULL);
			if (_data_->_state_ != 0) {
				while (!g_task_get_completed (_data_->_async_result)) {
					g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
				}
			}
			g_object_unref (_data_->_async_result);
			return FALSE;
		}
		_data_->_tmp49_ = _data_->join_result;
		_data_->_tmp50_ = _data_->_tmp49_->muc_error;
		if (_data_->_tmp50_ != NULL) {
			_data_->_tmp51_ = _data_->join_result;
			_data_->_tmp52_ = _data_->_tmp51_->muc_error;
			switch (*_data_->_tmp52_) {
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_PASSWORD_REQUIRED:
				{
					_data_->_tmp53_ = dino_gettext ("Password required to enter room");
					_data_->_tmp54_ = g_strdup (_data_->_tmp53_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp54_;
					_data_->_tmp55_ = _data_->self->priv->password_text_label;
					gtk_widget_set_visible ((GtkWidget*) _data_->_tmp55_, TRUE);
					_data_->_tmp56_ = _data_->self->priv->password_stack;
					gtk_widget_set_visible ((GtkWidget*) _data_->_tmp56_, TRUE);
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_BANNED:
				{
					_data_->_tmp57_ = dino_gettext ("Banned from joining or creating conference");
					_data_->_tmp58_ = g_strdup (_data_->_tmp57_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp58_;
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_ROOM_DOESNT_EXIST:
				{
					_data_->_tmp59_ = dino_gettext ("Room does not exist");
					_data_->_tmp60_ = g_strdup (_data_->_tmp59_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp60_;
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_CREATION_RESTRICTED:
				{
					_data_->_tmp61_ = dino_gettext ("Not allowed to create room");
					_data_->_tmp62_ = g_strdup (_data_->_tmp61_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp62_;
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_NOT_IN_MEMBER_LIST:
				{
					_data_->_tmp63_ = dino_gettext ("Members-only room");
					_data_->_tmp64_ = g_strdup (_data_->_tmp63_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp64_;
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_USE_RESERVED_ROOMNICK:
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_NICK_CONFLICT:
				{
					_data_->_tmp65_ = dino_gettext ("Choose a different nick");
					_data_->_tmp66_ = g_strdup (_data_->_tmp65_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp66_;
					break;
				}
				case XMPP_XEP_MUC_MUC_ENTER_ERROR_OCCUPANT_LIMIT_REACHED:
				{
					_data_->_tmp67_ = dino_gettext ("Too many occupants in room");
					_data_->_tmp68_ = g_strdup (_data_->_tmp67_);
					_g_free0 (_data_->label_text);
					_data_->label_text = _data_->_tmp68_;
					break;
				}
				default:
				break;
			}
		} else {
			_data_->_tmp69_ = _data_->join_result;
			_data_->_tmp70_ = _data_->_tmp69_->stanza_error;
			if (_data_->_tmp70_ != NULL) {
				_data_->_tmp72_ = dino_ui_conference_details_fragment_get_jid (_data_->self);
				_data_->_tmp73_ = _data_->_tmp72_;
				_data_->_tmp74_ = xmpp_jid_new (_data_->_tmp73_, &_data_->_inner_error0_);
				_data_->_tmp71_ = _data_->_tmp74_;
				if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
					_xmpp_xep_muc_join_result_unref0 (_data_->join_result);
					_xmpp_jid_unref0 (_data_->parsed_jid);
					if (_data_->_inner_error0_->domain == XMPP_INVALID_JID_ERROR) {
						goto __catch0_xmpp_invalid_jid_error;
					}
					_g_free0 (_data_->label_text);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
					g_clear_error (&_data_->_inner_error0_);
					g_object_unref (_data_->_async_result);
					return FALSE;
				}
				_data_->_tmp75_ = dino_gettext ("Could not connect to %s");
				_data_->_tmp76_ = _data_->_tmp71_->domainpart;
				_data_->_tmp77_ = g_strdup_printf (_data_->_tmp75_, _data_->_tmp76_);
				_g_free0 (_data_->label_text);
				_data_->label_text = _data_->_tmp77_;
				_xmpp_jid_unref0 (_data_->_tmp71_);
			}
		}
		_xmpp_xep_muc_join_result_unref0 (_data_->join_result);
		_xmpp_jid_unref0 (_data_->parsed_jid);
	}
	goto __finally0;
	__catch0_xmpp_invalid_jid_error:
	{
		g_clear_error (&_data_->_inner_error0_);
		_data_->_tmp78_ = dino_gettext ("Invalid address");
		_data_->_tmp79_ = g_strdup (_data_->_tmp78_);
		_g_free0 (_data_->label_text);
		_data_->label_text = _data_->_tmp79_;
	}
	__finally0:
	if (G_UNLIKELY (_data_->_inner_error0_ != NULL)) {
		_g_free0 (_data_->label_text);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error0_->message, g_quark_to_string (_data_->_inner_error0_->domain), _data_->_inner_error0_->code);
		g_clear_error (&_data_->_inner_error0_);
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	_data_->_tmp80_ = _data_->self->priv->notification_label;
	_data_->_tmp81_ = _data_->label_text;
	gtk_label_set_label (_data_->_tmp80_, _data_->_tmp81_);
	_data_->_tmp82_ = _data_->self->priv->notification_revealer;
	gtk_revealer_set_reveal_child (_data_->_tmp82_, TRUE);
	_g_free0 (_data_->label_text);
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
	if (_data_->_state_ != 0) {
		while (!g_task_get_completed (_data_->_async_result)) {
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
		}
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}

static void
dino_ui_conference_details_fragment_check_if_done (DinoUiConferenceDetailsFragment* self)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	{
		XmppJid* parsed_jid = NULL;
		const gchar* _tmp0_;
		const gchar* _tmp1_;
		XmppJid* _tmp2_;
		gboolean _tmp3_ = FALSE;
		gboolean _tmp4_ = FALSE;
		const gchar* _tmp5_;
		_tmp0_ = dino_ui_conference_details_fragment_get_jid (self);
		_tmp1_ = _tmp0_;
		_tmp2_ = xmpp_jid_new (_tmp1_, &_inner_error0_);
		parsed_jid = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == XMPP_INVALID_JID_ERROR) {
				goto __catch0_xmpp_invalid_jid_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
		_tmp5_ = parsed_jid->localpart;
		if (_tmp5_ != NULL) {
			const gchar* _tmp6_;
			_tmp6_ = parsed_jid->resourcepart;
			_tmp4_ = _tmp6_ == NULL;
		} else {
			_tmp4_ = FALSE;
		}
		if (_tmp4_) {
			const gchar* _tmp7_;
			const gchar* _tmp8_;
			_tmp7_ = dino_ui_conference_details_fragment_get_nick (self);
			_tmp8_ = _tmp7_;
			_tmp3_ = _tmp8_ != NULL;
		} else {
			_tmp3_ = FALSE;
		}
		dino_ui_conference_details_fragment_set_done (self, _tmp3_);
		_xmpp_jid_unref0 (parsed_jid);
	}
	goto __finally0;
	__catch0_xmpp_invalid_jid_error:
	{
		g_clear_error (&_inner_error0_);
		dino_ui_conference_details_fragment_set_done (self, FALSE);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return;
	}
}

static void
dino_ui_conference_details_fragment_set_active_stack (DinoUiConferenceDetailsFragment* self,
                                                      GtkStack* stack)
{
	GtkStack* _tmp0_;
	GtkStack* _tmp2_;
	GtkStack* _tmp4_;
	GtkStack* _tmp6_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (stack != NULL);
	gtk_stack_set_visible_child_name (stack, "entry");
	_tmp0_ = self->priv->accounts_stack;
	if (stack != _tmp0_) {
		GtkStack* _tmp1_;
		_tmp1_ = self->priv->accounts_stack;
		gtk_stack_set_visible_child_name (_tmp1_, "label");
	}
	_tmp2_ = self->priv->jid_stack;
	if (stack != _tmp2_) {
		GtkStack* _tmp3_;
		_tmp3_ = self->priv->jid_stack;
		gtk_stack_set_visible_child_name (_tmp3_, "label");
	}
	_tmp4_ = self->priv->nick_stack;
	if (stack != _tmp4_) {
		GtkStack* _tmp5_;
		_tmp5_ = self->priv->nick_stack;
		gtk_stack_set_visible_child_name (_tmp5_, "label");
	}
	_tmp6_ = self->priv->password_stack;
	if (stack != _tmp6_) {
		GtkStack* _tmp7_;
		_tmp7_ = self->priv->password_stack;
		gtk_stack_set_visible_child_name (_tmp7_, "label");
	}
}

gboolean
dino_ui_conference_details_fragment_get_done (DinoUiConferenceDetailsFragment* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_done;
	return result;
}

static void
dino_ui_conference_details_fragment_set_done (DinoUiConferenceDetailsFragment* self,
                                              gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_conference_details_fragment_get_done (self);
	if (old_value != value) {
		self->priv->_done = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY]);
	}
}

DinoEntitiesAccount*
dino_ui_conference_details_fragment_get_account (DinoUiConferenceDetailsFragment* self)
{
	DinoEntitiesAccount* result;
	DinoUiAccountComboBox* _tmp0_;
	DinoEntitiesAccount* _tmp1_;
	DinoEntitiesAccount* _tmp2_;
	DinoEntitiesAccount* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->account_combobox;
	_tmp1_ = dino_ui_account_combo_box_get_selected (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = _g_object_ref0 (_tmp2_);
	result = _tmp3_;
	return result;
}

void
dino_ui_conference_details_fragment_set_account (DinoUiConferenceDetailsFragment* self,
                                                 DinoEntitiesAccount* value)
{
	GtkLabel* _tmp0_;
	XmppJid* _tmp1_;
	XmppJid* _tmp2_;
	XmppJid* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	DinoUiAccountComboBox* _tmp6_;
	gboolean _tmp7_ = FALSE;
	const gchar* _tmp8_;
	const gchar* _tmp9_;
	GtkStack* _tmp14_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->accounts_label;
	_tmp1_ = dino_entities_account_get_bare_jid (value);
	_tmp2_ = _tmp1_;
	_tmp3_ = _tmp2_;
	_tmp4_ = xmpp_jid_to_string (_tmp3_);
	_tmp5_ = _tmp4_;
	gtk_label_set_label (_tmp0_, _tmp5_);
	_g_free0 (_tmp5_);
	_xmpp_jid_unref0 (_tmp3_);
	_tmp6_ = self->priv->account_combobox;
	dino_ui_account_combo_box_set_selected (_tmp6_, value);
	_tmp8_ = dino_ui_conference_details_fragment_get_nick (self);
	_tmp9_ = _tmp8_;
	if (_tmp9_ == NULL) {
		const gchar* _tmp10_;
		const gchar* _tmp11_;
		_tmp10_ = dino_entities_account_get_alias (value);
		_tmp11_ = _tmp10_;
		_tmp7_ = _tmp11_ != NULL;
	} else {
		_tmp7_ = FALSE;
	}
	if (_tmp7_) {
		const gchar* _tmp12_;
		const gchar* _tmp13_;
		_tmp12_ = dino_entities_account_get_alias (value);
		_tmp13_ = _tmp12_;
		dino_ui_conference_details_fragment_set_nick (self, _tmp13_);
	}
	_tmp14_ = self->priv->accounts_stack;
	gtk_stack_set_visible_child_name (_tmp14_, "label");
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY]);
}

const gchar*
dino_ui_conference_details_fragment_get_jid (DinoUiConferenceDetailsFragment* self)
{
	const gchar* result;
	GtkEntry* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->jid_entry;
	_tmp1_ = gtk_editable_get_text ((GtkEditable*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_conference_details_fragment_set_jid (DinoUiConferenceDetailsFragment* self,
                                             const gchar* value)
{
	GtkLabel* _tmp0_;
	GtkEntry* _tmp1_;
	GtkStack* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->jid_label;
	gtk_label_set_label (_tmp0_, value);
	_tmp1_ = self->priv->jid_entry;
	gtk_editable_set_text ((GtkEditable*) _tmp1_, value);
	_tmp2_ = self->priv->jid_stack;
	gtk_stack_set_visible_child_name (_tmp2_, "label");
	dino_ui_conference_details_fragment_check_if_done (self);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY]);
}

const gchar*
dino_ui_conference_details_fragment_get_nick (DinoUiConferenceDetailsFragment* self)
{
	const gchar* result;
	const gchar* _tmp0_ = NULL;
	GtkEntry* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = self->priv->nick_entry;
	_tmp2_ = gtk_editable_get_text ((GtkEditable*) _tmp1_);
	_tmp3_ = _tmp2_;
	if (g_strcmp0 (_tmp3_, "") != 0) {
		GtkEntry* _tmp4_;
		const gchar* _tmp5_;
		const gchar* _tmp6_;
		_tmp4_ = self->priv->nick_entry;
		_tmp5_ = gtk_editable_get_text ((GtkEditable*) _tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_;
	} else {
		_tmp0_ = NULL;
	}
	result = _tmp0_;
	return result;
}

void
dino_ui_conference_details_fragment_set_nick (DinoUiConferenceDetailsFragment* self,
                                              const gchar* value)
{
	const gchar* _tmp0_ = NULL;
	GtkLabel* _tmp1_;
	const gchar* _tmp2_ = NULL;
	GtkEntry* _tmp3_;
	GtkStack* _tmp4_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	if (_tmp0_ == NULL) {
		_tmp0_ = "";
	}
	_tmp1_ = self->priv->nick_label;
	gtk_label_set_label (_tmp1_, _tmp0_);
	_tmp2_ = value;
	if (_tmp2_ == NULL) {
		_tmp2_ = "";
	}
	_tmp3_ = self->priv->nick_entry;
	gtk_editable_set_text ((GtkEditable*) _tmp3_, _tmp2_);
	_tmp4_ = self->priv->nick_stack;
	gtk_stack_set_visible_child_name (_tmp4_, "label");
	dino_ui_conference_details_fragment_check_if_done (self);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY]);
}

const gchar*
dino_ui_conference_details_fragment_get_password (DinoUiConferenceDetailsFragment* self)
{
	const gchar* result;
	const gchar* _tmp0_ = NULL;
	GtkEntry* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = self->priv->password_entry;
	_tmp2_ = gtk_editable_get_text ((GtkEditable*) _tmp1_);
	_tmp3_ = _tmp2_;
	if (g_strcmp0 (_tmp3_, "") == 0) {
		_tmp0_ = NULL;
	} else {
		GtkEntry* _tmp4_;
		const gchar* _tmp5_;
		const gchar* _tmp6_;
		_tmp4_ = self->priv->password_entry;
		_tmp5_ = gtk_editable_get_text ((GtkEditable*) _tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_;
	}
	result = _tmp0_;
	return result;
}

void
dino_ui_conference_details_fragment_set_password (DinoUiConferenceDetailsFragment* self,
                                                  const gchar* value)
{
	GtkLabel* _tmp0_;
	GtkEntry* _tmp1_;
	GtkStack* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->password_label;
	gtk_label_set_label (_tmp0_, value);
	_tmp1_ = self->priv->password_entry;
	gtk_editable_set_text ((GtkEditable*) _tmp1_, value);
	_tmp2_ = self->priv->nick_stack;
	gtk_stack_set_visible_child_name (_tmp2_, "label");
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY]);
}

gboolean
dino_ui_conference_details_fragment_get_fragment_active (DinoUiConferenceDetailsFragment* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_fragment_active;
	return result;
}

void
dino_ui_conference_details_fragment_set_fragment_active (DinoUiConferenceDetailsFragment* self,
                                                         gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_conference_details_fragment_get_fragment_active (self);
	if (old_value != value) {
		self->priv->_fragment_active = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY]);
	}
}

GtkButton*
dino_ui_conference_details_fragment_get_ok_button (DinoUiConferenceDetailsFragment* self)
{
	GtkButton* result;
	GtkButton* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->ok_button_;
	result = _tmp0_;
	return result;
}

static void
_dino_ui_conference_details_fragment_ok_button____lambda299_ (DinoUiConferenceDetailsFragment* self)
{
	dino_ui_conference_details_fragment_on_ok_button_clicked (self, NULL, NULL);
}

static void
__dino_ui_conference_details_fragment_ok_button____lambda299__gtk_button_clicked (GtkButton* _sender,
                                                                                  gpointer self)
{
	_dino_ui_conference_details_fragment_ok_button____lambda299_ ((DinoUiConferenceDetailsFragment*) self);
}

void
dino_ui_conference_details_fragment_set_ok_button (DinoUiConferenceDetailsFragment* self,
                                                   GtkButton* value)
{
	g_return_if_fail (self != NULL);
	if (value != NULL) {
		GtkButton* _tmp0_;
		g_signal_connect_object (value, "clicked", (GCallback) __dino_ui_conference_details_fragment_ok_button____lambda299__gtk_button_clicked, self, 0);
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->ok_button_);
		self->priv->ok_button_ = _tmp0_;
	}
	g_object_notify_by_pspec ((GObject *) self, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY]);
}

static void
dino_ui_conference_details_fragment_class_init (DinoUiConferenceDetailsFragmentClass * klass,
                                                gpointer klass_data)
{
	dino_ui_conference_details_fragment_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiConferenceDetailsFragment_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_conference_details_fragment_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_conference_details_fragment_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_conference_details_fragment_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY] = g_param_spec_boolean ("done", "done", "done", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY] = g_param_spec_object ("account", "account", "account", DINO_ENTITIES_TYPE_ACCOUNT, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY] = g_param_spec_string ("jid", "jid", "jid", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY] = g_param_spec_string ("nick", "nick", "nick", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY] = g_param_spec_string ("password", "password", "password", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY] = g_param_spec_boolean ("fragment-active", "fragment-active", "fragment-active", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY, dino_ui_conference_details_fragment_properties[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY] = g_param_spec_object ("ok-button", "ok-button", "ok-button", gtk_button_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/im/dino/Dino/add_conversation/conference_details_fragment.ui");
	dino_ui_conference_details_fragment_signals[DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JOINED_SIGNAL] = g_signal_new ("joined", DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "accounts_stack", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, accounts_stack));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "accounts_button", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, accounts_button));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "accounts_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, accounts_label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "account_combobox", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, account_combobox));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "jid_stack", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, jid_stack));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "jid_button", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, jid_button));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "jid_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, jid_label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "jid_entry", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, jid_entry));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "nick_stack", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, nick_stack));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "nick_button", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, nick_button));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "nick_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, nick_label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "nick_entry", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, nick_entry));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "password_stack", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, password_stack));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "password_button", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, password_button));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "password_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, password_label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "password_text_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, password_text_label));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "password_entry", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, password_entry));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "notification_revealer", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, notification_revealer));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "notification_button", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, notification_button));
	gtk_widget_class_bind_template_child_full (GTK_WIDGET_CLASS (klass), "notification_label", FALSE, DinoUiConferenceDetailsFragment_private_offset + G_STRUCT_OFFSET (DinoUiConferenceDetailsFragmentPrivate, notification_label));
}

static void
dino_ui_conference_details_fragment_instance_init (DinoUiConferenceDetailsFragment * self,
                                                   gpointer klass)
{
	self->priv = dino_ui_conference_details_fragment_get_instance_private (self);
	self->priv->_fragment_active = TRUE;
	g_type_ensure (DINO_UI_TYPE_ACCOUNT_COMBO_BOX);
	gtk_widget_init_template (GTK_WIDGET (self));
}

static void
dino_ui_conference_details_fragment_finalize (GObject * obj)
{
	DinoUiConferenceDetailsFragment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragment);
	_g_object_unref0 (self->priv->stream_interactor);
	_g_object_unref0 (self->priv->ok_button_);
	G_OBJECT_CLASS (dino_ui_conference_details_fragment_parent_class)->finalize (obj);
}

static GType
dino_ui_conference_details_fragment_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiConferenceDetailsFragmentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_conference_details_fragment_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiConferenceDetailsFragment), 0, (GInstanceInitFunc) dino_ui_conference_details_fragment_instance_init, NULL };
	GType dino_ui_conference_details_fragment_type_id;
	dino_ui_conference_details_fragment_type_id = g_type_register_static (gtk_box_get_type (), "DinoUiConferenceDetailsFragment", &g_define_type_info, 0);
	DinoUiConferenceDetailsFragment_private_offset = g_type_add_instance_private (dino_ui_conference_details_fragment_type_id, sizeof (DinoUiConferenceDetailsFragmentPrivate));
	return dino_ui_conference_details_fragment_type_id;
}

GType
dino_ui_conference_details_fragment_get_type (void)
{
	static volatile gsize dino_ui_conference_details_fragment_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_conference_details_fragment_type_id__once)) {
		GType dino_ui_conference_details_fragment_type_id;
		dino_ui_conference_details_fragment_type_id = dino_ui_conference_details_fragment_get_type_once ();
		g_once_init_leave (&dino_ui_conference_details_fragment_type_id__once, dino_ui_conference_details_fragment_type_id);
	}
	return dino_ui_conference_details_fragment_type_id__once;
}

static void
_vala_dino_ui_conference_details_fragment_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec)
{
	DinoUiConferenceDetailsFragment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragment);
	switch (property_id) {
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY:
		g_value_set_boolean (value, dino_ui_conference_details_fragment_get_done (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY:
		g_value_take_object (value, dino_ui_conference_details_fragment_get_account (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY:
		g_value_set_string (value, dino_ui_conference_details_fragment_get_jid (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY:
		g_value_set_string (value, dino_ui_conference_details_fragment_get_nick (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY:
		g_value_set_string (value, dino_ui_conference_details_fragment_get_password (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY:
		g_value_set_boolean (value, dino_ui_conference_details_fragment_get_fragment_active (self));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY:
		g_value_set_object (value, dino_ui_conference_details_fragment_get_ok_button (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_conference_details_fragment_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec)
{
	DinoUiConferenceDetailsFragment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_TYPE_CONFERENCE_DETAILS_FRAGMENT, DinoUiConferenceDetailsFragment);
	switch (property_id) {
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_DONE_PROPERTY:
		dino_ui_conference_details_fragment_set_done (self, g_value_get_boolean (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_ACCOUNT_PROPERTY:
		dino_ui_conference_details_fragment_set_account (self, g_value_get_object (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_JID_PROPERTY:
		dino_ui_conference_details_fragment_set_jid (self, g_value_get_string (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_NICK_PROPERTY:
		dino_ui_conference_details_fragment_set_nick (self, g_value_get_string (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_PASSWORD_PROPERTY:
		dino_ui_conference_details_fragment_set_password (self, g_value_get_string (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_FRAGMENT_ACTIVE_PROPERTY:
		dino_ui_conference_details_fragment_set_fragment_active (self, g_value_get_boolean (value));
		break;
		case DINO_UI_CONFERENCE_DETAILS_FRAGMENT_OK_BUTTON_PROPERTY:
		dino_ui_conference_details_fragment_set_ok_button (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

