/* session_info.c generated by valac 0.56.19, the Vala compiler
 * generated from session_info.vala, do not modify */

#include "xmpp-vala.h"
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType xmpp_xep_jingle_session_info_ns_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

void
xmpp_xep_jingle_session_info_ns_handle_content_session_info (XmppXepJingleSessionInfoNs* self,
                                                             XmppXmppStream* stream,
                                                             XmppXepJingleSession* session,
                                                             XmppStanzaNode* info,
                                                             XmppIqStanza* iq,
                                                             GError** error)
{
	XmppXepJingleSessionInfoNsIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = XMPP_XEP_JINGLE_SESSION_INFO_NS_GET_INTERFACE (self);
	if (_iface_->handle_content_session_info) {
		_iface_->handle_content_session_info (self, stream, session, info, iq, error);
	}
}

const gchar*
xmpp_xep_jingle_session_info_ns_get_ns_uri (XmppXepJingleSessionInfoNs* self)
{
	XmppXepJingleSessionInfoNsIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = XMPP_XEP_JINGLE_SESSION_INFO_NS_GET_INTERFACE (self);
	if (_iface_->get_ns_uri) {
		return _iface_->get_ns_uri (self);
	}
	return NULL;
}

static void
xmpp_xep_jingle_session_info_ns_default_init (XmppXepJingleSessionInfoNsIface * iface,
                                              gpointer iface_data)
{
	g_object_interface_install_property (iface, g_param_spec_string ("ns-uri", "ns-uri", "ns-uri", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
xmpp_xep_jingle_session_info_ns_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (XmppXepJingleSessionInfoNsIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) xmpp_xep_jingle_session_info_ns_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType xmpp_xep_jingle_session_info_ns_type_id;
	xmpp_xep_jingle_session_info_ns_type_id = g_type_register_static (G_TYPE_INTERFACE, "XmppXepJingleSessionInfoNs", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (xmpp_xep_jingle_session_info_ns_type_id, G_TYPE_OBJECT);
	return xmpp_xep_jingle_session_info_ns_type_id;
}

GType
xmpp_xep_jingle_session_info_ns_get_type (void)
{
	static volatile gsize xmpp_xep_jingle_session_info_ns_type_id__once = 0;
	if (g_once_init_enter (&xmpp_xep_jingle_session_info_ns_type_id__once)) {
		GType xmpp_xep_jingle_session_info_ns_type_id;
		xmpp_xep_jingle_session_info_ns_type_id = xmpp_xep_jingle_session_info_ns_get_type_once ();
		g_once_init_leave (&xmpp_xep_jingle_session_info_ns_type_id__once, xmpp_xep_jingle_session_info_ns_type_id);
	}
	return xmpp_xep_jingle_session_info_ns_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

