/* label_hybrid.c generated by valac 0.56.19, the Vala compiler
 * generated from label_hybrid.vala, do not modify */

#include <gtk/gtk.h>
#include <glib-object.h>
#include <pango/pango.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_UTIL_TYPE_LABEL_HYBRID (dino_ui_util_label_hybrid_get_type ())
#define DINO_UI_UTIL_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybrid))
#define DINO_UI_UTIL_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybridClass))
#define DINO_UI_UTIL_IS_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID))
#define DINO_UI_UTIL_IS_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_UTIL_TYPE_LABEL_HYBRID))
#define DINO_UI_UTIL_LABEL_HYBRID_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybridClass))

typedef struct _DinoUiUtilLabelHybrid DinoUiUtilLabelHybrid;
typedef struct _DinoUiUtilLabelHybridClass DinoUiUtilLabelHybridClass;
typedef struct _DinoUiUtilLabelHybridPrivate DinoUiUtilLabelHybridPrivate;
enum  {
	DINO_UI_UTIL_LABEL_HYBRID_0_PROPERTY,
	DINO_UI_UTIL_LABEL_HYBRID_NUM_PROPERTIES
};
static GParamSpec* dino_ui_util_label_hybrid_properties[DINO_UI_UTIL_LABEL_HYBRID_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID (dino_ui_util_entry_label_hybrid_get_type ())
#define DINO_UI_UTIL_ENTRY_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybrid))
#define DINO_UI_UTIL_ENTRY_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybridClass))
#define DINO_UI_UTIL_IS_ENTRY_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID))
#define DINO_UI_UTIL_IS_ENTRY_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID))
#define DINO_UI_UTIL_ENTRY_LABEL_HYBRID_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybridClass))

typedef struct _DinoUiUtilEntryLabelHybrid DinoUiUtilEntryLabelHybrid;
typedef struct _DinoUiUtilEntryLabelHybridClass DinoUiUtilEntryLabelHybridClass;
typedef struct _DinoUiUtilEntryLabelHybridPrivate DinoUiUtilEntryLabelHybridPrivate;
enum  {
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_0_PROPERTY,
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY,
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY,
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY,
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY,
	DINO_UI_UTIL_ENTRY_LABEL_HYBRID_NUM_PROPERTIES
};
static GParamSpec* dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

#define DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID (dino_ui_util_combo_box_text_label_hybrid_get_type ())
#define DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybrid))
#define DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybridClass))
#define DINO_UI_UTIL_IS_COMBO_BOX_TEXT_LABEL_HYBRID(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID))
#define DINO_UI_UTIL_IS_COMBO_BOX_TEXT_LABEL_HYBRID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID))
#define DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybridClass))

typedef struct _DinoUiUtilComboBoxTextLabelHybrid DinoUiUtilComboBoxTextLabelHybrid;
typedef struct _DinoUiUtilComboBoxTextLabelHybridClass DinoUiUtilComboBoxTextLabelHybridClass;
typedef struct _DinoUiUtilComboBoxTextLabelHybridPrivate DinoUiUtilComboBoxTextLabelHybridPrivate;
enum  {
	DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_0_PROPERTY,
	DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY,
	DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY,
	DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY,
	DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_NUM_PROPERTIES
};
static GParamSpec* dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_NUM_PROPERTIES];
typedef struct _Block71Data Block71Data;

#define DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP (dino_ui_util_label_hybrid_group_get_type ())
#define DINO_UI_UTIL_LABEL_HYBRID_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP, DinoUiUtilLabelHybridGroup))
#define DINO_UI_UTIL_LABEL_HYBRID_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP, DinoUiUtilLabelHybridGroupClass))
#define DINO_UI_UTIL_IS_LABEL_HYBRID_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP))
#define DINO_UI_UTIL_IS_LABEL_HYBRID_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP))
#define DINO_UI_UTIL_LABEL_HYBRID_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP, DinoUiUtilLabelHybridGroupClass))

typedef struct _DinoUiUtilLabelHybridGroup DinoUiUtilLabelHybridGroup;
typedef struct _DinoUiUtilLabelHybridGroupClass DinoUiUtilLabelHybridGroupClass;
typedef struct _DinoUiUtilLabelHybridGroupPrivate DinoUiUtilLabelHybridGroupPrivate;
typedef struct _Block72Data Block72Data;
#define _dino_ui_util_label_hybrid_group_unref0(var) ((var == NULL) ? NULL : (var = (dino_ui_util_label_hybrid_group_unref (var), NULL)))
typedef struct _DinoUiUtilParamSpecLabelHybridGroup DinoUiUtilParamSpecLabelHybridGroup;

struct _DinoUiUtilLabelHybrid {
	GtkWidget parent_instance;
	DinoUiUtilLabelHybridPrivate * priv;
	GtkStack* stack;
	GtkLabel* label;
	GtkButton* button;
};

struct _DinoUiUtilLabelHybridClass {
	GtkWidgetClass parent_class;
	void (*init) (DinoUiUtilLabelHybrid* self, GtkWidget* widget);
};

struct _DinoUiUtilEntryLabelHybrid {
	DinoUiUtilLabelHybrid parent_instance;
	DinoUiUtilEntryLabelHybridPrivate * priv;
};

struct _DinoUiUtilEntryLabelHybridClass {
	DinoUiUtilLabelHybridClass parent_class;
};

struct _DinoUiUtilEntryLabelHybridPrivate {
	GtkEntry* entry_;
};

struct _DinoUiUtilComboBoxTextLabelHybrid {
	DinoUiUtilLabelHybrid parent_instance;
	DinoUiUtilComboBoxTextLabelHybridPrivate * priv;
};

struct _DinoUiUtilComboBoxTextLabelHybridClass {
	DinoUiUtilLabelHybridClass parent_class;
};

struct _DinoUiUtilComboBoxTextLabelHybridPrivate {
	GtkComboBoxText* combobox_;
};

struct _Block71Data {
	int _ref_count_;
	DinoUiUtilComboBoxTextLabelHybrid* self;
	GtkComboBoxText* combobox;
};

struct _DinoUiUtilLabelHybridGroup {
	GTypeInstance parent_instance;
	volatile int ref_count;
	DinoUiUtilLabelHybridGroupPrivate * priv;
};

struct _DinoUiUtilLabelHybridGroupClass {
	GTypeClass parent_class;
	void (*finalize) (DinoUiUtilLabelHybridGroup *self);
};

struct _DinoUiUtilLabelHybridGroupPrivate {
	GeeList* hybrids;
};

struct _Block72Data {
	int _ref_count_;
	DinoUiUtilLabelHybridGroup* self;
	DinoUiUtilLabelHybrid* hybrid;
};

struct _DinoUiUtilParamSpecLabelHybridGroup {
	GParamSpec parent_instance;
};

static gpointer dino_ui_util_label_hybrid_parent_class = NULL;
static gint DinoUiUtilEntryLabelHybrid_private_offset;
static gpointer dino_ui_util_entry_label_hybrid_parent_class = NULL;
static gint DinoUiUtilComboBoxTextLabelHybrid_private_offset;
static gpointer dino_ui_util_combo_box_text_label_hybrid_parent_class = NULL;
static gint DinoUiUtilLabelHybridGroup_private_offset;
static gpointer dino_ui_util_label_hybrid_group_parent_class = NULL;

VALA_EXTERN GType dino_ui_util_label_hybrid_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiUtilLabelHybrid, g_object_unref)
VALA_EXTERN void dino_ui_util_label_hybrid_init (DinoUiUtilLabelHybrid* self,
                                     GtkWidget* widget);
static void dino_ui_util_label_hybrid_real_init (DinoUiUtilLabelHybrid* self,
                                          GtkWidget* widget);
static void __lambda192_ (DinoUiUtilLabelHybrid* self);
VALA_EXTERN void dino_ui_util_label_hybrid_show_widget (DinoUiUtilLabelHybrid* self);
static void ___lambda192__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
VALA_EXTERN void dino_ui_util_label_hybrid_show_label (DinoUiUtilLabelHybrid* self);
static void dino_ui_util_label_hybrid_real_dispose (GObject* base);
VALA_EXTERN DinoUiUtilLabelHybrid* dino_ui_util_label_hybrid_new (void);
VALA_EXTERN DinoUiUtilLabelHybrid* dino_ui_util_label_hybrid_construct (GType object_type);
static void dino_ui_util_label_hybrid_finalize (GObject * obj);
static GType dino_ui_util_label_hybrid_get_type_once (void);
VALA_EXTERN GType dino_ui_util_entry_label_hybrid_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiUtilEntryLabelHybrid, g_object_unref)
VALA_EXTERN DinoUiUtilEntryLabelHybrid* dino_ui_util_entry_label_hybrid_new_wrap (GtkEntry* e);
VALA_EXTERN DinoUiUtilEntryLabelHybrid* dino_ui_util_entry_label_hybrid_construct_wrap (GType object_type,
                                                                            GtkEntry* e);
static void dino_ui_util_entry_label_hybrid_real_init (DinoUiUtilLabelHybrid* base,
                                                GtkWidget* widget);
VALA_EXTERN void dino_ui_util_entry_label_hybrid_set_entry (DinoUiUtilEntryLabelHybrid* self,
                                                GtkEntry* value);
VALA_EXTERN GtkEntry* dino_ui_util_entry_label_hybrid_get_entry (DinoUiUtilEntryLabelHybrid* self);
static void dino_ui_util_entry_label_hybrid_update_label (DinoUiUtilEntryLabelHybrid* self);
static void dino_ui_util_entry_label_hybrid_on_key_released (DinoUiUtilEntryLabelHybrid* self,
                                                      guint keyval);
static void _dino_ui_util_entry_label_hybrid_on_key_released_gtk_event_controller_key_key_released (GtkEventControllerKey* _sender,
                                                                                             guint keyval,
                                                                                             guint keycode,
                                                                                             GdkModifierType state,
                                                                                             gpointer self);
static void _dino_ui_util_entry_label_hybrid_update_label_gtk_editable_changed (GtkEditable* _sender,
                                                                         gpointer self);
static void _dino_ui_util_entry_label_hybrid_update_label_gtk_event_controller_focus_leave (GtkEventControllerFocus* _sender,
                                                                                     gpointer self);
VALA_EXTERN gboolean dino_ui_util_entry_label_hybrid_get_visibility (DinoUiUtilEntryLabelHybrid* self);
VALA_EXTERN DinoUiUtilEntryLabelHybrid* dino_ui_util_entry_label_hybrid_new (void);
VALA_EXTERN DinoUiUtilEntryLabelHybrid* dino_ui_util_entry_label_hybrid_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_util_entry_label_hybrid_get_text (DinoUiUtilEntryLabelHybrid* self);
VALA_EXTERN void dino_ui_util_entry_label_hybrid_set_text (DinoUiUtilEntryLabelHybrid* self,
                                               const gchar* value);
VALA_EXTERN void dino_ui_util_entry_label_hybrid_set_visibility (DinoUiUtilEntryLabelHybrid* self,
                                                     gboolean value);
VALA_EXTERN gfloat dino_ui_util_entry_label_hybrid_get_xalign (DinoUiUtilEntryLabelHybrid* self);
VALA_EXTERN void dino_ui_util_entry_label_hybrid_set_xalign (DinoUiUtilEntryLabelHybrid* self,
                                                 gfloat value);
static void dino_ui_util_entry_label_hybrid_finalize (GObject * obj);
static GType dino_ui_util_entry_label_hybrid_get_type_once (void);
static void _vala_dino_ui_util_entry_label_hybrid_get_property (GObject * object,
                                                         guint property_id,
                                                         GValue * value,
                                                         GParamSpec * pspec);
static void _vala_dino_ui_util_entry_label_hybrid_set_property (GObject * object,
                                                         guint property_id,
                                                         const GValue * value,
                                                         GParamSpec * pspec);
VALA_EXTERN GType dino_ui_util_combo_box_text_label_hybrid_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiUtilComboBoxTextLabelHybrid, g_object_unref)
VALA_EXTERN DinoUiUtilComboBoxTextLabelHybrid* dino_ui_util_combo_box_text_label_hybrid_new_wrap (GtkComboBoxText* cb);
VALA_EXTERN DinoUiUtilComboBoxTextLabelHybrid* dino_ui_util_combo_box_text_label_hybrid_construct_wrap (GType object_type,
                                                                                            GtkComboBoxText* cb);
VALA_EXTERN void dino_ui_util_combo_box_text_label_hybrid_append (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                      const gchar* id,
                                                      const gchar* text);
VALA_EXTERN GtkComboBoxText* dino_ui_util_combo_box_text_label_hybrid_get_combobox (DinoUiUtilComboBoxTextLabelHybrid* self);
VALA_EXTERN gchar* dino_ui_util_combo_box_text_label_hybrid_get_active_text (DinoUiUtilComboBoxTextLabelHybrid* self);
static void dino_ui_util_combo_box_text_label_hybrid_real_init (DinoUiUtilLabelHybrid* base,
                                                         GtkWidget* widget);
static Block71Data* block71_data_ref (Block71Data* _data71_);
static void block71_data_unref (void * _userdata_);
static void dino_ui_util_combo_box_text_label_hybrid_update_label (DinoUiUtilComboBoxTextLabelHybrid* self);
static void __lambda193_ (DinoUiUtilComboBoxTextLabelHybrid* self);
static void ___lambda193__gtk_combo_box_changed (GtkComboBox* _sender,
                                          gpointer self);
static void __lambda194_ (Block71Data* _data71_);
static void ___lambda194__gtk_button_clicked (GtkButton* _sender,
                                       gpointer self);
static void dino_ui_util_combo_box_text_label_hybrid_on_focus_leave (DinoUiUtilComboBoxTextLabelHybrid* self);
static void _dino_ui_util_combo_box_text_label_hybrid_on_focus_leave_gtk_event_controller_focus_leave (GtkEventControllerFocus* _sender,
                                                                                                gpointer self);
VALA_EXTERN DinoUiUtilComboBoxTextLabelHybrid* dino_ui_util_combo_box_text_label_hybrid_new (void);
VALA_EXTERN DinoUiUtilComboBoxTextLabelHybrid* dino_ui_util_combo_box_text_label_hybrid_construct (GType object_type);
VALA_EXTERN gint dino_ui_util_combo_box_text_label_hybrid_get_active (DinoUiUtilComboBoxTextLabelHybrid* self);
VALA_EXTERN void dino_ui_util_combo_box_text_label_hybrid_set_active (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                          gint value);
VALA_EXTERN gfloat dino_ui_util_combo_box_text_label_hybrid_get_xalign (DinoUiUtilComboBoxTextLabelHybrid* self);
VALA_EXTERN void dino_ui_util_combo_box_text_label_hybrid_set_xalign (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                          gfloat value);
VALA_EXTERN void dino_ui_util_combo_box_text_label_hybrid_set_combobox (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                            GtkComboBoxText* value);
static void dino_ui_util_combo_box_text_label_hybrid_finalize (GObject * obj);
static GType dino_ui_util_combo_box_text_label_hybrid_get_type_once (void);
static void _vala_dino_ui_util_combo_box_text_label_hybrid_get_property (GObject * object,
                                                                  guint property_id,
                                                                  GValue * value,
                                                                  GParamSpec * pspec);
static void _vala_dino_ui_util_combo_box_text_label_hybrid_set_property (GObject * object,
                                                                  guint property_id,
                                                                  const GValue * value,
                                                                  GParamSpec * pspec);
VALA_EXTERN gpointer dino_ui_util_label_hybrid_group_ref (gpointer instance);
VALA_EXTERN void dino_ui_util_label_hybrid_group_unref (gpointer instance);
VALA_EXTERN GParamSpec* dino_ui_util_param_spec_label_hybrid_group (const gchar* name,
                                                        const gchar* nick,
                                                        const gchar* blurb,
                                                        GType object_type,
                                                        GParamFlags flags);
VALA_EXTERN void dino_ui_util_value_set_label_hybrid_group (GValue* value,
                                                gpointer v_object);
VALA_EXTERN void dino_ui_util_value_take_label_hybrid_group (GValue* value,
                                                 gpointer v_object);
VALA_EXTERN gpointer dino_ui_util_value_get_label_hybrid_group (const GValue* value);
VALA_EXTERN GType dino_ui_util_label_hybrid_group_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiUtilLabelHybridGroup, dino_ui_util_label_hybrid_group_unref)
VALA_EXTERN void dino_ui_util_label_hybrid_group_add (DinoUiUtilLabelHybridGroup* self,
                                          DinoUiUtilLabelHybrid* hybrid);
static Block72Data* block72_data_ref (Block72Data* _data72_);
static void block72_data_unref (void * _userdata_);
static void __lambda337_ (Block72Data* _data72_);
static void ___lambda337__g_object_notify (GObject* _sender,
                                    GParamSpec* pspec,
                                    gpointer self);
VALA_EXTERN DinoUiUtilLabelHybridGroup* dino_ui_util_label_hybrid_group_new (void);
VALA_EXTERN DinoUiUtilLabelHybridGroup* dino_ui_util_label_hybrid_group_construct (GType object_type);
static void dino_ui_util_label_hybrid_group_finalize (DinoUiUtilLabelHybridGroup * obj);
static GType dino_ui_util_label_hybrid_group_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static void
__lambda192_ (DinoUiUtilLabelHybrid* self)
{
	dino_ui_util_label_hybrid_show_widget (self);
}

static void
___lambda192__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda192_ ((DinoUiUtilLabelHybrid*) self);
}

static void
dino_ui_util_label_hybrid_real_init (DinoUiUtilLabelHybrid* self,
                                     GtkWidget* widget)
{
	GtkBinLayout* _tmp0_;
	GtkStack* _tmp1_;
	GtkButton* _tmp2_;
	GtkLabel* _tmp3_;
	GtkStack* _tmp4_;
	GtkButton* _tmp5_;
	GtkStack* _tmp6_;
	GtkButton* _tmp7_;
	g_return_if_fail (widget != NULL);
	_tmp0_ = (GtkBinLayout*) gtk_bin_layout_new ();
	gtk_widget_set_layout_manager ((GtkWidget*) self, (GtkLayoutManager*) _tmp0_);
	_tmp1_ = self->stack;
	gtk_widget_set_parent ((GtkWidget*) _tmp1_, (GtkWidget*) self);
	_tmp2_ = self->button;
	_tmp3_ = self->label;
	gtk_button_set_child (_tmp2_, (GtkWidget*) _tmp3_);
	_tmp4_ = self->stack;
	_tmp5_ = self->button;
	gtk_stack_add_named (_tmp4_, (GtkWidget*) _tmp5_, "label");
	_tmp6_ = self->stack;
	gtk_stack_add_named (_tmp6_, widget, "widget");
	_tmp7_ = self->button;
	g_signal_connect_object (_tmp7_, "clicked", (GCallback) ___lambda192__gtk_button_clicked, self, 0);
}

void
dino_ui_util_label_hybrid_init (DinoUiUtilLabelHybrid* self,
                                GtkWidget* widget)
{
	DinoUiUtilLabelHybridClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = DINO_UI_UTIL_LABEL_HYBRID_GET_CLASS (self);
	if (_klass_->init) {
		_klass_->init (self, widget);
	}
}

void
dino_ui_util_label_hybrid_show_widget (DinoUiUtilLabelHybrid* self)
{
	GtkStack* _tmp0_;
	GtkStack* _tmp1_;
	GtkWidget* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->stack;
	gtk_stack_set_visible_child_name (_tmp0_, "widget");
	_tmp1_ = self->stack;
	_tmp2_ = gtk_stack_get_child_by_name (_tmp1_, "widget");
	gtk_widget_grab_focus (_tmp2_);
}

void
dino_ui_util_label_hybrid_show_label (DinoUiUtilLabelHybrid* self)
{
	GtkStack* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->stack;
	gtk_stack_set_visible_child_name (_tmp0_, "label");
}

static void
dino_ui_util_label_hybrid_real_dispose (GObject* base)
{
	DinoUiUtilLabelHybrid * self;
	GtkStack* _tmp0_;
	self = (DinoUiUtilLabelHybrid*) base;
	_tmp0_ = self->stack;
	gtk_widget_unparent ((GtkWidget*) _tmp0_);
}

DinoUiUtilLabelHybrid*
dino_ui_util_label_hybrid_construct (GType object_type)
{
	DinoUiUtilLabelHybrid * self = NULL;
	self = (DinoUiUtilLabelHybrid*) g_object_new (object_type, NULL);
	return self;
}

DinoUiUtilLabelHybrid*
dino_ui_util_label_hybrid_new (void)
{
	return dino_ui_util_label_hybrid_construct (DINO_UI_UTIL_TYPE_LABEL_HYBRID);
}

static void
dino_ui_util_label_hybrid_class_init (DinoUiUtilLabelHybridClass * klass,
                                      gpointer klass_data)
{
	dino_ui_util_label_hybrid_parent_class = g_type_class_peek_parent (klass);
	((DinoUiUtilLabelHybridClass *) klass)->init = (void (*) (DinoUiUtilLabelHybrid*, GtkWidget*)) dino_ui_util_label_hybrid_real_init;
	((GObjectClass *) klass)->dispose = (void (*) (GObject*)) dino_ui_util_label_hybrid_real_dispose;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_util_label_hybrid_finalize;
}

static void
dino_ui_util_label_hybrid_instance_init (DinoUiUtilLabelHybrid * self,
                                         gpointer klass)
{
	GtkStack* _tmp0_;
	GtkLabel* _tmp1_ = NULL;
	GtkButton* _tmp2_ = NULL;
	_tmp0_ = (GtkStack*) gtk_stack_new ();
	g_object_ref_sink (_tmp0_);
	self->stack = _tmp0_;
	_tmp1_ = (GtkLabel*) gtk_label_new ("");
	gtk_label_set_max_width_chars (_tmp1_, 1);
	gtk_label_set_ellipsize (_tmp1_, PANGO_ELLIPSIZE_END);
	g_object_ref_sink (_tmp1_);
	self->label = _tmp1_;
	_tmp2_ = (GtkButton*) gtk_button_new ();
	gtk_button_set_has_frame (_tmp2_, FALSE);
	g_object_ref_sink (_tmp2_);
	self->button = _tmp2_;
}

static void
dino_ui_util_label_hybrid_finalize (GObject * obj)
{
	DinoUiUtilLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybrid);
	_g_object_unref0 (self->stack);
	_g_object_unref0 (self->label);
	_g_object_unref0 (self->button);
	G_OBJECT_CLASS (dino_ui_util_label_hybrid_parent_class)->finalize (obj);
}

static GType
dino_ui_util_label_hybrid_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiUtilLabelHybridClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_util_label_hybrid_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiUtilLabelHybrid), 0, (GInstanceInitFunc) dino_ui_util_label_hybrid_instance_init, NULL };
	GType dino_ui_util_label_hybrid_type_id;
	dino_ui_util_label_hybrid_type_id = g_type_register_static (gtk_widget_get_type (), "DinoUiUtilLabelHybrid", &g_define_type_info, 0);
	return dino_ui_util_label_hybrid_type_id;
}

GType
dino_ui_util_label_hybrid_get_type (void)
{
	static volatile gsize dino_ui_util_label_hybrid_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_util_label_hybrid_type_id__once)) {
		GType dino_ui_util_label_hybrid_type_id;
		dino_ui_util_label_hybrid_type_id = dino_ui_util_label_hybrid_get_type_once ();
		g_once_init_leave (&dino_ui_util_label_hybrid_type_id__once, dino_ui_util_label_hybrid_type_id);
	}
	return dino_ui_util_label_hybrid_type_id__once;
}

static inline gpointer
dino_ui_util_entry_label_hybrid_get_instance_private (DinoUiUtilEntryLabelHybrid* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiUtilEntryLabelHybrid_private_offset);
}

DinoUiUtilEntryLabelHybrid*
dino_ui_util_entry_label_hybrid_construct_wrap (GType object_type,
                                                GtkEntry* e)
{
	DinoUiUtilEntryLabelHybrid * self = NULL;
	g_return_val_if_fail (e != NULL, NULL);
	self = (DinoUiUtilEntryLabelHybrid*) dino_ui_util_label_hybrid_construct (object_type);
	dino_ui_util_label_hybrid_init ((DinoUiUtilLabelHybrid*) self, (GtkWidget*) e);
	return self;
}

DinoUiUtilEntryLabelHybrid*
dino_ui_util_entry_label_hybrid_new_wrap (GtkEntry* e)
{
	return dino_ui_util_entry_label_hybrid_construct_wrap (DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, e);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
_dino_ui_util_entry_label_hybrid_on_key_released_gtk_event_controller_key_key_released (GtkEventControllerKey* _sender,
                                                                                        guint keyval,
                                                                                        guint keycode,
                                                                                        GdkModifierType state,
                                                                                        gpointer self)
{
	dino_ui_util_entry_label_hybrid_on_key_released ((DinoUiUtilEntryLabelHybrid*) self, keyval);
}

static void
_dino_ui_util_entry_label_hybrid_update_label_gtk_editable_changed (GtkEditable* _sender,
                                                                    gpointer self)
{
	dino_ui_util_entry_label_hybrid_update_label ((DinoUiUtilEntryLabelHybrid*) self);
}

static void
_dino_ui_util_entry_label_hybrid_update_label_gtk_event_controller_focus_leave (GtkEventControllerFocus* _sender,
                                                                                gpointer self)
{
	dino_ui_util_entry_label_hybrid_update_label ((DinoUiUtilEntryLabelHybrid*) self);
}

static void
dino_ui_util_entry_label_hybrid_real_init (DinoUiUtilLabelHybrid* base,
                                           GtkWidget* widget)
{
	DinoUiUtilEntryLabelHybrid * self;
	GtkEntry* e = NULL;
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
	GtkEntry* _tmp2_;
	GtkEntry* _tmp3_;
	GtkEntry* _tmp4_;
	GtkEventControllerKey* key_events = NULL;
	GtkEventControllerKey* _tmp5_;
	GtkEventControllerKey* _tmp6_;
	GtkEntry* _tmp7_;
	GtkEntry* _tmp8_;
	GtkEventControllerKey* _tmp9_;
	GtkEventController* _tmp10_;
	GtkEntry* _tmp11_;
	GtkEntry* _tmp12_;
	GtkEventControllerFocus* focus_events = NULL;
	GtkEventControllerFocus* _tmp13_;
	GtkEventControllerFocus* _tmp14_;
	GtkEntry* _tmp15_;
	GtkEntry* _tmp16_;
	GtkEventControllerFocus* _tmp17_;
	GtkEventController* _tmp18_;
	self = (DinoUiUtilEntryLabelHybrid*) base;
	g_return_if_fail (widget != NULL);
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (widget, gtk_entry_get_type ()) ? ((GtkEntry*) widget) : NULL);
	e = _tmp0_;
	_tmp1_ = e;
	if (_tmp1_ == NULL) {
		_g_object_unref0 (e);
		return;
	}
	_tmp2_ = e;
	dino_ui_util_entry_label_hybrid_set_entry (self, _tmp2_);
	_tmp3_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp4_ = _tmp3_;
	DINO_UI_UTIL_LABEL_HYBRID_CLASS (dino_ui_util_entry_label_hybrid_parent_class)->init (G_TYPE_CHECK_INSTANCE_CAST (self, DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybrid), (GtkWidget*) _tmp4_);
	dino_ui_util_entry_label_hybrid_update_label (self);
	_tmp5_ = (GtkEventControllerKey*) gtk_event_controller_key_new ();
	key_events = _tmp5_;
	_tmp6_ = key_events;
	g_signal_connect_object (_tmp6_, "key-released", (GCallback) _dino_ui_util_entry_label_hybrid_on_key_released_gtk_event_controller_key_key_released, self, 0);
	_tmp7_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp8_ = _tmp7_;
	_tmp9_ = key_events;
	_tmp10_ = _g_object_ref0 ((GtkEventController*) _tmp9_);
	gtk_widget_add_controller ((GtkWidget*) _tmp8_, _tmp10_);
	_tmp11_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp12_ = _tmp11_;
	g_signal_connect_object ((GtkEditable*) _tmp12_, "changed", (GCallback) _dino_ui_util_entry_label_hybrid_update_label_gtk_editable_changed, self, 0);
	_tmp13_ = (GtkEventControllerFocus*) gtk_event_controller_focus_new ();
	focus_events = _tmp13_;
	_tmp14_ = focus_events;
	g_signal_connect_object (_tmp14_, "leave", (GCallback) _dino_ui_util_entry_label_hybrid_update_label_gtk_event_controller_focus_leave, self, 0);
	_tmp15_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp16_ = _tmp15_;
	_tmp17_ = focus_events;
	_tmp18_ = _g_object_ref0 ((GtkEventController*) _tmp17_);
	gtk_widget_add_controller ((GtkWidget*) _tmp16_, _tmp18_);
	_g_object_unref0 (focus_events);
	_g_object_unref0 (key_events);
	_g_object_unref0 (e);
}

static void
dino_ui_util_entry_label_hybrid_on_key_released (DinoUiUtilEntryLabelHybrid* self,
                                                 guint keyval)
{
	g_return_if_fail (self != NULL);
	if (keyval == GDK_KEY_Return) {
		dino_ui_util_label_hybrid_show_label ((DinoUiUtilLabelHybrid*) self);
	}
}

static gchar*
g_unichar_to_string (gunichar self)
{
	gchar* str = NULL;
	gchar* _tmp0_;
	gchar* result;
	_tmp0_ = g_new0 (gchar, 7);
	str = (gchar*) _tmp0_;
	g_unichar_to_utf8 (self, str);
	result = str;
	return result;
}

static void
dino_ui_util_entry_label_hybrid_update_label (DinoUiUtilEntryLabelHybrid* self)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = dino_ui_util_entry_label_hybrid_get_visibility (self);
	_tmp1_ = _tmp0_;
	if (_tmp1_) {
		GtkLabel* _tmp2_;
		GtkEntry* _tmp3_;
		GtkEntry* _tmp4_;
		const gchar* _tmp5_;
		const gchar* _tmp6_;
		_tmp2_ = ((DinoUiUtilLabelHybrid*) self)->label;
		_tmp3_ = dino_ui_util_entry_label_hybrid_get_entry (self);
		_tmp4_ = _tmp3_;
		_tmp5_ = gtk_editable_get_text ((GtkEditable*) _tmp4_);
		_tmp6_ = _tmp5_;
		gtk_label_set_label (_tmp2_, _tmp6_);
	} else {
		gchar* filler = NULL;
		gchar* _tmp7_;
		GtkLabel* _tmp22_;
		const gchar* _tmp23_;
		_tmp7_ = g_strdup ("");
		filler = _tmp7_;
		{
			gint i = 0;
			i = 0;
			{
				gboolean _tmp8_ = FALSE;
				_tmp8_ = TRUE;
				while (TRUE) {
					GtkEntry* _tmp10_;
					GtkEntry* _tmp11_;
					const gchar* _tmp12_;
					const gchar* _tmp13_;
					gint _tmp14_;
					gint _tmp15_;
					const gchar* _tmp16_;
					GtkEntry* _tmp17_;
					GtkEntry* _tmp18_;
					gchar* _tmp19_;
					gchar* _tmp20_;
					gchar* _tmp21_;
					if (!_tmp8_) {
						gint _tmp9_;
						_tmp9_ = i;
						i = _tmp9_ + 1;
					}
					_tmp8_ = FALSE;
					_tmp10_ = dino_ui_util_entry_label_hybrid_get_entry (self);
					_tmp11_ = _tmp10_;
					_tmp12_ = gtk_editable_get_text ((GtkEditable*) _tmp11_);
					_tmp13_ = _tmp12_;
					_tmp14_ = strlen (_tmp13_);
					_tmp15_ = _tmp14_;
					if (!(i < _tmp15_)) {
						break;
					}
					_tmp16_ = filler;
					_tmp17_ = dino_ui_util_entry_label_hybrid_get_entry (self);
					_tmp18_ = _tmp17_;
					_tmp19_ = g_unichar_to_string (gtk_entry_get_invisible_char (_tmp18_));
					_tmp20_ = _tmp19_;
					_tmp21_ = g_strconcat (_tmp16_, _tmp20_, NULL);
					_g_free0 (filler);
					filler = _tmp21_;
					_g_free0 (_tmp20_);
				}
			}
		}
		_tmp22_ = ((DinoUiUtilLabelHybrid*) self)->label;
		_tmp23_ = filler;
		gtk_label_set_label (_tmp22_, _tmp23_);
		_g_free0 (filler);
	}
}

DinoUiUtilEntryLabelHybrid*
dino_ui_util_entry_label_hybrid_construct (GType object_type)
{
	DinoUiUtilEntryLabelHybrid * self = NULL;
	self = (DinoUiUtilEntryLabelHybrid*) dino_ui_util_label_hybrid_construct (object_type);
	return self;
}

DinoUiUtilEntryLabelHybrid*
dino_ui_util_entry_label_hybrid_new (void)
{
	return dino_ui_util_entry_label_hybrid_construct (DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID);
}

const gchar*
dino_ui_util_entry_label_hybrid_get_text (DinoUiUtilEntryLabelHybrid* self)
{
	const gchar* result;
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = gtk_editable_get_text ((GtkEditable*) _tmp1_);
	_tmp3_ = _tmp2_;
	result = _tmp3_;
	return result;
}

void
dino_ui_util_entry_label_hybrid_set_text (DinoUiUtilEntryLabelHybrid* self,
                                          const gchar* value)
{
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	g_return_if_fail (self != NULL);
	_tmp0_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = g_strdup (value);
	_tmp3_ = _tmp2_;
	gtk_editable_set_text ((GtkEditable*) _tmp1_, _tmp3_);
	_g_free0 (_tmp3_);
	dino_ui_util_entry_label_hybrid_update_label (self);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY]);
}

gboolean
dino_ui_util_entry_label_hybrid_get_visibility (DinoUiUtilEntryLabelHybrid* self)
{
	gboolean result;
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = gtk_entry_get_visibility (_tmp1_);
	_tmp3_ = _tmp2_;
	result = _tmp3_;
	return result;
}

void
dino_ui_util_entry_label_hybrid_set_visibility (DinoUiUtilEntryLabelHybrid* self,
                                                gboolean value)
{
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp1_ = _tmp0_;
	gtk_entry_set_visibility (_tmp1_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY]);
}

gfloat
dino_ui_util_entry_label_hybrid_get_xalign (DinoUiUtilEntryLabelHybrid* self)
{
	gfloat result;
	GtkLabel* _tmp0_;
	gfloat _tmp1_;
	gfloat _tmp2_;
	g_return_val_if_fail (self != NULL, -1.0F);
	_tmp0_ = ((DinoUiUtilLabelHybrid*) self)->label;
	_tmp1_ = gtk_label_get_xalign (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_util_entry_label_hybrid_set_xalign (DinoUiUtilEntryLabelHybrid* self,
                                            gfloat value)
{
	GtkLabel* _tmp0_;
	GtkEntry* _tmp1_;
	GtkEntry* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((DinoUiUtilLabelHybrid*) self)->label;
	gtk_label_set_xalign (_tmp0_, value);
	_tmp1_ = dino_ui_util_entry_label_hybrid_get_entry (self);
	_tmp2_ = _tmp1_;
	gtk_entry_set_alignment (_tmp2_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY]);
}

GtkEntry*
dino_ui_util_entry_label_hybrid_get_entry (DinoUiUtilEntryLabelHybrid* self)
{
	GtkEntry* result;
	GtkEntry* _tmp0_;
	GtkEntry* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->entry_;
	if (_tmp0_ == NULL) {
		GtkEntry* _tmp1_;
		GtkEntry* _tmp2_;
		_tmp1_ = (GtkEntry*) gtk_entry_new ();
		g_object_ref_sink (_tmp1_);
		_g_object_unref0 (self->priv->entry_);
		self->priv->entry_ = _tmp1_;
		_tmp2_ = self->priv->entry_;
		dino_ui_util_label_hybrid_init ((DinoUiUtilLabelHybrid*) self, (GtkWidget*) _tmp2_);
	}
	_tmp3_ = self->priv->entry_;
	result = _tmp3_;
	return result;
}

void
dino_ui_util_entry_label_hybrid_set_entry (DinoUiUtilEntryLabelHybrid* self,
                                           GtkEntry* value)
{
	GtkEntry* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _g_object_ref0 (value);
	_g_object_unref0 (self->priv->entry_);
	self->priv->entry_ = _tmp0_;
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY]);
}

static void
dino_ui_util_entry_label_hybrid_class_init (DinoUiUtilEntryLabelHybridClass * klass,
                                            gpointer klass_data)
{
	dino_ui_util_entry_label_hybrid_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiUtilEntryLabelHybrid_private_offset);
	((DinoUiUtilLabelHybridClass *) klass)->init = (void (*) (DinoUiUtilLabelHybrid*, GtkWidget*)) dino_ui_util_entry_label_hybrid_real_init;
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_util_entry_label_hybrid_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_util_entry_label_hybrid_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_util_entry_label_hybrid_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY] = g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY] = g_param_spec_boolean ("visibility", "visibility", "visibility", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY] = g_param_spec_float ("xalign", "xalign", "xalign", -G_MAXFLOAT, G_MAXFLOAT, 0.0F, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY, dino_ui_util_entry_label_hybrid_properties[DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY] = g_param_spec_object ("entry", "entry", "entry", gtk_entry_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_util_entry_label_hybrid_instance_init (DinoUiUtilEntryLabelHybrid * self,
                                               gpointer klass)
{
	self->priv = dino_ui_util_entry_label_hybrid_get_instance_private (self);
}

static void
dino_ui_util_entry_label_hybrid_finalize (GObject * obj)
{
	DinoUiUtilEntryLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybrid);
	_g_object_unref0 (self->priv->entry_);
	G_OBJECT_CLASS (dino_ui_util_entry_label_hybrid_parent_class)->finalize (obj);
}

static GType
dino_ui_util_entry_label_hybrid_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiUtilEntryLabelHybridClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_util_entry_label_hybrid_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiUtilEntryLabelHybrid), 0, (GInstanceInitFunc) dino_ui_util_entry_label_hybrid_instance_init, NULL };
	GType dino_ui_util_entry_label_hybrid_type_id;
	dino_ui_util_entry_label_hybrid_type_id = g_type_register_static (DINO_UI_UTIL_TYPE_LABEL_HYBRID, "DinoUiUtilEntryLabelHybrid", &g_define_type_info, 0);
	DinoUiUtilEntryLabelHybrid_private_offset = g_type_add_instance_private (dino_ui_util_entry_label_hybrid_type_id, sizeof (DinoUiUtilEntryLabelHybridPrivate));
	return dino_ui_util_entry_label_hybrid_type_id;
}

GType
dino_ui_util_entry_label_hybrid_get_type (void)
{
	static volatile gsize dino_ui_util_entry_label_hybrid_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_util_entry_label_hybrid_type_id__once)) {
		GType dino_ui_util_entry_label_hybrid_type_id;
		dino_ui_util_entry_label_hybrid_type_id = dino_ui_util_entry_label_hybrid_get_type_once ();
		g_once_init_leave (&dino_ui_util_entry_label_hybrid_type_id__once, dino_ui_util_entry_label_hybrid_type_id);
	}
	return dino_ui_util_entry_label_hybrid_type_id__once;
}

static void
_vala_dino_ui_util_entry_label_hybrid_get_property (GObject * object,
                                                    guint property_id,
                                                    GValue * value,
                                                    GParamSpec * pspec)
{
	DinoUiUtilEntryLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybrid);
	switch (property_id) {
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY:
		g_value_set_string (value, dino_ui_util_entry_label_hybrid_get_text (self));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY:
		g_value_set_boolean (value, dino_ui_util_entry_label_hybrid_get_visibility (self));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY:
		g_value_set_float (value, dino_ui_util_entry_label_hybrid_get_xalign (self));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY:
		g_value_set_object (value, dino_ui_util_entry_label_hybrid_get_entry (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_util_entry_label_hybrid_set_property (GObject * object,
                                                    guint property_id,
                                                    const GValue * value,
                                                    GParamSpec * pspec)
{
	DinoUiUtilEntryLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_UTIL_TYPE_ENTRY_LABEL_HYBRID, DinoUiUtilEntryLabelHybrid);
	switch (property_id) {
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_TEXT_PROPERTY:
		dino_ui_util_entry_label_hybrid_set_text (self, g_value_get_string (value));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_VISIBILITY_PROPERTY:
		dino_ui_util_entry_label_hybrid_set_visibility (self, g_value_get_boolean (value));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_XALIGN_PROPERTY:
		dino_ui_util_entry_label_hybrid_set_xalign (self, g_value_get_float (value));
		break;
		case DINO_UI_UTIL_ENTRY_LABEL_HYBRID_ENTRY_PROPERTY:
		dino_ui_util_entry_label_hybrid_set_entry (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_util_combo_box_text_label_hybrid_get_instance_private (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiUtilComboBoxTextLabelHybrid_private_offset);
}

DinoUiUtilComboBoxTextLabelHybrid*
dino_ui_util_combo_box_text_label_hybrid_construct_wrap (GType object_type,
                                                         GtkComboBoxText* cb)
{
	DinoUiUtilComboBoxTextLabelHybrid * self = NULL;
	GtkComboBoxText* _tmp0_;
	g_return_val_if_fail (cb != NULL, NULL);
	self = (DinoUiUtilComboBoxTextLabelHybrid*) dino_ui_util_label_hybrid_construct (object_type);
	_tmp0_ = _g_object_ref0 (cb);
	_g_object_unref0 (self->priv->combobox_);
	self->priv->combobox_ = _tmp0_;
	dino_ui_util_label_hybrid_init ((DinoUiUtilLabelHybrid*) self, (GtkWidget*) cb);
	return self;
}

DinoUiUtilComboBoxTextLabelHybrid*
dino_ui_util_combo_box_text_label_hybrid_new_wrap (GtkComboBoxText* cb)
{
	return dino_ui_util_combo_box_text_label_hybrid_construct_wrap (DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, cb);
}

void
dino_ui_util_combo_box_text_label_hybrid_append (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                 const gchar* id,
                                                 const gchar* text)
{
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (text != NULL);
	_tmp0_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp1_ = _tmp0_;
	gtk_combo_box_text_append (_tmp1_, id, text);
}

gchar*
dino_ui_util_combo_box_text_label_hybrid_get_active_text (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	gchar* _tmp2_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = gtk_combo_box_text_get_active_text (_tmp1_);
	result = _tmp2_;
	return result;
}

static Block71Data*
block71_data_ref (Block71Data* _data71_)
{
	g_atomic_int_inc (&_data71_->_ref_count_);
	return _data71_;
}

static void
block71_data_unref (void * _userdata_)
{
	Block71Data* _data71_;
	_data71_ = (Block71Data*) _userdata_;
	if (g_atomic_int_dec_and_test (&_data71_->_ref_count_)) {
		DinoUiUtilComboBoxTextLabelHybrid* self;
		self = _data71_->self;
		_g_object_unref0 (_data71_->combobox);
		_g_object_unref0 (self);
		g_slice_free (Block71Data, _data71_);
	}
}

static void
__lambda193_ (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	dino_ui_util_combo_box_text_label_hybrid_update_label (self);
	dino_ui_util_label_hybrid_show_label ((DinoUiUtilLabelHybrid*) self);
}

static void
___lambda193__gtk_combo_box_changed (GtkComboBox* _sender,
                                     gpointer self)
{
	__lambda193_ ((DinoUiUtilComboBoxTextLabelHybrid*) self);
}

static void
__lambda194_ (Block71Data* _data71_)
{
	DinoUiUtilComboBoxTextLabelHybrid* self;
	GtkComboBoxText* _tmp0_;
	self = _data71_->self;
	_tmp0_ = _data71_->combobox;
	gtk_combo_box_popup ((GtkComboBox*) _tmp0_);
}

static void
___lambda194__gtk_button_clicked (GtkButton* _sender,
                                  gpointer self)
{
	__lambda194_ (self);
}

static void
_dino_ui_util_combo_box_text_label_hybrid_on_focus_leave_gtk_event_controller_focus_leave (GtkEventControllerFocus* _sender,
                                                                                           gpointer self)
{
	dino_ui_util_combo_box_text_label_hybrid_on_focus_leave ((DinoUiUtilComboBoxTextLabelHybrid*) self);
}

static void
dino_ui_util_combo_box_text_label_hybrid_real_init (DinoUiUtilLabelHybrid* base,
                                                    GtkWidget* widget)
{
	DinoUiUtilComboBoxTextLabelHybrid * self;
	Block71Data* _data71_;
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	GtkComboBoxText* _tmp2_;
	GtkComboBoxText* _tmp3_;
	GtkButton* _tmp4_;
	GtkEventControllerFocus* focus_events = NULL;
	GtkEventControllerFocus* _tmp5_;
	GtkEventControllerFocus* _tmp6_;
	GtkComboBoxText* _tmp7_;
	GtkEventControllerFocus* _tmp8_;
	GtkEventController* _tmp9_;
	self = (DinoUiUtilComboBoxTextLabelHybrid*) base;
	g_return_if_fail (widget != NULL);
	_data71_ = g_slice_new0 (Block71Data);
	_data71_->_ref_count_ = 1;
	_data71_->self = g_object_ref (self);
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (widget, gtk_combo_box_text_get_type ()) ? ((GtkComboBoxText*) widget) : NULL);
	_data71_->combobox = _tmp0_;
	_tmp1_ = _data71_->combobox;
	if (_tmp1_ == NULL) {
		block71_data_unref (_data71_);
		_data71_ = NULL;
		return;
	}
	_tmp2_ = _data71_->combobox;
	DINO_UI_UTIL_LABEL_HYBRID_CLASS (dino_ui_util_combo_box_text_label_hybrid_parent_class)->init (G_TYPE_CHECK_INSTANCE_CAST (self, DINO_UI_UTIL_TYPE_LABEL_HYBRID, DinoUiUtilLabelHybrid), (GtkWidget*) _tmp2_);
	dino_ui_util_combo_box_text_label_hybrid_update_label (self);
	_tmp3_ = _data71_->combobox;
	g_signal_connect_object ((GtkComboBox*) _tmp3_, "changed", (GCallback) ___lambda193__gtk_combo_box_changed, self, 0);
	_tmp4_ = ((DinoUiUtilLabelHybrid*) self)->button;
	g_signal_connect_data (_tmp4_, "clicked", (GCallback) ___lambda194__gtk_button_clicked, block71_data_ref (_data71_), (GClosureNotify) block71_data_unref, 0);
	_tmp5_ = (GtkEventControllerFocus*) gtk_event_controller_focus_new ();
	focus_events = _tmp5_;
	_tmp6_ = focus_events;
	g_signal_connect_object (_tmp6_, "leave", (GCallback) _dino_ui_util_combo_box_text_label_hybrid_on_focus_leave_gtk_event_controller_focus_leave, self, 0);
	_tmp7_ = _data71_->combobox;
	_tmp8_ = focus_events;
	_tmp9_ = _g_object_ref0 ((GtkEventController*) _tmp8_);
	gtk_widget_add_controller ((GtkWidget*) _tmp7_, _tmp9_);
	_g_object_unref0 (focus_events);
	block71_data_unref (_data71_);
	_data71_ = NULL;
}

static void
dino_ui_util_combo_box_text_label_hybrid_on_focus_leave (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	g_return_if_fail (self != NULL);
	dino_ui_util_combo_box_text_label_hybrid_update_label (self);
	dino_ui_util_label_hybrid_show_label ((DinoUiUtilLabelHybrid*) self);
}

static void
dino_ui_util_combo_box_text_label_hybrid_update_label (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	GtkLabel* _tmp0_;
	GtkComboBoxText* _tmp1_;
	GtkComboBoxText* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((DinoUiUtilLabelHybrid*) self)->label;
	_tmp1_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp2_ = _tmp1_;
	_tmp3_ = gtk_combo_box_text_get_active_text (_tmp2_);
	_tmp4_ = _tmp3_;
	gtk_label_set_label (_tmp0_, _tmp4_);
	_g_free0 (_tmp4_);
}

DinoUiUtilComboBoxTextLabelHybrid*
dino_ui_util_combo_box_text_label_hybrid_construct (GType object_type)
{
	DinoUiUtilComboBoxTextLabelHybrid * self = NULL;
	self = (DinoUiUtilComboBoxTextLabelHybrid*) dino_ui_util_label_hybrid_construct (object_type);
	return self;
}

DinoUiUtilComboBoxTextLabelHybrid*
dino_ui_util_combo_box_text_label_hybrid_new (void)
{
	return dino_ui_util_combo_box_text_label_hybrid_construct (DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID);
}

gint
dino_ui_util_combo_box_text_label_hybrid_get_active (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	gint result;
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	gint _tmp2_;
	gint _tmp3_;
	g_return_val_if_fail (self != NULL, -1);
	_tmp0_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = gtk_combo_box_get_active ((GtkComboBox*) _tmp1_);
	_tmp3_ = _tmp2_;
	result = _tmp3_;
	return result;
}

void
dino_ui_util_combo_box_text_label_hybrid_set_active (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                     gint value)
{
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp1_ = _tmp0_;
	gtk_combo_box_set_active ((GtkComboBox*) _tmp1_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY]);
}

gfloat
dino_ui_util_combo_box_text_label_hybrid_get_xalign (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	gfloat result;
	GtkLabel* _tmp0_;
	gfloat _tmp1_;
	gfloat _tmp2_;
	g_return_val_if_fail (self != NULL, -1.0F);
	_tmp0_ = ((DinoUiUtilLabelHybrid*) self)->label;
	_tmp1_ = gtk_label_get_xalign (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

void
dino_ui_util_combo_box_text_label_hybrid_set_xalign (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                     gfloat value)
{
	GtkLabel* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((DinoUiUtilLabelHybrid*) self)->label;
	gtk_label_set_xalign (_tmp0_, value);
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY]);
}

GtkComboBoxText*
dino_ui_util_combo_box_text_label_hybrid_get_combobox (DinoUiUtilComboBoxTextLabelHybrid* self)
{
	GtkComboBoxText* result;
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->combobox_;
	if (_tmp0_ == NULL) {
		GtkComboBoxText* _tmp1_;
		GtkComboBoxText* _tmp2_;
		_tmp1_ = (GtkComboBoxText*) gtk_combo_box_text_new ();
		g_object_ref_sink (_tmp1_);
		_g_object_unref0 (self->priv->combobox_);
		self->priv->combobox_ = _tmp1_;
		_tmp2_ = self->priv->combobox_;
		dino_ui_util_label_hybrid_init ((DinoUiUtilLabelHybrid*) self, (GtkWidget*) _tmp2_);
	}
	_tmp3_ = self->priv->combobox_;
	result = _tmp3_;
	return result;
}

void
dino_ui_util_combo_box_text_label_hybrid_set_combobox (DinoUiUtilComboBoxTextLabelHybrid* self,
                                                       GtkComboBoxText* value)
{
	GtkComboBoxText* _tmp0_;
	GtkComboBoxText* _tmp1_;
	GtkComboBoxText* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = dino_ui_util_combo_box_text_label_hybrid_get_combobox (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = _g_object_ref0 (_tmp1_);
	_g_object_unref0 (self->priv->combobox_);
	self->priv->combobox_ = _tmp2_;
	g_object_notify_by_pspec ((GObject *) self, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY]);
}

static void
dino_ui_util_combo_box_text_label_hybrid_class_init (DinoUiUtilComboBoxTextLabelHybridClass * klass,
                                                     gpointer klass_data)
{
	dino_ui_util_combo_box_text_label_hybrid_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiUtilComboBoxTextLabelHybrid_private_offset);
	((DinoUiUtilLabelHybridClass *) klass)->init = (void (*) (DinoUiUtilLabelHybrid*, GtkWidget*)) dino_ui_util_combo_box_text_label_hybrid_real_init;
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_util_combo_box_text_label_hybrid_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_util_combo_box_text_label_hybrid_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_util_combo_box_text_label_hybrid_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY] = g_param_spec_int ("active", "active", "active", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY] = g_param_spec_float ("xalign", "xalign", "xalign", -G_MAXFLOAT, G_MAXFLOAT, 0.0F, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY, dino_ui_util_combo_box_text_label_hybrid_properties[DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY] = g_param_spec_object ("combobox", "combobox", "combobox", gtk_combo_box_text_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_util_combo_box_text_label_hybrid_instance_init (DinoUiUtilComboBoxTextLabelHybrid * self,
                                                        gpointer klass)
{
	self->priv = dino_ui_util_combo_box_text_label_hybrid_get_instance_private (self);
}

static void
dino_ui_util_combo_box_text_label_hybrid_finalize (GObject * obj)
{
	DinoUiUtilComboBoxTextLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybrid);
	_g_object_unref0 (self->priv->combobox_);
	G_OBJECT_CLASS (dino_ui_util_combo_box_text_label_hybrid_parent_class)->finalize (obj);
}

static GType
dino_ui_util_combo_box_text_label_hybrid_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiUtilComboBoxTextLabelHybridClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_util_combo_box_text_label_hybrid_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiUtilComboBoxTextLabelHybrid), 0, (GInstanceInitFunc) dino_ui_util_combo_box_text_label_hybrid_instance_init, NULL };
	GType dino_ui_util_combo_box_text_label_hybrid_type_id;
	dino_ui_util_combo_box_text_label_hybrid_type_id = g_type_register_static (DINO_UI_UTIL_TYPE_LABEL_HYBRID, "DinoUiUtilComboBoxTextLabelHybrid", &g_define_type_info, 0);
	DinoUiUtilComboBoxTextLabelHybrid_private_offset = g_type_add_instance_private (dino_ui_util_combo_box_text_label_hybrid_type_id, sizeof (DinoUiUtilComboBoxTextLabelHybridPrivate));
	return dino_ui_util_combo_box_text_label_hybrid_type_id;
}

GType
dino_ui_util_combo_box_text_label_hybrid_get_type (void)
{
	static volatile gsize dino_ui_util_combo_box_text_label_hybrid_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_util_combo_box_text_label_hybrid_type_id__once)) {
		GType dino_ui_util_combo_box_text_label_hybrid_type_id;
		dino_ui_util_combo_box_text_label_hybrid_type_id = dino_ui_util_combo_box_text_label_hybrid_get_type_once ();
		g_once_init_leave (&dino_ui_util_combo_box_text_label_hybrid_type_id__once, dino_ui_util_combo_box_text_label_hybrid_type_id);
	}
	return dino_ui_util_combo_box_text_label_hybrid_type_id__once;
}

static void
_vala_dino_ui_util_combo_box_text_label_hybrid_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec)
{
	DinoUiUtilComboBoxTextLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybrid);
	switch (property_id) {
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY:
		g_value_set_int (value, dino_ui_util_combo_box_text_label_hybrid_get_active (self));
		break;
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY:
		g_value_set_float (value, dino_ui_util_combo_box_text_label_hybrid_get_xalign (self));
		break;
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY:
		g_value_set_object (value, dino_ui_util_combo_box_text_label_hybrid_get_combobox (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_util_combo_box_text_label_hybrid_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec)
{
	DinoUiUtilComboBoxTextLabelHybrid * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_UTIL_TYPE_COMBO_BOX_TEXT_LABEL_HYBRID, DinoUiUtilComboBoxTextLabelHybrid);
	switch (property_id) {
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_ACTIVE_PROPERTY:
		dino_ui_util_combo_box_text_label_hybrid_set_active (self, g_value_get_int (value));
		break;
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_XALIGN_PROPERTY:
		dino_ui_util_combo_box_text_label_hybrid_set_xalign (self, g_value_get_float (value));
		break;
		case DINO_UI_UTIL_COMBO_BOX_TEXT_LABEL_HYBRID_COMBOBOX_PROPERTY:
		dino_ui_util_combo_box_text_label_hybrid_set_combobox (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_util_label_hybrid_group_get_instance_private (DinoUiUtilLabelHybridGroup* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiUtilLabelHybridGroup_private_offset);
}

static Block72Data*
block72_data_ref (Block72Data* _data72_)
{
	g_atomic_int_inc (&_data72_->_ref_count_);
	return _data72_;
}

static void
block72_data_unref (void * _userdata_)
{
	Block72Data* _data72_;
	_data72_ = (Block72Data*) _userdata_;
	if (g_atomic_int_dec_and_test (&_data72_->_ref_count_)) {
		DinoUiUtilLabelHybridGroup* self;
		self = _data72_->self;
		_g_object_unref0 (_data72_->hybrid);
		_dino_ui_util_label_hybrid_group_unref0 (self);
		g_slice_free (Block72Data, _data72_);
	}
}

static void
__lambda337_ (Block72Data* _data72_)
{
	DinoUiUtilLabelHybridGroup* self;
	GtkStack* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	self = _data72_->self;
	_tmp0_ = _data72_->hybrid->stack;
	_tmp1_ = gtk_stack_get_visible_child_name (_tmp0_);
	_tmp2_ = _tmp1_;
	if (g_strcmp0 (_tmp2_, "label") == 0) {
		return;
	}
	{
		GeeList* _h_list = NULL;
		GeeList* _tmp3_;
		gint _h_size = 0;
		GeeList* _tmp4_;
		gint _tmp5_;
		gint _tmp6_;
		gint _h_index = 0;
		_tmp3_ = self->priv->hybrids;
		_h_list = _tmp3_;
		_tmp4_ = _h_list;
		_tmp5_ = gee_collection_get_size ((GeeCollection*) _tmp4_);
		_tmp6_ = _tmp5_;
		_h_size = _tmp6_;
		_h_index = -1;
		while (TRUE) {
			gint _tmp7_;
			gint _tmp8_;
			DinoUiUtilLabelHybrid* h = NULL;
			GeeList* _tmp9_;
			gpointer _tmp10_;
			DinoUiUtilLabelHybrid* _tmp11_;
			_h_index = _h_index + 1;
			_tmp7_ = _h_index;
			_tmp8_ = _h_size;
			if (!(_tmp7_ < _tmp8_)) {
				break;
			}
			_tmp9_ = _h_list;
			_tmp10_ = gee_list_get (_tmp9_, _h_index);
			h = (DinoUiUtilLabelHybrid*) _tmp10_;
			_tmp11_ = h;
			if (_tmp11_ != _data72_->hybrid) {
				DinoUiUtilLabelHybrid* _tmp12_;
				GtkStack* _tmp13_;
				_tmp12_ = h;
				_tmp13_ = _tmp12_->stack;
				gtk_stack_set_visible_child_name (_tmp13_, "label");
			}
			_g_object_unref0 (h);
		}
	}
}

static void
___lambda337__g_object_notify (GObject* _sender,
                               GParamSpec* pspec,
                               gpointer self)
{
	__lambda337_ (self);
}

void
dino_ui_util_label_hybrid_group_add (DinoUiUtilLabelHybridGroup* self,
                                     DinoUiUtilLabelHybrid* hybrid)
{
	Block72Data* _data72_;
	DinoUiUtilLabelHybrid* _tmp0_;
	GeeList* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (hybrid != NULL);
	_data72_ = g_slice_new0 (Block72Data);
	_data72_->_ref_count_ = 1;
	_data72_->self = dino_ui_util_label_hybrid_group_ref (self);
	_tmp0_ = _g_object_ref0 (hybrid);
	_g_object_unref0 (_data72_->hybrid);
	_data72_->hybrid = _tmp0_;
	_tmp1_ = self->priv->hybrids;
	gee_collection_add ((GeeCollection*) _tmp1_, _data72_->hybrid);
	g_signal_connect_data ((GObject*) _data72_->hybrid, "notify::visible-child-name", (GCallback) ___lambda337__g_object_notify, block72_data_ref (_data72_), (GClosureNotify) block72_data_unref, 0);
	block72_data_unref (_data72_);
	_data72_ = NULL;
}

DinoUiUtilLabelHybridGroup*
dino_ui_util_label_hybrid_group_construct (GType object_type)
{
	DinoUiUtilLabelHybridGroup* self = NULL;
	self = (DinoUiUtilLabelHybridGroup*) g_type_create_instance (object_type);
	return self;
}

DinoUiUtilLabelHybridGroup*
dino_ui_util_label_hybrid_group_new (void)
{
	return dino_ui_util_label_hybrid_group_construct (DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP);
}

static void
dino_ui_util_value_label_hybrid_group_init (GValue* value)
{
	value->data[0].v_pointer = NULL;
}

static void
dino_ui_util_value_label_hybrid_group_free_value (GValue* value)
{
	if (value->data[0].v_pointer) {
		dino_ui_util_label_hybrid_group_unref (value->data[0].v_pointer);
	}
}

static void
dino_ui_util_value_label_hybrid_group_copy_value (const GValue* src_value,
                                                  GValue* dest_value)
{
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = dino_ui_util_label_hybrid_group_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}

static gpointer
dino_ui_util_value_label_hybrid_group_peek_pointer (const GValue* value)
{
	return value->data[0].v_pointer;
}

static gchar*
dino_ui_util_value_label_hybrid_group_collect_value (GValue* value,
                                                     guint n_collect_values,
                                                     GTypeCValue* collect_values,
                                                     guint collect_flags)
{
	if (collect_values[0].v_pointer) {
		DinoUiUtilLabelHybridGroup * object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = dino_ui_util_label_hybrid_group_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}

static gchar*
dino_ui_util_value_label_hybrid_group_lcopy_value (const GValue* value,
                                                   guint n_collect_values,
                                                   GTypeCValue* collect_values,
                                                   guint collect_flags)
{
	DinoUiUtilLabelHybridGroup ** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = dino_ui_util_label_hybrid_group_ref (value->data[0].v_pointer);
	}
	return NULL;
}

GParamSpec*
dino_ui_util_param_spec_label_hybrid_group (const gchar* name,
                                            const gchar* nick,
                                            const gchar* blurb,
                                            GType object_type,
                                            GParamFlags flags)
{
	DinoUiUtilParamSpecLabelHybridGroup* spec;
	g_return_val_if_fail (g_type_is_a (object_type, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}

gpointer
dino_ui_util_value_get_label_hybrid_group (const GValue* value)
{
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP), NULL);
	return value->data[0].v_pointer;
}

void
dino_ui_util_value_set_label_hybrid_group (GValue* value,
                                           gpointer v_object)
{
	DinoUiUtilLabelHybridGroup * old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		dino_ui_util_label_hybrid_group_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		dino_ui_util_label_hybrid_group_unref (old);
	}
}

void
dino_ui_util_value_take_label_hybrid_group (GValue* value,
                                            gpointer v_object)
{
	DinoUiUtilLabelHybridGroup * old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		dino_ui_util_label_hybrid_group_unref (old);
	}
}

static void
dino_ui_util_label_hybrid_group_class_init (DinoUiUtilLabelHybridGroupClass * klass,
                                            gpointer klass_data)
{
	dino_ui_util_label_hybrid_group_parent_class = g_type_class_peek_parent (klass);
	((DinoUiUtilLabelHybridGroupClass *) klass)->finalize = dino_ui_util_label_hybrid_group_finalize;
	g_type_class_adjust_private_offset (klass, &DinoUiUtilLabelHybridGroup_private_offset);
}

static void
dino_ui_util_label_hybrid_group_instance_init (DinoUiUtilLabelHybridGroup * self,
                                               gpointer klass)
{
	GeeArrayList* _tmp0_;
	self->priv = dino_ui_util_label_hybrid_group_get_instance_private (self);
	_tmp0_ = gee_array_list_new (DINO_UI_UTIL_TYPE_LABEL_HYBRID, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
	self->priv->hybrids = (GeeList*) _tmp0_;
	self->ref_count = 1;
}

static void
dino_ui_util_label_hybrid_group_finalize (DinoUiUtilLabelHybridGroup * obj)
{
	DinoUiUtilLabelHybridGroup * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_UTIL_TYPE_LABEL_HYBRID_GROUP, DinoUiUtilLabelHybridGroup);
	g_signal_handlers_destroy (self);
	_g_object_unref0 (self->priv->hybrids);
}

static GType
dino_ui_util_label_hybrid_group_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { dino_ui_util_value_label_hybrid_group_init, dino_ui_util_value_label_hybrid_group_free_value, dino_ui_util_value_label_hybrid_group_copy_value, dino_ui_util_value_label_hybrid_group_peek_pointer, "p", dino_ui_util_value_label_hybrid_group_collect_value, "p", dino_ui_util_value_label_hybrid_group_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiUtilLabelHybridGroupClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_util_label_hybrid_group_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiUtilLabelHybridGroup), 0, (GInstanceInitFunc) dino_ui_util_label_hybrid_group_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType dino_ui_util_label_hybrid_group_type_id;
	dino_ui_util_label_hybrid_group_type_id = g_type_register_fundamental (g_type_fundamental_next (), "DinoUiUtilLabelHybridGroup", &g_define_type_info, &g_define_type_fundamental_info, 0);
	DinoUiUtilLabelHybridGroup_private_offset = g_type_add_instance_private (dino_ui_util_label_hybrid_group_type_id, sizeof (DinoUiUtilLabelHybridGroupPrivate));
	return dino_ui_util_label_hybrid_group_type_id;
}

GType
dino_ui_util_label_hybrid_group_get_type (void)
{
	static volatile gsize dino_ui_util_label_hybrid_group_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_util_label_hybrid_group_type_id__once)) {
		GType dino_ui_util_label_hybrid_group_type_id;
		dino_ui_util_label_hybrid_group_type_id = dino_ui_util_label_hybrid_group_get_type_once ();
		g_once_init_leave (&dino_ui_util_label_hybrid_group_type_id__once, dino_ui_util_label_hybrid_group_type_id);
	}
	return dino_ui_util_label_hybrid_group_type_id__once;
}

gpointer
dino_ui_util_label_hybrid_group_ref (gpointer instance)
{
	DinoUiUtilLabelHybridGroup * self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}

void
dino_ui_util_label_hybrid_group_unref (gpointer instance)
{
	DinoUiUtilLabelHybridGroup * self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		DINO_UI_UTIL_LABEL_HYBRID_GROUP_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

