/* preferences_row.c generated by valac 0.56.19, the Vala compiler
 * generated from preferences_row.vala, do not modify */

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gee.h>
#include <gtk/gtk.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY (dino_ui_view_model_preferences_row_any_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAny))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAnyClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_ANY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_ANY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAnyClass))

typedef struct _DinoUiViewModelPreferencesRowAny DinoUiViewModelPreferencesRowAny;
typedef struct _DinoUiViewModelPreferencesRowAnyClass DinoUiViewModelPreferencesRowAnyClass;
typedef struct _DinoUiViewModelPreferencesRowAnyPrivate DinoUiViewModelPreferencesRowAnyPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_any_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT (dino_ui_view_model_preferences_row_text_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowText))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowTextClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowTextClass))

typedef struct _DinoUiViewModelPreferencesRowText DinoUiViewModelPreferencesRowText;
typedef struct _DinoUiViewModelPreferencesRowTextClass DinoUiViewModelPreferencesRowTextClass;
typedef struct _DinoUiViewModelPreferencesRowTextPrivate DinoUiViewModelPreferencesRowTextPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_NUM_PROPERTIES];

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY (dino_ui_view_model_preferences_row_entry_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntry))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntryClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntryClass))

typedef struct _DinoUiViewModelPreferencesRowEntry DinoUiViewModelPreferencesRowEntry;
typedef struct _DinoUiViewModelPreferencesRowEntryClass DinoUiViewModelPreferencesRowEntryClass;
typedef struct _DinoUiViewModelPreferencesRowEntryPrivate DinoUiViewModelPreferencesRowEntryPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_entry_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_NUM_PROPERTIES];
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_CHANGED_SIGNAL,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_NUM_SIGNALS
};
static guint dino_ui_view_model_preferences_row_entry_signals[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_NUM_SIGNALS] = {0};

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT (dino_ui_view_model_preferences_row_private_text_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateText))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateTextClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_PRIVATE_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_PRIVATE_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateTextClass))

typedef struct _DinoUiViewModelPreferencesRowPrivateText DinoUiViewModelPreferencesRowPrivateText;
typedef struct _DinoUiViewModelPreferencesRowPrivateTextClass DinoUiViewModelPreferencesRowPrivateTextClass;
typedef struct _DinoUiViewModelPreferencesRowPrivateTextPrivate DinoUiViewModelPreferencesRowPrivateTextPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_private_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_NUM_PROPERTIES];
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_CHANGED_SIGNAL,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_NUM_SIGNALS
};
static guint dino_ui_view_model_preferences_row_private_text_signals[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_NUM_SIGNALS] = {0};

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE (dino_ui_view_model_preferences_row_toggle_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggle))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggleClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_TOGGLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_TOGGLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggleClass))

typedef struct _DinoUiViewModelPreferencesRowToggle DinoUiViewModelPreferencesRowToggle;
typedef struct _DinoUiViewModelPreferencesRowToggleClass DinoUiViewModelPreferencesRowToggleClass;
typedef struct _DinoUiViewModelPreferencesRowTogglePrivate DinoUiViewModelPreferencesRowTogglePrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_toggle_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_NUM_PROPERTIES];

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX (dino_ui_view_model_preferences_row_combo_box_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBox))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBoxClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_COMBO_BOX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBoxClass))

typedef struct _DinoUiViewModelPreferencesRowComboBox DinoUiViewModelPreferencesRowComboBox;
typedef struct _DinoUiViewModelPreferencesRowComboBoxClass DinoUiViewModelPreferencesRowComboBoxClass;
typedef struct _DinoUiViewModelPreferencesRowComboBoxPrivate DinoUiViewModelPreferencesRowComboBoxPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_combo_box_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED (dino_ui_view_model_preferences_row_widget_deprecated_get_type ())
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED, DinoUiViewModelPreferencesRowWidgetDeprecated))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED, DinoUiViewModelPreferencesRowWidgetDeprecatedClass))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_WIDGET_DEPRECATED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_IS_WIDGET_DEPRECATED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED))
#define DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED, DinoUiViewModelPreferencesRowWidgetDeprecatedClass))

typedef struct _DinoUiViewModelPreferencesRowWidgetDeprecated DinoUiViewModelPreferencesRowWidgetDeprecated;
typedef struct _DinoUiViewModelPreferencesRowWidgetDeprecatedClass DinoUiViewModelPreferencesRowWidgetDeprecatedClass;
typedef struct _DinoUiViewModelPreferencesRowWidgetDeprecatedPrivate DinoUiViewModelPreferencesRowWidgetDeprecatedPrivate;
enum  {
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED_0_PROPERTY,
	DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED_NUM_PROPERTIES
};
static GParamSpec* dino_ui_view_model_preferences_row_widget_deprecated_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_WIDGET_DEPRECATED_NUM_PROPERTIES];

struct _DinoUiViewModelPreferencesRowAny {
	GObject parent_instance;
	DinoUiViewModelPreferencesRowAnyPrivate * priv;
};

struct _DinoUiViewModelPreferencesRowAnyClass {
	GObjectClass parent_class;
};

struct _DinoUiViewModelPreferencesRowAnyPrivate {
	gchar* _title;
};

struct _DinoUiViewModelPreferencesRowText {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowTextPrivate * priv;
};

struct _DinoUiViewModelPreferencesRowTextClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

struct _DinoUiViewModelPreferencesRowTextPrivate {
	gchar* _text;
};

struct _DinoUiViewModelPreferencesRowEntry {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowEntryPrivate * priv;
};

struct _DinoUiViewModelPreferencesRowEntryClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

struct _DinoUiViewModelPreferencesRowEntryPrivate {
	gchar* _text;
};

struct _DinoUiViewModelPreferencesRowPrivateText {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowPrivateTextPrivate * priv;
};

struct _DinoUiViewModelPreferencesRowPrivateTextClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

struct _DinoUiViewModelPreferencesRowPrivateTextPrivate {
	gchar* _text;
};

struct _DinoUiViewModelPreferencesRowToggle {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowTogglePrivate * priv;
};

struct _DinoUiViewModelPreferencesRowToggleClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

struct _DinoUiViewModelPreferencesRowTogglePrivate {
	gchar* _subtitle;
	gboolean _state;
};

struct _DinoUiViewModelPreferencesRowComboBox {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowComboBoxPrivate * priv;
	GeeList* items;
};

struct _DinoUiViewModelPreferencesRowComboBoxClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

struct _DinoUiViewModelPreferencesRowComboBoxPrivate {
	gint _active_item;
};

struct _DinoUiViewModelPreferencesRowWidgetDeprecated {
	DinoUiViewModelPreferencesRowAny parent_instance;
	DinoUiViewModelPreferencesRowWidgetDeprecatedPrivate * priv;
	GtkWidget* widget;
};

struct _DinoUiViewModelPreferencesRowWidgetDeprecatedClass {
	DinoUiViewModelPreferencesRowAnyClass parent_class;
};

static gint DinoUiViewModelPreferencesRowAny_private_offset;
static gpointer dino_ui_view_model_preferences_row_any_parent_class = NULL;
static gint DinoUiViewModelPreferencesRowText_private_offset;
static gpointer dino_ui_view_model_preferences_row_text_parent_class = NULL;
static gint DinoUiViewModelPreferencesRowEntry_private_offset;
static gpointer dino_ui_view_model_preferences_row_entry_parent_class = NULL;
static gint DinoUiViewModelPreferencesRowPrivateText_private_offset;
static gpointer dino_ui_view_model_preferences_row_private_text_parent_class = NULL;
static gint DinoUiViewModelPreferencesRowToggle_private_offset;
static gpointer dino_ui_view_model_preferences_row_toggle_parent_class = NULL;
static gint DinoUiViewModelPreferencesRowComboBox_private_offset;
static gpointer dino_ui_view_model_preferences_row_combo_box_parent_class = NULL;
static gpointer dino_ui_view_model_preferences_row_widget_deprecated_parent_class = NULL;

VALA_EXTERN GType dino_ui_view_model_preferences_row_any_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowAny, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowAny* dino_ui_view_model_preferences_row_any_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_view_model_preferences_row_any_get_title (DinoUiViewModelPreferencesRowAny* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_any_set_title (DinoUiViewModelPreferencesRowAny* self,
                                                       const gchar* value);
static void dino_ui_view_model_preferences_row_any_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_any_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_any_get_property (GObject * object,
                                                                guint property_id,
                                                                GValue * value,
                                                                GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_any_set_property (GObject * object,
                                                                guint property_id,
                                                                const GValue * value,
                                                                GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_text_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowText, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowText* dino_ui_view_model_preferences_row_text_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowText* dino_ui_view_model_preferences_row_text_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_view_model_preferences_row_text_get_text (DinoUiViewModelPreferencesRowText* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_text_set_text (DinoUiViewModelPreferencesRowText* self,
                                                       const gchar* value);
static void dino_ui_view_model_preferences_row_text_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_text_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_text_get_property (GObject * object,
                                                                 guint property_id,
                                                                 GValue * value,
                                                                 GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_text_set_property (GObject * object,
                                                                 guint property_id,
                                                                 const GValue * value,
                                                                 GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowEntry, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowEntry* dino_ui_view_model_preferences_row_entry_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowEntry* dino_ui_view_model_preferences_row_entry_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_view_model_preferences_row_entry_get_text (DinoUiViewModelPreferencesRowEntry* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_entry_set_text (DinoUiViewModelPreferencesRowEntry* self,
                                                        const gchar* value);
static void dino_ui_view_model_preferences_row_entry_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_entry_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_entry_get_property (GObject * object,
                                                                  guint property_id,
                                                                  GValue * value,
                                                                  GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_entry_set_property (GObject * object,
                                                                  guint property_id,
                                                                  const GValue * value,
                                                                  GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_private_text_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowPrivateText, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowPrivateText* dino_ui_view_model_preferences_row_private_text_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowPrivateText* dino_ui_view_model_preferences_row_private_text_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_view_model_preferences_row_private_text_get_text (DinoUiViewModelPreferencesRowPrivateText* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_private_text_set_text (DinoUiViewModelPreferencesRowPrivateText* self,
                                                               const gchar* value);
static void dino_ui_view_model_preferences_row_private_text_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_private_text_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_private_text_get_property (GObject * object,
                                                                         guint property_id,
                                                                         GValue * value,
                                                                         GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_private_text_set_property (GObject * object,
                                                                         guint property_id,
                                                                         const GValue * value,
                                                                         GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_toggle_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowToggle, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowToggle* dino_ui_view_model_preferences_row_toggle_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowToggle* dino_ui_view_model_preferences_row_toggle_construct (GType object_type);
VALA_EXTERN const gchar* dino_ui_view_model_preferences_row_toggle_get_subtitle (DinoUiViewModelPreferencesRowToggle* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_toggle_set_subtitle (DinoUiViewModelPreferencesRowToggle* self,
                                                             const gchar* value);
VALA_EXTERN gboolean dino_ui_view_model_preferences_row_toggle_get_state (DinoUiViewModelPreferencesRowToggle* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_toggle_set_state (DinoUiViewModelPreferencesRowToggle* self,
                                                          gboolean value);
static void dino_ui_view_model_preferences_row_toggle_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_toggle_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_toggle_get_property (GObject * object,
                                                                   guint property_id,
                                                                   GValue * value,
                                                                   GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_toggle_set_property (GObject * object,
                                                                   guint property_id,
                                                                   const GValue * value,
                                                                   GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_combo_box_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowComboBox, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowComboBox* dino_ui_view_model_preferences_row_combo_box_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowComboBox* dino_ui_view_model_preferences_row_combo_box_construct (GType object_type);
VALA_EXTERN gint dino_ui_view_model_preferences_row_combo_box_get_active_item (DinoUiViewModelPreferencesRowComboBox* self);
VALA_EXTERN void dino_ui_view_model_preferences_row_combo_box_set_active_item (DinoUiViewModelPreferencesRowComboBox* self,
                                                                   gint value);
static void dino_ui_view_model_preferences_row_combo_box_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_combo_box_get_type_once (void);
static void _vala_dino_ui_view_model_preferences_row_combo_box_get_property (GObject * object,
                                                                      guint property_id,
                                                                      GValue * value,
                                                                      GParamSpec * pspec);
static void _vala_dino_ui_view_model_preferences_row_combo_box_set_property (GObject * object,
                                                                      guint property_id,
                                                                      const GValue * value,
                                                                      GParamSpec * pspec);
VALA_EXTERN GType dino_ui_view_model_preferences_row_widget_deprecated_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DinoUiViewModelPreferencesRowWidgetDeprecated, g_object_unref)
VALA_EXTERN DinoUiViewModelPreferencesRowWidgetDeprecated* dino_ui_view_model_preferences_row_widget_deprecated_new (void);
VALA_EXTERN DinoUiViewModelPreferencesRowWidgetDeprecated* dino_ui_view_model_preferences_row_widget_deprecated_construct (GType object_type);
static void dino_ui_view_model_preferences_row_widget_deprecated_finalize (GObject * obj);
static GType dino_ui_view_model_preferences_row_widget_deprecated_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
dino_ui_view_model_preferences_row_any_get_instance_private (DinoUiViewModelPreferencesRowAny* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowAny_private_offset);
}

DinoUiViewModelPreferencesRowAny*
dino_ui_view_model_preferences_row_any_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowAny * self = NULL;
	self = (DinoUiViewModelPreferencesRowAny*) g_object_new (object_type, NULL);
	return self;
}

const gchar*
dino_ui_view_model_preferences_row_any_get_title (DinoUiViewModelPreferencesRowAny* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_title;
	result = _tmp0_;
	return result;
}

void
dino_ui_view_model_preferences_row_any_set_title (DinoUiViewModelPreferencesRowAny* self,
                                                  const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_any_get_title (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_title);
		self->priv->_title = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_any_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_any_class_init (DinoUiViewModelPreferencesRowAnyClass * klass,
                                                   gpointer klass_data)
{
	dino_ui_view_model_preferences_row_any_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowAny_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_any_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_any_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_any_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY, dino_ui_view_model_preferences_row_any_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY] = g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_view_model_preferences_row_any_instance_init (DinoUiViewModelPreferencesRowAny * self,
                                                      gpointer klass)
{
	self->priv = dino_ui_view_model_preferences_row_any_get_instance_private (self);
}

static void
dino_ui_view_model_preferences_row_any_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowAny * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAny);
	_g_free0 (self->priv->_title);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_any_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_any_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowAnyClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_any_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowAny), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_any_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_any_type_id;
	dino_ui_view_model_preferences_row_any_type_id = g_type_register_static (G_TYPE_OBJECT, "DinoUiViewModelPreferencesRowAny", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	DinoUiViewModelPreferencesRowAny_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_any_type_id, sizeof (DinoUiViewModelPreferencesRowAnyPrivate));
	return dino_ui_view_model_preferences_row_any_type_id;
}

GType
dino_ui_view_model_preferences_row_any_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_any_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_any_type_id__once)) {
		GType dino_ui_view_model_preferences_row_any_type_id;
		dino_ui_view_model_preferences_row_any_type_id = dino_ui_view_model_preferences_row_any_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_any_type_id__once, dino_ui_view_model_preferences_row_any_type_id);
	}
	return dino_ui_view_model_preferences_row_any_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_any_get_property (GObject * object,
                                                           guint property_id,
                                                           GValue * value,
                                                           GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowAny * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAny);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY:
		g_value_set_string (value, dino_ui_view_model_preferences_row_any_get_title (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_any_set_property (GObject * object,
                                                           guint property_id,
                                                           const GValue * value,
                                                           GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowAny * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, DinoUiViewModelPreferencesRowAny);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ANY_TITLE_PROPERTY:
		dino_ui_view_model_preferences_row_any_set_title (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_view_model_preferences_row_text_get_instance_private (DinoUiViewModelPreferencesRowText* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowText_private_offset);
}

DinoUiViewModelPreferencesRowText*
dino_ui_view_model_preferences_row_text_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowText * self = NULL;
	self = (DinoUiViewModelPreferencesRowText*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowText*
dino_ui_view_model_preferences_row_text_new (void)
{
	return dino_ui_view_model_preferences_row_text_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT);
}

const gchar*
dino_ui_view_model_preferences_row_text_get_text (DinoUiViewModelPreferencesRowText* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_text;
	result = _tmp0_;
	return result;
}

void
dino_ui_view_model_preferences_row_text_set_text (DinoUiViewModelPreferencesRowText* self,
                                                  const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_text_get_text (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_text);
		self->priv->_text = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_text_class_init (DinoUiViewModelPreferencesRowTextClass * klass,
                                                    gpointer klass_data)
{
	dino_ui_view_model_preferences_row_text_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowText_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_text_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_text_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_text_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY, dino_ui_view_model_preferences_row_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY] = g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_view_model_preferences_row_text_instance_init (DinoUiViewModelPreferencesRowText * self,
                                                       gpointer klass)
{
	self->priv = dino_ui_view_model_preferences_row_text_get_instance_private (self);
}

static void
dino_ui_view_model_preferences_row_text_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowText);
	_g_free0 (self->priv->_text);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_text_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_text_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowTextClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_text_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowText), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_text_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_text_type_id;
	dino_ui_view_model_preferences_row_text_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowText", &g_define_type_info, 0);
	DinoUiViewModelPreferencesRowText_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_text_type_id, sizeof (DinoUiViewModelPreferencesRowTextPrivate));
	return dino_ui_view_model_preferences_row_text_type_id;
}

GType
dino_ui_view_model_preferences_row_text_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_text_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_text_type_id__once)) {
		GType dino_ui_view_model_preferences_row_text_type_id;
		dino_ui_view_model_preferences_row_text_type_id = dino_ui_view_model_preferences_row_text_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_text_type_id__once, dino_ui_view_model_preferences_row_text_type_id);
	}
	return dino_ui_view_model_preferences_row_text_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_text_get_property (GObject * object,
                                                            guint property_id,
                                                            GValue * value,
                                                            GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowText);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY:
		g_value_set_string (value, dino_ui_view_model_preferences_row_text_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_text_set_property (GObject * object,
                                                            guint property_id,
                                                            const GValue * value,
                                                            GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TEXT, DinoUiViewModelPreferencesRowText);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TEXT_TEXT_PROPERTY:
		dino_ui_view_model_preferences_row_text_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_view_model_preferences_row_entry_get_instance_private (DinoUiViewModelPreferencesRowEntry* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowEntry_private_offset);
}

DinoUiViewModelPreferencesRowEntry*
dino_ui_view_model_preferences_row_entry_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowEntry * self = NULL;
	self = (DinoUiViewModelPreferencesRowEntry*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowEntry*
dino_ui_view_model_preferences_row_entry_new (void)
{
	return dino_ui_view_model_preferences_row_entry_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY);
}

const gchar*
dino_ui_view_model_preferences_row_entry_get_text (DinoUiViewModelPreferencesRowEntry* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_text;
	result = _tmp0_;
	return result;
}

void
dino_ui_view_model_preferences_row_entry_set_text (DinoUiViewModelPreferencesRowEntry* self,
                                                   const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_entry_get_text (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_text);
		self->priv->_text = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_entry_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_entry_class_init (DinoUiViewModelPreferencesRowEntryClass * klass,
                                                     gpointer klass_data)
{
	dino_ui_view_model_preferences_row_entry_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowEntry_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_entry_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_entry_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_entry_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY, dino_ui_view_model_preferences_row_entry_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY] = g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	dino_ui_view_model_preferences_row_entry_signals[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_CHANGED_SIGNAL] = g_signal_new ("changed", DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
dino_ui_view_model_preferences_row_entry_instance_init (DinoUiViewModelPreferencesRowEntry * self,
                                                        gpointer klass)
{
	self->priv = dino_ui_view_model_preferences_row_entry_get_instance_private (self);
}

static void
dino_ui_view_model_preferences_row_entry_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntry);
	_g_free0 (self->priv->_text);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_entry_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_entry_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowEntry), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_entry_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_entry_type_id;
	dino_ui_view_model_preferences_row_entry_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowEntry", &g_define_type_info, 0);
	DinoUiViewModelPreferencesRowEntry_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_entry_type_id, sizeof (DinoUiViewModelPreferencesRowEntryPrivate));
	return dino_ui_view_model_preferences_row_entry_type_id;
}

GType
dino_ui_view_model_preferences_row_entry_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_entry_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_entry_type_id__once)) {
		GType dino_ui_view_model_preferences_row_entry_type_id;
		dino_ui_view_model_preferences_row_entry_type_id = dino_ui_view_model_preferences_row_entry_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_entry_type_id__once, dino_ui_view_model_preferences_row_entry_type_id);
	}
	return dino_ui_view_model_preferences_row_entry_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_entry_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntry);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY:
		g_value_set_string (value, dino_ui_view_model_preferences_row_entry_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_entry_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowEntry * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ENTRY, DinoUiViewModelPreferencesRowEntry);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_ENTRY_TEXT_PROPERTY:
		dino_ui_view_model_preferences_row_entry_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_view_model_preferences_row_private_text_get_instance_private (DinoUiViewModelPreferencesRowPrivateText* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowPrivateText_private_offset);
}

DinoUiViewModelPreferencesRowPrivateText*
dino_ui_view_model_preferences_row_private_text_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowPrivateText * self = NULL;
	self = (DinoUiViewModelPreferencesRowPrivateText*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowPrivateText*
dino_ui_view_model_preferences_row_private_text_new (void)
{
	return dino_ui_view_model_preferences_row_private_text_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT);
}

const gchar*
dino_ui_view_model_preferences_row_private_text_get_text (DinoUiViewModelPreferencesRowPrivateText* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_text;
	result = _tmp0_;
	return result;
}

void
dino_ui_view_model_preferences_row_private_text_set_text (DinoUiViewModelPreferencesRowPrivateText* self,
                                                          const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_private_text_get_text (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_text);
		self->priv->_text = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_private_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_private_text_class_init (DinoUiViewModelPreferencesRowPrivateTextClass * klass,
                                                            gpointer klass_data)
{
	dino_ui_view_model_preferences_row_private_text_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowPrivateText_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_private_text_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_private_text_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_private_text_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY, dino_ui_view_model_preferences_row_private_text_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY] = g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	dino_ui_view_model_preferences_row_private_text_signals[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_CHANGED_SIGNAL] = g_signal_new ("changed", DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
dino_ui_view_model_preferences_row_private_text_instance_init (DinoUiViewModelPreferencesRowPrivateText * self,
                                                               gpointer klass)
{
	self->priv = dino_ui_view_model_preferences_row_private_text_get_instance_private (self);
}

static void
dino_ui_view_model_preferences_row_private_text_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowPrivateText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateText);
	_g_free0 (self->priv->_text);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_private_text_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_private_text_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowPrivateTextClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_private_text_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowPrivateText), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_private_text_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_private_text_type_id;
	dino_ui_view_model_preferences_row_private_text_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowPrivateText", &g_define_type_info, 0);
	DinoUiViewModelPreferencesRowPrivateText_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_private_text_type_id, sizeof (DinoUiViewModelPreferencesRowPrivateTextPrivate));
	return dino_ui_view_model_preferences_row_private_text_type_id;
}

GType
dino_ui_view_model_preferences_row_private_text_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_private_text_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_private_text_type_id__once)) {
		GType dino_ui_view_model_preferences_row_private_text_type_id;
		dino_ui_view_model_preferences_row_private_text_type_id = dino_ui_view_model_preferences_row_private_text_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_private_text_type_id__once, dino_ui_view_model_preferences_row_private_text_type_id);
	}
	return dino_ui_view_model_preferences_row_private_text_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_private_text_get_property (GObject * object,
                                                                    guint property_id,
                                                                    GValue * value,
                                                                    GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowPrivateText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateText);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY:
		g_value_set_string (value, dino_ui_view_model_preferences_row_private_text_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_private_text_set_property (GObject * object,
                                                                    guint property_id,
                                                                    const GValue * value,
                                                                    GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowPrivateText * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_PRIVATE_TEXT, DinoUiViewModelPreferencesRowPrivateText);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_PRIVATE_TEXT_TEXT_PROPERTY:
		dino_ui_view_model_preferences_row_private_text_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_view_model_preferences_row_toggle_get_instance_private (DinoUiViewModelPreferencesRowToggle* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowToggle_private_offset);
}

DinoUiViewModelPreferencesRowToggle*
dino_ui_view_model_preferences_row_toggle_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowToggle * self = NULL;
	self = (DinoUiViewModelPreferencesRowToggle*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowToggle*
dino_ui_view_model_preferences_row_toggle_new (void)
{
	return dino_ui_view_model_preferences_row_toggle_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE);
}

const gchar*
dino_ui_view_model_preferences_row_toggle_get_subtitle (DinoUiViewModelPreferencesRowToggle* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_subtitle;
	result = _tmp0_;
	return result;
}

void
dino_ui_view_model_preferences_row_toggle_set_subtitle (DinoUiViewModelPreferencesRowToggle* self,
                                                        const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_toggle_get_subtitle (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_subtitle);
		self->priv->_subtitle = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_toggle_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY]);
	}
}

gboolean
dino_ui_view_model_preferences_row_toggle_get_state (DinoUiViewModelPreferencesRowToggle* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_state;
	return result;
}

void
dino_ui_view_model_preferences_row_toggle_set_state (DinoUiViewModelPreferencesRowToggle* self,
                                                     gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_toggle_get_state (self);
	if (old_value != value) {
		self->priv->_state = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_toggle_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_toggle_class_init (DinoUiViewModelPreferencesRowToggleClass * klass,
                                                      gpointer klass_data)
{
	dino_ui_view_model_preferences_row_toggle_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowToggle_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_toggle_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_toggle_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_toggle_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY, dino_ui_view_model_preferences_row_toggle_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY] = g_param_spec_string ("subtitle", "subtitle", "subtitle", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY, dino_ui_view_model_preferences_row_toggle_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY] = g_param_spec_boolean ("state", "state", "state", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_view_model_preferences_row_toggle_instance_init (DinoUiViewModelPreferencesRowToggle * self,
                                                         gpointer klass)
{
	self->priv = dino_ui_view_model_preferences_row_toggle_get_instance_private (self);
}

static void
dino_ui_view_model_preferences_row_toggle_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowToggle * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggle);
	_g_free0 (self->priv->_subtitle);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_toggle_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_toggle_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowToggleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_toggle_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowToggle), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_toggle_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_toggle_type_id;
	dino_ui_view_model_preferences_row_toggle_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowToggle", &g_define_type_info, 0);
	DinoUiViewModelPreferencesRowToggle_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_toggle_type_id, sizeof (DinoUiViewModelPreferencesRowTogglePrivate));
	return dino_ui_view_model_preferences_row_toggle_type_id;
}

GType
dino_ui_view_model_preferences_row_toggle_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_toggle_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_toggle_type_id__once)) {
		GType dino_ui_view_model_preferences_row_toggle_type_id;
		dino_ui_view_model_preferences_row_toggle_type_id = dino_ui_view_model_preferences_row_toggle_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_toggle_type_id__once, dino_ui_view_model_preferences_row_toggle_type_id);
	}
	return dino_ui_view_model_preferences_row_toggle_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_toggle_get_property (GObject * object,
                                                              guint property_id,
                                                              GValue * value,
                                                              GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowToggle * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggle);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY:
		g_value_set_string (value, dino_ui_view_model_preferences_row_toggle_get_subtitle (self));
		break;
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY:
		g_value_set_boolean (value, dino_ui_view_model_preferences_row_toggle_get_state (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_toggle_set_property (GObject * object,
                                                              guint property_id,
                                                              const GValue * value,
                                                              GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowToggle * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_TOGGLE, DinoUiViewModelPreferencesRowToggle);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_SUBTITLE_PROPERTY:
		dino_ui_view_model_preferences_row_toggle_set_subtitle (self, g_value_get_string (value));
		break;
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TOGGLE_STATE_PROPERTY:
		dino_ui_view_model_preferences_row_toggle_set_state (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static inline gpointer
dino_ui_view_model_preferences_row_combo_box_get_instance_private (DinoUiViewModelPreferencesRowComboBox* self)
{
	return G_STRUCT_MEMBER_P (self, DinoUiViewModelPreferencesRowComboBox_private_offset);
}

DinoUiViewModelPreferencesRowComboBox*
dino_ui_view_model_preferences_row_combo_box_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowComboBox * self = NULL;
	self = (DinoUiViewModelPreferencesRowComboBox*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowComboBox*
dino_ui_view_model_preferences_row_combo_box_new (void)
{
	return dino_ui_view_model_preferences_row_combo_box_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX);
}

gint
dino_ui_view_model_preferences_row_combo_box_get_active_item (DinoUiViewModelPreferencesRowComboBox* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, -1);
	result = self->priv->_active_item;
	return result;
}

void
dino_ui_view_model_preferences_row_combo_box_set_active_item (DinoUiViewModelPreferencesRowComboBox* self,
                                                              gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = dino_ui_view_model_preferences_row_combo_box_get_active_item (self);
	if (old_value != value) {
		self->priv->_active_item = value;
		g_object_notify_by_pspec ((GObject *) self, dino_ui_view_model_preferences_row_combo_box_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY]);
	}
}

static void
dino_ui_view_model_preferences_row_combo_box_class_init (DinoUiViewModelPreferencesRowComboBoxClass * klass,
                                                         gpointer klass_data)
{
	dino_ui_view_model_preferences_row_combo_box_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DinoUiViewModelPreferencesRowComboBox_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_dino_ui_view_model_preferences_row_combo_box_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_dino_ui_view_model_preferences_row_combo_box_set_property;
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_combo_box_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY, dino_ui_view_model_preferences_row_combo_box_properties[DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY] = g_param_spec_int ("active-item", "active-item", "active-item", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
dino_ui_view_model_preferences_row_combo_box_instance_init (DinoUiViewModelPreferencesRowComboBox * self,
                                                            gpointer klass)
{
	GeeArrayList* _tmp0_;
	self->priv = dino_ui_view_model_preferences_row_combo_box_get_instance_private (self);
	_tmp0_ = gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL);
	self->items = (GeeList*) _tmp0_;
}

static void
dino_ui_view_model_preferences_row_combo_box_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowComboBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBox);
	_g_object_unref0 (self->items);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_combo_box_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_combo_box_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowComboBoxClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_combo_box_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowComboBox), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_combo_box_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_combo_box_type_id;
	dino_ui_view_model_preferences_row_combo_box_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowComboBox", &g_define_type_info, 0);
	DinoUiViewModelPreferencesRowComboBox_private_offset = g_type_add_instance_private (dino_ui_view_model_preferences_row_combo_box_type_id, sizeof (DinoUiViewModelPreferencesRowComboBoxPrivate));
	return dino_ui_view_model_preferences_row_combo_box_type_id;
}

GType
dino_ui_view_model_preferences_row_combo_box_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_combo_box_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_combo_box_type_id__once)) {
		GType dino_ui_view_model_preferences_row_combo_box_type_id;
		dino_ui_view_model_preferences_row_combo_box_type_id = dino_ui_view_model_preferences_row_combo_box_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_combo_box_type_id__once, dino_ui_view_model_preferences_row_combo_box_type_id);
	}
	return dino_ui_view_model_preferences_row_combo_box_type_id__once;
}

static void
_vala_dino_ui_view_model_preferences_row_combo_box_get_property (GObject * object,
                                                                 guint property_id,
                                                                 GValue * value,
                                                                 GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowComboBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBox);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY:
		g_value_set_int (value, dino_ui_view_model_preferences_row_combo_box_get_active_item (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_dino_ui_view_model_preferences_row_combo_box_set_property (GObject * object,
                                                                 guint property_id,
                                                                 const GValue * value,
                                                                 GParamSpec * pspec)
{
	DinoUiViewModelPreferencesRowComboBox * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_COMBO_BOX, DinoUiViewModelPreferencesRowComboBox);
	switch (property_id) {
		case DINO_UI_VIEW_MODEL_PREFERENCES_ROW_COMBO_BOX_ACTIVE_ITEM_PROPERTY:
		dino_ui_view_model_preferences_row_combo_box_set_active_item (self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

DinoUiViewModelPreferencesRowWidgetDeprecated*
dino_ui_view_model_preferences_row_widget_deprecated_construct (GType object_type)
{
	DinoUiViewModelPreferencesRowWidgetDeprecated * self = NULL;
	self = (DinoUiViewModelPreferencesRowWidgetDeprecated*) dino_ui_view_model_preferences_row_any_construct (object_type);
	return self;
}

DinoUiViewModelPreferencesRowWidgetDeprecated*
dino_ui_view_model_preferences_row_widget_deprecated_new (void)
{
	return dino_ui_view_model_preferences_row_widget_deprecated_construct (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED);
}

static void
dino_ui_view_model_preferences_row_widget_deprecated_class_init (DinoUiViewModelPreferencesRowWidgetDeprecatedClass * klass,
                                                                 gpointer klass_data)
{
	dino_ui_view_model_preferences_row_widget_deprecated_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = dino_ui_view_model_preferences_row_widget_deprecated_finalize;
}

static void
dino_ui_view_model_preferences_row_widget_deprecated_instance_init (DinoUiViewModelPreferencesRowWidgetDeprecated * self,
                                                                    gpointer klass)
{
}

static void
dino_ui_view_model_preferences_row_widget_deprecated_finalize (GObject * obj)
{
	DinoUiViewModelPreferencesRowWidgetDeprecated * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_WIDGET_DEPRECATED, DinoUiViewModelPreferencesRowWidgetDeprecated);
	_g_object_unref0 (self->widget);
	G_OBJECT_CLASS (dino_ui_view_model_preferences_row_widget_deprecated_parent_class)->finalize (obj);
}

static GType
dino_ui_view_model_preferences_row_widget_deprecated_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DinoUiViewModelPreferencesRowWidgetDeprecatedClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dino_ui_view_model_preferences_row_widget_deprecated_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DinoUiViewModelPreferencesRowWidgetDeprecated), 0, (GInstanceInitFunc) dino_ui_view_model_preferences_row_widget_deprecated_instance_init, NULL };
	GType dino_ui_view_model_preferences_row_widget_deprecated_type_id;
	dino_ui_view_model_preferences_row_widget_deprecated_type_id = g_type_register_static (DINO_UI_VIEW_MODEL_PREFERENCES_ROW_TYPE_ANY, "DinoUiViewModelPreferencesRowWidgetDeprecated", &g_define_type_info, 0);
	return dino_ui_view_model_preferences_row_widget_deprecated_type_id;
}

GType
dino_ui_view_model_preferences_row_widget_deprecated_get_type (void)
{
	static volatile gsize dino_ui_view_model_preferences_row_widget_deprecated_type_id__once = 0;
	if (g_once_init_enter (&dino_ui_view_model_preferences_row_widget_deprecated_type_id__once)) {
		GType dino_ui_view_model_preferences_row_widget_deprecated_type_id;
		dino_ui_view_model_preferences_row_widget_deprecated_type_id = dino_ui_view_model_preferences_row_widget_deprecated_get_type_once ();
		g_once_init_leave (&dino_ui_view_model_preferences_row_widget_deprecated_type_id__once, dino_ui_view_model_preferences_row_widget_deprecated_type_id);
	}
	return dino_ui_view_model_preferences_row_widget_deprecated_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

